#include "main.h"

FLASH_EraseInitTypeDef EraseInitStruct;
uint32_t Address = 0, PageError = 0;
uint32_t FirstPage = 0, NbOfPages = 0, BankNumber = 0;

static uint32_t GetPage(uint32_t Addr)
{
  uint32_t page = 0;

  if (Addr < (FLASH_BASE + FLASH_BANK_SIZE))
  {
    /* Bank 1 */
    page = (Addr - FLASH_BASE) / FLASH_PAGE_SIZE;
  }
  else
  {
    /* Bank 2 */
    page = (Addr - (FLASH_BASE + FLASH_BANK_SIZE)) / FLASH_PAGE_SIZE;
  }

  return page;
}

static uint32_t GetBank(uint32_t Addr)
{

  if (Addr < (FLASH_BASE + FLASH_BANK_SIZE))
  {
    /* Bank 1 */
	  return FLASH_BANK_1;
  }
  else
  {
    /* Bank 2 */
	  return FLASH_BANK_2;
  }
}

//void Flash_WR_UpdateFlag(uint32_t flag)
//{
//	uint32_t QuadWord[4] = {0x00000000,0x00000000,0x00000000,0x00000000};
//
//	QuadWord[0] = Flash_RD_DevID();
//	QuadWord[1] = Flash_RD_FireFlag();
//	QuadWord[2] = flag;
//
//	Flash_ErasePage(MISC_DATA_SAVE_ADDR);
//
//	HAL_FLASH_Unlock();
//
//	if(HAL_FLASH_Program(FLASH_TYPEPROGRAM_QUADWORD, MISC_DATA_SAVE_ADDR, QuadWord) != HAL_OK)
//	{
//		while (1)
//		{
//			Error_Handler();
//		}
//	}
//
//	HAL_FLASH_Lock();
//}

uint32_t Flash_RD_UpdateFlag(void)
{
	return *(uint32_t*)UPDATE_FLAG_ADDR;
}

//void Flash_WR_FireFlag(uint32_t flag)
//{
//	uint32_t QuadWord[4] = {0x00000000,0x00000000,0x00000000,0x00000000};
//
//	QuadWord[0] = Flash_RD_DevID();
//	QuadWord[1] = flag;
//	QuadWord[2] = Flash_RD_UpdateFlag();
//
//	Flash_ErasePage(MISC_DATA_SAVE_ADDR);
//
//	HAL_FLASH_Unlock();
//
//	if(HAL_FLASH_Program(FLASH_TYPEPROGRAM_QUADWORD, MISC_DATA_SAVE_ADDR, QuadWord) != HAL_OK)
//	{
//		while (1)
//		{
//			Error_Handler();
//		}
//	}
//
//	HAL_FLASH_Lock();
//}

uint32_t Flash_RD_FireFlag(void)
{
	return *(uint32_t*)FIRE_FLAG_ADDR;
}

//void Flash_WR_DevID(uint32_t id)
//{
//	uint32_t QuadWord[4] = {0x00000000,0x00000000,0x00000000,0x00000000};
//
//	QuadWord[0] = id;
//	QuadWord[1] = Flash_RD_FireFlag();
//	QuadWord[2] = Flash_RD_UpdateFlag();
//
//	Flash_ErasePage(MISC_DATA_SAVE_ADDR);
//
//	HAL_FLASH_Unlock();
//
//	if(HAL_FLASH_Program(FLASH_TYPEPROGRAM_QUADWORD, MISC_DATA_SAVE_ADDR, QuadWord) != HAL_OK)
//	{
//		while (1)
//		{
//			Error_Handler();
//		}
//	}
//
//	HAL_FLASH_Lock();
//}

uint32_t Flash_RD_DevID(void)
{
	return *(uint32_t*)DEV_ID_ADDR;
}

void Flash_ReadPage(uint32_t addr)
{

}

void Flash_ErasePage(uint32_t addr)
{
	HAL_FLASH_Unlock();

	FirstPage = GetPage(addr);
	NbOfPages = GetPage(addr) - FirstPage + 1;
	BankNumber = GetBank(addr);

	EraseInitStruct.TypeErase   = FLASH_TYPEERASE_PAGES;
	EraseInitStruct.Banks       = BankNumber;
	EraseInitStruct.Page        = FirstPage;
	EraseInitStruct.NbPages     = NbOfPages;

	if (HAL_FLASHEx_Erase(&EraseInitStruct, &PageError) != HAL_OK)
	{
		while (1)
		{
			Error_Handler();
		}
	}

	HAL_FLASH_Lock();
}

uint32_t FlashEDdrAccData_WR(unsigned char protectFlag,EDR_AccData *data,uint32_t dataLen)
{
	static uint8_t protectIndex = 0;
	static uint8_t index = 0;
	uint32_t addr[3] = {FLASH_PROTECKT_ACC_DATA_ADDR0,FLASH_PROTECKT_ACC_DATA_ADDR1,FLASH_PROTECKT_ACC_DATA_ADDR2};
	uint32_t addr1[3] = {FLASH_ACC_DATA_ADDR0,FLASH_ACC_DATA_ADDR1,FLASH_ACC_DATA_ADDR2};
	uint32_t usrAddr = 0;

	if(protectFlag)
	{
		if(protectIndex == 3)
			return;
		usrAddr = addr[protectIndex++];
	}
	else
	{
		usrAddr = addr1[index++];
		Flash_ErasePage(usrAddr);
		if(index == 3) index = 0;
	}

	HAL_FLASH_Unlock();

	while(dataLen)
	{
		if(HAL_FLASH_Program(FLASH_TYPEPROGRAM_QUADWORD, usrAddr, (uint32_t*)data) != HAL_OK)
		{
			while (1)
			{
				Error_Handler();
			}
		}
		dataLen -= 4;
		data += 4;
		usrAddr += 16;
	}

	HAL_FLASH_Lock();

	return 0;
}
