/*
 * flash_user.c
 *
 *  Created on: Mar 7, 2024
 *      Author: liuzhimin
 */

#ifndef SRC_FLASH_USER_C_
#define SRC_FLASH_USER_C_


#include "main.h"



#define FLASH_ADDR_WR    		 ((uint32_t)0x0802B000)

#define FLASH_ID_ADDR   		 (FLASH_ADDR_WR)
#define FLASH_SN_ADDR    		 (FLASH_ADDR_WR+4)
#define FLASH_UPDATE_ADDR   	 (FLASH_ADDR_WR+8)
#define FLASH_SN2_ADDR   		 (FLASH_ADDR_WR+12)

#define POWER1_FLAG_ADDR  		 (FLASH_ADDR_WR+16)
#define POWER2_FLAG_ADDR  		 (FLASH_ADDR_WR+20)
#define POWER3_FLAG_ADDR  		 (FLASH_ADDR_WR+24)
#define POWER4_FLAG_ADDR  		 (FLASH_ADDR_WR+28)

#define ACC_OFFSET1_FLAG_ADDR  	 (FLASH_ADDR_WR+32)
#define ACC_OFFSET2_FLAG_ADDR  	 (FLASH_ADDR_WR+36)
#define SN1_CHAR1_FLASH_ADDR  	 (FLASH_ADDR_WR+40)
#define SN2_CHAR1_FLASH_ADDR  	 (FLASH_ADDR_WR+44)

#define SN1_CHAR2_FLASH_ADDR  	 (FLASH_ADDR_WR+48)
#define SN1_CHAR3_FLASH_ADDR  	 (FLASH_ADDR_WR+52)
#define SN1_CHAR4_FLASH_ADDR  	 (FLASH_ADDR_WR+56)
#define SN1_CHAR5_FLASH_ADDR  	 (FLASH_ADDR_WR+60)

#define SN2_CHAR2_FLASH_ADDR  	 (FLASH_ADDR_WR+64)
#define SN2_CHAR3_FLASH_ADDR  	 (FLASH_ADDR_WR+68)
#define SN2_CHAR4_FLASH_ADDR  	 (FLASH_ADDR_WR+72)
#define SN2_CHAR5_FLASH_ADDR  	 (FLASH_ADDR_WR+76)


#define RAM_QUADWORD_LEN   20


#define  FLASH_DATA_ACC_OFFSET_MASK  0x00FFFFFF

#define  FLASH_DATA_NULL		  	 0xFFFFFFFF
#define  FLASH_DATA_DEFAULT		  	 0x00000000

typedef enum {
	FLASH_INDEX_DEV_ID = 0,
	FLASH_INDEX_DEV_SN,
	FLASH_INDEX_UPDATE_FLG,
	FLASH_INDEX_DEV_SN2,
	FLASH_INDEX_POWER_ON_FLG1,
	FLASH_INDEX_POWER_ON_FLG2,
	FLASH_INDEX_POWER_ON_FLG3,
	FLASH_INDEX_POWER_ON_FLG4,
	FLASH_INDEX_OFFSET1_FLG,
	FLASH_INDEX_OFFSET2_FLG,

	FLASH_INDEX_SN1_CHAR1,
	FLASH_INDEX_SN2_CHAR1,

	FLASH_INDEX_SN1_CHAR2,
	FLASH_INDEX_SN1_CHAR3,
	FLASH_INDEX_SN1_CHAR4,
	FLASH_INDEX_SN1_CHAR5,

	FLASH_INDEX_SN2_CHAR2,
	FLASH_INDEX_SN2_CHAR3,
	FLASH_INDEX_SN2_CHAR4,
	FLASH_INDEX_SN2_CHAR5,

//	FLASH_INDEX_RESERVED_FLG1,
//	FLASH_INDEX_RESERVED_FLG2,
//	FLASH_INDEX_RESERVED_FLG3,
//	FLASH_INDEX_RESERVED_FLG4,
//	FLASH_INDEX_RESERVED_FLG5,
//	FLASH_INDEX_RESERVED_FLG6,

	FLASH_INDEX_MAX,
} FLASH_INDEX;

void MX_FLASH_Init(void)
{





#ifdef  DOWNLOAD_SELECTION_ENABLE

	FLASH_OBProgramInitTypeDef OBInit;
	HAL_FLASH_Unlock();
	HAL_FLASH_OB_Unlock();

	HAL_FLASHEx_OBGetConfig(&OBInit);
	if(OBInit.RDPLevel != OB_RDP_LEVEL_1){
		OBInit.OptionType = OPTIONBYTE_RDP;
		OBInit.RDPLevel = OB_RDP_LEVEL_1;
		HAL_FLASHEx_OBProgram(&OBInit);
	}

//	if(OBInit.RDPLevel != OB_RDP_LEVEL_0){
//		OBInit.OptionType = OPTIONBYTE_USER;
//		OBInit.RDPLevel = OB_RDP_LEVEL_0;
//		HAL_FLASHEx_OBProgram(&OBInit);
//	}



	HAL_FLASH_OB_Lock();
	HAL_FLASH_Lock();
#else

#endif
}


void MX_FLASH_UnInit(void)
{
	FLASH_OBProgramInitTypeDef OBInit;

	HAL_FLASH_Unlock();
	HAL_FLASH_OB_Unlock();

	HAL_FLASHEx_OBGetConfig(&OBInit);
	if(OBInit.RDPLevel != OB_RDP_LEVEL_0){
		OBInit.OptionType = OPTIONBYTE_USER;
		OBInit.RDPLevel = OB_RDP_LEVEL_0;
		HAL_FLASHEx_OBProgram(&OBInit);
	}

	HAL_FLASH_OB_Lock();
	HAL_FLASH_Lock();
}




void MX_Flash_RDAllData(uint32_t* DatBuf)
{
	DatBuf[FLASH_INDEX_DEV_ID] = *(uint32_t*)FLASH_ID_ADDR;
	DatBuf[FLASH_INDEX_DEV_SN] = *(uint32_t*)FLASH_SN_ADDR;
	DatBuf[FLASH_INDEX_UPDATE_FLG] = *(uint32_t*)FLASH_UPDATE_ADDR;
	DatBuf[FLASH_INDEX_DEV_SN2] = *(uint32_t*)FLASH_SN2_ADDR;


	DatBuf[FLASH_INDEX_POWER_ON_FLG1] = *(uint32_t*)POWER1_FLAG_ADDR;
	DatBuf[FLASH_INDEX_POWER_ON_FLG2] = *(uint32_t*)POWER2_FLAG_ADDR;
	DatBuf[FLASH_INDEX_POWER_ON_FLG3] = *(uint32_t*)POWER3_FLAG_ADDR;
	DatBuf[FLASH_INDEX_POWER_ON_FLG4] = *(uint32_t*)POWER4_FLAG_ADDR;


	DatBuf[FLASH_INDEX_OFFSET1_FLG] = *(uint32_t*)ACC_OFFSET1_FLAG_ADDR;
	DatBuf[FLASH_INDEX_OFFSET2_FLG] = *(uint32_t*)ACC_OFFSET2_FLAG_ADDR;


	DatBuf[FLASH_INDEX_SN1_CHAR1] = *(uint32_t*)SN1_CHAR1_FLASH_ADDR;
	DatBuf[FLASH_INDEX_SN1_CHAR2] = *(uint32_t*)SN1_CHAR2_FLASH_ADDR;
	DatBuf[FLASH_INDEX_SN1_CHAR3] = *(uint32_t*)SN1_CHAR3_FLASH_ADDR;
	DatBuf[FLASH_INDEX_SN1_CHAR4] = *(uint32_t*)SN1_CHAR4_FLASH_ADDR;
	DatBuf[FLASH_INDEX_SN1_CHAR5] = *(uint32_t*)SN1_CHAR5_FLASH_ADDR;
	DatBuf[FLASH_INDEX_SN2_CHAR1] = *(uint32_t*)SN2_CHAR1_FLASH_ADDR;
	DatBuf[FLASH_INDEX_SN2_CHAR2] = *(uint32_t*)SN2_CHAR2_FLASH_ADDR;
	DatBuf[FLASH_INDEX_SN2_CHAR3] = *(uint32_t*)SN2_CHAR3_FLASH_ADDR;
	DatBuf[FLASH_INDEX_SN2_CHAR4] = *(uint32_t*)SN2_CHAR4_FLASH_ADDR;
	DatBuf[FLASH_INDEX_SN2_CHAR5] = *(uint32_t*)SN2_CHAR5_FLASH_ADDR;


//	DatBuf[FLASH_INDEX_RESERVED_FLG3] = *(uint32_t*)ACC_RESERVED3_FLAG_ADDR;
//	DatBuf[FLASH_INDEX_RESERVED_FLG4] = *(uint32_t*)ACC_RESERVED4_FLAG_ADDR;
//	DatBuf[FLASH_INDEX_RESERVED_FLG5] = *(uint32_t*)ACC_RESERVED5_FLAG_ADDR;
//	DatBuf[FLASH_INDEX_RESERVED_FLG6] = *(uint32_t*)ACC_RESERVED6_FLAG_ADDR;
}

UNS8 MX_Flash_WDAllData(uint32_t* DatBuf)
{
	UNS8 err = 0;
	Flash_ErasePage(FLASH_ADDR_WR);
	HAL_FLASH_Unlock();
	err |= HAL_FLASH_Program(FLASH_TYPEPROGRAM_QUADWORD, FLASH_ADDR_WR, (uint32_t)DatBuf);
	err |= HAL_FLASH_Program(FLASH_TYPEPROGRAM_QUADWORD, FLASH_ADDR_WR+16, (uint32_t)(DatBuf+4));
	err |= HAL_FLASH_Program(FLASH_TYPEPROGRAM_QUADWORD, FLASH_ADDR_WR+32, (uint32_t)(DatBuf+8));
	err |= HAL_FLASH_Program(FLASH_TYPEPROGRAM_QUADWORD, FLASH_ADDR_WR+48, (uint32_t)(DatBuf+12));
	err |= HAL_FLASH_Program(FLASH_TYPEPROGRAM_QUADWORD, FLASH_ADDR_WR+64, (uint32_t)(DatBuf+16));


	HAL_FLASH_Lock();

	return err;
}




void MX_Flash_WR_UpdateFlag(uint32_t flag)
{

	UNS32 QuadWord[RAM_QUADWORD_LEN] = {0};
	UNS32 QuadWordRead[RAM_QUADWORD_LEN] = {0};

	MX_Flash_RDAllData(QuadWord);

	QuadWord[FLASH_INDEX_UPDATE_FLG] = flag;

	int i= 3;
	while(i--){


		MX_Flash_WDAllData(QuadWord);
		BSP_FeedWWDG();
		MX_Flash_RDAllData(QuadWordRead);

		if(0 == memcmp(QuadWord,QuadWordRead,RAM_QUADWORD_LEN * 4)){
			break;
		}
	}
}



void MX_Flash_WR_DevSN(uint32_t flag)
{
	UNS32 QuadWord[RAM_QUADWORD_LEN] = {0};
	UNS32 QuadWordRead[RAM_QUADWORD_LEN] = {0};

	MX_Flash_RDAllData(QuadWord);

	QuadWord[FLASH_INDEX_DEV_SN] = flag;

	int i= 3;
	while(i--){


		MX_Flash_WDAllData(QuadWord);
		BSP_FeedWWDG();
		MX_Flash_RDAllData(QuadWordRead);

		if(0 == memcmp(QuadWord,QuadWordRead,RAM_QUADWORD_LEN * 4)){
			break;
		}
	}
}
void MX_Flash_WR_DevSN2(uint32_t id)
{
	UNS32 QuadWord[RAM_QUADWORD_LEN] = {0};
	UNS32 QuadWordRead[RAM_QUADWORD_LEN] = {0};

	MX_Flash_RDAllData(QuadWord);

	QuadWord[FLASH_INDEX_DEV_SN2] = id;


	int i= 3;
	while(i--){


		MX_Flash_WDAllData(QuadWord);
		BSP_FeedWWDG();
		MX_Flash_RDAllData(QuadWordRead);
		if(0 == memcmp(QuadWord,QuadWordRead,RAM_QUADWORD_LEN * 4)){
			break;
		}
	}
}
void MX_Flash_WR_DevID(uint32_t id)
{
	UNS32 QuadWord[RAM_QUADWORD_LEN] = {0};
	UNS32 QuadWordRead[RAM_QUADWORD_LEN] = {0};

	MX_Flash_RDAllData(QuadWord);

	QuadWord[FLASH_INDEX_DEV_ID] = id;
	int i= 3;
	while(i--){
		MX_Flash_WDAllData(QuadWord);
		BSP_FeedWWDG();
		MX_Flash_RDAllData(QuadWordRead);
		if(0 == memcmp(QuadWord,QuadWordRead,RAM_QUADWORD_LEN * 4)){
			break;
		}
	}

}

UNS8 MX_Flash_Uint32checkSum(uint8_t *data,uint16_t len)
{
	uint8_t sum = 0;
	for(int i = 0;i < len;i++){
		sum += data[i];
	}
	data[len] = sum;
	return sum;
}

UNS16 MX_Flash_buffercheckSum(uint16_t *data,uint16_t len)
{
	uint16_t sum = 0;
	for(int i = 0; i< len;i++){
		sum += data[i];
	}
	return sum;
}

void MX_Flash_WR_AccOffset(uint32_t *offset)
{
	UNS32 QuadWord[RAM_QUADWORD_LEN] = {0};
	UNS32 QuadWordRead[RAM_QUADWORD_LEN] = {0};

	MX_Flash_RDAllData(QuadWord);
	*offset = *offset & FLASH_DATA_ACC_OFFSET_MASK;
	MX_Flash_Uint32checkSum((uint8_t*)offset,3);

	QuadWord[FLASH_INDEX_OFFSET1_FLG] = *offset;
	QuadWord[FLASH_INDEX_OFFSET2_FLG] = *offset;
	int i= 3;
	while(i--){
		MX_Flash_WDAllData(QuadWord);
		BSP_FeedWWDG();
		MX_Flash_RDAllData(QuadWordRead);

		if(0 == memcmp(QuadWord,QuadWordRead,RAM_QUADWORD_LEN * 4)){
			break;
		}
	}
}



/*18 *//*20 */
UNS8 MX_Flash_WR_SN_CharBuffer(uint8_t *buffer)
{
#define SN_CHAR_LEN_MAX  9

	UNS8 ret = 0;



	UNS8 SN_buffer[20] = {0};
	memcpy(SN_buffer,buffer,20);

	UNS32 QuadWord[RAM_QUADWORD_LEN] = {0};
	memset(QuadWord,0,SN_CHAR_LEN_MAX);

	UNS32 QuadWordRead[RAM_QUADWORD_LEN] = {0};
	memset(QuadWordRead,0,SN_CHAR_LEN_MAX);

	MX_Flash_RDAllData(QuadWord);

	UNS16 checkChar = MX_Flash_buffercheckSum((UNS16*)SN_buffer,SN_CHAR_LEN_MAX);
	uint32_t  *pbuf32 =(uint32_t*)SN_buffer;
	QuadWord[FLASH_INDEX_SN1_CHAR1] = pbuf32[4] | checkChar << 16;
	QuadWord[FLASH_INDEX_SN1_CHAR2] = pbuf32[3];
	QuadWord[FLASH_INDEX_SN1_CHAR3] = pbuf32[2];
	QuadWord[FLASH_INDEX_SN1_CHAR4] = pbuf32[1];
	QuadWord[FLASH_INDEX_SN1_CHAR5] = pbuf32[0];


	QuadWord[FLASH_INDEX_SN2_CHAR1] = pbuf32[4] | checkChar << 16;
	QuadWord[FLASH_INDEX_SN2_CHAR2] = pbuf32[3];
	QuadWord[FLASH_INDEX_SN2_CHAR3] = pbuf32[2];
	QuadWord[FLASH_INDEX_SN2_CHAR4] = pbuf32[1];
	QuadWord[FLASH_INDEX_SN2_CHAR5] = pbuf32[0];
	int i= 3;
	while(i--){
		ret = MX_Flash_WDAllData(QuadWord);
		BSP_FeedWWDG();
		MX_Flash_RDAllData(QuadWordRead);

		if(0 == memcmp(QuadWord,QuadWordRead,SN_CHAR_LEN_MAX * 4)){
			break;
		}
	}

	return ret;
}

/*18 *//*20 */
UNS8 MX_Flash_RD_SN_CharBuffer(uint8_t *buffer)
{
#define SN_CHAR_LEN_MAX  9

	UNS32 ret = TRUE;
	UNS32 QuadWord[RAM_QUADWORD_LEN] = {0};

	memset(buffer,0,20);

	QuadWord[0] = *(uint32_t*)SN1_CHAR5_FLASH_ADDR;
	QuadWord[1] = *(uint32_t*)SN1_CHAR4_FLASH_ADDR;
	QuadWord[2] = *(uint32_t*)SN1_CHAR3_FLASH_ADDR;
	QuadWord[3] = *(uint32_t*)SN1_CHAR2_FLASH_ADDR;
	QuadWord[4] = *(uint32_t*)SN1_CHAR1_FLASH_ADDR;

	QuadWord[5] = *(uint32_t*)SN2_CHAR5_FLASH_ADDR;
	QuadWord[6] = *(uint32_t*)SN2_CHAR4_FLASH_ADDR;
	QuadWord[7] = *(uint32_t*)SN2_CHAR3_FLASH_ADDR;
	QuadWord[8] = *(uint32_t*)SN2_CHAR2_FLASH_ADDR;
	QuadWord[9] = *(uint32_t*)SN2_CHAR1_FLASH_ADDR;



	UNS16 checkchar1 =  (QuadWord[4]>>16)&0xffff;
	UNS16 checkchar2 =  (QuadWord[9]>>16)&0xffff;

	UNS16 *pchar1 =  ((UNS16*)&QuadWord[0]);
	UNS16 *pchar2 =  ((UNS16*)&QuadWord[5]);


//	memcpy(buffer,(UNS8*)pchar2,SN_CHAR_LEN_MAX *2);

//	UNS16 charsum1 = MX_Flash_buffercheckSum(pchar1,SN_CHAR_LEN_MAX);
//	UNS16 charsum2 = MX_Flash_buffercheckSum(pchar2,SN_CHAR_LEN_MAX);


	if(checkchar1 != checkchar2){
		if((checkchar1 == MX_Flash_buffercheckSum(pchar1,SN_CHAR_LEN_MAX))){
			memcpy(buffer,(UNS8*)pchar1 ,SN_CHAR_LEN_MAX *2);
		}else{
				if(checkchar2 == MX_Flash_buffercheckSum(pchar1,SN_CHAR_LEN_MAX)){
					memcpy(buffer,(UNS8*)pchar2 ,SN_CHAR_LEN_MAX *2);
				}else{
					ret = FALSE;
				}
		}
	}else{
		if((checkchar1 == MX_Flash_buffercheckSum(pchar2,SN_CHAR_LEN_MAX))){
			memcpy(buffer,(UNS8*)pchar2 ,SN_CHAR_LEN_MAX *2);
		}else{
			ret = FALSE;
		}
	}

	return ret;
}



uint32_t MX_Flash_RD_DevSN2(void)
{
	return *(uint32_t*)FLASH_SN2_ADDR;
}

uint32_t MX_Flash_RD_DevSN(void)
{
	return *(uint32_t*)FLASH_SN_ADDR;
}
uint32_t MX_Flash_RD_UpdateFlag(void)
{
	return *(uint32_t*)FLASH_UPDATE_ADDR;
}

uint32_t MX_Flash_RD_DevID(void)
{
	return *(uint32_t*)FLASH_ID_ADDR;
}

uint32_t MX_Flash_RD_PowerOnRecordFlag(void)
{
	#define NO_CODE   0xFFFFFFFF
//	#define ERR_CODE  0xFFFFFFFE
//	UNS32 flg1 = *(uint32_t*)POWER1_FLAG_ADDR;
//	UNS32 flg2 = *(uint32_t*)POWER2_FLAG_ADDR;
//	UNS32 flg3 = *(uint32_t*)POWER3_FLAG_ADDR;
//	UNS32 flg4 = *(uint32_t*)POWER4_FLAG_ADDR;
//	UNS32 ret = ERR_CODE;
//	if(flg1 == flg2){
//		if((flg1 == NO_CODE)&&(flg3 == flg4)&&(flg3 == NO_CODE)){
//			ret  = 0;
//		}else{
//			if((flg3 == flg4)
//			&&(flg3 == VER_CRC32(flg1,0))){
//					ret = flg1;
//			}
//		}
//	}
	return (0);
}





uint32_t MX_Flash_RD_AccOffset(uint32_t *offset)
{
	UNS32 ret = TRUE;

	UNS32 data1 = *(uint32_t*)ACC_OFFSET1_FLAG_ADDR;
	UNS32 data2 = *(uint32_t*)ACC_OFFSET2_FLAG_ADDR;
	*offset  = FLASH_DATA_DEFAULT;
	if(data1 != data2){
		if((data1 != FLASH_DATA_NULL)&&(*(uint8_t*)(data2 >> 24) == MX_Flash_Uint32checkSum((uint8_t*)&data1,3))){
			*offset = data1 & FLASH_DATA_ACC_OFFSET_MASK;
		}else{
			if(data2 != FLASH_DATA_NULL){
				if(*(uint8_t*)(data2 >> 24) == MX_Flash_Uint32checkSum((uint8_t*)&data2,3)){
					*offset = data2 & FLASH_DATA_ACC_OFFSET_MASK;
				}else{
					ret = FALSE;
				}
			}
		}
	}else{
		if(data1 != FLASH_DATA_NULL){
			if( ((uint8_t)(data2 >> 24) == MX_Flash_Uint32checkSum((uint8_t*)&data1,3))){
				*offset = data1 & FLASH_DATA_ACC_OFFSET_MASK;
			}else{
				ret = FALSE;
			}
		}
	}
	return (ret);
}




void MX_Flash_ReadPage(uint32_t addr)
{

}

int MX_FlashProgram(uint32_t Address,uint32_t *DataAddress,uint32_t len)
{
	HAL_StatusTypeDef err = HAL_OK;
	uint32_t progAddr;
	uint32_t addrlen = 0;
	addrlen = ((len/16)+((len%16)>0?1:0));
	HAL_FLASH_Unlock();

	progAddr = Address;
	while(addrlen){
		err = HAL_FLASH_Program(FLASH_NSCR_PG,(uint32_t)progAddr,(uint32_t)DataAddress);
		if(err != HAL_OK){
			break;
		}
		addrlen--;
		progAddr += 16;
		DataAddress += 4;
	}
	HAL_FLASH_Lock();

	return err;
}

uint32_t MX_FlashGetPage(uint32_t Addr)
{
  uint32_t page = 0;

  if (Addr < (FLASH_BASE + FLASH_BANK_SIZE))
  {
    /* Bank 1 */
    page = (Addr - FLASH_BASE) / FLASH_PAGE_SIZE;
  }
  else
  {
    /* Bank 2 */
    page = (Addr - (FLASH_BASE + FLASH_BANK_SIZE)) / FLASH_PAGE_SIZE;
  }
  return page;
}

uint32_t MX_FlashGetBank(uint32_t Addr)
{
	if(Addr < (FLASH_BASE + FLASH_BANK_SIZE)){

		return FLASH_BANK_1;
	}else{

		return FLASH_BANK_2;
	}
}


int MX_FlashErasePage(uint32_t addr)
{
	HAL_StatusTypeDef err = HAL_OK;
	uint32_t FirstPage = 0,NbOfPages = 0;
	uint32_t BankNumber = 0;
	FLASH_EraseInitTypeDef EraseInitStruct;
	uint32_t PageError = 0;

	FirstPage = MX_FlashGetPage(addr);
	NbOfPages = MX_FlashGetPage(addr) - FirstPage + 1;
	BankNumber = MX_FlashGetBank(addr);
	HAL_FLASH_Unlock();

	EraseInitStruct.TypeErase   = FLASH_TYPEERASE_PAGES;
	EraseInitStruct.Banks       = BankNumber;
	EraseInitStruct.Page        = FirstPage;
	EraseInitStruct.NbPages     = NbOfPages;
	err = HAL_FLASHEx_Erase(&EraseInitStruct, &PageError);

	HAL_FLASH_Lock();
	return err;
}


#endif /* SRC_FLASH_USER_C_ */









