/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file    spi.c
  * @brief   This file provides code for the configuration
  *          of the SPI instances.
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2023 STMicroelectronics.
  * All rights reserved.
  *
  * This software is licensed under terms that can be found in the LICENSE file
  * in the root directory of this software component.
  * If no LICENSE file comes with this software, it is provided AS-IS.
  *
  ******************************************************************************
  */
/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "spi.h"

/* USER CODE BEGIN 0 */
//#include "main.h"

#define SPI_WAIT_TIMEOUT 500

/*SPI2_CS1		- PB12 Output*/
/*SPI2_CS2		- PC6  Output*/



/* CS_CTRL_PACKAGE_T */
typedef struct
{
	GPIO_TypeDef* GpioType[SPI_CS_MAX];
	UNS32 		  GpioMask[SPI_CS_MAX];
} CS_CTRL_PACKAGE_T;

const CS_CTRL_PACKAGE_T CsCtrlPackage ={
		.GpioType[SPI2_CS_1] = GPIOB,
		.GpioType[SPI2_CS_2] = GPIOC,
		.GpioType[SPI1_CS] = GPIOB,
		.GpioMask[SPI2_CS_1] = LL_GPIO_PIN_12,
		.GpioMask[SPI2_CS_2] = LL_GPIO_PIN_6,
		.GpioMask[SPI1_CS] 	= LL_GPIO_PIN_0,
};

#define SPI_CS_HIGH(inx)       {CsCtrlPackage.GpioType[inx]->BSRR = CsCtrlPackage.GpioMask[inx];}
#define SPI_CS_LOW(inx)        {CsCtrlPackage.GpioType[inx]->BRR = CsCtrlPackage.GpioMask[inx];}

/* USER CODE END 0 */

/* SPI1 init function */
void MX_SPI1_Init(void)
{

  /* USER CODE BEGIN SPI1_Init 0 */

  /* USER CODE END SPI1_Init 0 */

  LL_SPI_InitTypeDef SPI_InitStruct = {0};

  LL_GPIO_InitTypeDef GPIO_InitStruct = {0};

  LL_RCC_SetSPIClockSource(LL_RCC_SPI1_CLKSOURCE_MSIK);

  /* Peripheral clock enable */
  LL_APB2_GRP1_EnableClock(LL_APB2_GRP1_PERIPH_SPI1);

  LL_AHB2_GRP1_EnableClock(LL_AHB2_GRP1_PERIPH_GPIOA);
  LL_AHB2_GRP1_EnableClock(LL_AHB2_GRP1_PERIPH_GPIOB);
  /**SPI1 GPIO Configuration
  PA5   ------> SPI1_SCK
  PA7   ------> SPI1_MOSI
  PB0   ------> SPI1_NSS
  */
  GPIO_InitStruct.Pin = LL_GPIO_PIN_5|LL_GPIO_PIN_7;
  GPIO_InitStruct.Mode = LL_GPIO_MODE_ALTERNATE;
  GPIO_InitStruct.Speed = LL_GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.OutputType = LL_GPIO_OUTPUT_PUSHPULL;
  GPIO_InitStruct.Pull = LL_GPIO_PULL_DOWN;
  GPIO_InitStruct.Alternate = LL_GPIO_AF_5;
  LL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  GPIO_InitStruct.Pin = LL_GPIO_PIN_0;
  GPIO_InitStruct.Mode = LL_GPIO_MODE_ALTERNATE;
  GPIO_InitStruct.Speed = LL_GPIO_SPEED_FREQ_LOW;
  GPIO_InitStruct.OutputType = LL_GPIO_OUTPUT_PUSHPULL;
  GPIO_InitStruct.Pull = LL_GPIO_PULL_NO;
  GPIO_InitStruct.Alternate = LL_GPIO_AF_5;
  LL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /* USER CODE BEGIN SPI1_Init 1 */

  /* USER CODE END SPI1_Init 1 */
  SPI_InitStruct.TransferDirection = LL_SPI_SIMPLEX_TX;
  SPI_InitStruct.Mode = LL_SPI_MODE_MASTER;
  SPI_InitStruct.DataWidth = LL_SPI_DATAWIDTH_32BIT;
  SPI_InitStruct.ClockPolarity = LL_SPI_POLARITY_LOW;
  SPI_InitStruct.ClockPhase = LL_SPI_PHASE_1EDGE;
  SPI_InitStruct.NSS = LL_SPI_NSS_HARD_OUTPUT;
  SPI_InitStruct.BaudRate = LL_SPI_BAUDRATEPRESCALER_DIV4;
  SPI_InitStruct.BitOrder = LL_SPI_MSB_FIRST;
  SPI_InitStruct.CRCCalculation = LL_SPI_CRCCALCULATION_DISABLE;
  SPI_InitStruct.CRCPoly = 0x7;
  LL_SPI_Init(SPI1, &SPI_InitStruct);
  LL_SPI_SetStandard(SPI1, LL_SPI_PROTOCOL_MOTOROLA);
  LL_SPI_EnableNSSPulseMgt(SPI1);
  /* USER CODE BEGIN SPI1_Init 2 */
  LL_SPI_SetFIFOThreshold(SPI1, LL_SPI_FIFO_TH_01DATA);
  /* Enable TXP   Interrupt */
//   LL_SPI_EnableIT_TXP(SPI1);
//   /* Enable SPI1 Error Interrupt */
//   LL_SPI_EnableIT_CRCERR(SPI1);

  LL_SPI_Enable(SPI1);
  LL_SPI_StartMasterTransfer(SPI1);
  /* USER CODE END SPI1_Init 2 */

}
/* SPI2 init function */
void MX_SPI2_Init(void)
{

  /* USER CODE BEGIN SPI2_Init 0 */

  /* USER CODE END SPI2_Init 0 */

  LL_SPI_InitTypeDef SPI_InitStruct = {0};

  LL_GPIO_InitTypeDef GPIO_InitStruct = {0};

  LL_RCC_SetSPIClockSource(LL_RCC_SPI2_CLKSOURCE_HSI);

  /* Peripheral clock enable */
  LL_APB1_GRP1_EnableClock(LL_APB1_GRP1_PERIPH_SPI2);

  LL_AHB2_GRP1_EnableClock(LL_AHB2_GRP1_PERIPH_GPIOB);
  /**SPI2 GPIO Configuration
  PB13   ------> SPI2_SCK
  PB14   ------> SPI2_MISO
  PB15   ------> SPI2_MOSI
  */
  GPIO_InitStruct.Pin = LL_GPIO_PIN_13|LL_GPIO_PIN_14|LL_GPIO_PIN_15;
  GPIO_InitStruct.Mode = LL_GPIO_MODE_ALTERNATE;
  GPIO_InitStruct.Speed = LL_GPIO_SPEED_FREQ_LOW;
  GPIO_InitStruct.OutputType = LL_GPIO_OUTPUT_PUSHPULL;
  GPIO_InitStruct.Pull = LL_GPIO_PULL_NO;
  GPIO_InitStruct.Alternate = LL_GPIO_AF_5;
  LL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /* USER CODE BEGIN SPI2_Init 1 */

  /* USER CODE END SPI2_Init 1 */
  SPI_InitStruct.TransferDirection = LL_SPI_FULL_DUPLEX;
  SPI_InitStruct.Mode = LL_SPI_MODE_MASTER;
  SPI_InitStruct.DataWidth = LL_SPI_DATAWIDTH_16BIT;
  SPI_InitStruct.ClockPolarity = LL_SPI_POLARITY_LOW;
  SPI_InitStruct.ClockPhase = LL_SPI_PHASE_1EDGE;
  SPI_InitStruct.NSS = LL_SPI_NSS_SOFT;
  SPI_InitStruct.BaudRate = LL_SPI_BAUDRATEPRESCALER_DIV4;
  SPI_InitStruct.BitOrder = LL_SPI_MSB_FIRST;
  SPI_InitStruct.CRCCalculation = LL_SPI_CRCCALCULATION_DISABLE;
  SPI_InitStruct.CRCPoly = 0x7;
  LL_SPI_Init(SPI2, &SPI_InitStruct);
  LL_SPI_SetStandard(SPI2, LL_SPI_PROTOCOL_MOTOROLA);
  LL_SPI_DisableNSSPulseMgt(SPI2);
  /* USER CODE BEGIN SPI2_Init 2 */

  LL_SPI_Enable(SPI2);
  LL_SPI_StartMasterTransfer(SPI2);


  /* USER CODE END SPI2_Init 2 */

}

/* USER CODE BEGIN 1 */



uint8_t SPI1_Write32Bit(uint32_t  data_buffer)
{
	uint8_t state = SPI_NO_ERROR;
	uint32_t timeout_cnt = 200;
	LL_SPI_TransmitData32(SPI1, data_buffer);
   // while((!LL_SPI_IsActiveFlag_RXP(SPI1)) && (timeout_cnt--)){}
	while((timeout_cnt--)){}

	return state;
}



uint8_t SPI1_TransmitBuffer(uint32_t data)
{
	uint8_t state = 0;
	//state = SPI1_Write16Bit(data);
	LL_SPI_TransmitData32(SPI1, data);
	return state;
}


UNS8 SPI2_Write16Bit(UNS8  cs,UNS16 *data_buffer)
{
	uint32_t timeout_cnt;
	uint8_t state = SPI_NO_ERROR;
//	__IO uint16_t  data = 0;
//	__IO uint16_t  cmd = *data_buffer;

	uint16_t  data = 0;
	uint16_t  cmd = *data_buffer;

	timeout_cnt = SPI_WAIT_TIMEOUT;
	/*LL_SPI_IsActiveFlag_TXE*/
	/* Wait until TXE flag is set to send data */
    while((!LL_SPI_IsActiveFlag_TXP(SPI2)) && (timeout_cnt > 0)){
		timeout_cnt--;
		if(!timeout_cnt){
			state = SPI_ERROR_WAIT_TXP;}
    }

	SPI_CS_LOW(cs);
	/* Transmit data in 8 Bit mode */
	LL_SPI_TransmitData16(SPI2, cmd);
	/*LL_SPI_IsActiveFlag_BSY(SPI2)*/
	/* Check BSY flag */
	timeout_cnt = SPI_WAIT_TIMEOUT;
	while((!LL_SPI_IsActiveFlag_RXP(SPI2))&&(timeout_cnt > 0)){
		timeout_cnt--;
		if(!timeout_cnt){
			state = SPI_ERROR_WAIT_RXP;}
	}
	/* Read 8-Bits in the data register*/
	  data =  LL_SPI_ReceiveData16(SPI2);
	  *data_buffer  = data;
	//  debug("2REG:%x \r\n",data);

	SPI_CS_HIGH(cs);
	return state;
}



uint8_t SPI2_WriteFixedMultipleBit(uint8_t  cs,uint8_t addr,uint32_t *data_buffer)
{
	uint32_t timeout_cnt;
	uint8_t state = SPI_NO_ERROR;

	timeout_cnt = 1000;
    while((!LL_SPI_IsActiveFlag_TXP(SPI2)) && (timeout_cnt > 0)){
		timeout_cnt--;
		if(!timeout_cnt){
			state = SPI_ERROR_WAIT_TXP;}
    }
	SPI_CS_LOW(cs);

	LL_SPI_TransmitData32(SPI2, (addr << 8 | 0x00000000));
	LL_SPI_TransmitData32(SPI2, 0x00);
	uint16_t  data8 = 0;
	uint16_t *pData16 = (uint16_t *)data_buffer;

	for(int i = 0;i < 4; i++){
		timeout_cnt = 80;
		while((timeout_cnt > 0)){
			timeout_cnt--;
		}
		data8 =  LL_SPI_ReceiveData16(SPI2);
		pData16[i] = (data8 << 8)|(data8 >> 8);
	}

	SPI_CS_HIGH(cs);
	return state;
}



/* USER CODE END 1 */
