/*
 * ACC.c
 *
 *  Created on: Jan 26, 2024
 *      Author: j21800440
 */

/** INCLUDE FILES **/
#include "main.h"
#include "aba2000.h"
//#include "GlobalVar.h"
//#include "flt.h"
//#include "standard.h"
//#include "conf.h"

#define ACCEL_INSTALL_NUM               (CONF_FROM.Data.AccConf.ACC_SENSOR_INSTALLED)
#define ACCEL_S1_PROG_SID1              0X04//(CONF_FROM.Data.AccConf.S1_PROG_SID1)
#define ACCEL_S1_PROG_SID2              0X04//(CONF_FROM.Data.AccConf.S1_PROG_SID2)
#define ACCEL_S2_PROG_SID1              0X04//(CONF_FROM.Data.AccConf.S2_PROG_SID1)
#define ACCEL_S2_PROG_SID2              0X04//(CONF_FROM.Data.AccConf.S2_PROG_SID2)



//********************************************************
#define SENSOR_VALUE_MULTI    (1U) /// 扩大倍数

#define SENSOR_VALUE_LIMIT_POSI 	2032
#define SENSOR_VALUE_LIMIT_NEGA 	-2032

#define SENSOR_SPI_CHN  SPI1_SS1
#define SPI_DATA_TRANSMIT(CH,CMD,DATA)  SPI2_Write32Bit(CH,CMD,DATA)

#define BIST_THRES_VALUE_POSIFACT 600   /// LSB
#define BIST_THRES_VALUE_NEGAFACT -500   /// LSB
#define BIST_THRES_PER_FAC 40   /// %
#define BIST_THRES_PER_OTP 12   /// %

#define OPERATION_CYCLE     ((uint16_t) 500)

#define CMD_CRC_MODE               ((uint8_t)0x00UL)
#define RSP_CRC_MODE               ((uint8_t)0x01UL)
#define RSP_CRC_MASK		       ((uint32_t)0xFFFFFFF0UL)

#define RSP_CMD_REG                ((uint8_t)0x00UL)
#define RSP_CMD_DAT                ((uint8_t)0x01UL)

#define ACC_DATA_ERROR_MAX    	   ((uint8_t)60U)   //3*20=40  time:10ms
#define ACC_DATA_ERROR_MIN         ((uint8_t)2U)    //         time 10ms

#define ACC_SPI_ERROR_MAX    	   ((uint8_t)10U)
#define ACC_SPI_ERROR_MIN         ((uint8_t)2U)    //         time 10ms

/*Request Sensor Data Channel 1*/
#define ACC_CMD_RD_DATA_X	   ((uint32_t)0x8000000CU)
/*Request Sensor Data Channel 2*/
#define ACC_CMD_RD_DATA_Y	   ((uint32_t)0xC0000008U)
/*Request Register Monitor 1*/
#define ACC_CMD_RD_MONI_1	   ((uint32_t)0x3200001CU)
/*Request Register Monitor 2*/
#define ACC_CMD_RD_MONI_2	   ((uint32_t)0x33000014U)

/*12bit +-2047LSB*/
#define S8BIT_MAX    				(0x7FU)   //S8bit max 127()
#define S12BIT_MAX    				(0x7FFU)  //S12bit max 2047()
#define S16BIT_MAX    			 	(0x1000U) //U12bit max 4095 (S12)4096 =0

#define  CONVERT_MULT        			    ((uint32_t)7071U)     //
#define  ROUND_BASE                         ((uint32_t)5000U)     //rounding base
#define  MULT_BASE                          ((uint32_t)10000U)    //mult base 10000
#define  SID_MASK                           (0x1FU)              //config prog sid mask
typedef void (*FUNCTION_PTR)(void);

/****Define connector harness orientation******/
#define CONNECT_HARNESS_TO_LFT_FWD            ((uint8_t)1U)
#define CONNECT_HARNESS_TO_RGT_FWD            ((uint8_t)2U)
#define CONNECT_HARNESS_TO_LFT_BCK            ((uint8_t)4U)
#define CONNECT_HARNESS_TO_RGT_BCK            ((uint8_t)8U)
#define CONNECT_HAENESS_FRON				  ((uint8_t)0U)
#define CONNECT_HAENESS_BACK				  ((uint8_t)1U)

#define CONNECT_HARNESS_DIRE                  (CONNECT_HARNESS_TO_LFT_FWD )
#define CONNECT_HARNESS_PST                   (CONNECT_HAENESS_BACK)



uint8_t CRC3_Table3bits[8] = {
    2U,3U,0U,1U,6U,7U,4U,5U
 };

uint8_t CRC3_Table8bits[8][256] = {
 {
    0U,1U,2U,3U,4U,5U,6U,7U,3U,2U,1U,0U,7U,6U,5U,4U,6U,7U,4U,5U,2U,3U,0U,1U,5U,4U,7U,6U,1U,0U,3U,2U,
    7U,6U,5U,4U,3U,2U,1U,0U,4U,5U,6U,7U,0U,1U,2U,3U,1U,0U,3U,2U,5U,4U,7U,6U,2U,3U,0U,1U,6U,7U,4U,5U,
    5U,4U,7U,6U,1U,0U,3U,2U,6U,7U,4U,5U,2U,3U,0U,1U,3U,2U,1U,0U,7U,6U,5U,4U,0U,1U,2U,3U,4U,5U,6U,7U,
    2U,3U,0U,1U,6U,7U,4U,5U,1U,0U,3U,2U,5U,4U,7U,6U,4U,5U,6U,7U,0U,1U,2U,3U,7U,6U,5U,4U,3U,2U,1U,0U,
    1U,0U,3U,2U,5U,4U,7U,6U,2U,3U,0U,1U,6U,7U,4U,5U,7U,6U,5U,4U,3U,2U,1U,0U,4U,5U,6U,7U,0U,1U,2U,3U,
    6U,7U,4U,5U,2U,3U,0U,1U,5U,4U,7U,6U,1U,0U,3U,2U,0U,1U,2U,3U,4U,5U,6U,7U,3U,2U,1U,0U,7U,6U,5U,4U,
    4U,5U,6U,7U,0U,1U,2U,3U,7U,6U,5U,4U,3U,2U,1U,0U,2U,3U,0U,1U,6U,7U,4U,5U,1U,0U,3U,2U,5U,4U,7U,6U,
    3U,2U,1U,0U,7U,6U,5U,4U,0U,1U,2U,3U,4U,5U,6U,7U,5U,4U,7U,6U,1U,0U,3U,2U,6U,7U,4U,5U,2U,3U,0U,1U},
 {
    2U,3U,0U,1U,6U,7U,4U,5U,1U,0U,3U,2U,5U,4U,7U,6U,4U,5U,6U,7U,0U,1U,2U,3U,7U,6U,5U,4U,3U,2U,1U,0U,
    5U,4U,7U,6U,1U,0U,3U,2U,6U,7U,4U,5U,2U,3U,0U,1U,3U,2U,1U,0U,7U,6U,5U,4U,0U,1U,2U,3U,4U,5U,6U,7U,
    7U,6U,5U,4U,3U,2U,1U,0U,4U,5U,6U,7U,0U,1U,2U,3U,1U,0U,3U,2U,5U,4U,7U,6U,2U,3U,0U,1U,6U,7U,4U,5U,
    0U,1U,2U,3U,4U,5U,6U,7U,3U,2U,1U,0U,7U,6U,5U,4U,6U,7U,4U,5U,2U,3U,0U,1U,5U,4U,7U,6U,1U,0U,3U,2U,
    3U,2U,1U,0U,7U,6U,5U,4U,0U,1U,2U,3U,4U,5U,6U,7U,5U,4U,7U,6U,1U,0U,3U,2U,6U,7U,4U,5U,2U,3U,0U,1U,
    4U,5U,6U,7U,0U,1U,2U,3U,7U,6U,5U,4U,3U,2U,1U,0U,2U,3U,0U,1U,6U,7U,4U,5U,1U,0U,3U,2U,5U,4U,7U,6U,
    6U,7U,4U,5U,2U,3U,0U,1U,5U,4U,7U,6U,1U,0U,3U,2U,0U,1U,2U,3U,4U,5U,6U,7U,3U,2U,1U,0U,7U,6U,5U,4U,
    1U,0U,3U,2U,5U,4U,7U,6U,2U,3U,0U,1U,6U,7U,4U,5U,7U,6U,5U,4U,3U,2U,1U,0U,4U,5U,6U,7U,0U,1U,2U,3U
 },{
    4U,5U,6U,7U,0U,1U,2U,3U,7U,6U,5U,4U,3U,2U,1U,0U,2U,3U,0U,1U,6U,7U,4U,5U,1U,0U,3U,2U,5U,4U,7U,6U,
    3U,2U,1U,0U,7U,6U,5U,4U,0U,1U,2U,3U,4U,5U,6U,7U,5U,4U,7U,6U,1U,0U,3U,2U,6U,7U,4U,5U,2U,3U,0U,1U,
    1U,0U,3U,2U,5U,4U,7U,6U,2U,3U,0U,1U,6U,7U,4U,5U,7U,6U,5U,4U,3U,2U,1U,0U,4U,5U,6U,7U,0U,1U,2U,3U,
    6U,7U,4U,5U,2U,3U,0U,1U,5U,4U,7U,6U,1U,0U,3U,2U,0U,1U,2U,3U,4U,5U,6U,7U,3U,2U,1U,0U,7U,6U,5U,4U,
    5U,4U,7U,6U,1U,0U,3U,2U,6U,7U,4U,5U,2U,3U,0U,1U,3U,2U,1U,0U,7U,6U,5U,4U,0U,1U,2U,3U,4U,5U,6U,7U,
    2U,3U,0U,1U,6U,7U,4U,5U,1U,0U,3U,2U,5U,4U,7U,6U,4U,5U,6U,7U,0U,1U,2U,3U,7U,6U,5U,4U,3U,2U,1U,0U,
    0U,1U,2U,3U,4U,5U,6U,7U,3U,2U,1U,0U,7U,6U,5U,4U,6U,7U,4U,5U,2U,3U,0U,1U,5U,4U,7U,6U,1U,0U,3U,2U,
    7U,6U,5U,4U,3U,2U,1U,0U,4U,5U,6U,7U,0U,1U,2U,3U,1U,0U,3U,2U,5U,4U,7U,6U,2U,3U,0U,1U,6U,7U,4U,5U
 },{
    6U,7U,4U,5U,2U,3U,0U,1U,5U,4U,7U,6U,1U,0U,3U,2U,0U,1U,2U,3U,4U,5U,6U,7U,3U,2U,1U,0U,7U,6U,5U,4U,
    1U,0U,3U,2U,5U,4U,7U,6U,2U,3U,0U,1U,6U,7U,4U,5U,7U,6U,5U,4U,3U,2U,1U,0U,4U,5U,6U,7U,0U,1U,2U,3U,
    3U,2U,1U,0U,7U,6U,5U,4U,0U,1U,2U,3U,4U,5U,6U,7U,5U,4U,7U,6U,1U,0U,3U,2U,6U,7U,4U,5U,2U,3U,0U,1U,
    4U,5U,6U,7U,0U,1U,2U,3U,7U,6U,5U,4U,3U,2U,1U,0U,2U,3U,0U,1U,6U,7U,4U,5U,1U,0U,3U,2U,5U,4U,7U,6U,
    7U,6U,5U,4U,3U,2U,1U,0U,4U,5U,6U,7U,0U,1U,2U,3U,1U,0U,3U,2U,5U,4U,7U,6U,2U,3U,0U,1U,6U,7U,4U,5U,
    0U,1U,2U,3U,4U,5U,6U,7U,3U,2U,1U,0U,7U,6U,5U,4U,6U,7U,4U,5U,2U,3U,0U,1U,5U,4U,7U,6U,1U,0U,3U,2U,
    2U,3U,0U,1U,6U,7U,4U,5U,1U,0U,3U,2U,5U,4U,7U,6U,4U,5U,6U,7U,0U,1U,2U,3U,7U,6U,5U,4U,3U,2U,1U,0U,
    5U,4U,7U,6U,1U,0U,3U,2U,6U,7U,4U,5U,2U,3U,0U,1U,3U,2U,1U,0U,7U,6U,5U,4U,0U,1U,2U,3U,4U,5U,6U,7U
 },{
    3U,2U,1U,0U,7U,6U,5U,4U,0U,1U,2U,3U,4U,5U,6U,7U,5U,4U,7U,6U,1U,0U,3U,2U,6U,7U,4U,5U,2U,3U,0U,1U,
    4U,5U,6U,7U,0U,1U,2U,3U,7U,6U,5U,4U,3U,2U,1U,0U,2U,3U,0U,1U,6U,7U,4U,5U,1U,0U,3U,2U,5U,4U,7U,6U,
    6U,7U,4U,5U,2U,3U,0U,1U,5U,4U,7U,6U,1U,0U,3U,2U,0U,1U,2U,3U,4U,5U,6U,7U,3U,2U,1U,0U,7U,6U,5U,4U,
    1U,0U,3U,2U,5U,4U,7U,6U,2U,3U,0U,1U,6U,7U,4U,5U,7U,6U,5U,4U,3U,2U,1U,0U,4U,5U,6U,7U,0U,1U,2U,3U,
    2U,3U,0U,1U,6U,7U,4U,5U,1U,0U,3U,2U,5U,4U,7U,6U,4U,5U,6U,7U,0U,1U,2U,3U,7U,6U,5U,4U,3U,2U,1U,0U,
    5U,4U,7U,6U,1U,0U,3U,2U,6U,7U,4U,5U,2U,3U,0U,1U,3U,2U,1U,0U,7U,6U,5U,4U,0U,1U,2U,3U,4U,5U,6U,7U,
    7U,6U,5U,4U,3U,2U,1U,0U,4U,5U,6U,7U,0U,1U,2U,3U,1U,0U,3U,2U,5U,4U,7U,6U,2U,3U,0U,1U,6U,7U,4U,5U,
    0U,1U,2U,3U,4U,5U,6U,7U,3U,2U,1U,0U,7U,6U,5U,4U,6U,7U,4U,5U,2U,3U,0U,1U,5U,4U,7U,6U,1U,0U,3U,2U
 },{
    1U,0U,3U,2U,5U,4U,7U,6U,2U,3U,0U,1U,6U,7U,4U,5U,7U,6U,5U,4U,3U,2U,1U,0U,4U,5U,6U,7U,0U,1U,2U,3U,
    6U,7U,4U,5U,2U,3U,0U,1U,5U,4U,7U,6U,1U,0U,3U,2U,0U,1U,2U,3U,4U,5U,6U,7U,3U,2U,1U,0U,7U,6U,5U,4U,
    4U,5U,6U,7U,0U,1U,2U,3U,7U,6U,5U,4U,3U,2U,1U,0U,2U,3U,0U,1U,6U,7U,4U,5U,1U,0U,3U,2U,5U,4U,7U,6U,
    3U,2U,1U,0U,7U,6U,5U,4U,0U,1U,2U,3U,4U,5U,6U,7U,5U,4U,7U,6U,1U,0U,3U,2U,6U,7U,4U,5U,2U,3U,0U,1U,
    0U,1U,2U,3U,4U,5U,6U,7U,3U,2U,1U,0U,7U,6U,5U,4U,6U,7U,4U,5U,2U,3U,0U,1U,5U,4U,7U,6U,1U,0U,3U,2U,
    7U,6U,5U,4U,3U,2U,1U,0U,4U,5U,6U,7U,0U,1U,2U,3U,1U,0U,3U,2U,5U,4U,7U,6U,2U,3U,0U,1U,6U,7U,4U,5U,
    5U,4U,7U,6U,1U,0U,3U,2U,6U,7U,4U,5U,2U,3U,0U,1U,3U,2U,1U,0U,7U,6U,5U,4U,0U,1U,2U,3U,4U,5U,6U,7U,
    2U,3U,0U,1U,6U,7U,4U,5U,1U,0U,3U,2U,5U,4U,7U,6U,4U,5U,6U,7U,0U,1U,2U,3U,7U,6U,5U,4U,3U,2U,1U,0U
 },{
    7U,6U,5U,4U,3U,2U,1U,0U,4U,5U,6U,7U,0U,1U,2U,3U,1U,0U,3U,2U,5U,4U,7U,6U,2U,3U,0U,1U,6U,7U,4U,5U,
    0U,1U,2U,3U,4U,5U,6U,7U,3U,2U,1U,0U,7U,6U,5U,4U,6U,7U,4U,5U,2U,3U,0U,1U,5U,4U,7U,6U,1U,0U,3U,2U,
    2U,3U,0U,1U,6U,7U,4U,5U,1U,0U,3U,2U,5U,4U,7U,6U,4U,5U,6U,7U,0U,1U,2U,3U,7U,6U,5U,4U,3U,2U,1U,0U,
    5U,4U,7U,6U,1U,0U,3U,2U,6U,7U,4U,5U,2U,3U,0U,1U,3U,2U,1U,0U,7U,6U,5U,4U,0U,1U,2U,3U,4U,5U,6U,7U,
    6U,7U,4U,5U,2U,3U,0U,1U,5U,4U,7U,6U,1U,0U,3U,2U,0U,1U,2U,3U,4U,5U,6U,7U,3U,2U,1U,0U,7U,6U,5U,4U,
    1U,0U,3U,2U,5U,4U,7U,6U,2U,3U,0U,1U,6U,7U,4U,5U,7U,6U,5U,4U,3U,2U,1U,0U,4U,5U,6U,7U,0U,1U,2U,3U,
    3U,2U,1U,0U,7U,6U,5U,4U,0U,1U,2U,3U,4U,5U,6U,7U,5U,4U,7U,6U,1U,0U,3U,2U,6U,7U,4U,5U,2U,3U,0U,1U,
    4U,5U,6U,7U,0U,1U,2U,3U,7U,6U,5U,4U,3U,2U,1U,0U,2U,3U,0U,1U,6U,7U,4U,5U,1U,0U,3U,2U,5U,4U,7U,6U
 },{
    5U,4U,7U,6U,1U,0U,3U,2U,6U,7U,4U,5U,2U,3U,0U,1U,3U,2U,1U,0U,7U,6U,5U,4U,0U,1U,2U,3U,4U,5U,6U,7U,
    2U,3U,0U,1U,6U,7U,4U,5U,1U,0U,3U,2U,5U,4U,7U,6U,4U,5U,6U,7U,0U,1U,2U,3U,7U,6U,5U,4U,3U,2U,1U,0U,
    0U,1U,2U,3U,4U,5U,6U,7U,3U,2U,1U,0U,7U,6U,5U,4U,6U,7U,4U,5U,2U,3U,0U,1U,5U,4U,7U,6U,1U,0U,3U,2U,
    7U,6U,5U,4U,3U,2U,1U,0U,4U,5U,6U,7U,0U,1U,2U,3U,1U,0U,3U,2U,5U,4U,7U,6U,2U,3U,0U,1U,6U,7U,4U,5U,
    4U,5U,6U,7U,0U,1U,2U,3U,7U,6U,5U,4U,3U,2U,1U,0U,2U,3U,0U,1U,6U,7U,4U,5U,1U,0U,3U,2U,5U,4U,7U,6U,
    3U,2U,1U,0U,7U,6U,5U,4U,0U,1U,2U,3U,4U,5U,6U,7U,5U,4U,7U,6U,1U,0U,3U,2U,6U,7U,4U,5U,2U,3U,0U,1U,
    1U,0U,3U,2U,5U,4U,7U,6U,2U,3U,0U,1U,6U,7U,4U,5U,7U,6U,5U,4U,3U,2U,1U,0U,4U,5U,6U,7U,0U,1U,2U,3U,
    6U,7U,4U,5U,2U,3U,0U,1U,5U,4U,7U,6U,1U,0U,3U,2U,0U,1U,2U,3U,4U,5U,6U,7U,3U,2U,1U,0U,7U,6U,5U,4U
 }
};

/*Sensor request response result*/
typedef enum{
	ENDINIT_NOT_SET = 0,
	NORMAL_MODE,
	FOC_SOC_MODE,
	SELF_TEST_MODE,
	OC_ERR,       //Internal Error Present
	NOCLK_ERR,
	OTP_ERR,
	SOC_ERR,
	REF_ERR,
	ME_ERR,
	VOLTAGE_ERR,
	GND_ERR,
	FOC_ERR,
	OTHER_ERR
}SENSOR_DEVSTAT_T;

/*Sensor request response result*/
typedef enum{
	SENSOR_NORMAL = 0,
	SPI_COMMU_FAULT,
	REQUEST_ERROR,
	INCORRECT_CONDITION,
	NO_EOP,
	VDDA_UNDERVLOT,
	GND_LOSE,
	VDDI_UNDERVLOT,
    VDD_UNDERVLOT
}SENSOR_FAULT;

uint8_t ACC_FALUT_CODE;

/*Request Response Format*/
typedef struct
{
	uint32_t Send_Cmd;
	uint32_t Resp_Data;
}ACC_REQ_RES_FORMAT_T;

typedef union
{
	struct
	{
		uint32_t CRC1			:3;		/*bit0~2 */
		uint32_t NRO            :1;     /*bit3 */
		uint32_t DATA		    :16;	/*bit4~19 */
		uint32_t UNUSED20TO24   :5;     /*bit20~24 */
		uint32_t SEN            :1;     /*bit25 */
		uint32_t PF	            :1;	    /*bit26 */
		uint32_t TF	            :1;	    /*bit27 */
		uint32_t UNUSED28       :1;     /*bit28 */
		uint32_t EEOP           :1;     /*bit29 */
		uint32_t TST            :1;     /*bit30 */
		uint32_t TFF		    :1;		/*bit31 */
	} Bit;
	uint8_t	Byte[4];
	uint32_t	DWord;
} ACC_REGISTER_RESP_U;

typedef union
{
	struct
	{
		uint32_t CRC1			:3;		/*bit0~2 */
		uint32_t NRO            :1;     /*bit3 */
		uint32_t UNUSED4TO7     :4;     /*bit4~7 */
		uint32_t DATA		    :12;	/*bit8~19 */
		uint32_t CH_n		    :6;     /*bit20~25 */
		uint32_t PF	            :1;	    /*bit26 */
		uint32_t TF	            :1;	    /*bit27 */
		uint32_t UNUSED28       :1;     /*bit28 */
		uint32_t EEOP           :1;     /*bit29 */
		uint32_t TST            :1;     /*bit30 */
		uint32_t TFF		    :1;		/*bit31 */

	} Bit;
	uint8_t	Byte[4];
	uint32_t	DWord;
} ACC_SENSOR_RESP_U;

typedef enum
{
	ACCEL_STATE_SOFTRESET = 0,
    ACCEL_STATE_CONFIG_DEV_MODE,
    ACCEL_STATE_MONITOR_DATA,
    ACCEL_STATE_FOC,
    ACCEL_STATE_BIST_POSI_ACTV,
    ACCEL_STATE_BIST_POSI_VERF,
    ACCEL_STATE_BIST_NEGA_ACTV,
    ACCEL_STATE_BIST_NEGA_VERF,
    ACCEL_STATE_CONFIG_SOC,
    ACCEL_STATE_EOP,
    ACCEL_STATE_NORMAL_MODE_ENTRY,

	ACCEL_STATE_RESET_RETRY_HANDLE,
	ACCEL_STATE_RE_ENTER_HANDLE,
	ACCEL_STATE_FAIL,
	ACCEL_STATE_MAX,
	ACCEL_STATE_NORMAL
} ACCEL_STATE_E;

typedef struct
{
    uint16_t	       ExecDelayCnt;
    ACCEL_STATE_E      State;
    SENSOR_DEVSTAT_T   SensorStat;
    uint32_t		   SensorFault;
    uint8_t 		   SensorDataErrCnt;
    uint8_t 		   CheckDataErrCnt;
    uint8_t 		   CheckDataErrFlg;
    uint8_t 		   SensorDataErrFlg;
    uint8_t 		   SensorSPIErrCnt;
    uint8_t 		   SensorSPIErrFlg;
    uint8_t 		   csCh;
    uint8_t 		   RangeDet;
    uint16_t 		   RangeCnt;
    AXIS_DATA32BIT_T   offset[ACC_MAX];


} ACCEL_CTRL_PACKAGE_T;

ACCEL_GLOBAL_DATA_T ACCEL;
static ACCEL_CTRL_PACKAGE_T AccelCtrlPkg;
static ACCEL_STATE_E ACCEL_STEP_PRESENT;


S32 g_s32_mSensorXval = 0;
S32 g_s32_mSensorYval = 0;
S32 g_s32_sSensorXval = 0;
S32 g_s32_sSensorYval = 0;


static int16_t  S12ToS16(uint16_t  val);
static uint16_t ABSCAL(int16_t VALA, int16_t VALB);
static int16_t AccelGetValue(UNS8 csCh,const uint32_t requset_sensor_ch);

static void AccelSoftReset(void);
static void AccelConfigureDevMode(void);
static void AccelMonitorRead(void);
static void AccelFoc(void);
static void AccelBISTPositiveActive(void);
static void VccelBISTPositiveVerify(void);
static void AccelBISTNegativeActive(void);
static void AccelBISTNegativeVerify(void);
static void AccelSOC(void);
static void AccelEOP(void);
static void AccelNormalModeEntry(void);

static void AcceResetAndRetryHandle(void);
static void AcceReEnterHandle(void);
static void AccelStateFail(void);

static void AccelGetXYValue(void);

static const FUNCTION_PTR AccelExecRtnTable[ACCEL_STATE_MAX] =
{
	AccelSoftReset,
    AccelConfigureDevMode,
    AccelMonitorRead,
    AccelFoc,
    AccelBISTPositiveActive,
    VccelBISTPositiveVerify,
    AccelBISTNegativeActive,
    AccelBISTNegativeVerify,
    AccelSOC,
    AccelEOP,
    AccelNormalModeEntry,

	AcceResetAndRetryHandle,
	AcceReEnterHandle,
	AccelStateFail						/* ACCEL_STATE_FAIL */
};

uint8_t SPI_3Bit_CRC32Check(const uint32_t Data){
	uint8_t crc3 = 0x7;
	crc3 = CRC3_Table3bits[(Data>>24)&0x07];
	crc3 = CRC3_Table8bits[crc3][(Data>>16)&0xFF];
	crc3 = CRC3_Table8bits[crc3][(Data>> 8)&0xFF];
	crc3 = CRC3_Table8bits[crc3][(Data>> 0)&0xFF];
	return crc3;
}

/*check bit31~bit5**/
UNS32 SPI_Crc3(const UNS32 data)
{
    UNS8  temp;
    UNS8  crc[3] =  {0x01,0x01,0x01};
    UNS8  crc_next[3] ;
	UNS32 ret = 0;

	UNS32 data1 = data;
    for (UNS16 i = 0; i < 30; i++)
    {
        crc_next[2] = crc[1];
        crc_next[1] = crc[0]^crc[2];
        temp = (UNS8)((data1 >>(31 - i)) & 1);
        crc_next[0] = crc[2] ^ temp;

        crc[0] =  crc_next[0];
        crc[1] =  crc_next[1];
        crc[2] =  crc_next[2];
    }
    temp = 0;
    temp |= (UNS8)((crc[2] & 0x01) << 2);
    temp |= (UNS8)((crc[1] & 0x01) << 1);
    temp |= (UNS8)(crc[0] & 0x01);
    temp = (UNS8)(temp << 2);
    ret  = data1 | temp;

	return (ret);

}

static int16_t  S8ToS16(const uint8_t    val)
{
	if(val > S8BIT_MAX ){return (int16_t)(0xFFFF - (0XFF-val));}
	else {return ((int16_t)val);}
}

static int16_t  S12ToS16(const uint16_t    val)
{
	if(val > S12BIT_MAX ){return (int16_t)(val - S16BIT_MAX);}
	else {return ((int16_t)val);}
}

static uint16_t ABSCAL(int16_t VALA, int16_t VALB){
	int16_t dif = VALA - VALB;
	if(dif < 0){
		dif = 0 - dif;
	}
	return dif;
}

BOOL ACCEL_IsStatusGood(void)
{
	BOOL ret = TRUE ;
	if((AccelCtrlPkg.SensorStat != NORMAL_MODE ) || (AccelCtrlPkg.RangeDet == FALSE))
	{
		ret = FALSE;
	}
	return (ret);
}

BOOL ACCEL_IsSPIGood(void)
{
	BOOL isMsgVal = TRUE;
//	if ( AccelCtrlPkg.SensorStat >= OC_ERR )
//	{
//		isMsgVal = FALSE;
//	}
	if(AccelCtrlPkg.State == ACCEL_STATE_FAIL || (AccelCtrlPkg.State == ACCEL_STATE_NORMAL && AccelCtrlPkg.SensorSPIErrFlg)){
		isMsgVal = FALSE;
	}else{;}

	return (isMsgVal);
}


void BSP_SPI_ACC_TranRevOpenIRQ(UNS32 CMD,UNS32 * DATA)
{


}



static void ACCEL_SetDelayUS(uint32_t timeUS){
    AccelCtrlPkg.ExecDelayCnt = (timeUS + (OPERATION_CYCLE>> 1)) / OPERATION_CYCLE;
    if(AccelCtrlPkg.ExecDelayCnt == 0){
    	AccelCtrlPkg.ExecDelayCnt = 1;
    }
}

static SENSOR_DEVSTAT_T ACCEL_STATUS_DETECT(uint32_t data){

    ACC_REGISTER_RESP_U  rspRegisterData;
    ACC_SENSOR_RESP_U	rspSentorData;
    uint32_t monitor1,monitor2 = 0;

    rspSentorData.DWord = data;


    AccelCtrlPkg.SensorFault = 0;

    if(!rspSentorData.Bit.NRO){ return NORMAL_MODE; }
    else if(rspSentorData.Bit.EEOP){return ENDINIT_NOT_SET;}
    else if(rspSentorData.Bit.TST){return SELF_TEST_MODE;}
    else{;}

    SPI_DATA_TRANSMIT(AccelCtrlPkg.csCh,ACC_CMD_RD_MONI_1, &rspRegisterData.DWord);
	if (SPI_3Bit_CRC32Check(rspRegisterData.DWord & 0xFFFFFFF8) == rspRegisterData.Bit.CRC1) {
		monitor1 = rspRegisterData.Bit.DATA;
	}

	SPI_DATA_TRANSMIT(AccelCtrlPkg.csCh,ACC_CMD_RD_MONI_2, &rspRegisterData.DWord);
	if (SPI_3Bit_CRC32Check(rspRegisterData.DWord & 0xFFFFFFF8) == rspRegisterData.Bit.CRC1) {
		monitor2 = rspRegisterData.Bit.DATA;
	}

	AccelCtrlPkg.SensorFault = (monitor1 << 16) |  monitor2;

	/* Sort in order of importance */
	/* order 1 */
	if (monitor1 & 0x0010) {return NOCLK_ERR;}
	if (monitor2 & 0x000E) {return VOLTAGE_ERR;}
	if (monitor2 & 0x000E) {return GND_ERR;}
	if (monitor2 & 0x0020) {return OTP_ERR;}
	/* order2 */
	if (monitor2 & 0x4000) {return ME_ERR;}
	if (monitor1 & 0x2000) {return REF_ERR;}
	if (monitor2 & 0x0700) {return FOC_ERR;}
	if (monitor1 & 0x0C00) {return SOC_ERR;}
	if (monitor1 & 0x0003) {return OC_ERR;}
	return OTHER_ERR;

}

void ACCEL_RamInit(void)
{
	(void)memset(&AccelCtrlPkg, 0, sizeof(AccelCtrlPkg));
	ACCEL_SetDelayUS(1000);
    AccelCtrlPkg.State        = ACCEL_STATE_SOFTRESET;
	AccelCtrlPkg.SensorStat   = ENDINIT_NOT_SET;
	ACC_FALUT_CODE 			  = 0;
	AccelCtrlPkg.csCh = 0;
}



BOOL ACC_StCalRngChk()
{

#define ACC_RNG_MAX      				(512)
#define ACC_X_OFFSET_HIGH_DEV_MAX  		(32)
#define ACC_X_OFFSET_LOW_DEV_MAX  		(-32)
#define ACC_Y_OFFSET_HIGH_DEV_MAX  		(32)
#define ACC_Y_OFFSET_LOW_DEV_MAX  		(-32)


	UNS8 ret = FALSE;
	UNS8 retval = 0;
	AXIS_DATA_T master_acc,slave_acc;

	if(AccelCtrlPkg.RangeCnt < ACC_RNG_MAX){
		AccelCtrlPkg.RangeCnt++;
		AccelGetXYValue();
		master_acc.X 	= g_s32_mSensorXval;
		master_acc.Y 	= g_s32_mSensorYval;
		slave_acc.X 	= g_s32_sSensorXval;
		slave_acc.Y 	= g_s32_sSensorYval;

		AccelCtrlPkg.offset[ACC_MASTER].X += master_acc.X;
		AccelCtrlPkg.offset[ACC_MASTER].Y += master_acc.Y;
		AccelCtrlPkg.offset[ACC_SLAVE].X += slave_acc.X;
		AccelCtrlPkg.offset[ACC_SLAVE].Y += slave_acc.Y;

	}else{

		AccelCtrlPkg.RangeCnt = 0;
		master_acc.X = AccelCtrlPkg.offset[ACC_MASTER].X/ACC_RNG_MAX;
		master_acc.Y = AccelCtrlPkg.offset[ACC_MASTER].Y/ACC_RNG_MAX;
		slave_acc.X = AccelCtrlPkg.offset[ACC_SLAVE].X/ACC_RNG_MAX;
		slave_acc.Y = AccelCtrlPkg.offset[ACC_SLAVE].Y/ACC_RNG_MAX;

		retval |= IS_OUT_RANGE(master_acc.X,ACC_X_OFFSET_LOW_DEV_MAX,ACC_X_OFFSET_HIGH_DEV_MAX);
		retval |= IS_OUT_RANGE(master_acc.Y,ACC_Y_OFFSET_LOW_DEV_MAX,ACC_Y_OFFSET_HIGH_DEV_MAX);

		retval |= IS_OUT_RANGE(slave_acc.X,ACC_X_OFFSET_LOW_DEV_MAX,ACC_X_OFFSET_HIGH_DEV_MAX);
		retval |= IS_OUT_RANGE(slave_acc.Y,ACC_Y_OFFSET_LOW_DEV_MAX,ACC_Y_OFFSET_HIGH_DEV_MAX);

		if(retval > 0){

			AccelCtrlPkg.offset[ACC_MASTER].X = 0;
			AccelCtrlPkg.offset[ACC_MASTER].Y = 0;
			AccelCtrlPkg.offset[ACC_SLAVE].X = 0;
			AccelCtrlPkg.offset[ACC_SLAVE].Y = 0;
			AccelCtrlPkg.RangeCnt = 0;
			ret = FALSE;
		}else{
			ret = TRUE;
		}
	}
	return ret;
}
void ACCEL_TimerRoutine(void)
{
	if ((uint16_t)0 != AccelCtrlPkg.ExecDelayCnt)
	{
		AccelCtrlPkg.ExecDelayCnt--;
	}
	else if (ACCEL_STATE_MAX > AccelCtrlPkg.State)
	{
		AccelExecRtnTable[AccelCtrlPkg.State]();
		if(AccelCtrlPkg.State == ACCEL_STATE_NORMAL){
			if(AccelCtrlPkg.csCh == 0)
			{
				AccelCtrlPkg.csCh++;
				AccelCtrlPkg.State = ACCEL_STATE_SOFTRESET;
			}
		}
	}
	else if (AccelCtrlPkg.RangeDet == FALSE)
	{
		/*Range detection, low risk! The hardware comes with zero bias calibration, it is recommended to delete it!*/
		if(TRUE == ACC_StCalRngChk()){
			AccelCtrlPkg.RangeDet = TRUE;
		}
	}
	else if(ACCEL_IsStatusGood())
	{
		AccelGetXYValue();
	}
    else{}
}


/*0 Complete a soft reset*/
static void AccelSoftReset(void){
    typedef enum{
        RESET_CMD_STEP_1 = 0,
        RESET_CMD_STEP_2,
        RESET_CMD_END
    }SOFTRESET_STAT;

    const uint32_t soft_reset[RESET_CMD_END] = {0x379BD5AC, 0x379BD5AC};

    ACC_REGISTER_RESP_U  rspRegisterData;SOFTRESET_STAT step;

    for(step = RESET_CMD_STEP_1; step < RESET_CMD_END; step++){
        SPI_DATA_TRANSMIT(AccelCtrlPkg.csCh,soft_reset[step],&rspRegisterData.DWord);
    }

    AccelCtrlPkg.State++;
    ACCEL_SetDelayUS(5000);
}/* End of AccelSoftReset */


/*config sensor sid*/

static void AccelProgSid(const uint8_t sid1,const uint8_t sid2)
{
	uint8_t prog_sid1 = sid1 & SID_MASK;
	uint8_t prog_sid2 = sid2 & SID_MASK;
	uint32_t cmd = 0x23800000;
//	uint32_t cmd1 = cmd;

	ACC_REGISTER_RESP_U  rspRegisterData;

	if(prog_sid1 != prog_sid2)
	{
        cmd = cmd | (prog_sid2 << 10) | (prog_sid1 << 5);
	}

	cmd = SPI_Crc3(cmd);
//	cmd1 = cmd | SPI_3Bit_CRC32Check(cmd);

	SPI_DATA_TRANSMIT(AccelCtrlPkg.csCh,cmd,&rspRegisterData.DWord);
}

/*1 Configure Device Mode*/
static void AccelConfigureDevMode(void){
    typedef enum{
        CONFIG_CMD_SPI = 0,
        CONFIG_CMD_POWER,
        CONFIG_CMD_RD_POWER_MODE,
        CONFIG_CMD_END
    }INITCONFIG_STAT;

    const uint32_t config_init[CONFIG_CMD_END] = {0xFF00FF00, 0x20800100, 0x20000010};

    ACC_REGISTER_RESP_U  rspRegisterData;INITCONFIG_STAT step;
    for(step = CONFIG_CMD_SPI; step < CONFIG_CMD_END; step++){
    	SPI_DATA_TRANSMIT(AccelCtrlPkg.csCh,config_init[step], &rspRegisterData.DWord);
        if(step == CONFIG_CMD_RD_POWER_MODE){
            if((rspRegisterData.Bit.DATA&0x000F) == ((config_init[CONFIG_CMD_POWER]>>5)&0x000F)){
                ACCEL.FltDataForVerifyReg = 0;
            }
            else{
               ACCEL.FltDataForVerifyReg = (uint32_t)(AccelCtrlPkg.State << 28)|(config_init[CONFIG_CMD_POWER] >> 4);
               break;
            }
        }
    }

    AccelProgSid(ACCEL_S1_PROG_SID1,ACCEL_S1_PROG_SID2);

    if(ACCEL.FltDataForVerifyReg){
	    AccelCtrlPkg.State = ACCEL_STATE_RESET_RETRY_HANDLE;
    }else{ AccelCtrlPkg.State++;}

    ACCEL_SetDelayUS(1000);
}/* End of Configure Device Mode */

/*2 Read the status monitor register*/
static void AccelMonitorRead(void){
    typedef enum{
        MONITOR_CMD_MONITOR1 = 0,
        MONITOR_CMD_MONITOR2,
        MONITOR_CMD_END
    }MONITOR_READ_STAT;

    const ACC_REQ_RES_FORMAT_T monitor_read[MONITOR_CMD_END] = {{0x3200001C,0x2800C00E},{0x33000014,0x2800001B}};

    ACC_REGISTER_RESP_U  rspRegisterData; MONITOR_READ_STAT step;

    for(step = MONITOR_CMD_MONITOR1; step < MONITOR_CMD_END; step++){
        SPI_DATA_TRANSMIT(AccelCtrlPkg.csCh,monitor_read[step].Send_Cmd,&rspRegisterData.DWord);
        if(rspRegisterData.DWord != monitor_read[step].Resp_Data){
            ACCEL.FltDataForVerifyReg = (uint32_t)(AccelCtrlPkg.State << 28)|(monitor_read[step].Send_Cmd >> 4);
            break;
        }
    }

    if(ACCEL.FltDataForVerifyReg){
	    AccelCtrlPkg.State = ACCEL_STATE_RESET_RETRY_HANDLE;
    }else{ AccelCtrlPkg.State++;}

}/* End of Read the status monitor register */

/*3 FOC */
static void AccelFoc(void){

    typedef enum{
        FOC_CMD_SET = 0,
        FOC_CMD_READ,
        FOC_CMD_END
    }FOC_STAT;

    const uint32_t FOC[FOC_CMD_END] = {0x2C81BDE0,0x2C000004};

    ACC_REGISTER_RESP_U  rspRegisterData; FOC_STAT step = FOC_CMD_SET;

    for(step = FOC_CMD_SET; step < FOC_CMD_END; step++){
        SPI_DATA_TRANSMIT(AccelCtrlPkg.csCh,FOC[step], &(rspRegisterData.DWord));
        if(SPI_3Bit_CRC32Check(rspRegisterData.DWord&0xFFFFFFF8) == rspRegisterData.Bit.CRC1){
        	AccelCtrlPkg.SensorStat = FOC_SOC_MODE;
            if(step == FOC_CMD_READ){
                if((rspRegisterData.Bit.DATA&0x0FFF)!=((FOC[step-1]>>5)&0x0FFF)){
                    ACCEL.FltDataForVerifyReg = (uint32_t)(AccelCtrlPkg.State << 28)|(FOC[FOC_CMD_SET] >> 4);
                }
            }
        }else{
            ACCEL.FltDataForVerifyReg = (uint32_t)(AccelCtrlPkg.State << 28)|(FOC[step] >> 4);
            break;
        }
    }

    if(ACCEL.FltDataForVerifyReg){
	    AccelCtrlPkg.State = ACCEL_STATE_RESET_RETRY_HANDLE;
    }else{ AccelCtrlPkg.State++;}

    ACCEL_SetDelayUS(65000);
}/* End of FOC */

/*4 BIST Positive Active*/
static void  AccelBISTPositiveActive(void){
    typedef enum{
        BISTPA_CMD_CONF_SET = 0,
        BISTPA_CMD_CONF_READ,
        BISTPA_CMD_POSI_SET,
        BISTPA_CMD_POSI_READ,
        BISTPA_CMD_END
    }BIST_POSI_ACTV_STAT;

    const uint32_t bistp_actv[BISTPA_CMD_END] = {0x34800200,0x34000000,0x3180033C,0x31000004};

    ACC_REGISTER_RESP_U  rspRegisterData; BIST_POSI_ACTV_STAT step;

    for(step = BISTPA_CMD_CONF_SET; step < BISTPA_CMD_END; step++){
        SPI_DATA_TRANSMIT(AccelCtrlPkg.csCh,bistp_actv[step],&rspRegisterData.DWord);
        if(SPI_3Bit_CRC32Check(rspRegisterData.DWord&0xFFFFFFF8) == rspRegisterData.Bit.CRC1){
        	AccelCtrlPkg.SensorStat = SELF_TEST_MODE;
            if(step == BISTPA_CMD_CONF_READ){
                if(rspRegisterData.Bit.DATA != ((bistp_actv[BISTPA_CMD_CONF_SET]>>5)&0x003F)){
                    ACCEL.FltDataForVerifyReg = (uint32_t)(AccelCtrlPkg.State << 28)|(bistp_actv[step] >> 4);
                }
            }else if(step == BISTPA_CMD_POSI_READ){
                if(rspRegisterData.Bit.DATA != ((bistp_actv[BISTPA_CMD_POSI_SET]>>5)&0x003F)){
                    ACCEL.FltDataForVerifyReg = (uint32_t)(AccelCtrlPkg.State << 28)|(bistp_actv[step] >> 4);
                }
            }else{};
        }else{
            ACCEL.FltDataForVerifyReg = (uint32_t)(AccelCtrlPkg.State << 28)|(bistp_actv[step] >> 4);
            break;
        }
    }

    if(ACCEL.FltDataForVerifyReg){
        AccelCtrlPkg.State = ACCEL_STATE_RESET_RETRY_HANDLE;
    }else{ AccelCtrlPkg.State++;}

    ACCEL_SetDelayUS(4500);

}/* End of BIST Positive Active */

/*5 BIST Positive Verify*/
static void  VccelBISTPositiveVerify(void){
    typedef enum{
        BISTPV_CMD_READ_BIST_CH1 = 0,
        BISTPV_CMD_READ_BIST_CH2,
        BISTPV_CMD_READ_VAL_CH1,
        BISTPV_CMD_READ_VAL_CH2,
        BISTPV_CMD_STOP,
        BISTPV_CMD_READ,
        BISTPV_CMD_END
    }BIST_POSI_STAT;

    const uint32_t bist_verf[BISTPV_CMD_END] = {0x0D000018,0x0E000000,0x8000000C,0xC0000008,0x31800000,0x31000004};

    ACC_REGISTER_RESP_U  rspRegisterData; BIST_POSI_STAT step; ACC_SENSOR_RESP_U    rspSensorData;
    uint16_t difPosi,difNega,difThrePosi,difThreNega = 0;
    int16_t valBISTCH1,valBISTCH2 = 0;
    int16_t valVAlCH1,valVAlCH2 = 0;
    uint8_t rltCMDSta = 0;

    for(step = BISTPV_CMD_READ_BIST_CH1; step < BISTPV_CMD_END; step++){
        SPI_DATA_TRANSMIT(AccelCtrlPkg.csCh,bist_verf[step],&rspRegisterData.DWord);
        rspSensorData.DWord = rspRegisterData.DWord;
        if(SPI_3Bit_CRC32Check(rspRegisterData.DWord&0xFFFFFFF8) != rspRegisterData.Bit.CRC1){
        	ACCEL.FltDataForSelfTest = (uint32_t)(AccelCtrlPkg.State << 28)|(bist_verf[step] >> 4);
        	break;
        }
        switch (step)
        {
        case BISTPV_CMD_READ_BIST_CH1:
            valBISTCH1 = (rspRegisterData.Bit.DATA>>4)&0xFFF0;
            break;
        case BISTPV_CMD_READ_BIST_CH2:
            valBISTCH2 = S8ToS16(rspRegisterData.Bit.DATA&0x00FF)<<4;
            break;
        case BISTPV_CMD_READ_VAL_CH1:
            valVAlCH1 = S12ToS16(rspSensorData.Bit.DATA);
            break;
        case BISTPV_CMD_READ_VAL_CH2:
            valVAlCH2 = S12ToS16(rspSensorData.Bit.DATA);
            break;
        case BISTPV_CMD_STOP:
            difPosi = ABSCAL(valVAlCH1,BIST_THRES_VALUE_POSIFACT);
            difNega = ABSCAL(valVAlCH2,BIST_THRES_VALUE_NEGAFACT);
            difThrePosi = BIST_THRES_VALUE_POSIFACT * BIST_THRES_PER_FAC / 100;
            difThreNega = (0 - BIST_THRES_VALUE_NEGAFACT) * BIST_THRES_PER_FAC / 100;
            if(rltCMDSta == 2){break;}
            if(difPosi > difThrePosi || difNega > difThreNega){
                rltCMDSta = 1;
                break;
            }
            difPosi = ABSCAL(valVAlCH1,valBISTCH1);
            difNega = ABSCAL(valVAlCH2,valBISTCH2);
            difThrePosi = valBISTCH1 * BIST_THRES_PER_OTP / 100;
            difThreNega = (0 -valBISTCH2) * BIST_THRES_PER_OTP / 100;
            if(difPosi > difThrePosi || difNega > difThreNega){
                rltCMDSta = 1;
            }
            break;
        case BISTPV_CMD_READ:
            if(rspRegisterData.Bit.DATA != ((bist_verf[step-1]>>5)&0x3F)){
                ACCEL.FltDataForVerifyReg = (uint32_t)(AccelCtrlPkg.State << 28)|(bist_verf[step] >> 4);
            }
            break;
        default:
            break;
        }
        if(rltCMDSta == 1){
            rltCMDSta = 2;
            ACCEL.FltDataForSelfTest = (uint32_t)(AccelCtrlPkg.State << 28)|(bist_verf[step] >> 4);
            if(step < BISTPV_CMD_STOP){
                step = BISTPV_CMD_STOP;
            }
        }
    }

    if(ACCEL.FltDataForSelfTest){
    	ACCEL_STEP_PRESENT = AccelCtrlPkg.State;
	    AccelCtrlPkg.State = ACCEL_STATE_RE_ENTER_HANDLE;
    }else if(ACCEL.FltDataForVerifyReg){
        AccelCtrlPkg.State = ACCEL_STATE_RESET_RETRY_HANDLE;
    }
    else{ AccelCtrlPkg.State++;}
}/* End of BIST Positive Verify */

/*6 BIST Negative Active*/
static void AccelBISTNegativeActive(void){
    typedef enum{
        BISTNA_CMD_POSI_SET,
        BISTNA_CMD_POSI_READ,
        BISTNA_CMD_END
    }BIST_NEGA_ACTV_STAT;

    const uint32_t bistp_actv[BISTNA_CMD_END] = {0x318004CC,0x31000004};

    ACC_REGISTER_RESP_U  rspRegisterData; BIST_NEGA_ACTV_STAT step;

    for (step = BISTNA_CMD_POSI_SET; step < BISTNA_CMD_END; step++) {
		SPI_DATA_TRANSMIT(AccelCtrlPkg.csCh,bistp_actv[step],&rspRegisterData.DWord);
		if (SPI_3Bit_CRC32Check(rspRegisterData.DWord & 0xFFFFFFF8)== rspRegisterData.Bit.CRC1) {
			if (step == BISTNA_CMD_POSI_READ) {
				if (rspRegisterData.Bit.DATA != ((bistp_actv[step - 1] >> 5) & 0x003F)) {
					ACCEL.FltDataForVerifyReg = (uint32_t) (AccelCtrlPkg.State<< 28) | (bistp_actv[step] >> 4);
				}
			}
		}else{
			ACCEL.FltDataForVerifyReg = (uint32_t) (AccelCtrlPkg.State<< 28) | (bistp_actv[step] >> 4);
		}
	}

    if(ACCEL.FltDataForVerifyReg){
        AccelCtrlPkg.State = ACCEL_STATE_RESET_RETRY_HANDLE;
    }else{ AccelCtrlPkg.State++;}

    ACCEL_SetDelayUS(4500);
}/* End of BIST Negative Active */

/*7 BIST Negative Verify */
static void AccelBISTNegativeVerify(void){
    typedef enum{
        BISTNV_CMD_READ_BIST_CH1 = 0,
        BISTNV_CMD_READ_BIST_CH2,
        BISTNV_CMD_READ_VAL_CH1,
        BISTNV_CMD_READ_VAL_CH2,
        BISTNV_CMD_STOP,
        BISTNV_CMD_READ,
        BISTNV_CMD_END
    }BIST_NEGA_STAT;

    const uint32_t bist_verf[BISTNV_CMD_END] = {0x0D000018,0x0E000000,0x8000000C,0xC0000008,0x31800000,0x31000004};

    ACC_REGISTER_RESP_U rspRegisterData; BIST_NEGA_STAT step; ACC_SENSOR_RESP_U rspSensorData;
    uint16_t difPosi,difNega,difThrePosi,difThreNega = 0;
    int16_t valBISTCH1,valBISTCH2 = 0;
    int16_t valVAlCH1,valVAlCH2 = 0;
    uint8_t rltCMDSta = 0;

    for(step = BISTNV_CMD_READ_BIST_CH1; step < BISTNV_CMD_END; step++){
        SPI_DATA_TRANSMIT(AccelCtrlPkg.csCh,bist_verf[step],&rspRegisterData.DWord);
        rspSensorData.DWord = rspRegisterData.DWord;
        if (SPI_3Bit_CRC32Check(rspRegisterData.DWord & 0xFFFFFFF8) != rspRegisterData.Bit.CRC1) {
        	ACCEL.FltDataForSelfTest = (uint32_t)(AccelCtrlPkg.State << 28)|(bist_verf[step] >> 4);
        	break;
        }
        switch (step)
        {
        case BISTNV_CMD_READ_BIST_CH1:
            valBISTCH1 = S8ToS16(rspRegisterData.Bit.DATA&0x00FF)<<4;
            break;
        case BISTNV_CMD_READ_BIST_CH2:
            valBISTCH2 = (rspRegisterData.Bit.DATA>>4)&0xFFF0;
            break;
        case BISTNV_CMD_READ_VAL_CH1:
            valVAlCH1 = S12ToS16(rspSensorData.Bit.DATA);
            break;
        case BISTNV_CMD_READ_VAL_CH2:
            valVAlCH2 = S12ToS16(rspSensorData.Bit.DATA);
            break;
        case BISTNV_CMD_STOP:
            if(rltCMDSta == 2){break;}
            difNega = ABSCAL(valVAlCH1,BIST_THRES_VALUE_NEGAFACT);
            difPosi = ABSCAL(valVAlCH2,BIST_THRES_VALUE_POSIFACT);
            difThrePosi = BIST_THRES_VALUE_POSIFACT * BIST_THRES_PER_FAC / 100;
            difThreNega = (0 - BIST_THRES_VALUE_NEGAFACT) * BIST_THRES_PER_FAC / 100;
            if(difPosi > difThrePosi || difNega > difThreNega){
                rltCMDSta = 1;
                break;
            }
            difNega = ABSCAL(valVAlCH1,valBISTCH1);
            difPosi = ABSCAL(valVAlCH2,valBISTCH2);
            difThreNega = (0 - valBISTCH1) * BIST_THRES_PER_OTP / 100;
            difThrePosi = valBISTCH2 * BIST_THRES_PER_OTP / 100;
            if(difPosi > difThrePosi || difNega > difThreNega){
                rltCMDSta = 1;
            }
            break;
        case BISTNV_CMD_READ:
            if(rspRegisterData.Bit.DATA != ((bist_verf[step-1]>>5)&0x3F)){
                ACCEL.FltDataForVerifyReg = (uint32_t)(AccelCtrlPkg.State << 28)|(bist_verf[step] >> 4);
            }
            break;
        default:
            break;
        }
        if(rltCMDSta == 1){
            rltCMDSta = 2;
            ACCEL.FltDataForSelfTest = (uint32_t)(AccelCtrlPkg.State << 28)|(bist_verf[step] >> 4);
            if(step < BISTNV_CMD_STOP){
                step = BISTNV_CMD_STOP;
            }
        }
    }

    if(ACCEL.FltDataForSelfTest){
    	ACCEL_STEP_PRESENT = AccelCtrlPkg.State;
	    AccelCtrlPkg.State = ACCEL_STATE_RE_ENTER_HANDLE;
    }else if(ACCEL.FltDataForVerifyReg){
        AccelCtrlPkg.State = ACCEL_STATE_RESET_RETRY_HANDLE;
    }
    else{ AccelCtrlPkg.State++;}

}/* End of BIST Negative Verify */

/*8 SOC Configure */
static void AccelSOC(void){
     typedef enum{
        SOC_CMD_SET = 0,
        SOC_CMD_READ,
        SOC_CMD_END
    }BIST_NEGA_STAT;

    const uint32_t SOC[SOC_CMD_END] = {0x2C80B5AC,0x2C000004};

    ACC_REGISTER_RESP_U  rspRegisterData; BIST_NEGA_STAT step;

    for(step = SOC_CMD_SET; step < SOC_CMD_END; step++){
        SPI_DATA_TRANSMIT(AccelCtrlPkg.csCh,SOC[step],&rspRegisterData.DWord);
        if (SPI_3Bit_CRC32Check(rspRegisterData.DWord & 0xFFFFFFF8)== rspRegisterData.Bit.CRC1) {
        	AccelCtrlPkg.SensorStat = FOC_SOC_MODE;
			if (step == SOC_CMD_READ) {
				if ((rspRegisterData.Bit.DATA & 0x0FFF)!= ((SOC[SOC_CMD_SET] >> 5) & 0x0FFF)) {
					ACCEL.FltDataForVerifyReg = (uint32_t) (AccelCtrlPkg.State<< 28) | (SOC[SOC_CMD_SET] >> 4);
				}
			}
        }
        else{
        	ACCEL.FltDataForVerifyReg = (uint32_t) (AccelCtrlPkg.State<< 28) | (SOC[SOC_CMD_SET] >> 4);
        }
    }

    if(ACCEL.FltDataForVerifyReg){
        AccelCtrlPkg.State = ACCEL_STATE_RESET_RETRY_HANDLE;
    }
    else{ AccelCtrlPkg.State++;}

}/* End of SOC Configure */

/*9 EOP */
static void AccelEOP(void){
    typedef enum{
        EOP_CMD_SET = 0,
        EOP_CMD_END
    }CONFIG_EOP_STAT;

    const uint32_t EEOP[EOP_CMD_END] = {0x3897DDF4};

    ACC_REGISTER_RESP_U rspRegisterData; CONFIG_EOP_STAT step;

    for(step = EOP_CMD_SET; step < EOP_CMD_END; step++){
    	AccelCtrlPkg.SensorStat = ENDINIT_NOT_SET;
    	SPI_DATA_TRANSMIT(AccelCtrlPkg.csCh,EEOP[step],&rspRegisterData.DWord);
    }

    if(ACCEL.FltDataForVerifyReg){
        AccelCtrlPkg.State = ACCEL_STATE_RESET_RETRY_HANDLE;
    }else{ AccelCtrlPkg.State++;}

    ACCEL_SetDelayUS(3000);
}/* End of EOP */

/*10 Entry Normal Mode */
static void AccelNormalModeEntry(void){
	typedef enum {
		NM_CMD_SENSORX1,
		NM_CMD_SENSORY1,
		NM_CMD_END
	} CONFIG_NORMALMODE_STAT;

	const uint32_t normal_mode[NM_CMD_END] = {0x8000000C, 0xC0000008};

	ACC_REGISTER_RESP_U rspRegisterData; CONFIG_NORMALMODE_STAT step;

	for(step = NM_CMD_SENSORX1; step < NM_CMD_END; step++){
		SPI_DATA_TRANSMIT(AccelCtrlPkg.csCh,normal_mode[step],&rspRegisterData.DWord);
		if (SPI_3Bit_CRC32Check(rspRegisterData.DWord & 0xFFFFFFF8) != rspRegisterData.Bit.CRC1){
			ACCEL.FltDataForVerifyReg = (uint32_t) (AccelCtrlPkg.State<< 28) | (normal_mode[step] >> 4);
			break;
		}
		if(step == NM_CMD_SENSORY1){
			AccelCtrlPkg.SensorStat = ACCEL_STATUS_DETECT(rspRegisterData.DWord);
			if (AccelCtrlPkg.SensorStat != NORMAL_MODE) {
				ACCEL.FltDataForVerifyReg = (uint32_t) (AccelCtrlPkg.State << 28) | (normal_mode[step] >> 4);
				ACC_FALUT_CODE |= SENSOR_INIT_FAIL;
			}
		}
	}

	if (ACCEL.FltDataForVerifyReg) {
		AccelCtrlPkg.State = ACCEL_STATE_RESET_RETRY_HANDLE;
	} else {
		/// entry normal mode
		ACC_FALUT_CODE &= ~SENSOR_INIT_FAIL;
		AccelCtrlPkg.State = ACCEL_STATE_NORMAL;
	}
}


static void AcceResetAndRetryHandle(void){
	static uint8_t ACC_TEST = 0;
	ACC_TEST++;
    if (ACC_TEST > 10) //TRUE == FLT_UpdateStatus(FLT_ACCL_SELF_TEST, ACTIVE)) /// 记录次数，未达到阈值则重启
    {
        /* Fault has matured, abort retry */
        AccelCtrlPkg.State = ACCEL_STATE_FAIL;
    }
    else
    {
        /* Fault has not matured yet, reset the sensor to retry */
        ACCEL_RamInit();
    }

}

static void AcceReEnterHandle(void){
	static uint8_t ACC_TEST = 0;
	ACC_TEST++;

	switch (ACCEL_STEP_PRESENT) {
	case ACCEL_STATE_BIST_POSI_ACTV:
		if(ACC_TEST > 5){
			AccelCtrlPkg.State = ACCEL_STATE_BIST_NEGA_ACTV;
			ACC_FALUT_CODE |= SENSOR_BIST_FAIL;
			ACC_TEST = 0;
		}
		break;
	case ACCEL_STATE_BIST_POSI_VERF:
		if(ACC_TEST > 5){
			AccelCtrlPkg.State = ACCEL_STATE_BIST_NEGA_ACTV;
			ACC_FALUT_CODE |= SENSOR_BIST_FAIL;
			ACC_TEST = 0;
		}else{
			AccelCtrlPkg.State = ACCEL_STATE_BIST_POSI_ACTV;
		}
		break;
	case ACCEL_STATE_BIST_NEGA_ACTV:
		if (ACC_TEST > 5) {
			AccelCtrlPkg.State = ACCEL_STATE_BIST_NEGA_VERF + 1;
			ACC_FALUT_CODE |= SENSOR_BIST_FAIL;
			ACC_TEST = 0;
		}
		break;
	case ACCEL_STATE_BIST_NEGA_VERF:
		if (ACC_TEST > 5) {
			AccelCtrlPkg.State = ACCEL_STATE_BIST_NEGA_VERF + 1;
			ACC_FALUT_CODE |= SENSOR_BIST_FAIL;
			ACC_TEST = 0;
		} else {
			AccelCtrlPkg.State = ACCEL_STATE_BIST_NEGA_ACTV;
		}
		break;
	}
}

static void AccelStateFail(void){
	ACC_FALUT_CODE |= SENSOR_INIT_FAIL;
}

static int32_t CalcuConverResult(const int16_t val)
{
    int32_t  tmp_val = val;
    BOOL flag = FALSE ;
    int32_t  res_val;
    if(val < 0){
        tmp_val = ~tmp_val + 1;
        flag = TRUE ;
    }
	res_val = (((int32_t)tmp_val) *  CONVERT_MULT ) / MULT_BASE ;
    if(flag) res_val = ~res_val + 1;
    return (res_val);

}




static void MSensorDeg(const int16_t data_x ,const int16_t data_y)
{
	g_s32_mSensorXval = data_y;
	g_s32_mSensorYval = data_x;

}
static void SSensorDeg(const int16_t data_x ,const int16_t data_y)
{

	g_s32_sSensorXval =  (~(data_x)+ 1);
	g_s32_sSensorYval =  data_y;

}


UNS32 XValueErrorCunot_test = 0;
UNS32 YValueErrorCunot_test = 0;

/*High risk operating conditions, if any problems occur, please prioritize inspection*/
void AccelDoubDataCheck(void)
{

#define	 DOUBLE_SENSOR_ERR_ENABLE  			1

#define	 DIFFERENCE_SIZE_MAX  				32 /* 16lsb = 1g  32/16 = 2g */


#if	DOUBLE_SENSOR_ERR_ENABLE

	BOOL err = FALSE;

	if((g_s32_mSensorXval - g_s32_sSensorXval) > DIFFERENCE_SIZE_MAX){
		XValueErrorCunot_test++;
		/*The difference between the two sensors is greater than 2g, exceeding the threshold output of 0, high risk!*/
		g_s32_mSensorXval = 0;
		err = TRUE;
	}

	if((g_s32_mSensorYval - g_s32_sSensorYval) > DIFFERENCE_SIZE_MAX){
		YValueErrorCunot_test++;
		/*The difference between the two sensors is greater than 2g, exceeding the threshold output of 0, high risk!*/
		g_s32_mSensorYval = 0;
		err = TRUE;
	}

	if (err == TRUE) {
		if ( AccelCtrlPkg.CheckDataErrCnt >= ACC_DATA_ERROR_MAX) {
			AccelCtrlPkg.CheckDataErrFlg = TRUE;
		} else {
			AccelCtrlPkg.CheckDataErrCnt++;
		}
	} else {
		if (ACC_DATA_ERROR_MIN >= AccelCtrlPkg.CheckDataErrCnt) {
			AccelCtrlPkg.CheckDataErrFlg = FALSE;
			AccelCtrlPkg.CheckDataErrCnt= ACC_DATA_ERROR_MIN;
		} else {
			AccelCtrlPkg.CheckDataErrCnt--;
		}
	}

	/*Double sensor difference greater than 2g exceeding threshold error, high risk!*/
	if(AccelCtrlPkg.CheckDataErrFlg == FALSE){
		ACC_FALUT_CODE &= ~SENSOR_DATA_FAULT;
	} else {
		ACC_FALUT_CODE |= SENSOR_DATA_FAULT;
	}

#endif

}


void AccelGetXYValue(void)
{
	int16_t master_x = 0, master_y = 0,s_x = 0, s_y = 0;
	master_x = AccelGetValue(ACC_MASTER,ACC_CMD_RD_DATA_X);
	master_y = AccelGetValue(ACC_MASTER,ACC_CMD_RD_DATA_Y);
	s_x = AccelGetValue(ACC_SLAVE,ACC_CMD_RD_DATA_X);
	s_y = AccelGetValue(ACC_SLAVE,ACC_CMD_RD_DATA_Y);
	SSensorDeg(s_x,s_y);
	MSensorDeg(master_x,master_y);
	AccelDoubDataCheck();
}

static int16_t AccelGetValue(UNS8 csCh ,const uint32_t requset_sensor_ch)
{
	ACC_SENSOR_RESP_U respSensorVal;
	int16_t           xy_val = 0;

	respSensorVal.DWord = 0;

	SPI_DATA_TRANSMIT(csCh,requset_sensor_ch,&respSensorVal.DWord);

	if (SPI_3Bit_CRC32Check(respSensorVal.DWord & 0xFFFFFFF8) != respSensorVal.Bit.CRC1){
		ACCEL.FltDataForSensorData = (requset_sensor_ch&0xF0000000)|(respSensorVal.DWord >> 4);
		AccelCtrlPkg.SensorFault |= 0x80000000;
	}else{
		if(ACCEL_STATUS_DETECT(respSensorVal.DWord) != NORMAL_MODE){
			ACCEL.FltDataForSensorData = (requset_sensor_ch&0xF0000000)|(respSensorVal.DWord >> 4);
		}else{
			ACCEL.FltDataForSensorData = 0;
			xy_val = S12ToS16((uint16_t)respSensorVal.Bit.DATA);
		}
	}

	if (ACCEL.FltDataForSensorData) {
		if ( AccelCtrlPkg.SensorDataErrCnt >= ACC_DATA_ERROR_MAX) {
			AccelCtrlPkg.SensorDataErrFlg = 1;
		} else {
			AccelCtrlPkg.SensorDataErrCnt++;
		}
		if(AccelCtrlPkg.SensorFault & 0x80000000){
			if(AccelCtrlPkg.SensorSPIErrCnt >= ACC_SPI_ERROR_MAX){
				AccelCtrlPkg.SensorSPIErrFlg = 1;
			} else{
				AccelCtrlPkg.SensorSPIErrCnt++;
			}
		}else{;}

	} else {
		if (ACC_DATA_ERROR_MIN >= AccelCtrlPkg.SensorDataErrCnt) {
			AccelCtrlPkg.SensorDataErrFlg = 0;
			AccelCtrlPkg.SensorDataErrCnt= ACC_DATA_ERROR_MIN;
		} else {
			AccelCtrlPkg.SensorDataErrCnt--;
		}
		if (ACC_SPI_ERROR_MIN >= AccelCtrlPkg.SensorSPIErrCnt) {
			AccelCtrlPkg.SensorSPIErrFlg = 0;
			AccelCtrlPkg.SensorSPIErrCnt = ACC_SPI_ERROR_MIN;
		} else {
			AccelCtrlPkg.SensorSPIErrCnt--;
		}
	}

	if(AccelCtrlPkg.SensorDataErrFlg == 0){
		ACC_FALUT_CODE &= ~SENSOR_DATA_FAULT;
	} else {
		ACC_FALUT_CODE |= SENSOR_DATA_FAULT;
	}

	return (xy_val);
}























