/*
 * accel.c
 *
 *  Created on: 2023年12月20日
 *      Author: liuzhimin
 */


/* Includes ------------------------------------------------------------------*/
#include <main.h>



#define ACC_WRITE_BIT     		(0x00)
#define ACC_READ_BIT    	 	(0x80)
#define ACC_WRITE_REG_MASK     	(0x08)
#define ACC_READ_REG_MASK      	(0x08)
#define ACC_DATA_MASK      		(0x00FF)
#define ACC_PM				    5


#define ACC_CALI_DELAY_MAX      		(500)

#define ACC_CALI_CNT_MAX      			(32)

#define ACC_FAULT_COUNT_MAX  		20
#define ACC_TWO_FAULT_COUNT_MAX  	3

#define ACC_INIT_ERR_COUNT_MAX  5


static S32  Filter_master_value[2] = {0 , 0 };
static S32  Filter_slave_value[2] =  {0 , 0 };


typedef  enum {
    MORE_90  = 0,
    BETWEEN_45_90 ,
    BETWEEN_NEGAT45_45 ,
    BETWEEN_NEGAT90_NEGAT45 ,
    LESS_THAN_NEGAT90 ,
}X_AXIS_BOUND_RANGE_U;





typedef enum {
  ACC_ERR_NO = 0,
  ACC_ERR_WHO_READ,
  ACC_ERR_WHO_NO,
  ACC_ERR_CFG_MODE_WRIT,
  ACC_ERR_CFG_MODE_READ,
  ACC_ERR_CFG_MODE_WRIT_READ,
  ACC_ERR_INT1_THRES_WRIT,
  ACC_ERR_INT1_THRES_READ,
  ACC_ERR_INT1_THRES_WRIT_READ,
  ACC_ERR_INT2_THRES_WRIT,
  ACC_ERR_INT2_THRES_READ,
  ACC_ERR_INT2_THRES_WRIT_READ,
  ACC_ERR_OUT_X_READ,
  ACC_ERR_OUT_Y_READ,
  ACC_ERR_CODE_CHECK,
  ACC_ERR_SLEEP_WRIT,
  ACC_ERR_SLEEP_READ,
  ACC_ERR_SLEEP_WRIT_READ,
  ACC_ERR_HPFILTER_RESET_READ,
  ACC_ERR_HPFILTER_READ,
  ACC_ERR_HPFILTER_WRIT,

  ACC_ERR_HPFILTER_WRIT_READ,

} ACC_ERROR_E;


typedef ACC_ERROR_E (*FUNCTION_PTR)(ACC_CH_E);

typedef enum {
	ACCEL_STATE_SOFTRESET = 0,
	ACCEL_STATE_INIT_MASTER,
	ACCEL_STATE_INIT_SLAVE,
	ACCEL_STATE_FIRST_CALI,
	ACCEL_STATE_DELAY_CALI,
	ACCEL_STATE_TWO_CALI,
	ACCEL_STATE_CALI_CHECK,
	ACCEL_STATE_NORMAL,
	ACCEL_STATE_ERROR,
	ACCEL_STATE_MAX,

} ACCEL_STATE_E;

typedef enum {
	ACCEL_INIT_SOFTRESET = 0,
	ACCEL_INIT_CHECK_DEVICE,
    ACCEL_INIT_CONIFG_MODE,
    ACCEL_INIT_UPDATE_SPI,
	ACCEL_INIT_INT1_THRESHOLD,
	ACCEL_INIT_INT1_CONFIG,
	ACCEL_INIT_ENABLE_INT1,
	ACCEL_INIT_INT2_THRESHOLD,
	ACCEL_INIT_INT2_CONFIG,
	ACCEL_INIT_ENABLE_INT2,
	ACCEL_INIT_CTRL_REG,
//	ACCEL_INIT_SET_HPFCUTOFF,
//	ACCEL_INIT_SET_HPFMODE,
//	ACCEL_INIT_FILTER_RESET,
//	ACCEL_INIT_SET_FDS,
	ACCEL_INIT_DONE,
	ACCEL_INIT_MAX,
} ACCEL_INIT_E;

typedef enum {
  ACC_LL_ERROR			      =		0x00,
  ACC_LL_SUCCESS              =		0x01,
} ACC_LL_STATUS_E;

#define ACC_WHO_AM_I				0x0F
#define ACC_DEFAULT_DEVICE			0x32

#define ACC_CTRL_REG1       		(0x20)
#define ACC_CTRL_REG5        		(0x24)
typedef enum {

  ACC_ODR_POWER_DOWN        =		0x00,
  ACC_ODR_1Hz               =		0x10,
  ACC_ODR_10Hz              =		0x20,
  ACC_ODR_25Hz              =		0x30,
  ACC_ODR_50Hz              =		0x40,
  ACC_ODR_100Hz		        =		0x50,
  ACC_ODR_200Hz		        =		0x60,
  ACC_ODR_400Hz		      	=		0x70,
  ACC_ODR_LP_1600Hz		    =		0x80,
  ACC_ODR_LP_5376Hz		    =		0x90,

} ACC_ODR_E;

typedef enum {
  ACC_X_ENABLE              =    	0x01,
  ACC_X_DISABLE             =    	0x00,
  ACC_Y_ENABLE              =    	0x02,
  ACC_Y_DISABLE             =    	0x00,
  ACC_Z_ENABLE              =    	0x04,
  ACC_Z_DISABLE             =    	0x00,
  ACC_LP_ENABLE             =    	0x08,
  ACC_LP_DISABLE            =    	0x00
} ACC_AXIS_ENABLE_E;

typedef enum {
  ACC_SLEEP_OFF       		= 0x00,
  ACC_SLEEP_ON       		= 0x03,
} ACC_SLEEP_E;


typedef enum {
  ACC_POWER_DOWN           	=		0x00,
  ACC_NORMAL 			  	=		0x20,
  ACC_LOW_POWER_05		  	=		0x40,
  ACC_LOW_POWER_1 		  	=		0x60,
  ACC_LOW_POWER_2		  	=		0x80,
  ACC_LOW_POWER_5		  	=		0xA0,
  ACC_LOW_POWER_10		  	=		0xC0,
} ACC_MODE_E;

#define ACC_INT1_THS_REG          	(0x32)
#define ACC_INT2_THS_REG          	(0x36)

#define INT1_THRESHOLD            	(0x0F)////////////门限

#define INT2_THRESHOLD            	(0x03)

#define ACC_REFERENCE_VAL			(0)//////HP过滤器的参考（0-255）


#define ACC_INT1_CFG				0x30
#define ACC_INT2_CFG				0x34

#define ACC_INT1_SRC            	0x31
#define ACC_INT2_SRC				0x35

#define ACC_INT_CFG_ENABLE_MASK		0x80
#define ACC_INT_CFG_IA_MASK			0x3F

typedef enum {
	ACC_INT_AND           =   0x80,
	ACC_INT_OR            =   0x00,
	ACC_INT_ZHIE_ENABLE   =   0x20,
	ACC_INT_ZLIE_ENABLE   =   0x10,
	ACC_INT_YHIE_ENABLE   =   0x08,
	ACC_INT_YLIE_ENABLE   =   0x04,
	ACC_INT_XHIE_ENABLE   =   0x02,
	ACC_INT_XLIE_ENABLE   =   0x01,
}ACC_INT_CFG_E;

#define ACC_INT_6D               (6)

typedef enum {
  ACC_INT_MODE_OR            =    0x00,
  ACC_INT_MODE_6D_MOVEMENT   =    0x01,
  ACC_INT_MODE_AND           =    0x02,
  ACC_INT_MODE_6D_POSITION   =    0x03
}  ACC_INT_MODE_E;



#define ACC_REG_OUT_X_L              	0x28
#define ACC_REG_OUT_X_H              	0x29
#define ACC_REG_OUT_Y_L              	0x2A
#define ACC_REG_OUT_Y_H              	0x2B
#define ACC_REG_OUT_Z_L              	0x2C
#define ACC_REG_OUT_Z_H              	0x2D


#define ACC_CTRL_REG3					0x22
//CONTROL REGISTER 2
#define ACC_CTRL_REG4					0x23


//CONTROL REGISTER 2
#define ACC_CTRL_REG2					0x21

#define ACC_CTRL_BOOT                   (7)
#define ACC_CTRL_HPM     				(5)
#define ACC_CTRL_FDS     				(4)
#define ACC_CTRL_HPEN2					(3)
#define ACC_CTRL_HPEN1					(2)
#define ACC_CTRL_HPCF					(0)


typedef enum {
  ACC_FILTER_INT_BYPASS        	=    0x00,
  ACC_FILTER_INT_ENABLED   		=    0x01,
} ACC_FILTER_INT_E;




typedef	union
{
	struct
	{
		UNS8 err_state:1;
		UNS8 err_device:1;
		UNS8 err_Type:6;
	} Bit;
	UNS8 Dword;

} FAULT_STATE_T;

typedef struct {
  UNS8  faultCnt[ACC_MAX];
  FAULT_STATE_T record;
  UNS8 	Reserved[1];
} ACC_FAULT_T;



typedef struct
{
	ACC_FAULT_T         fault;
	AXIS_DATA_T         axis;
	AXIS_DATA_T         axis_Verify;
	AXIS_DATA_T 		offset[ACC_MAX];
	AXIS_DATA_T 		sumset[ACC_MAX];
//	AXIS_DATA32BIT_T    average[ACC_MAX];
	ACCEL_STATE_E  		State;
	ACCEL_INIT_E        InitStep;
	UNS16 				cali_cnt;
	UNS8 				err_cnt;
	UNS8 				autocali;
	BOOL 				CaliFlag;
	UNS8 				Reserved[2];
} ACC_CTRL_PACKAGE_T;

ACC_CTRL_PACKAGE_T AccelCtrlPkg;
//status_t response;

ACC_ERROR_E ACC_SoftReset(ACC_CH_E ch);
ACC_ERROR_E ACC_ConfigMode(ACC_CH_E ch);
ACC_ERROR_E ACC_ConfigCtrlReg(ACC_CH_E ch);
ACC_ERROR_E ACC_SetInt1Threshold(ACC_CH_E ch);
ACC_ERROR_E ACC_SetInt1Config(ACC_CH_E ch);
ACC_ERROR_E ACC_EnableInt1(ACC_CH_E ch);
ACC_ERROR_E ACC_SetInt2Threshold(ACC_CH_E ch);
ACC_ERROR_E ACC_SetInt2Config(ACC_CH_E ch);
ACC_ERROR_E ACC_EnableInt2(ACC_CH_E ch);
ACC_ERROR_E ACC_Init_Done(ACC_CH_E ch);
ACC_ERROR_E ACC_CheckDevice(ACC_CH_E ch);
ACC_ERROR_E ACC_SetHPFCutOFF(ACC_CH_E ch);
ACC_ERROR_E ACC_SetHPFMode(ACC_CH_E ch);
ACC_ERROR_E ACC_HPFilterReset(ACC_CH_E ch);
ACC_ERROR_E ACC_SetFDS(ACC_CH_E ch);
ACC_ERROR_E ACC_UpdateSpiBlock(ACC_CH_E ch);

static const FUNCTION_PTR AccelExecRtnTable[ACCEL_INIT_MAX] ={
	ACC_SoftReset,
	ACC_CheckDevice,
	ACC_ConfigMode,
	ACC_UpdateSpiBlock,
	ACC_SetInt1Threshold,
	ACC_SetInt1Config,
	ACC_EnableInt1,
	ACC_SetInt2Threshold,
	ACC_SetInt2Config,
	ACC_EnableInt2,
	ACC_ConfigCtrlReg,
//	ACC_SetHPFCutOFF,
//	ACC_SetHPFMode,
//	ACC_HPFilterReset,
//	ACC_SetFDS,
	ACC_Init_Done,
};


ACC_LL_STATUS_E ACC_WriteReg(ACC_CH_E ch, UNS8 Reg, UNS8 *pBuf)
{
	ACC_LL_STATUS_E status = ACC_LL_ERROR;
	UNS16 cmd = (((Reg | ACC_WRITE_BIT) << ACC_WRITE_REG_MASK) | *pBuf);
	if(SPI_NO_ERROR == SPI2_Write16Bit(ch,&cmd)){
		status = ACC_LL_SUCCESS;}
  return status;
}

ACC_LL_STATUS_E ACC_ReadReg(ACC_CH_E ch, UNS8 Reg, UNS8 *pBuf)
{
	ACC_LL_STATUS_E status = ACC_LL_ERROR;
	UNS16 cmd = ((Reg | ACC_READ_BIT) << ACC_READ_REG_MASK);
	if(SPI_NO_ERROR == SPI2_Write16Bit(ch,&cmd)){
		*pBuf = (cmd & ACC_DATA_MASK);
		status = ACC_LL_SUCCESS;}
	return status;
}



ACC_ERROR_E ACC_SoftReset(ACC_CH_E ch)
{
	/*Waiting to add*/
	return ACC_ERR_NO;
}

ACC_ERROR_E ACC_CheckDevice(ACC_CH_E ch)
{
	UNS8 reg = 0;
	if(ACC_LL_ERROR == ACC_ReadReg ( ch, ACC_WHO_AM_I, &reg)){
		return ACC_ERR_WHO_READ;}
	if(reg != ACC_DEFAULT_DEVICE)
		return ACC_ERR_WHO_NO;
	/*Waiting to add*/
	return ACC_ERR_NO;
}

/*Config Mode reg1*/
ACC_ERROR_E ACC_ConfigMode(ACC_CH_E ch)
{
	UNS8 cmd,reg = 0;
	cmd = (ACC_ODR_LP_1600Hz | ACC_LP_ENABLE | ACC_X_ENABLE | ACC_Y_ENABLE| ACC_Z_ENABLE);
	if(ACC_LL_ERROR == ACC_WriteReg ( ch, ACC_CTRL_REG1, &cmd)){
		return ACC_ERR_CFG_MODE_WRIT;}

	if(ACC_LL_ERROR == ACC_ReadReg ( ch, ACC_CTRL_REG1, &reg)){
		return ACC_ERR_CFG_MODE_READ;}

	if(cmd != reg){
		return ACC_ERR_CFG_MODE_WRIT_READ;}
	return ACC_ERR_NO;
}






typedef enum {
	ACC_INT1_I1_CLICK_ENABLE       	 =    0x80,
	ACC_INT1_I1_IA1_ENABLE     	  	 =    0x40,
	ACC_INT1_I1_IA2_ENABLE         	 =    0x20,
	ACC_INT1_I1_ZYXDA_ENABLE      	 =    0x10,
	ACC_INT1_I1_321DA_ENABLE  	  	 =    0x08,
	ACC_INT1_I1_FIFO_WTM_ENABLE      =    0x04,
	ACC_INT1_I1_FIFO_OVERRUN_ENABLE  =    0x02,
} ACC_INT_CONF_T;



ACC_ERROR_E ACC_ConfigCtrlReg(ACC_CH_E ch)
{
	UNS8 cmd,reg = 0;
	cmd = (ACC_INT1_I1_IA1_ENABLE);
	if(ACC_LL_ERROR == ACC_WriteReg (ch,ACC_CTRL_REG3, &cmd)){
		return ACC_ERR_CFG_MODE_WRIT;}

	if(ACC_LL_ERROR == ACC_ReadReg ( ch, ACC_CTRL_REG3, &reg)){
		return ACC_ERR_CFG_MODE_READ;}

	if(cmd != reg){
		return ACC_ERR_CFG_MODE_WRIT_READ;}
	return ACC_ERR_NO;
}



ACC_ERROR_E ACC_SetInt1Threshold(ACC_CH_E ch)
{
	UNS8 cmd,reg = 0;
	cmd = INT1_THRESHOLD;
	if(ACC_LL_ERROR == ACC_WriteReg(ch, ACC_INT1_THS_REG, &cmd)){
		return ACC_ERR_INT1_THRES_WRIT;}

	if(ACC_LL_ERROR == ACC_ReadReg (ch, ACC_INT1_THS_REG, &reg)){
		return ACC_ERR_INT1_THRES_READ;}

	if(cmd != reg){
		return ACC_ERR_INT1_THRES_WRIT_READ;}

	return ACC_ERR_NO;
}


ACC_ERROR_E ACC_SetInt1Config(ACC_CH_E ch)
{
	UNS8 cmd,reg;


	cmd = (ACC_INT_YHIE_ENABLE | ACC_INT_XHIE_ENABLE);

	if(ACC_LL_ERROR == ACC_WriteReg (ch, ACC_INT1_CFG, &cmd)){
		return ACC_ERR_CFG_MODE_WRIT;}

	if(ACC_LL_ERROR == ACC_ReadReg ( ch, ACC_INT1_CFG, &reg)){
		return ACC_ERR_CFG_MODE_READ;}

	if(cmd != reg){
		return ACC_ERR_CFG_MODE_WRIT_READ;}

	return ACC_ERR_NO;
}



ACC_ERROR_E ACC_UpdateSpiBlock(ACC_CH_E ch)
{
#define ACC_SIP_UPDATE_ENABLE		0x80
#define ACC_BLE_MSB_ENABLE			0x40
#define ACC_FULL_SCALE_2G_ENABLE	0x00
#define ACC_FULL_SCALE_4G_ENABLE	0x10
#define ACC_FULL_SCALE_8G_ENABLE	0x20
#define ACC_FULL_SCALE_16G_ENABLE	0x04
#define ACC_HIGH_RES_ENABLE			0x01
#define ACC_SIM_4WIRE_ENABLE		0x00

	UNS8 cmd,reg;

	cmd = ACC_SIP_UPDATE_ENABLE | ACC_BLE_MSB_ENABLE | ACC_FULL_SCALE_16G_ENABLE | ACC_SIM_4WIRE_ENABLE;
	if(ACC_LL_ERROR == ACC_WriteReg (ch, ACC_CTRL_REG4, &cmd)){
		return ACC_ERR_CFG_MODE_WRIT;}

	if(ACC_LL_ERROR == ACC_ReadReg ( ch, ACC_CTRL_REG4, &reg)){
		return ACC_ERR_CFG_MODE_READ;}

	if(cmd != reg){
		return ACC_ERR_CFG_MODE_WRIT_READ;}

	return ACC_ERR_NO;
}

ACC_ERROR_E ACC_EnableInt1(ACC_CH_E ch)
{

	return ACC_ERR_NO;
}




ACC_ERROR_E ACC_SetInt2Threshold(ACC_CH_E ch)
{
	return ACC_ERR_NO;
}
ACC_ERROR_E ACC_SetInt2Config(ACC_CH_E ch)
{
	return ACC_ERR_NO;
}
ACC_ERROR_E ACC_EnableInt2(ACC_CH_E ch)
{
	return ACC_ERR_NO;
}
ACC_ERROR_E ACC_Init_Done(ACC_CH_E ch)
{
	/*Waiting to add*/
	return ACC_ERR_NO;
}



ACC_ERROR_E ACC_SetFDS(ACC_CH_E ch)/*Set Filter Data Selection*/
{
#define ACC_HP_FILTER_FDS_MASK			0xEF
	ACC_FILTER_INT_E SETFDS = ACC_FILTER_INT_ENABLED;

	UNS8 cmd,reg;
	if(ACC_LL_ERROR == ACC_ReadReg ( ch, ACC_CTRL_REG2, &cmd)){
		return ACC_ERR_HPFILTER_READ;}
	cmd &= ACC_HP_FILTER_FDS_MASK;
	cmd |= (SETFDS << ACC_CTRL_FDS);
	if(ACC_LL_ERROR == ACC_WriteReg (ch, ACC_CTRL_REG2, &cmd)){
		return ACC_ERR_HPFILTER_WRIT;}

	if(ACC_LL_ERROR == ACC_ReadReg ( ch, ACC_CTRL_REG2, &reg)){
		return ACC_ERR_HPFILTER_READ;}

	if(cmd != reg){
		return ACC_ERR_HPFILTER_WRIT_READ;}

	return ACC_ERR_NO;
}



ACC_ERROR_E ACC_HPFilterReset(ACC_CH_E ch)
{
#define ACC_HP_FILTER_RESET			0x25
	UNS8 reg = 0;
	if(ACC_LL_ERROR == ACC_ReadReg (ch, ACC_HP_FILTER_RESET, &reg)){
		return ACC_ERR_HPFILTER_RESET_READ;}

	return ACC_ERR_NO;
}


ACC_ERROR_E ACC_SetReference(ACC_CH_E ch)
{
#define ACC_REFERENCE_REG		                0x26

	UNS8 cmd,reg = 0;
	cmd = ACC_REFERENCE_VAL;
	if(ACC_LL_ERROR == ACC_WriteReg(ch, ACC_REFERENCE_REG, &cmd)){
		return ACC_ERR_HPFILTER_WRIT;}

	if(ACC_LL_ERROR == ACC_ReadReg (ch, ACC_REFERENCE_REG, &reg)){
		return ACC_ERR_HPFILTER_READ;}

	if(cmd != reg){
		return ACC_ERR_HPFILTER_WRIT_READ;}

	return ACC_ERR_NO;
}

ACC_ERROR_E ACC_SetHPFMode(ACC_CH_E ch)
{
#define ACC_SETHPF_MODE_MASK			0x9F

typedef enum {
  ACC_HPM_NORMAL_MODE_RES            =             0x00,
  ACC_HPM_REF_SIGNAL                 =             0x01,
  ACC_HPM_NORMAL_MODE                =             0x02,
} ACC_HPFMODE_T;

ACC_HPFMODE_T acc_mode = ACC_HPM_NORMAL_MODE_RES;

	UNS8 cmd,reg;
	if(ACC_LL_ERROR == ACC_ReadReg ( ch, ACC_CTRL_REG2, &cmd)){
		return ACC_ERR_HPFILTER_READ;}
	cmd &= ACC_SETHPF_MODE_MASK;
	cmd |= (acc_mode << ACC_CTRL_HPM);
	if(ACC_LL_ERROR == ACC_WriteReg (ch, ACC_CTRL_REG2, &cmd)){
		return ACC_ERR_HPFILTER_WRIT;}

	if(ACC_LL_ERROR == ACC_ReadReg ( ch, ACC_CTRL_REG2, &reg)){
		return ACC_ERR_HPFILTER_READ;}

	if(cmd != reg){
		return ACC_ERR_HPFILTER_WRIT_READ;}

	return ACC_ERR_NO;
}

ACC_ERROR_E ACC_SetHPFCutOFF(ACC_CH_E ch)
{
#define ACC_SETHPF_CUTOFF_MASK			0xFC

typedef enum {
  ACC_HPFCF_0                        =             0x00,/* 20hz*/
  ACC_HPFCF_1                        =             0x01,/*10hz */
  ACC_HPFCF_2                        =             0x02,/*5hz */
  ACC_HPFCF_3                        =             0x03 /* 2.5hz*/
} ACC_HPFCUTOFFFREQ_T;

ACC_HPFCUTOFFFREQ_T hpfcf = ACC_HPFCF_2;

	UNS8 cmd,reg;
	if(ACC_LL_ERROR == ACC_ReadReg ( ch, ACC_CTRL_REG2, &cmd)){
		return ACC_ERR_HPFILTER_READ;}
	cmd &= ACC_SETHPF_CUTOFF_MASK;
	cmd |= (hpfcf << ACC_CTRL_HPCF);
	if(ACC_LL_ERROR == ACC_WriteReg (ch, ACC_CTRL_REG2, &cmd)){
		return ACC_ERR_HPFILTER_WRIT;}

	if(ACC_LL_ERROR == ACC_ReadReg ( ch, ACC_CTRL_REG2, &reg)){
		return ACC_ERR_HPFILTER_READ;}

	if(cmd != reg){
		return ACC_ERR_HPFILTER_WRIT_READ;}

	return ACC_ERR_NO;
}


ACC_ERROR_E ACC_SetInt2HPEnable(ACC_CH_E ch)
{
#define ACC_SETINT2HPF_ENABLE_MASK			0xF7

	ACC_FILTER_INT_E filter = ACC_FILTER_INT_ENABLED;

	UNS8 cmd,reg;
	if(ACC_LL_ERROR == ACC_ReadReg ( ch, ACC_CTRL_REG2, &cmd)){
		return ACC_ERR_HPFILTER_READ;}
	cmd &= ACC_SETINT2HPF_ENABLE_MASK;
	cmd |= (filter << ACC_CTRL_HPEN2);
	if(ACC_LL_ERROR == ACC_WriteReg (ch, ACC_CTRL_REG2, &cmd)){
		return ACC_ERR_HPFILTER_WRIT;}

	if(ACC_LL_ERROR == ACC_ReadReg ( ch, ACC_CTRL_REG2, &reg)){
		return ACC_ERR_HPFILTER_READ;}

	if(cmd != reg){
		return ACC_ERR_HPFILTER_WRIT_READ;}

	return ACC_ERR_NO;
}


ACC_ERROR_E ACC_SetInt1HPEnable(ACC_CH_E ch)
{
#define ACC_SETINT1HPF_ENABLE_MASK			0xFB

	ACC_FILTER_INT_E filter = ACC_FILTER_INT_ENABLED;

	UNS8 cmd,reg;
	if(ACC_LL_ERROR == ACC_ReadReg ( ch, ACC_CTRL_REG2, &cmd)){
		return ACC_ERR_HPFILTER_READ;}
	cmd &= ACC_SETINT2HPF_ENABLE_MASK;
	cmd |= (filter << ACC_CTRL_HPEN1);
	if(ACC_LL_ERROR == ACC_WriteReg (ch, ACC_CTRL_REG2, &cmd)){
		return ACC_ERR_HPFILTER_WRIT;}

	if(ACC_LL_ERROR == ACC_ReadReg ( ch, ACC_CTRL_REG2, &reg)){
		return ACC_ERR_HPFILTER_READ;}

	if(cmd != reg){
		return ACC_ERR_HPFILTER_WRIT_READ;}

	return ACC_ERR_NO;
}







BOOL ACC_FaultCount(ACC_CH_E ch,ACC_ERROR_E err,UNS16 CNT_MAX)
{
	BOOL ret = FALSE;
	if(err != ACC_ERR_NO){
		if(AccelCtrlPkg.fault.faultCnt[ch] < CNT_MAX){
		AccelCtrlPkg.fault.faultCnt[ch]++;
		}else{
			AccelCtrlPkg.fault.record.Bit.err_state = TRUE;
			AccelCtrlPkg.fault.record.Bit.err_device = ch;
			AccelCtrlPkg.fault.record.Bit.err_Type = err;
			AccelCtrlPkg.fault.faultCnt[ch] = 0;

		}
		ret	= TRUE;
	}else{
		AccelCtrlPkg.fault.faultCnt[ch] = 0;
	}
	return ret;

}

BOOL ACC_InitErrCount(ACC_CH_E ch,ACC_ERROR_E err)
{
	BOOL ret = FALSE;
	if(err == ACC_ERR_NO){
		AccelCtrlPkg.InitStep++;
		AccelCtrlPkg.err_cnt = 0;
	}else{
		if(AccelCtrlPkg.err_cnt < ACC_INIT_ERR_COUNT_MAX){
			AccelCtrlPkg.err_cnt++;
		}else{
			AccelCtrlPkg.err_cnt = 0;
			AccelCtrlPkg.InitStep = ACCEL_INIT_SOFTRESET;
			if(TRUE == ACC_FaultCount(ch,err,ACC_FAULT_COUNT_MAX)){
				AccelCtrlPkg.State = ACCEL_STATE_ERROR;
			}
			ret	= TRUE;
		}
	}
	if(AccelCtrlPkg.InitStep == ACCEL_INIT_MAX){
		AccelCtrlPkg.State++;
		AccelCtrlPkg.InitStep = 0;
	}
	return ret;
}


ACC_ERROR_E ACC_ReadAxisReg(ACC_CH_E ch,AXIS_DATA_T* data)
{
	S16 Acc_S = 0;
	UNS8 *Acc_L = (UNS8 *)(&Acc_S);
	UNS8 *Acc_H = ((UNS8 *)(&Acc_S)+1);
	ACC_ERROR_E err_x = ACC_ERR_NO;
	ACC_ERROR_E err_y = ACC_ERR_NO;
	ACC_ERROR_E err_z = ACC_ERR_NO;


	if(ACC_LL_ERROR == ACC_ReadReg(ch, ACC_REG_OUT_X_L,Acc_L)){
		err_x =  ACC_ERR_OUT_X_READ;
	}

	if(ACC_LL_ERROR == ACC_ReadReg(ch, ACC_REG_OUT_X_H,Acc_H)){
		err_x =   ACC_ERR_OUT_X_READ;
	}
	if(err_x == ACC_ERR_NO){
		data->X = Acc_S>>4;
	}else{
		data->X = 0;
	}


	if(ACC_LL_ERROR == ACC_ReadReg(ch, ACC_REG_OUT_Y_L, Acc_L)){
		err_y =   ACC_ERR_OUT_Y_READ;
	}
	if(ACC_LL_ERROR == ACC_ReadReg(ch, ACC_REG_OUT_Y_H, Acc_H)){
		err_y =   ACC_ERR_OUT_Y_READ;
	}
	if(err_y == ACC_ERR_NO){
		data->Y = Acc_S>>4;
	}else{
		data->Y = 0;
	}

	if(ACC_LL_ERROR == ACC_ReadReg(ch, ACC_REG_OUT_Z_L, Acc_L)){
		err_z =   ACC_ERR_OUT_Y_READ;
	}
	if(ACC_LL_ERROR == ACC_ReadReg(ch, ACC_REG_OUT_Z_H, Acc_H)){
		err_z =   ACC_ERR_OUT_Y_READ;
	}
	if(err_z == ACC_ERR_NO){
		data->Z = Acc_S>>4;
	}else{
		data->Z =0;
	}
	return 0;
}


ACC_ERROR_E ACC_ReadAxisRegCail(ACC_CH_E ch,AXIS_DATA_T* data)
{
	S16 Acc_S;
	UNS8 *Acc_L = (UNS8 *)(&Acc_S);
	UNS8 *Acc_H = ((UNS8 *)(&Acc_S)+1);


	if(ACC_LL_ERROR == ACC_ReadReg(ch, ACC_REG_OUT_X_L,Acc_L))
	return ACC_ERR_OUT_X_READ;


	if(ACC_LL_ERROR == ACC_ReadReg(ch, ACC_REG_OUT_X_H,Acc_H))
	return ACC_ERR_OUT_X_READ;
	data->X = Acc_S>>4;

	return ACC_ERR_NO;
}

ACC_ERROR_E ACC_PowerModeSet(ACC_CH_E ch,UNS8 mode)
{
	UNS8 cmd,reg = 0;

	if(ACC_LL_ERROR == ACC_ReadReg(ch, ACC_CTRL_REG1, &cmd)){
		return ACC_ERR_SLEEP_READ;}
	cmd &= 0x1F;
	cmd |= mode;


	if(ACC_LL_ERROR == ACC_WriteReg(ch, ACC_CTRL_REG1, &cmd)){
		return ACC_ERR_SLEEP_WRIT;}

	if(ACC_LL_ERROR == ACC_ReadReg(ch, ACC_CTRL_REG1, &reg)){
		return ACC_ERR_SLEEP_READ;}

	if(cmd != reg){
		return ACC_ERR_SLEEP_WRIT_READ;}

	return ACC_ERR_NO;
}

ACC_ERROR_E ACC_WkupToSleep(ACC_CH_E ch)
{
	UNS8 cmd,reg = 0;
	cmd = ACC_SLEEP_ON;
	if(ACC_LL_ERROR == ACC_WriteReg ( ch, ACC_CTRL_REG5, &cmd)){
		return ACC_ERR_SLEEP_WRIT;}

	if(ACC_LL_ERROR == ACC_ReadReg ( ch, ACC_CTRL_REG5, &reg)){
		return ACC_ERR_SLEEP_READ;}

	if(cmd != reg){
		return ACC_ERR_SLEEP_WRIT_READ;}



	return ACC_ERR_NO;
}



BOOL ACC_ClearSRC(void)
{
	UNS8 reg = 0;
	ACC_ReadReg(ACC_MASTER, ACC_INT1_SRC, &reg);
	ACC_ReadReg(ACC_SLAVE, ACC_INT1_SRC, &reg);
	return 0;
}

void ACC_RamInit(void)
{
	(void)memset(&AccelCtrlPkg, 0, sizeof(AccelCtrlPkg));
}

AXIS_DATA_T MasterSensorTo180Deg(AXIS_DATA_T* axis)
{
	AXIS_DATA_T acc;
	acc.X = ~(axis->X) + 1;
	acc.Y = ~(axis->Y) + 1 ;
	acc.Z = ~(axis->Z) + 1 ;
	return acc;
}

AXIS_DATA_T SlaveSensorTo45Deg(AXIS_DATA_T* axis)
 {
	AXIS_DATA_T acc;


	acc.X =   ~(axis->Y) + 1 ;
	acc.Y = axis->X;
	acc.Z = ~(axis->Z) + 1 ;
	return acc;
}







static X_AXIS_BOUND_RANGE_U  FindXAxisBoundRange(const S32 master_accel_x)
{

    if(master_accel_x >  90)   {return MORE_90 ;}
    if(master_accel_x >  45)   {return BETWEEN_45_90 ;}
    if(master_accel_x >  -45)   {return BETWEEN_NEGAT45_45;}
    if(master_accel_x >  -90)   {return  BETWEEN_NEGAT90_NEGAT45 ;}

    return LESS_THAN_NEGAT90;


}
static BOOL  SlaveAccVerifyMasterAcc(const S32 master_accel_x ,const S32 slave_accel_x )
{
	ACC_ERROR_E err = ACC_ERR_CODE_CHECK;
    S32  high_x ,low_x ;
    X_AXIS_BOUND_RANGE_U x_axis_range = FindXAxisBoundRange(master_accel_x);

     switch(x_axis_range){

        case LESS_THAN_NEGAT90:
	        high_x  = master_accel_x >> 1;
	        low_x   = master_accel_x << 1 ;

	        break;

        case BETWEEN_NEGAT90_NEGAT45:
            high_x = 45 + master_accel_x ;
            low_x =  master_accel_x << 1 ;

            break;
        case BETWEEN_NEGAT45_45:
        	high_x = 45 + master_accel_x;
            low_x  =  (-45)+ master_accel_x;
            break;

        case BETWEEN_45_90:
            high_x =  master_accel_x << 1 ;
            low_x =  (-45) + master_accel_x ;
            break;

        case  MORE_90 :
            high_x  =  master_accel_x << 1 ;
            low_x   =  master_accel_x >> 1 ;
            break;

        default :

            break;
    }

     if(slave_accel_x <= high_x && slave_accel_x >=  low_x){
    	 err =  ACC_ERR_NO;
     }
     return  err;
}








 S32 AccelFilterType( const S32 accel_data,S32 filter_value[2])
{
    S32 tmp_val,ret_val;

    tmp_val = (accel_data << 6 ) * 7 + filter_value[0] * 25 ;
    if(tmp_val < 0){
    	filter_value[0] = (tmp_val + 32 - 1) >> 5 ;
    }else filter_value[0] = tmp_val >> 5 ;

    tmp_val = (filter_value[0] << 6 ) * 7 + filter_value[1] * 25 ;
    if(tmp_val < 0){
    	filter_value[1] = (tmp_val + 32 - 1) >> 5 ;
    }else filter_value[1] = tmp_val >> 5 ;

    if(filter_value[1] >= 0){
    	ret_val = (filter_value[1] + 32) >> 12 ;
    }
    else {
    	ret_val = (filter_value[1] + 4096 - 1 - 32) >> 12 ;
    }

    return ret_val;

}




ACC_ERROR_E DoubleAccelVerify(AXIS_DATA_T *m_data,AXIS_DATA_T *s_data)
{
    S32 master_filter_x , slave_filter_x;
	ACC_ERROR_E err = ACC_ERR_NO;

	master_filter_x  = AccelFilterType(m_data->X,Filter_master_value);
	slave_filter_x   = AccelFilterType(s_data->X,Filter_slave_value);

	err = SlaveAccVerifyMasterAcc(master_filter_x,slave_filter_x);
	if(err == ACC_ERR_NO){

	}


	return err;

}





BOOL ACC_StartCalibration(AXIS_DATA_T *m_data,AXIS_DATA_T *s_data)
{
	ACC_ERROR_E err = ACC_ERR_NO;
	AXIS_DATA_T m_acc,s_acc;
	if(AccelCtrlPkg.cali_cnt < ACC_CALI_CNT_MAX){
		AccelCtrlPkg.cali_cnt++;
		err =  ACC_ReadAxisReg(ACC_MASTER,&m_acc);
		ACC_FaultCount(ACC_MASTER,err,ACC_FAULT_COUNT_MAX);
		err =  ACC_ReadAxisReg(ACC_SLAVE,&s_acc);
		ACC_FaultCount(ACC_SLAVE,err,ACC_FAULT_COUNT_MAX);

		m_data->X += m_acc.X;
		m_data->Y += m_acc.Y;
		m_data->Z += m_acc.Z;
		s_data->X += s_acc.X;
		s_data->Y += s_acc.Y;
		s_data->Z += s_acc.Z;

	}else{
		AccelCtrlPkg.cali_cnt = 0;
		m_data->X = m_data->X/ACC_CALI_CNT_MAX;
		m_data->Y = m_data->Y/ACC_CALI_CNT_MAX;
		m_data->Z = m_data->Z/ACC_CALI_CNT_MAX;
		s_data->X = s_data->X/ACC_CALI_CNT_MAX;
		s_data->Y = s_data->Y/ACC_CALI_CNT_MAX;
		s_data->Z = s_data->Z/ACC_CALI_CNT_MAX;
		m_acc = MasterSensorTo180Deg(m_data);
		s_acc = SlaveSensorTo45Deg(s_data);

		if(ACC_ERR_NO == DoubleAccelVerify(&m_acc,&s_acc)){
			return TRUE;
		}
		AccelCtrlPkg.cali_cnt = 0;
		m_data->X = 0;
		m_data->Y = 0;
		m_data->Z = 0;
		s_data->X = 0;
		s_data->Y = 0;
		s_data->Z = 0;
	}
	return FALSE;
}




BOOL ACC_StartCalibrationCheck(void)
{
#define ACC_OFFSET_HIGH_DEV_MAX  		40
#define ACC_OFFSET_LOW_DEV_MAX  		-40
#define ACC_CURRENT_HIGH_DEV_MAX  		10
#define ACC_CURRENT_LOW_DEV_MAX  		-10
UNS8 retval = 0;

	if(IS_OUT_RANGE(AccelCtrlPkg.offset[ACC_MASTER].X,ACC_OFFSET_LOW_DEV_MAX,ACC_OFFSET_HIGH_DEV_MAX)){
		retval++;
	}
	if(IS_OUT_RANGE(AccelCtrlPkg.offset[ACC_MASTER].Y,ACC_OFFSET_LOW_DEV_MAX,ACC_OFFSET_HIGH_DEV_MAX)){
		retval++;
	}
	if(IS_OUT_RANGE(AccelCtrlPkg.offset[ACC_SLAVE].X,ACC_OFFSET_LOW_DEV_MAX,ACC_OFFSET_HIGH_DEV_MAX)){
		retval++;
	}

	if(IS_OUT_RANGE(AccelCtrlPkg.offset[ACC_SLAVE].Y,ACC_OFFSET_LOW_DEV_MAX,ACC_OFFSET_HIGH_DEV_MAX)){
		retval++;
	}

	if(IS_OUT_RANGE((AccelCtrlPkg.sumset[ACC_MASTER].X - AccelCtrlPkg.offset[ACC_MASTER].X),ACC_CURRENT_LOW_DEV_MAX,ACC_CURRENT_HIGH_DEV_MAX)){
		retval++;
	}
	if(IS_OUT_RANGE((AccelCtrlPkg.sumset[ACC_MASTER].Y - AccelCtrlPkg.offset[ACC_MASTER].Y),ACC_CURRENT_LOW_DEV_MAX,ACC_CURRENT_HIGH_DEV_MAX)){
		retval++;
	}
	if(IS_OUT_RANGE((AccelCtrlPkg.sumset[ACC_SLAVE].X - AccelCtrlPkg.offset[ACC_SLAVE].X),ACC_CURRENT_LOW_DEV_MAX,ACC_CURRENT_HIGH_DEV_MAX)){
		retval++;
	}
	if(IS_OUT_RANGE((AccelCtrlPkg.sumset[ACC_SLAVE].Y - AccelCtrlPkg.offset[ACC_SLAVE].Y),ACC_CURRENT_LOW_DEV_MAX,ACC_CURRENT_HIGH_DEV_MAX)){
		retval++;
	}



	if(retval)
		return FALSE;
	return TRUE;
}

void ACC_InitCalibration(void){

	AccelCtrlPkg.CaliFlag = TRUE;
	AccelCtrlPkg.cali_cnt = 0;
	AccelCtrlPkg.sumset[ACC_MASTER].X = 0;
	AccelCtrlPkg.sumset[ACC_MASTER].Y = 0;
	AccelCtrlPkg.sumset[ACC_MASTER].Z = 0;
	AccelCtrlPkg.sumset[ACC_SLAVE].X = 0;
	AccelCtrlPkg.sumset[ACC_SLAVE].Y = 0;
	AccelCtrlPkg.sumset[ACC_SLAVE].Z = 0;
}



void ACC_CalibrationRoutine(void)
{

}


void ACC_DataRoutine(void)
{
	ACC_ERROR_E master_err = ACC_ERR_NO;
	ACC_ERROR_E slave_err = ACC_ERR_NO;
//	ACC_ERROR_E double_err = ACC_ERR_NO;
	AXIS_DATA_T master_acc,slave_acc;
	if(AccelCtrlPkg.State == ACCEL_STATE_NORMAL){
		master_err =  ACC_ReadAxisReg(ACC_MASTER,&master_acc);
		slave_err =  ACC_ReadAxisReg(ACC_SLAVE,&slave_acc);
//		master_acc.X  = master_acc.X - AccelCtrlPkg.offset[ACC_MASTER].X;
//		master_acc.Y  =  master_acc.Y - AccelCtrlPkg.offset[ACC_MASTER].Y;

		ACC_FaultCount(ACC_MASTER,master_err,ACC_FAULT_COUNT_MAX);
		ACC_FaultCount(ACC_SLAVE,slave_err,ACC_FAULT_COUNT_MAX);

//		slave_acc.X  = slave_acc.X - AccelCtrlPkg.offset[ACC_SLAVE].X;
//		slave_acc.Y  =  slave_acc.Y - AccelCtrlPkg.offset[ACC_SLAVE].Y;

		master_acc = MasterSensorTo180Deg(&master_acc);
		slave_acc = SlaveSensorTo45Deg(&slave_acc);

//		double_err = DoubleAccelVerify(&master_acc,&slave_acc);
//		ACC_FaultCount(ACC_DUAL_DATA,double_err,ACC_TWO_FAULT_COUNT_MAX);
		if(AccelCtrlPkg.fault.record.Bit.err_state == FALSE){
			AccelCtrlPkg.axis_Verify = slave_acc;
#if 1
			AccelCtrlPkg.axis.X = master_acc.X;
			AccelCtrlPkg.axis.Y = master_acc.Y;
			AccelCtrlPkg.axis.Z = master_acc.Z;
#else
			AccelCtrlPkg.axis.X = (master_acc.X + slave_acc.X)/2;
			AccelCtrlPkg.axis.Y = (master_acc.Y + slave_acc.Y)/2;
			AccelCtrlPkg.axis.Z = (master_acc.Z + slave_acc.Z)/2;
#endif

		}else{
			AccelCtrlPkg.axis.X = 0;
			AccelCtrlPkg.axis.Y = 0;
		}
	}else{
		AccelCtrlPkg.axis.X = 0;
		AccelCtrlPkg.axis.Y = 0;
	}


}



void ACC_Routine(void)
{
	ACC_ERROR_E err = ACC_ERR_NO;

		switch(AccelCtrlPkg.State)
		{
		case ACCEL_STATE_SOFTRESET:
			AccelCtrlPkg.State++;
			AccelCtrlPkg.InitStep = 0;
			AccelCtrlPkg.cali_cnt = 0;
			break;
		case ACCEL_STATE_INIT_MASTER:
			if(AccelCtrlPkg.InitStep < ACCEL_INIT_MAX)
			err = AccelExecRtnTable[AccelCtrlPkg.InitStep](ACC_MASTER);
			ACC_InitErrCount(ACC_MASTER,err);
			break;
		case ACCEL_STATE_INIT_SLAVE:
			if(AccelCtrlPkg.InitStep < ACCEL_INIT_MAX)
			err = AccelExecRtnTable[AccelCtrlPkg.InitStep](ACC_SLAVE);
			ACC_InitErrCount(ACC_SLAVE,err);
			AccelCtrlPkg.offset[ACC_MASTER].X = 0;
			AccelCtrlPkg.offset[ACC_MASTER].Y = 0;
			AccelCtrlPkg.offset[ACC_SLAVE].X = 0;
			AccelCtrlPkg.offset[ACC_SLAVE].Y = 0;
			AccelCtrlPkg.sumset[ACC_MASTER].X = 0;
			AccelCtrlPkg.sumset[ACC_MASTER].Y = 0;
			AccelCtrlPkg.sumset[ACC_SLAVE].X = 0;
			AccelCtrlPkg.sumset[ACC_SLAVE].Y = 0;

			break;
		case ACCEL_STATE_FIRST_CALI:
			if(ACC_StartCalibration(&AccelCtrlPkg.offset[ACC_MASTER],&AccelCtrlPkg.offset[ACC_SLAVE])){
				AccelCtrlPkg.State++;}

			break;
		case ACCEL_STATE_DELAY_CALI:
			if(AccelCtrlPkg.cali_cnt < ACC_CALI_DELAY_MAX){
				AccelCtrlPkg.cali_cnt++;
			}else{
				AccelCtrlPkg.cali_cnt = 0;
				AccelCtrlPkg.State++;
			}
			break;
		case ACCEL_STATE_TWO_CALI:
			if(ACC_StartCalibration(&AccelCtrlPkg.sumset[ACC_MASTER],&AccelCtrlPkg.sumset[ACC_SLAVE])){
				AccelCtrlPkg.State++;}
			break;
		case ACCEL_STATE_CALI_CHECK:
			if(ACC_StartCalibrationCheck()){
				AccelCtrlPkg.State++;
			}else{
				AccelCtrlPkg.State = ACCEL_STATE_FIRST_CALI;
				AccelCtrlPkg.offset[ACC_MASTER].X = 0;
				AccelCtrlPkg.offset[ACC_MASTER].Y = 0;
				AccelCtrlPkg.offset[ACC_SLAVE].X = 0;
				AccelCtrlPkg.offset[ACC_SLAVE].Y = 0;
				AccelCtrlPkg.sumset[ACC_MASTER].X = 0;
				AccelCtrlPkg.sumset[ACC_MASTER].Y = 0;
				AccelCtrlPkg.sumset[ACC_SLAVE].X = 0;
				AccelCtrlPkg.sumset[ACC_SLAVE].Y = 0;
			}
			break;
		case ACCEL_STATE_NORMAL:


			break;
		case ACCEL_STATE_ERROR:
			AccelCtrlPkg.State = ACCEL_STATE_SOFTRESET;

			break;
		default:
			AccelCtrlPkg.State = ACCEL_STATE_SOFTRESET;
			break;
		}


}


void ACC_SetSleepMode(UNS8 Switch)
{
	if(Switch == TRUE){
		ACC_PowerModeSet(ACC_MASTER,ACC_NORMAL);
		ACC_PowerModeSet(ACC_SLAVE,ACC_NORMAL);
	}else{
		ACC_PowerModeSet(ACC_MASTER,ACC_LOW_POWER_05);
		ACC_PowerModeSet(ACC_SLAVE,ACC_LOW_POWER_05);
	}
}
BOOL ACC_IsStatusGood(void)
{
	BOOL ret = FALSE;
	if((AccelCtrlPkg.State == ACCEL_STATE_NORMAL ) && (AccelCtrlPkg.fault.record.Bit.err_state == FALSE)){
		ret	= TRUE;}
	return ret;
}

BOOL ACCEL_GetDualSensorData(AXIS_DATA_T axis[])
{
	axis[ACC_MASTER] = AccelCtrlPkg.axis;
	axis[ACC_SLAVE] = AccelCtrlPkg.axis_Verify;


	return TRUE;
}

BOOL ACCEL_GetDualSensorDataCali(S16* X,S16* Y)
{
	*X = AccelCtrlPkg.offset[ACC_MASTER].X;
	*Y = AccelCtrlPkg.offset[ACC_MASTER].Y;
	return TRUE;
}

BOOL ACCEL_GetAccelData(S16_PTR const data_x, S16_PTR const data_y, S16_PTR const data_z)
{

	*data_x = AccelCtrlPkg.axis.X;
	*data_y = AccelCtrlPkg.axis.Y;
	*data_z = AccelCtrlPkg.axis.Z;
	return TRUE;

}
