
/*
 * DIAG_.c
 *
 *  Created on: Jan 9, 2024
 *      Author: liuzhimin
 */



#include "main.h"



#define DIAG_BSP_UP_POWER_ON()    	BSP_GPIO_Write(GPIO_OUT_RES_POWOR,0)
#define DIAG_BSP_UP_POWER_OFF()    	BSP_GPIO_Write(GPIO_OUT_RES_POWOR,1)

#define DIAG_SAFETY_UNLOCK() 		LL_GPIO_ResetOutputPin(GPIOA,LL_GPIO_PIN_11);/*MCU_CT3*/
#define DIAG_SAFETY_LOCK() 			LL_GPIO_SetOutputPin(GPIOA,LL_GPIO_PIN_11);/*MCU_CT3*/

#define DIAG_SAMP_CNT_MAX 		   32
#define DIAG_IDLE_CNT_MAX 		   400

#define DIAG_INIT_CNT_MAX 		   200

#define DIAG_RES_VLOT_REF  		((UNS16)3300)
#define DIAG_RES_ADC_REF  		((UNS16)1240)
#define DIAG_RES_ADC_MIN 		((UNS16)400)
#define DIAG_RES_ADC_MAX		((UNS16)4096)
#define DIAG_RES_DIODE_VLOT 	((UNS16)300)
#define DIAG_RES_UP 		   ((float)57)



#define DIAG_RES_REF_MAX 		((float)600)
#define DIAG_RES_REF_MIN  		((float)50)


#define DIAG_ADC_VLOT(adc) 		((adc*1000)/DIAG_RES_ADC_REF)
#define DIAG_RES_VLOT(adc) 		(DIAG_ADC_VLOT(adc)- DIAG_RES_DIODE_VLOT)
#define DIAG_RES_CURRENT(adc)	((DIAG_RES_VLOT_REF - DIAG_RES_VLOT(adc))/DIAG_RES_UP)






typedef enum {
	DIAG_STEP_RESET = 0,
	DIAG_STEP_CTRL,
	DIAG_STEP_SAMP,
	DIAG_STEP_FUALT,
	DIAG_STEP_IDLE,
	DIAG_STEP_LOWPOWER,
	DIAG_STEP_ERROR,
	DIAG_STEP_MAX,
} DIAG_STEP_T;




/* Test_DEBUG_CTRL_PACKAGE_T */
typedef struct
{
	/*  */
	UNS16 DeployTimes;
	UNS16 ResValue;
	UNS32 ResSum;
	UNS16 ResAdc;
	BOOL DiagRes;
	BOOL DiagUnlock;
	UNS8 DiagStep;
	UNS16 DelayCnt;
	UNS8 Safetyflg;
	UNS8 SampCnt;
	UNS16 IdleCnt;
	/* Data err */
	UNS8 ErrorCnt;
	UNS8 MaturityCnt;
	UNS8 Reserved[1];

} DIAG__CTRL_PACKAGE_T;

static DIAG__CTRL_PACKAGE_T DIAG_CtrlPkg;

/*****************************************************************************
 *	DIAG_RamInit:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Module memory initialization
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void DIAG_RamInit(void)
{
	(void)memset(&DIAG_CtrlPkg, 0, sizeof(DIAG_CtrlPkg));
	DIAG_CtrlPkg.DiagRes = TRUE;
}


/*****************************************************************************
 *	DIAG_GetResAdc:
 *
 *	PARAMETERS: UNS16
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Obtain resistance value
 *
 *	RETURNS: UNS16
 *
 *****************************************************************************
 */
UNS16 DIAG_GetResAdc(UNS16 resadc)
{
	UNS32 resvlot;
	if(resadc < DIAG_RES_ADC_MIN)
		resadc = DIAG_RES_ADC_MIN;
	if(resadc > DIAG_RES_ADC_MAX)
		resadc = DIAG_RES_ADC_MAX;

	resvlot = DIAG_RES_VLOT(resadc)*100 / DIAG_RES_CURRENT(resadc);
	if(resvlot > 10000)resvlot = 10000;
	return (UNS16)resvlot;
}

/*****************************************************************************
 *	DIAG_Routine:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Module main loop
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void DIAG_Routine(void)
{
	//	520
	if(FRONT_ALGO_ON != TRUE){
		switch(DIAG_CtrlPkg.DiagStep)
		{
		case DIAG_STEP_RESET:
			if(DIAG_CtrlPkg.IdleCnt < 500){
				DIAG_CtrlPkg.IdleCnt++;
			}else{
				if(PWR_GetLowBat() != BAT_STATUS_LOW ){
					DIAG_CtrlPkg.DiagStep = DIAG_STEP_CTRL;
					DIAG_CtrlPkg.IdleCnt = 0;
				}else{
					DIAG_CtrlPkg.DiagStep = DIAG_STEP_LOWPOWER;
				}
			}
			DIAG_CtrlPkg.DelayCnt = 0;
			DIAG_CtrlPkg.Safetyflg = 0;
			break;
		case DIAG_STEP_CTRL:

			if(DIAG_CtrlPkg.DelayCnt < 500){
				DIAG_CtrlPkg.DelayCnt++;
			}else{

				if(DPL_GetSquibInterLockFlag() == FALSE){
					if(DIAG_CtrlPkg.Safetyflg == 0){
						DIAG_SAFETY_UNLOCK();
						DIAG_CtrlPkg.Safetyflg = 1;
					}else{
						DIAG_CtrlPkg.Safetyflg = 0;
						DIAG_CtrlPkg.DiagStep = DIAG_STEP_SAMP;
						DIAG_BSP_UP_POWER_ON();
						DIAG_CtrlPkg.DelayCnt = 0;
					}
				}
			}

			DIAG_CtrlPkg.ResSum = 0;
			DIAG_CtrlPkg.SampCnt = 0;
			break;
		case DIAG_STEP_SAMP:
			if(DIAG_CtrlPkg.SampCnt < DIAG_SAMP_CNT_MAX){
				DIAG_CtrlPkg.ResSum += ADC_GetChannelValue(ADC_BUF_RES);
				DIAG_CtrlPkg.SampCnt++;

			}else{
				DIAG_CtrlPkg.ResAdc =  DIAG_CtrlPkg.ResSum /DIAG_CtrlPkg.SampCnt;
				DIAG_CtrlPkg.ResValue  = DIAG_GetResAdc(DIAG_CtrlPkg.ResAdc);

				DIAG_CtrlPkg.SampCnt = 0;
				DIAG_CtrlPkg.DiagStep = DIAG_STEP_FUALT;

			}
			break;
		case DIAG_STEP_FUALT:

			/*DIAG_BSP_DOWN_POWER_OFF();*/
			if(IS_IN_RANGE(DIAG_CtrlPkg.ResValue,DIAG_RES_REF_MIN,DIAG_RES_REF_MAX)){
				DIAG_CtrlPkg.DiagRes = TRUE;
			}else{
				DIAG_CtrlPkg.DiagRes = FALSE;
			}
			if(DPL_GetSquibInterLockFlag() == FALSE){
				DIAG_BSP_UP_POWER_OFF();
				DIAG_SAFETY_LOCK();
				DIAG_CtrlPkg.DiagStep = DIAG_STEP_IDLE;
				DIAG_CtrlPkg.IdleCnt = 0;
			}
			break;
		case DIAG_STEP_IDLE:

			if(DIAG_CtrlPkg.IdleCnt < DIAG_IDLE_CNT_MAX){
				DIAG_CtrlPkg.IdleCnt++;
			}else{
				if((DIAG_CtrlPkg.DiagUnlock == TRUE)&&((PWR_GetLowBat() != BAT_STATUS_LOW))){
					DIAG_CtrlPkg.DiagStep = DIAG_STEP_CTRL;
					DIAG_CtrlPkg.DiagUnlock = FALSE;
					DIAG_CtrlPkg.IdleCnt = 0;
					DIAG_CtrlPkg.DelayCnt = 0;
				}

			}
			break;
		case DIAG_STEP_ERROR:

			break;
		default:
			break;
		}
	}else{

	}

}

/*****************************************************************************
 *	DIAG_GetAirBagStep:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Obtain airbag status
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 DIAG_GetAirBagStep(void)
{
	return DIAG_CtrlPkg.DiagStep;
}

/*****************************************************************************
 *	DIAG_AirBagUpdate:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Activate airbag update
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
void DIAG_AirBagUpdate(void)
{
	DIAG_CtrlPkg.DiagUnlock = TRUE;
}


/*****************************************************************************
 *	DIAG_DiagAirBagRes:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get airbag resistance
 *
 *	RETURNS: UNS16
 *
 *****************************************************************************
 */
UNS16 DIAG_DiagAirBagRes(void)
{
	return	DIAG_CtrlPkg.ResValue;

}
/*****************************************************************************
 *	DIAG_GetAirBagFualt:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get airbag fualt
 *
 *	RETURNS: bool
 *
 *****************************************************************************
 */
BOOL DIAG_GetAirBagFualt(void)
{
	return DIAG_CtrlPkg.DiagRes;
}


/*****************************************************************************
 *	DIAG_DiagAirBagAdc:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: return airbag adc
 *
 *	RETURNS: UNS16
 *
 *****************************************************************************
 */
UNS16 DIAG_DiagAirBagAdc(void)
{
	return	DIAG_CtrlPkg.ResAdc;

}


