/*
 * ecap.c
 *
 *  Created on: Jan 9, 2024
 *      Author: liuzhimin
 */

#include "main.h"

/*********** VER ***********/

#define	VER_BOOT_VOLE    		(6000)
#define	VER_VOLT_RANGE    		(1000)

#define	VER_DWON_VOLE    		(3000)
#define	VER_DWON_RANGE    		(1000)


#define	VER_VOLT_THRE_MAX(volt,range)     	((volt + range) /3.127)
#define	VER_VOLT_THRE_MIN(volt,range)		((volt - range) /3.127)


#define ERCAP_BOOST_FINISH_LOWER_LIMIT        VER_VOLT_THRE_MIN(VER_BOOT_VOLE,VER_VOLT_RANGE)
#define ERCAP_BOOST_FINISH_UPPER_LIMIT        VER_VOLT_THRE_MAX(VER_BOOT_VOLE,VER_VOLT_RANGE)

#define ERCAP_POWERDWON_500MS_LOWER_LIMIT     VER_VOLT_THRE_MIN(VER_DWON_VOLE,VER_DWON_RANGE)
#define ERCAP_POWERDWON_500MS_UPPER_LIMIT     VER_VOLT_THRE_MAX(VER_DWON_VOLE,VER_DWON_RANGE)


/** MACROS AND TYPEDEFS **/
#define ERCAP_INTERVAL_TIME                 ((UNS16)2U)     //5 * 2 =10ms Detecting capacitor voltage
#define ERCAP_INTERVAL_BOOST_COMP           ((UNS16)20U)    //5*20 =100ms,Capacitor boost completion delay 100ms
#define ERCAP_WAITING_BOOST_MAX             ((UNS16)200U)   //10*200 >2s,Large capacitor charging fault
#define ERCAP_POWER_DOWN_TIME               ((UNS16)50U)    //Test large capacitor power down time 500ms
#define ERCAP_NORMAL_PERIOD                 ((UNS16)100U)   //500ms(100*5)
#define ERCAP_NORMAI_BOOST_MAX_TIMES        ((UNS16)20U)    //Boost failure times
#define ERCAP_DIAG_PERIOD                   (((UNS8)10U))
#define ERCAP_DIAG_RETRY_COUNTER_40         ((UNS8)40U)    //valid range: 0~255
#define ERCAP_DIAG_STEP_INIT_MAX_TIMES      ((UNS8)3U)     //valid range: 0~255
#define ERCAP_WAITING_A2D_OK_MAX_TIMES		((UNS8)20U)     //valid range: 0~255


typedef struct
{
	ERCAP_STEP_E ErcapStep;

	BOOL  IsBoostEnFlag ;
	BOOL  ChargeComplete;
	UNS16 ErcapWaitTimer;
	UNS8  ErcapSetpInitRetryCounter;
	UNS8  ErcapSwitchRetryCounter;
	UNS8  ErcapWaitA2DRetryCounter;
	UNS8  FltPwrtestCount;
	ERCAP_DIAG_STATUS_E ErcapDiagStatus;
	UNS8 Reserved;
} ERCAP_CTRL_PACKAGE_T;

/** PRIVATE DATA **/

static ERCAP_CTRL_PACKAGE_T ErcapCtrlPkg;


void ERCAP_RamInit(void)
{
	/* Global */
	/* Control Package */
	(void)memset(&ErcapCtrlPkg, 0, sizeof(ErcapCtrlPkg));
	ErcapCtrlPkg.IsBoostEnFlag = FALSE;
	ErcapCtrlPkg.ErcapStep  = ERCAP_STEP_INIT ;
	ErcapCtrlPkg.ErcapDiagStatus = ERCAP_DIAG_NOT_COMPLETE ;

} /* End ERCAP_RamInit */


void  BoostON(void)
{
	BSP_GPIO_Write(GPIO_OUT_BST_CTL2,0);
	ErcapCtrlPkg.IsBoostEnFlag = TRUE;
	BSP_GPIO_Write(GPIO_OUT_BST_CTL4,0);
}

void BoostOFF(void)
{
	BSP_GPIO_Write(GPIO_OUT_BST_CTL4,1);
	ErcapCtrlPkg.IsBoostEnFlag = FALSE ;
	BSP_GPIO_Write(GPIO_OUT_BST_CTL2,1);
}

UNS8 ERCAP_GetVerVlot(void)
{
	UNS16 vBattMonData = 0,AdVal,AdConver;
	UNS8 flag = 1;
	AdVal = ADC_GetChannelValue(ADC_BUF_VER);
	AdConver = ADC_GetChannelValue(ADC_BUF_BAT);
	GET_CH_VOLT(AdVal,AdConver,vBattMonData);

	if(IS_IN_RANGE(vBattMonData,ERCAP_BOOST_FINISH_LOWER_LIMIT,ERCAP_BOOST_FINISH_UPPER_LIMIT)){
		flag = 0;
	}
	return flag;
}
void ERCAP_ExecRoutine(void)
{
	UNS16 vBattMonData = 0,AdVal,AdConver;

	/* if test condition not meet, Interval Timer*/
	if (ErcapCtrlPkg.ErcapWaitTimer > ZERO)
	{
		ErcapCtrlPkg.ErcapWaitTimer--;
	}
	if (ZERO == ErcapCtrlPkg.ErcapWaitTimer)
	{
//		if (TRUE == PWR_IS_VBATT_IN_RANGE())
		{

		switch(ErcapCtrlPkg.ErcapStep)
		{
			case ERCAP_STEP_INIT:

			if(ZERO == ErcapCtrlPkg.ErcapWaitA2DRetryCounter)
			{
				BoostON();
				ErcapCtrlPkg.ErcapStep = ERCAP_STEP_TEST_BOOST;
				ErcapCtrlPkg.ErcapWaitTimer = ERCAP_INTERVAL_TIME;
				if(ErcapCtrlPkg.ErcapSetpInitRetryCounter >= ERCAP_DIAG_STEP_INIT_MAX_TIMES )
				{
					ErcapCtrlPkg.ErcapSetpInitRetryCounter = 0 ;
					ErcapCtrlPkg.ErcapStep = ERCAP_STEP_ERROR;
					ErcapCtrlPkg.ErcapWaitTimer = ERCAP_INTERVAL_TIME;
				}
				ErcapCtrlPkg.ErcapSwitchRetryCounter =  0 ;
				ErcapCtrlPkg.ErcapWaitA2DRetryCounter = 0 ;
				ErcapCtrlPkg.ErcapDiagStatus = ERCAP_DIAG_OK ;
			}
				break;
			case ERCAP_STEP_TEST_BOOST://0.942v   1.9V

				AdVal = ADC_GetChannelValue(ADC_BUF_VER);
				AdConver = ADC_GetChannelValue(ADC_BUF_BAT);
				GET_CH_VOLT(AdVal,AdConver,vBattMonData);

				if(IS_IN_RANGE(vBattMonData,ERCAP_BOOST_FINISH_LOWER_LIMIT,ERCAP_BOOST_FINISH_UPPER_LIMIT))
				{
					ErcapCtrlPkg.ChargeComplete = TRUE ;
					ErcapCtrlPkg.ErcapStep = ERCAP_STEP_DIAG_CAP;
					ErcapCtrlPkg.ErcapWaitTimer = ERCAP_INTERVAL_TIME;

				}else {
					ErcapCtrlPkg.FltPwrtestCount++;
					ErcapCtrlPkg.ErcapWaitTimer = ERCAP_INTERVAL_TIME;
				}

				if(ErcapCtrlPkg.FltPwrtestCount >= ERCAP_WAITING_BOOST_MAX)
				{
					ErcapCtrlPkg.FltPwrtestCount = 0 ;
					ErcapCtrlPkg.ErcapSetpInitRetryCounter++;

					BoostOFF();
					ErcapCtrlPkg.ErcapStep = ERCAP_STEP_INIT ;
				}

				break;
			case ERCAP_STEP_DIAG_CAP:
				if(ErcapCtrlPkg.ChargeComplete == TRUE)
				{
					ErcapCtrlPkg.ChargeComplete = FALSE ;
					ErcapCtrlPkg.FltPwrtestCount = ERCAP_POWER_DOWN_TIME ;
					BoostOFF();

					break;
				}
				if(ErcapCtrlPkg.FltPwrtestCount)
				{
					ErcapCtrlPkg.FltPwrtestCount--;
					ErcapCtrlPkg.ErcapWaitTimer = ERCAP_INTERVAL_TIME;
				}else {
					AdVal = ADC_GetChannelValue(ADC_BUF_VER);
					AdConver = ADC_GetChannelValue(ADC_BUF_BAT);
					GET_CH_VOLT(AdVal,AdConver,vBattMonData);

					if(IS_IN_RANGE(vBattMonData,ERCAP_POWERDWON_500MS_LOWER_LIMIT,ERCAP_POWERDWON_500MS_UPPER_LIMIT)){
						ErcapCtrlPkg.ErcapStep = ERCAP_STEP_NORMAL;
						ErcapCtrlPkg.ErcapWaitTimer = ERCAP_INTERVAL_TIME;
					}else {
						BoostOFF();
						ErcapCtrlPkg.ErcapStep = ERCAP_STEP_INIT ;
						ErcapCtrlPkg.ErcapWaitTimer = ERCAP_INTERVAL_TIME;
						ErcapCtrlPkg.ErcapSetpInitRetryCounter++;
					}
				}
				break;
			case ERCAP_STEP_NORMAL:
				if(ErcapCtrlPkg.ChargeComplete == FALSE)
				{
					ErcapCtrlPkg.ChargeComplete = TRUE ;
					BoostOFF();
					ErcapCtrlPkg.ErcapWaitTimer = ERCAP_NORMAL_PERIOD;
					ErcapCtrlPkg.FltPwrtestCount = 0 ;
					ErcapCtrlPkg.ChargeComplete = TRUE;
					break;
				}
				ErcapCtrlPkg.ErcapWaitTimer = ERCAP_NORMAL_PERIOD;
				break;
			case ERCAP_STEP_ERROR:
				BoostOFF();

				break;
			default :
				break;
			}
		}
	}
}/*  ERCAP_ExecRoutine */


ERCAP_STEP_E ERCAP_GetErcapState(void)
{
	return ErcapCtrlPkg.ErcapStep;
}

