/*
 * Display.c
 *
 *  Created on: Dec 1, 2023
 *      Author: liuzhimin
 */
#include "main.h"


#define LV595_ERRORS_CNT_MAX  					(10)
#define LV595_ERRORS_CNT_INIT  					(0)
#define LV595_MATURITY_CNT_MAX  				(0)
#define LV595_OUT_ALL_OFF  						(0xFFFFFFFF)
#define LV595_OUT_ALL_ON  						(0x00000000)


typedef enum
{
	LV595_NORMAL_TYPE = 0,
	LV595_lOW_POWER_TYPE,
} LV595_DATA_TYPE;

typedef enum
{
	LV595_NO_ERROR= 0,
	LV595_ERROR,
	/*other*/
} LV595_ERROR_E;


typedef enum
{
	LV595_CMD_NULL = 0,
	LV595_CMD_ON,
	LV595_CMD_REFRESH,
	LV595_CMD_OFF,

} LV595_CMD_E;


/* Test_DISP_CTRL_PACKAGE_T */
typedef struct
{
	/* Disp deploy number */
	UNS32 DeployNum;
	/* Flags for deploy  */
	LV595_DATA32_U DataFlag;
	/* For LED Module status */
	LV595_STATUS_E ModuleStatus;
	LV595_CMD_E ModuleCmd;
	UNS8 UpdateFlg;
	/* Data err */
	UNS8 ErrorCnt;
	UNS8 MaturityCnt;
	//UNS8 PauseTimes;
	//UNS8 Reserved[1];

} LV595_CTRL_PACKAGE_T;

static LV595_CTRL_PACKAGE_T LV595CtrlPkg;

/*****************************************************************************
 *	LV595_RamInit:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: initialization
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void LV595_RamInit(void)
{
	(void)memset(&LV595CtrlPkg, 0, sizeof(LV595CtrlPkg));
	LV595CtrlPkg.ModuleCmd = LV595_CMD_ON;
}
/*****************************************************************************
 *	LV595_EnablePower:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Enable the underlying layer
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */

void LV595_EnablePower(void)
{
	BSP_GPIO_Write(GPIO_OUT_LED_POWER,Bit_RESET);
}

/*****************************************************************************
 *	LV595_DisablePower:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Turn off the underlying disable
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */

void LV595_DisablePower(void)
{
	BSP_GPIO_Write(GPIO_OUT_LED_POWER,Bit_SET);
}


/*****************************************************************************
 *	LV595_Enable:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Sleep wake-up call
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void LV595_Enable(void)
{
	LV595CtrlPkg.ModuleCmd = LV595_CMD_ON;
}

/*****************************************************************************
 *	LV595_GetStatus:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get 595 status
 *
 *	RETURNS: Status
 *
 *****************************************************************************
 */

LV595_STATUS_E LV595_GetStatus(void)
{
	return LV595CtrlPkg.ModuleStatus;
}


/*****************************************************************************
 *	LV595_Disable:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Disable 595
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void LV595_Disable(void)
{
	LV595CtrlPkg.ModuleCmd = LV595_CMD_OFF;
}


/*****************************************************************************
 *	LV595_UpdateData:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Update data 595
 *
 *	RETURNS: UNS32 Data
 *
 *****************************************************************************
 */

void LV595_UpdateData(UNS32 data)
{
	if(LV595CtrlPkg.ModuleStatus != LV595_STATUS_OFF){
		LV595CtrlPkg.DataFlag.Dword = data;
		LV595CtrlPkg.UpdateFlg = TRUE;
	}
}




/*****************************************************************************
 *	Lv595_DetectionNumberErrors:
 *
 *	PARAMETERS: LV595_ERROR_E
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Detection Errors
 *
 *	RETURNS: SPI_ERROR_E SpiErr
 *
 *****************************************************************************
 */
LV595_ERROR_E Lv595_DetectionNumberErrors(SPI_ERROR_E SpiErr)
{
	LV595_ERROR_E Err = LV595_NO_ERROR;
	if(SpiErr != SPI_NO_ERROR)
	{
		if(LV595CtrlPkg.ErrorCnt < LV595_ERRORS_CNT_MAX){
			LV595CtrlPkg.ErrorCnt++;
		}else{LV595CtrlPkg.ModuleStatus = LV595_STATUS_FAILURE;
		LV595CtrlPkg.ModuleCmd = LV595_CMD_NULL;}
		Err = LV595_ERROR;
	}else{
		LV595CtrlPkg.ErrorCnt	= LV595_ERRORS_CNT_INIT;}
	return Err;
}


/*****************************************************************************
 *	LV595_RefreshData:
 *
 *	PARAMETERS: LV595_ERROR_E
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: 595 Refresh data
 *
 *	RETURNS: SPI_ERROR_E SpiErr
 *
 *****************************************************************************
 */

SPI_ERROR_E LV595_RefreshData(UNS32 DispData,LV595_DATA_TYPE type)
{

	SPI_ERROR_E err = SPI_NO_ERROR;
#ifdef	DISP_DIODE_REVERSAL_ENABLE
	DispData = ~DispData;
#endif
	err = SPI1_TransmitBuffer(DispData);


	return err;
}

/*****************************************************************************
 *	LV595_CmdUpdate:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: LV595 Update
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void LV595_CmdUpdate(void)
{
	SPI_ERROR_E err;
	switch(LV595CtrlPkg.ModuleCmd)
	{

		case LV595_CMD_NULL:

			break;
		case LV595_CMD_ON:
			LV595_EnablePower();
			LV595CtrlPkg.ModuleStatus = LV595_STATUS_INIT;
			LV595CtrlPkg.ModuleCmd = LV595_CMD_NULL;
			break;
		case LV595_CMD_REFRESH:

			LV595CtrlPkg.ModuleCmd = LV595_CMD_NULL;
			break;
		case LV595_CMD_OFF:
			if(LV595CtrlPkg.ModuleStatus != LV595_STATUS_OFF){
				err = LV595_RefreshData((UNS32)LV595_OUT_ALL_OFF,LV595_lOW_POWER_TYPE);//LV595_lOW_POWER_TYPE);
				if(LV595_NO_ERROR == Lv595_DetectionNumberErrors(err)){
					LV595CtrlPkg.ModuleCmd = LV595_CMD_NULL;
					LV595CtrlPkg.ModuleStatus = LV595_STATUS_OFF;
					LV595_DisablePower();
				}
			}
			break;
		default:
			break;
	}
}


/*****************************************************************************
 *	LV595_Routine:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: LV595 Main call
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void LV595_Routine(void)
{
	SPI_ERROR_E err;
	LV595_CmdUpdate();
	switch(LV595CtrlPkg.ModuleStatus)
	{

		case LV595_STATUS_INIT:
			LV595CtrlPkg.DeployNum = LV595_OUT_ALL_OFF;
			LV595CtrlPkg.DataFlag.Dword = LV595_OUT_ALL_OFF;
			LV595CtrlPkg.ErrorCnt = LV595_ERRORS_CNT_INIT;
			LV595CtrlPkg.ModuleStatus = LV595_STATUS_REFRESH;
			LV595CtrlPkg.UpdateFlg = TRUE;
		case LV595_STATUS_REFRESH :
			/*Refresh*/
			err = LV595_RefreshData((UNS32)LV595CtrlPkg.DeployNum,LV595_NORMAL_TYPE);
			if(LV595_NO_ERROR == Lv595_DetectionNumberErrors(err))
			{
				LV595CtrlPkg.ModuleStatus = LV595_STATUS_WAIT;
			}
		//	break;
		case LV595_STATUS_WAIT:
			/*Prevent jumping*/
			if(LV595CtrlPkg.UpdateFlg == TRUE){
				LV595CtrlPkg.UpdateFlg = FALSE;
				if(LV595CtrlPkg.MaturityCnt < LV595_MATURITY_CNT_MAX){
					LV595CtrlPkg.MaturityCnt++;
				}else{
					LV595CtrlPkg.DeployNum = LV595CtrlPkg.DataFlag.Dword;

					LV595CtrlPkg.ModuleStatus = LV595_STATUS_REFRESH;
					LV595CtrlPkg.MaturityCnt = 0;
				}
			}
			else{LV595CtrlPkg.MaturityCnt = 0;}
			break;
		case LV595_STATUS_FAILURE :
			/*waiting to reset,callback*/
			LV595_RefreshData((UNS32)LV595_OUT_ALL_OFF,LV595_lOW_POWER_TYPE);
			break;
		case LV595_STATUS_OFF :
			/*Waiting to enter sleep mode*/
			LV595_RefreshData((UNS32)LV595_OUT_ALL_OFF,LV595_lOW_POWER_TYPE);
			break;
		default:
			/*Waiting to enter sleep mode*/
			LV595_RefreshData((UNS32)LV595_OUT_ALL_OFF,LV595_lOW_POWER_TYPE);
			break;
	}
}


