/*
 * Alarm.c
 *
 *  Created on: 2023年12月19日
 *      Author: liuzhimin
 */



#include "main.h"



#define ALARM_INIT_TIME_MAX     	50

#define ALARM_BUZ_CONTINUOUS_CYCLE	300
#define ALARM_BUZ_SHORT_CYCLE		100
#define ALARM_BUZ_LONG_CYCLE		500


typedef enum
{
	AL_NO_ACTIVE = 0,
	AL_ACTIVE,
} AL_ACTIVE_E;



typedef enum {
	AL_BUZZER_OFF = 0,
	AL_BUZZER_ON,
	AL_BUZZER_MAX,
} AL_BUZZER_E;



typedef enum {
	AL_STEP_INIT = 0,
	AL_STEP_OFF,
	AL_STEP_ON,
} AL_STEP_E;

typedef struct
{
	AL_ACTIVE_E ActiveFlag;
	AL_STATUS_E AlarmFlag;
	AL_STEP_E CfgStep;
	UNS8 AlarmMode;
	UNS16 BuzzCnt;
	UNS8 buzzFlag;
	UNS8 buzzOld;
	UNS8 DelayCnt;
	UNS8 Reserved[3];

} AL_CTRL_PACKAGE_T;

AL_CTRL_PACKAGE_T AlarmCtrlPackage;

/*****************************************************************************
 *	AL_RamInit:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Module initialization
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void AL_RamInit(void)
{
	(void)memset(&AlarmCtrlPackage, 0, sizeof(AlarmCtrlPackage));
}


/*****************************************************************************
 *	AL_StatusUpdate:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Status update
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void AL_StatusUpdate(void)
{
	ADK_SWITCH_U SwVal;
	UNS8 ALstat = 0;
	ADKEY_DIR_E dir;
	SwVal = AdKey_GetSwitchAll();
	if(ADK2_VALUE_MODE_ON == SwVal.Bit.SW_SOS)
	{
		if(ADK2_VALUE_MODE_ON != SwVal.Bit.SW_BUCKLE){
			ALstat++;
		}
		/*Determine two ISOFIX*/
		if(	SWITCH_STATUS_ON != AdKey_IsLeftAndRightGate(SwVal.Bit.SW_ISOFIXR,SwVal.Bit.SW_ISOFIXL)){
			ALstat++;
		}
		dir = AdKey_IsFrontORBackGate(SwVal.Bit.SW_DIRF,SwVal.Bit.SW_DIRB);
		if((ADKEY_DIR_FRONT == dir)) {//20240926 Abnormal detection only occurs when moving forward
			if(ADK2_VALUE_MODE_ON != SwVal.Bit.SW_PILLOW){
				ALstat++;
			}
		}else if(ADKEY_DIR_BACK == dir){//20240926 Backward does not require buzzer sound
		}else{
			ALstat++;}
		if(ADK_VALUE_MODE_2 != SwVal.Bit.SW_LEG){
			ALstat++;
		}
	}
	if(ALstat)
	AlarmCtrlPackage.AlarmFlag = AL_STATUS_ON;
	else
		AlarmCtrlPackage.AlarmFlag = AL_STATUS_OFF;
}

/*****************************************************************************
 *	AL_SetBuzzMode:
 *
 *	PARAMETERS: duty
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Set buzzer mode
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void AL_SetBuzzMode(AL_BUZZER_E buzDuty)
{
	if(AlarmCtrlPackage.buzzOld != buzDuty){
		AlarmCtrlPackage.buzzOld = buzDuty;
		if(buzDuty == AL_BUZZER_ON){
			BSP_SetPwm2Compare(12);
		}else{
			BSP_SetPwm2Compare(0);
		}
	}
//	if(buzDuty < AL_BUZZER_MAX){
//		BSP_GPIO_Write(GPIO_OUT_LED_BUZZ,buzDuty);
//	}
}
/*****************************************************************************
 *	AL_SetBuzzMode:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get buzzer state
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 AL_GetFlag(void)
{
	return AlarmCtrlPackage.AlarmFlag;
}

/*****************************************************************************
 *	AL_BuzzerShortOut:
 *
 *	PARAMETERS: UNS8 UNS32 Max number of times
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get buzzer state
 *
 *	RETURNS:UNS8
 *
 *****************************************************************************
 */
UNS8 AL_BuzzerShortOut(UNS8* mode,UNS32 max)
{
	if(AlarmCtrlPackage.BuzzCnt < max){
		if(AlarmCtrlPackage.buzzFlag == TRUE){
			AL_SetBuzzMode(AL_BUZZER_ON);
		}else{
			AL_SetBuzzMode(AL_BUZZER_OFF);
		}
		AlarmCtrlPackage.BuzzCnt++;

	}else{

		if(AlarmCtrlPackage.buzzFlag == FALSE){
			AlarmCtrlPackage.buzzFlag = TRUE;
			AlarmCtrlPackage.BuzzCnt = 0;
			if(*mode)(*mode)--;
			return TRUE;
		}else{
			AlarmCtrlPackage.buzzFlag = FALSE;
			AlarmCtrlPackage.BuzzCnt = 0;
		}
	}
	return FALSE;
}

/*****************************************************************************
 *	AL_SetAlarmOut:
 *
 *	PARAMETERS: UNS8
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Set buzzer mode
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void AL_SetAlarmOut(AL_MODE_E mode)
{
	AlarmCtrlPackage.AlarmMode = mode;
	AlarmCtrlPackage.BuzzCnt = 0;
	AlarmCtrlPackage.buzzFlag = TRUE;
}
/*****************************************************************************
 *	AL_AlarmExecution:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: buzzer Execution
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void AL_AlarmExecution(void)
{
	UNS8 tmp = 1;
	switch(AlarmCtrlPackage.AlarmMode)
	{
	case AL_MODE_1S:
		if(AL_BuzzerShortOut(&tmp,ALARM_BUZ_LONG_CYCLE))
		AlarmCtrlPackage.AlarmMode = 0;
		AlarmCtrlPackage.DelayCnt = 0;
		break;
	case AL_MODE_ONE:
	case AL_MODE_TWO:
	case AL_MODE_THREE:
		AL_BuzzerShortOut(&AlarmCtrlPackage.AlarmMode,ALARM_BUZ_SHORT_CYCLE);
		AlarmCtrlPackage.DelayCnt = 0;
		break;
	case AL_MODE_OFF :
		if(AlarmCtrlPackage.AlarmFlag == AL_STATUS_ON){
			if(AlarmCtrlPackage.DelayCnt < ALARM_INIT_TIME_MAX){
				AlarmCtrlPackage.DelayCnt++;
				AL_SetBuzzMode(AL_BUZZER_OFF);
			}else{
				AL_BuzzerShortOut(&tmp,ALARM_BUZ_CONTINUOUS_CYCLE);
			}
		}else{AL_SetBuzzMode(AL_BUZZER_OFF);
			AlarmCtrlPackage.DelayCnt = 0;}
		break;
	default:
		break;
	}
}
/*****************************************************************************
 *	AL_RestAlarm:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: reset buzzer state
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void AL_RestAlarm(void)
{
	AlarmCtrlPackage.CfgStep = 0;
	AlarmCtrlPackage.DelayCnt = 0;
}


/*****************************************************************************
 *	AL_Routine:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Main loop
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void AL_Routine(void)
{
	AL_StatusUpdate();
	switch(AlarmCtrlPackage.CfgStep)
	{
	case AL_STEP_INIT:
		if(AlarmCtrlPackage.DelayCnt < ALARM_INIT_TIME_MAX){
			AlarmCtrlPackage.DelayCnt++;
		}else{
			AlarmCtrlPackage.DelayCnt = 0;
			AlarmCtrlPackage.CfgStep = AL_STEP_ON;}
		break;
	case AL_STEP_ON:
		AL_AlarmExecution();
		break;
	case AL_STEP_OFF:
		AL_SetBuzzMode(AL_BUZZER_OFF);
		break;
	default:
		break;
	}
}














