/*
 * adckey.c
 *
 *  Created on: Dec 14, 2023
 *      Author: liuzhimin
 */

#ifndef RTE_ADCKEY_C_
#define RTE_ADCKEY_C_


#include "adc.h"



#define	ADKEY_BASE_CNT  	(500)
#define	ADKEY_ACTIVE_CNT  	(1250)


#define	ADK_SW_BIT	 		(2)
#define	ADK_SW_MASK 		(3)


#define	ADK_SW2_LEVEL_MAX	(3)
#define	ADK_SW3_LEVEL_MAX	(4)


#define ADK_SW1_NOKEY    	(2047)
#define ADK_SW1_KEY1    	(1023)


#define ADK_SW2_NOKEY    	(2702)
#define ADK_SW2_KEY_ON		(654)
#define ADK_SW2_KEY_1		(327)


#define ADK_SW3_NOKEY    	(3039)
#define ADK_SW3_KEY_1		(1646)
#define ADK_SW3_KEY_2		(654)
#define ADK_SW3_KEY_3		(327)


/*********** 5V In***********/

#define	ADK_5V_THRE_MAX     (3500)
#define	ADK_5V_THRE_MIN		(1500)

#define	ADK_DEVICE_ON_STATUS_MAX  	1000
#define	ADK_DEVICE_OFF_STATUS_MAX  	1000


typedef struct
{
	ADK_SWITCH_U KeySw;
	UNS32 OldKey;
	UNS16 KeyCount;
	UNS16 DeviceStatusOnCnt;
	UNS16 DeviceStatusOffCnt;
	UNS16 Reserved16[0];
	UNS8 DeviceStatusFlag;
	ADK_ACTIVE_E ActiveFlag;
	UNS8 Reserved[2];

} ADKEY_CTRL_PACKAGE_T;


//ADC_BUF_CHANNEL_E AdkKeyValTable[ADC_BUF_CH_MAX];
ADKEY_CTRL_PACKAGE_T AdKeyCtrlPackge;



/*****************************************************************************
 *	AdKey_ConvertTherrSwData:
 *
 *	PARAMETERS: UNS16 key
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Three stage switch
 *
 *	RETURNS: UNS16 key
 *
 *****************************************************************************
 */
ADK_KEY_E AdKey_ConvertTherrSwData(UNS16 AdcKey)
{
	const uint16_t ADK_VAL_TAB[] = {ADK_SW3_NOKEY,ADK_SW3_KEY_1,ADK_SW3_KEY_2,ADK_SW3_KEY_3};
	const uint8_t ADK_KEY_MODE[] = {ADK_VALUE_MODE_OFF, ADK_VALUE_MODE_1 ,ADK_VALUE_MODE_2,ADK_VALUE_MODE_OFF,ADK_VALUE_MODE_OFF};
    uint8_t inx;
	ADK_KEY_E KeyValue = ADK_VALUE_MODE_OFF;
	for(inx = 0;inx < ADK_SW3_LEVEL_MAX;inx++){
		if(AdcKey > ADK_VAL_TAB[inx]){break;}}
	KeyValue = ADK_KEY_MODE[inx];
    return KeyValue;
}

/*****************************************************************************
 *	AdKey_ConvertTowSwData:
 *
 *	PARAMETERS: UNS16 key
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Tow stage switch
 *
 *	RETURNS: UNS16 key
 *
 *****************************************************************************
 */
ADK2_KEY_E AdKey_ConvertTowSwData(UNS16 AdcKey)
{
	const uint16_t ADK_VAL_TAB[] = {ADK_SW2_NOKEY,ADK_SW2_KEY_ON,ADK_SW2_KEY_1};
	const uint8_t ADK_KEY_MODE[] = {ADK2_VALUE_MODE_OFF, ADK2_VALUE_MODE_ON,ADK2_VALUE_MODE_OFF,ADK2_VALUE_MODE_OFF};
    uint8_t inx;
    ADK_KEY_E KeyValue = ADK_VALUE_MODE_OFF;
	for(inx = 0;inx < ADK_SW2_LEVEL_MAX;inx++){
		if(AdcKey > ADK_VAL_TAB[inx]){break;}}
	KeyValue = ADK_KEY_MODE[inx];
    return KeyValue;
}


/*****************************************************************************
 *	AdKey_ConvertOneSwData:
 *
 *	PARAMETERS: UNS16 key
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: One stage switch
 *
 *	RETURNS: UNS16 key
 *
 *****************************************************************************
 */
ADK2_KEY_E AdKey_ConvertOneSwData(UNS16 AdcKey)
{
    if(AdcKey < ADK_SW1_NOKEY)//&&(AdcKey > ADK_SW1_KEY1))
    return ADK2_VALUE_MODE_ON;
	return ADK2_VALUE_MODE_OFF;
}

/*****************************************************************************
 *	ADKey_GetModuleStatus:
 *
 *	PARAMETERS: UNS16 key
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get modeule status
 *
 *	RETURNS: UNS16 key
 *
 *****************************************************************************
 */
ADK_ACTIVE_E ADKey_GetModuleStatus(void)
{
	return AdKeyCtrlPackge.ActiveFlag;
}

/*****************************************************************************
 *	ADKey_filter:
 *
 *	PARAMETERS: UNS8
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: adc key filter
 *
 *	RETURNS: UNS32 key
 *
 *****************************************************************************
 */
UNS8 ADKey_filter(UNS32 key)
{
   UNS8 KeyStus = 0;
   if (AdKeyCtrlPackge.OldKey != key) {
	   AdKeyCtrlPackge.KeyCount = 0;
	   AdKeyCtrlPackge.OldKey = key;
		AdKeyCtrlPackge.ActiveFlag = ADK_STATUS_ACTIVE;
   } else {
   	if(AdKeyCtrlPackge.KeyCount < 0xFFF0){
   		AdKeyCtrlPackge.KeyCount ++;}
       if (AdKeyCtrlPackge.KeyCount == ADKEY_BASE_CNT) {
    	   AdKeyCtrlPackge.KeySw.Dword = key;
    	   KeyStus++;}
   }
   if(AdKeyCtrlPackge.ActiveFlag == ADK_STATUS_ACTIVE){
	   Disp_DelayOutput();
 	  if( AdKeyCtrlPackge.KeyCount > ADKEY_ACTIVE_CNT){
 		  AdKeyCtrlPackge.ActiveFlag = ADK_STATUS_INACTIVE;
 	  }}
   return KeyStus;
}

/*****************************************************************************
 *	AdKey_Convert5vInPutThreshold:
 *
 *	PARAMETERS: UNS16
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Convert 5v InPut Thres check up
 *
 *	RETURNS: UNS16
 *
 *****************************************************************************
 */
ADK2_KEY_E AdKey_Convert5vInPutThreshold(UNS16 vlot)
{
	ADK2_KEY_E ky5v = ADK2_VALUE_MODE_OFF;
	if(IS_IN_RANGE(vlot,ADK_5V_THRE_MIN,ADK_5V_THRE_MAX)){
		ky5v = ADK2_VALUE_MODE_ON;}
	return ky5v;
}

/*****************************************************************************
 *	AdKey_ConvertLowPowerThreshold:
 *
 *	PARAMETERS: UNS16
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Convert Low Power Threshold check up
 *
 *	RETURNS: ADK2_KEY_E
 *
 *****************************************************************************
 */
ADK2_KEY_E AdKey_ConvertLowPowerThreshold(UNS16 AdVal)
{
	ADK2_KEY_E Low = ADK2_VALUE_MODE_OFF;
//	if(IS_IN_RANGE(AdVal,ADK_BAT_THRE_UP,ADK_BAT_THRE_DOWN)){
//		Low = ADK2_VALUE_MODE_ON;}
	return Low;
}

/*****************************************************************************
 *	AdKey_ConvertBuckleThreshold:
 *
 *	PARAMETERS: UNS8
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Buckle threshold check up
 *
 *	RETURNS: ADK2_KEY_E
 *
 *****************************************************************************
 */
ADK2_KEY_E AdKey_ConvertBuckleThreshold(UNS8 AdVal)
{
	ADK2_KEY_E ky5v = ADK2_VALUE_MODE_OFF;
	if(AdVal == Bit_RESET){
		ky5v = ADK2_VALUE_MODE_ON;}
	return ky5v;
}


/*****************************************************************************
 *	AdKey_ReadKeyVal:
 *
 *	PARAMETERS: UNS32*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Read key
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 AdKey_ReadKeyVal(UNS32* Key)
{
	UNS16 AdVal,AdConver,Vlot;
	UNS8 ReadVal;
	ADK_SWITCH_U AdKey = {0};
//	ADK_READY_STATUS_E ReadyStus = ADK_READY_STATUS_DONE;

	/*ISOFIX0*/
	AdVal = ADC_GetChannelValue(ADC_BUF_ISOFIXR);
	AdKey.Bit.SW_ISOFIXR =  AdKey_ConvertTherrSwData(AdVal);
	/*ISOFIX1*/
	AdVal = ADC_GetChannelValue(ADC_BUF_ISOFIXL);
	AdKey.Bit.SW_ISOFIXL =  AdKey_ConvertTherrSwData(AdVal);
	/*DIR0*/
	AdVal = ADC_GetChannelValue(ADC_BUF_DIRF);
	AdKey.Bit.SW_DIRF =  AdKey_ConvertTherrSwData(AdVal);
	/*DIR1*/
	AdVal = ADC_GetChannelValue(ADC_BUF_DIRB);
	AdKey.Bit.SW_DIRB =  AdKey_ConvertTherrSwData(AdVal);
	/*LEG*/
	AdVal = ADC_GetChannelValue(ADC_BUF_LEG);
	AdKey.Bit.SW_LEG =  AdKey_ConvertTherrSwData(AdVal);

	/*Buckle*/
	ReadVal = BSP_GPIO_Read(GPIO_IN_BUCKLE);
	AdKey.Bit.SW_BUCKLE = AdKey_ConvertBuckleThreshold(ReadVal);

	/*PILLOW*/
	AdVal = ADC_GetChannelValue(ADC_BUF_PILLOW);
	AdKey.Bit.SW_PILLOW =  AdKey_ConvertTherrSwData(AdVal);

	/*sos*/
	AdVal = ADC_GetChannelValue(ADC_BUF_SOS);
	AdKey.Bit.SW_SOS = AdKey_ConvertOneSwData(AdVal);

	/*5v*/
	AdVal = ADC_GetChannelValue(ADC_BUF_5V);
	AdConver = ADC_GetChannelValue(ADC_BUF_BAT);
	GET_CH_VOLT(AdVal,AdConver,Vlot);
	AdKey.Bit.SW_5V = AdKey_Convert5vInPutThreshold(Vlot);

	/*back*/
	//*Key = AdKeyCtrlPackge.KeySw.Dword;//DEL
	//AdKeyCtrlPackge.KeySw.Dword = AdKey.Dword;
	/*filter*/
	return 	ADKey_filter(AdKey.Dword);
}

/*****************************************************************************
 *	AdKey_IsLeftAndRightGate:
 *
 *	PARAMETERS: UNS8
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Left and right gate check up
 *
 *	RETURNS: UNS16     
 *
 *****************************************************************************
 */
SWITCH_STATUS_E AdKey_IsLeftAndRightGate(UNS16 SwValL,UNS16 SwValR)
{
	SWITCH_STATUS_E Sw = SWITCH_STATUS_ON;

	//if((SWITCH_STATUS_OFF ==  SwValL) && (SWITCH_STATUS_OFF == SwValR)){
	if((SWITCH_STATUS_ON ==  SwValL) && (SWITCH_STATUS_ON == SwValR)){//20120123 update
		Sw = SWITCH_STATUS_ON;
	} else {
		if((SWITCH_STATUS_NO_INSEST ==  SwValL) || (SWITCH_STATUS_NO_INSEST == SwValR)){
			Sw = SWITCH_STATUS_NO_INSEST;
		}else{
			Sw = SWITCH_STATUS_OFF;
		}
	}
	return Sw;
}

/*****************************************************************************
 *	AdKey_IsLeftAndRightGateStaus:
 *
 *	PARAMETERS: BOOL
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Left and right staus
 *
 *	RETURNS: TRUE OR  FALSE
 *
 *****************************************************************************
 */
BOOL AdKey_IsLeftAndRightGateStaus(void)
{
	//if((SWITCH_STATUS_OFF ==  SwValL) && (SWITCH_STATUS_OFF == SwValR)){
	if((SWITCH_STATUS_ON ==  AdKeyCtrlPackge.KeySw.Bit.SW_ISOFIXR ) && (SWITCH_STATUS_ON == AdKeyCtrlPackge.KeySw.Bit.SW_ISOFIXL)){
		return TRUE;
	} else {
		return FALSE;
	}

}
/*****************************************************************************
 *	AdKey_IsFrontORBackGate:
 *
 *	PARAMETERS: BOOL
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Front or back check up
 *
 *	RETURNS: FRONT OR  BACK
 *
 *****************************************************************************
 */
ADKEY_DIR_E AdKey_IsFrontORBackGate(UNS16 SwValF,UNS16 SwValB)
{
	ADKEY_DIR_E dire = ADKEY_DIR_ERROR_NO;
	if((SWITCH_STATUS_ON ==  SwValF) && (SWITCH_STATUS_OFF == SwValB)){
		dire = ADKEY_DIR_FRONT;
	}else if((SWITCH_STATUS_OFF == SwValF)&& (SWITCH_STATUS_ON == SwValB)){
		dire = ADKEY_DIR_BACK;
	}
	return dire;
}



/*****************************************************************************
 *	AdKey_IsFrontORBackGate:
 *
 *	PARAMETERS: Seating status
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Device status check up
 *
 *	RETURNS: TRUE OR  FALSE
 *
 *****************************************************************************
 */
BOOL AdKey_DeviceStatusSwitchLogic(BOOL sos)
{
	ADK_SWITCH_U KeySw = AdKeyCtrlPackge.KeySw;
	if(sos == TRUE){
		if(TRUE == ACCEL_IsStatusGood()){
		KeySw.Bit.SW_SOS = ADK2_VALUE_MODE_ON;}
	}
	if((KeySw.Bit.SW_DIRF == SWITCH_STATUS_ON)
	&&(KeySw.Bit.SW_DIRB == SWITCH_STATUS_OFF)
	&&(KeySw.Bit.SW_ISOFIXR == SWITCH_STATUS_ON)
	&&(KeySw.Bit.SW_ISOFIXL == SWITCH_STATUS_ON)
	&&(KeySw.Bit.SW_BUCKLE == ADK2_VALUE_MODE_ON)
	&&(KeySw.Bit.SW_LEG == SWITCH_STATUS_ON)
	&&(KeySw.Bit.SW_SOS == ADK2_VALUE_MODE_ON)
	&&(KeySw.Bit.SW_PILLOW == ADK_VALUE_MODE_1)
	&&(DIAG_GetAirBagFualt() == TRUE)
	&&(FALSE == PWR_getBatEmptyStats())
	&&(ACC_INT_IsStatusGood() == TRUE)){
		return TRUE;
	}
#ifdef	NO_SWITCH_CHECK_FIRE_TEST_ENABLE
	return TRUE;
#else
	return FALSE;
#endif
}


/*****************************************************************************
 *	AdKey_IsFrontORBackGate:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Get Device status
 *
 *	RETURNS: TRUE OR  FALSE
 *
 *****************************************************************************
 */
BOOL AdKey_GetDeviceStatus(void)
{
	BOOL Status = FALSE;
	if((AdKeyCtrlPackge.DeviceStatusFlag == TRUE) &&(ACCEL_IsStatusGood() == TRUE)){
		Status = TRUE;
	}
	return Status;
}
/*****************************************************************************
 *	AdKey_IsFrontORBackGate:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Get Device status
 *
 *	RETURNS: TRUE OR  FALSE
 *
 *****************************************************************************
 */
BOOL AdKey_GetAccIntStatus(void)
{
	return AdKeyCtrlPackge.DeviceStatusFlag;
}

/*****************************************************************************
 *	AdKey_DeviceStatus:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Device status running
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void AdKey_DeviceStatus(void)
{
	if(FRONT_ALGO_ON != TRUE){
		if(AdKeyCtrlPackge.DeviceStatusFlag == FALSE){
			if(AdKey_DeviceStatusSwitchLogic(FALSE)){
				AdKeyCtrlPackge.DeviceStatusOffCnt = 0;
				if(AdKeyCtrlPackge.DeviceStatusOnCnt < ADK_DEVICE_ON_STATUS_MAX)
					AdKeyCtrlPackge.DeviceStatusOnCnt ++;
				else{
					AdKeyCtrlPackge.DeviceStatusFlag = TRUE;
					ACC_SetDataAccelInit();
				}
			}else{AdKeyCtrlPackge.DeviceStatusOnCnt  = 0;}
		}else{
			if(!AdKey_DeviceStatusSwitchLogic(TRUE)){
				AdKeyCtrlPackge.DeviceStatusOnCnt = 0;
				if(AdKeyCtrlPackge.DeviceStatusOffCnt < ADK_DEVICE_OFF_STATUS_MAX)
					AdKeyCtrlPackge.DeviceStatusOffCnt ++;
				else {
					AdKeyCtrlPackge.DeviceStatusFlag = FALSE;
				}
			}else{AdKeyCtrlPackge.DeviceStatusOffCnt  = 0;}
		}
	}
}

/*****************************************************************************
 *	AdKey_DeviceStatus:
 *
 *	PARAMETERS: UNS16
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Get device status
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 AdKey_GetSwitchVal(ADKEY_SW_CH_E ch)
{
	UNS8 adkey = ADK_VALUE_MODE_OFF;
	UNS32 Sw = AdKeyCtrlPackge.KeySw.Dword;
	if(ch < ADKEY_SW_MAX){
		adkey = ((Sw >> (ch * ADK_SW_BIT))) & ADK_SW_MASK;
	}



	return adkey;
}


/*****************************************************************************
 *	AdKey_GetSwitchAll:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Get all device status
 *
 *	RETURNS: UNS16
 *
 *****************************************************************************
 */
ADK_SWITCH_U AdKey_GetSwitchAll(void)
{
	ADK_SWITCH_U sw;
	sw = AdKeyCtrlPackge.KeySw;
	return sw;
}

/*****************************************************************************
 *	AdKey_RamInit:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Module initialization
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */

void AdKey_RamInit(void)
{
	(void)memset(&AdKeyCtrlPackge, 0, sizeof(AdKeyCtrlPackge));
//	(void)memset(&AdkKeyValTable, 0, sizeof(ADKEY_BUF_MAX));

}


/*****************************************************************************
 *	AdKey_Routine:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Module main loop
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void AdKey_Routine(void)
{
	UNS32 Adkey;

	if(AdKey_ReadKeyVal(&Adkey)){
		Disp_UpdateData(TRIGGE_TYPE_POLLING);
	}
	AdKey_DeviceStatus();
	//TEMP_GetTempValue();
}



#endif /* RTE_ADCKEY_C_ */
