/*
 * debug.c
 *
 *  Created on: Feb 19, 2024
 *      Author: liuzhimin
 */
#include "main.h"


#define DEBUG_CMD_LEN_MAX  			 9
#define DEBUG_CMD_CHAR_LEN_MAX   	40
#define DEBUG_CMD_DATA_LEN_MAX   	35
#define DEBUG_DEPLOY_TIMERS_MAX    2000


#define DEBUG_CMD_CHECK_SUM_MASK    	7
#define DEBUG_CMD_CHECK_SUM_STATS   	6

#define DEBUG_CMD_PARS_MASK   			0
#define DEBUG_DATA_PARS_MASK  			1

#define DEBUG_PASS_LEN  				13

typedef enum {
	PASS_EDR_RD = 0,
	PASS_EDR_ER,
	PASS_INFO_RD,
	PASS_LEN_MAX,
} DEBUG_PASS_TYPE;


const UNS8 debug_passkey[PASS_LEN_MAX][DEBUG_PASS_LEN] = {
		{0x59,0x58,0x32,0x30,0x32,0x34,0x52,0x44,0x45,0x44,0x52,0x00,0x00},	//YX2024RDEDR
		{0x59,0x58,0x32,0x30,0x32,0x34,0x45,0x52,0x45,0x44,0x52,0x00,0x00},	//YX2024EREDR
		{0x59,0x58,0x32,0x30,0x32,0x34,0x52,0x44,0x49,0x4E,0x46,0x4F,0x00},	//YX2024RDINFO
};

UNS8 EDR_CawDataFlag = FALSE;
BOOL EDR_BleActiveFlg = 0;
UNS16 EDR_BleActiveCnt = 0;
BOOL EDR_AccDataFlag = 0;
UNS16 EDR_AccDataInx = 0;

UNS16 EDR_CawPollingFlag = FALSE;
AXIS_DATA_T EDR_AccDataRoundRamTab[500] = {0};
AXIS_DATA_T EDR_AccDataRamTest[500] = {0};
static UNS8  EDR_FireFlag = 0;
static UNS8  EDR_FireLastStepFlag = 0;
static UNS16  EDR_FireLastStepTime = 0;
static UNS16  EDR_FireTime = 0;
//EDR_AccData EDR_AccDataRamTest[500] = {0};


const char const_cmd[DEBUG_CMD_LEN_MAX][DEBUG_CMD_CHAR_LEN_MAX] ={
"update\r",
"reset\r",
"edr\r",
"res\r",
"id\r",
"tesa\r",
"tesb\r",
"writ\r",
"read\r",
};


typedef enum
{
	DEBUG_CMD_NOT = 0,
	DEBUG_CMD_UPDATE,
	DEBUG_CMD_RESET,
	DEBUG_CMD_EDR,
	DEBUG_CMD_RES,
	DEBUG_CMD_ID,
	DEBUG_CMD_TEST1,
	DEBUG_CMD_TESTEX,
	DEBUG_CMD_WRITE,
	DEBUG_CMD_READ,
	DEBUG_CMD_ERROR,
} DEBUG_CMD_E;


typedef enum
{
	DEBUG_PORT_STEP_IDLE = 0,
	DEBUG_PORT_STEP_DATA,
} DEBUG_STEP_E;

typedef enum
{
	DEBUG_PORT_STATUS_INACTIVE = 0,
	DEBUG_PORT_STATUS_ACTIVE,
} DEBUG_STATUS_E;

typedef struct
{
	DEBUG_STATUS_E              Active;
	DEBUG_STEP_E               	Step;
	UNS8 						cmd;
	UNS8 						Inx;
	UNS8 						DataF;
	char               			Char[DEBUG_CMD_CHAR_LEN_MAX];
	char               			Dat[DEBUG_CMD_DATA_LEN_MAX];
	UNS8 						Reserved[1];
} DEBUG_PORT_T;



/* Test_DEBUG_CTRL_PACKAGE_T */
typedef struct
{
	/*  */
	UNS16 DeployTimes;
	/* */
	DEBUG_PORT_T PortRecv;
	/* Data err */
	UNS8 ErrorCnt;
	UNS8 MaturityCnt;
	UNS8 Reserved[1];

} DEBUG_CTRL_PACKAGE_T;

static DEBUG_CTRL_PACKAGE_T DEBUGCtrlPkg;


#define DEBUG_PORT_DATA_FLAG  	DEBUGCtrlPkg.PortRecv.DataF
#define DEBUG_PORT_STEP  	 	DEBUGCtrlPkg.PortRecv.Step
#define DEBUG_PORT_ACTIVE  	 	DEBUGCtrlPkg.PortRecv.Active
#define DEBUG_PORT_CHAR        	DEBUGCtrlPkg.PortRecv.Char
#define DEBUG_PORT_INX        	DEBUGCtrlPkg.PortRecv.Inx
#define DEBUG_PORT_CMD        	DEBUGCtrlPkg.PortRecv.cmd
#define DEBUG_DEPLOY_TIMERS     DEBUGCtrlPkg.DeployTimes
#define DEBUG_STEP_PRINT     	DEBUGCtrlPkg.StepPrint
#define DEBUG_CMD_DATA        	DEBUGCtrlPkg.PortRecv.Dat

void DEBUG_PrintfExec(void);
void DEBUG_printf(void);

extern S16 AccData_Z;

typedef UNS8 (*FUNCTION_PTR)(UNS8 *);

UNS8 DEBUG_ReadRecRam(UNS8 *buff);
UNS8 DEBUG_ReadRecFlash(UNS8 *buff);
UNS8 DEBUG_ReadRecNot(UNS8 *buff);
void DEBUG_PrintfClearState(void);
UNS8 DEBUG_WriteEraseID(UNS8 *buff);
UNS8 DEBUG_WriteEraseEEM(UNS8 *buff);
UNS8 DEBUG_WriteEraseEDR(UNS8 *buff);
UNS8 DEBUG_WriteRecNot(UNS8 *buff);
UNS8 DEBUG_WriteTest(UNS8 *buff);
UNS8 DEBUG_ReadRecUnlock(UNS8 *buff);

typedef enum {
	DEBUG_EXEC_READ_NOT = 0,
	DEBUG_EXEC_READ_EDR_RAM,
	DEBUG_EXEC_READ_EDR_FLASH,
	DEBUG_EXEC_READ_EDR_UNLOCK,
	DEBUG_EXEC_READ_MAX,
} DEBUG_READ_EXEC_E;

typedef enum {
	DEBUG_EXEC_WRITE_NOT = 0,
	DEBUG_EXEC_WRITE_ERASE_EEM,
	DEBUG_EXEC_WRITE_ERASE_EDR,
	DEBUG_EXEC_WRITE_ERASE_ID,
	DEBUG_EXEC_WRITE_MAX,
} DEBUG_WRITE_EXEC_E;


static const FUNCTION_PTR DebugReadExecRtnTable[DEBUG_EXEC_READ_MAX] ={
		DEBUG_ReadRecNot,
		DEBUG_ReadRecRam,
		DEBUG_ReadRecFlash,
		DEBUG_ReadRecUnlock,
};

static const FUNCTION_PTR DebugWriteExecRtnTable[DEBUG_EXEC_WRITE_MAX] ={
		DEBUG_WriteRecNot,
		DEBUG_WriteEraseEEM,
		DEBUG_WriteEraseEDR,
		DEBUG_WriteEraseID,
};


UNS8 DEBUG_ReadRecNot(UNS8 *buff)
{

	return 1;
}




/*****************************************************************************
 *	DEBUG_UpdateFlag:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: boot update
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void DEBUG_UpdateFlag(void)
{
#define UART_UPDATE_ROM_FLAG 0xffffffff

	if(MX_Flash_RD_UpdateFlag() != UART_UPDATE_ROM_FLAG)
		MX_Flash_WR_UpdateFlag(UART_UPDATE_ROM_FLAG);
}
/*****************************************************************************
 *	DEBUG_RamInit:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Module initialization
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void DEBUG_RamInit(void)
{
	(void)memset(&DEBUGCtrlPkg, 0, sizeof(DEBUGCtrlPkg));
	DEBUG_UpdateFlag();
	EDR_AccDataFlag  = TRUE;
}

/*****************************************************************************
 *	DEBUG_PortInterruptRecv:
 *
 *	PARAMETERS: UNS8
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Serial port data reception completed
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */

void DEBUG_PortInterruptRecv(UNS8 rx)
{
	if((rx != 0) || (DEBUG_PORT_INX > 4)){
		if(DEBUG_PORT_INX < DEBUG_CMD_CHAR_LEN_MAX){
			DEBUG_PORT_CHAR[DEBUG_PORT_INX++] = rx;
			DEBUG_PORT_DATA_FLAG = 1;
		}
		if(DEBUG_PORT_ACTIVE  == DEBUG_PORT_STATUS_INACTIVE){
			DEBUG_PORT_ACTIVE = DEBUG_PORT_STATUS_ACTIVE;
		}
		PWR_ResetSleepTime();
	}
}
/*****************************************************************************
 *	DEBUG_PortInterruptDone:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Serial port data reception
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */
void DEBUG_PortInterruptDone(void)
{
	if(DEBUG_PORT_STEP == DEBUG_PORT_STEP_IDLE){
		if( DEBUG_PORT_DATA_FLAG > 0){
			DEBUG_PORT_DATA_FLAG--;
		}else{
			if(DEBUG_PORT_INX > 0){
			DEBUG_PORT_STEP = DEBUG_PORT_STEP_DATA;
			DEBUG_PORT_INX = 0;
			}
		}

	}
}



/*****************************************************************************
 *	DEBUG_RestPortRecv:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Clear port data
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void DEBUG_RestPortRecv(void)
{
	memset(DEBUG_PORT_CHAR,0,DEBUG_CMD_CHAR_LEN_MAX);
	DEBUG_PORT_INX = 0;
	DEBUG_PORT_STEP = DEBUG_PORT_STEP_IDLE;
}


/*****************************************************************************
 *	DEBUG_CmdToLower:
 *
 *	PARAMETERS: int
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Character judgment
 *
 *	RETURNS: int
 *
 *****************************************************************************
 */
int DEBUG_CmdToLower(int c)
{
    if ((c >= 'A') && (c <= 'Z'))
        return c + ('a' - 'A');
    return c;
}


/*****************************************************************************
 *	DEBUG_CmdMatch:
 *
 *	PARAMETERS: str*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Character comparison
 *
 *	RETURNS: int
 *
 *****************************************************************************
 */
int DEBUG_CmdMatch(const char *str, const char *cmd)
{
    int c1, c2;
    do {
        c1 = DEBUG_CmdToLower(*str++);
        c2 = DEBUG_CmdToLower(*cmd++);
    } while((c1 == c2) && c1);

    return c1 - c2;
}

/*****************************************************************************
 *	DEBUG_ActiveCnt:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Work activity level
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void DEBUG_ActiveCnt(void)
{

	if(DEBUG_PORT_ACTIVE  == DEBUG_PORT_STATUS_ACTIVE){
		Disp_DelayOutput();
		if(DEBUG_DEPLOY_TIMERS < DEBUG_DEPLOY_TIMERS_MAX){
			DEBUG_DEPLOY_TIMERS++;
		}else{
			DEBUG_DEPLOY_TIMERS = 0;
			DEBUG_PORT_ACTIVE = DEBUG_PORT_STATUS_INACTIVE;
		}
	}else{
		DEBUG_DEPLOY_TIMERS = 0;
	}

	if(EDR_BleActiveFlg  == TRUE){
//		Disp_DelayOutput();
		if(EDR_BleActiveCnt < DEBUG_DEPLOY_TIMERS_MAX){
			EDR_BleActiveCnt++;
		}else{
			EDR_BleActiveCnt = 0;
			EDR_BleActiveFlg = FALSE;
		}
	}else{
		EDR_BleActiveCnt = 0;
	}
}

/*****************************************************************************
 *	DEBUG_ReadEDRfire:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Read edr fire
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
//UNS8 DEBUG_ReadEDRfire(void)
//{
//	return EDR_FireFlag;
//}
/*****************************************************************************
 *	DEBUG_EDRfireFlag:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Save edr fire data
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
UNS8 EDR_PollTimer = 0;
void DEBUG_EDRfireFlag(void)
{
	EDR_FireFlag = TRUE;
	EDR_FireTime = EDR_AccDataInx;

	EDR_CawPollingFlag = FALSE;

}
/*****************************************************************************
 *	DEBUG_EDRfireFlagClr:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  clr edr fire data
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void DEBUG_EDRfireFlagClr(void)
{
	EDR_FireFlag = FALSE;
	EDR_FireTime = 0;
	EDR_CawPollingFlag = FALSE;
	EDR_PollTimer = 0;
	EDR_FireLastStepFlag = FALSE;
	EDR_FireLastStepTime = 0;
}

/*****************************************************************************
 *	DEBUG_EDRFireFlagLast:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Save off mos fire data
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void DEBUG_EDRFireFlagLast(void)
{
	EDR_FireLastStepFlag = TRUE;
	EDR_FireLastStepTime = EDR_AccDataInx;
}

UNS16 edrmax;
UNS16 edrlen;
UNS16 edrflg;


/*****************************************************************************
 *	DEBUG_CmdProcessing:
 *
 *	PARAMETERS: UNS8
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION:  Module processing
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void  DEBUG_CmdProcessing(DEBUG_CMD_E cmd)
{
	UNS16 resvlot;
	switch(cmd)
	{
	case DEBUG_CMD_WRITE :
		DEBUG_PrintfClearState();
		DEBUG_PORT_CMD = cmd;
		break;
	case DEBUG_CMD_READ :
		DEBUG_PrintfClearState();
		DEBUG_PORT_CMD = cmd;
		break;
	case DEBUG_CMD_UPDATE :

		DEBUG_PORT_CMD = cmd;
	    debug("Debug> update app\r\n");
		break;
	case DEBUG_CMD_RESET :

		DEBUG_PORT_CMD = cmd;
	    debug("Debug> sys reset!\r\n");
		break;
	case DEBUG_CMD_EDR :
		break;
//		DEBUG_PORT_CMD = cmd;
//		debug("Debug> Read EDR \r\n");
//		if(EDR_FireFlag)
//			debug("Debug> Fire mark!\r\n");
//		else
//		    debug("Debug> Not fire mark!\r\n");
//
//		debug("Debug> Fire time:%d \r\n",EDR_FireTime);
//		debug("Debug> Exit time:%d\r\n",EDR_FireLastStepTime);
//		edrmax = 500;//sizeof(EDR_AccDataRamTest);
//		edrlen = 0;
//		edrflg = 0;
//	    debug("Debug> -----------------------\r\n");
//		break;
	case DEBUG_CMD_RES :
		DEBUG_PORT_CMD = cmd;

		resvlot = DIAG_DiagAirBagRes();
	    debug("Debug> Read AiaBag\r\n");
		debug("Debug> res:%d.%d\r\n",(resvlot/100),(resvlot%100));

		break;
	case DEBUG_CMD_ID :
		DEBUG_PORT_CMD = cmd;

		break;
	case DEBUG_CMD_TEST1:
		DEBUG_PORT_CMD = cmd;
		break;
	case DEBUG_CMD_TESTEX:
		DEBUG_PORT_CMD = cmd;
		break;
	case DEBUG_CMD_ERROR :
	default:

	    debug("Debug> input error!\r\n");
		break;
	}
	if((cmd != DEBUG_CMD_NOT) && (cmd != DEBUG_CMD_ERROR)){
	   Disp_UpdateData(TRIGGE_TYPE_KEY);
	}
}

/*****************************************************************************
 *	DEBUG_CmdProcessing:
 *
 *	PARAMETERS: string
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: data analysis
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
DEBUG_CMD_E  DEBUG_CmdParsing(char *str)
{
	for (UNS8 index = 0; index <  DEBUG_CMD_LEN_MAX; index++) { {
		if (DEBUG_CmdMatch(const_cmd[index],str) == 0) {
			index = index + 1;
			if((index == DEBUG_CMD_WRITE)||(index == DEBUG_CMD_READ||(index == DEBUG_CMD_TESTEX))){
				UNS8 Sum = 0;
				for(int i = 0 ;i < (DEBUG_CMD_DATA_LEN_MAX - 2);i++){
					Sum += str[i + DEBUG_CMD_CHECK_SUM_MASK];
				}
				if(Sum == str[DEBUG_CMD_CHECK_SUM_STATS]){
					memcpy(DEBUG_CMD_DATA,str + DEBUG_CMD_CHECK_SUM_MASK,DEBUG_CMD_DATA_LEN_MAX - 1);
				}else{
					return DEBUG_CMD_ERROR;
				}
			}
			return index;
			}
		}
	}
	return DEBUG_CMD_ERROR;
}
/*****************************************************************************
 *	DEBUG_PrintfClearState:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Clear state
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void DEBUG_PrintfClearState(void)
{
	REC_ReadRecRamClr();
}

/*****************************************************************************
 *	DEBUG_Routine:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Module main call
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void DEBUG_Routine(void)
{
	DEBUG_CMD_E cmd = 0;
	DEBUG_PortInterruptDone();
    if(DEBUG_PORT_STEP == DEBUG_PORT_STEP_DATA){
    	cmd = DEBUG_CmdParsing((char*)DEBUG_PORT_CHAR);
    	DEBUG_RestPortRecv();
        DEBUG_CmdProcessing(cmd);
    }
    DEBUG_PrintfExec();
    DEBUG_ActiveCnt();
}

/*****************************************************************************
 *	DEBUG_DelayShutdown:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Sleep delay
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void DEBUG_DelayShutdown(void)
{
	DEBUG_DEPLOY_TIMERS = 0;
	Disp_DelayOutput();

}

/*****************************************************************************
 *	DEBUG_PrintfID:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: printf id
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void DEBUG_PrintfID(void)
{
	uint32_t uid[4];
	VER_GetChipId(uid);
	BOOL Asat= ACC_INT_IsStatusGood();
	AXIS_DATA_T axis[2];
	ACCEL_GetDualSensorData(axis);
	UNS8 acc_accint = DPL_ACC_IntState();
	UNS8 acc_algo = DPL_AlgorithmState();
	UNS8 acc_devsta = AdKey_GetDeviceStatus();
	UNS8 pwr_sta = PWR_GetLoopMode();


	debug("%08X %08X %08X %08X\r\n",(int)uid[2],(int)uid[1],(int)uid[0],(int)uid[3]);
	debug("%s%d%01d%01d-%s \r\n",VERSION,YEAR,MONTH,DAY,BUILD__);
	debug("%d %d %d %d %d %d %d %d %d %d %d\r\n",
			Asat,axis[ACC_MASTER].X,axis[ACC_MASTER].Y,axis[ACC_MASTER].Z,
			axis[ACC_SLAVE].X,axis[ACC_SLAVE].Y,axis[ACC_SLAVE].Z,
			acc_accint,acc_algo,acc_devsta,pwr_sta
			);


}

/*****************************************************************************
 *	DEBUG_PrintfExec:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Print main run
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void DEBUG_PrintfExec(void)
{

	switch(DEBUG_PORT_CMD)
	{
	case DEBUG_CMD_UPDATE :
#define UART_UPDATE_STATUS 	0xAA33AA33
		DEBUG_PORT_CMD = DEBUG_CMD_NOT;
		MX_Flash_WR_UpdateFlag(UART_UPDATE_STATUS);
		NVIC_SystemReset();
		DEBUG_DelayShutdown();
		break;
	case DEBUG_CMD_EDR:
//		if(edrlen < edrmax){
//			debug("%8d  %8d  %8d  %8d  \r\n",edrlen,EDR_AccDataRamTest[edrlen].X,EDR_AccDataRamTest[edrlen].Y,EDR_AccDataRamTest[edrlen].Z);
//			edrlen++;
//		}else{
//			debug("-------------------\r\n");
//			DEBUG_PORT_CMD = DEBUG_CMD_NOT;
//		}
//		DEBUG_DelayShutdown();
//
		DEBUG_PORT_CMD = DEBUG_CMD_NOT;
		break;
	case DEBUG_CMD_RESET :

		NVIC_SystemReset();
		DEBUG_PORT_CMD = DEBUG_CMD_NOT;
		break;
	case DEBUG_CMD_ID :
		DEBUG_PORT_CMD = DEBUG_CMD_NOT;
		DEBUG_PrintfID();

//		UNS32 pwr_on =	PWR_GetPowerSupplyOnfreq();
//		debug("Power on:%d\r\n",(int)pwr_on);
			break;
	case DEBUG_CMD_TEST1 :

		if(TSET_Read()){
			DEBUG_PORT_CMD = DEBUG_CMD_NOT;
		}else{
			DEBUG_DelayShutdown();
		}
		break;
	case DEBUG_CMD_TESTEX:
		if(TSET_HandleEx((UNS8*)DEBUG_CMD_DATA)){
			DEBUG_PORT_CMD = DEBUG_CMD_NOT;
		}else{
			DEBUG_DelayShutdown();
		}
		break;
	case DEBUG_CMD_WRITE :
		if((UNS8)DEBUG_CMD_DATA[DEBUG_CMD_PARS_MASK] < DEBUG_EXEC_WRITE_MAX){
			if(DebugWriteExecRtnTable[(UNS8)DEBUG_CMD_DATA[DEBUG_CMD_PARS_MASK]]((UNS8*)DEBUG_CMD_DATA + DEBUG_DATA_PARS_MASK)){
				DEBUG_PORT_CMD = DEBUG_CMD_NOT;
			}
			DEBUG_DelayShutdown();
		}else{
			DEBUG_PORT_CMD = DEBUG_CMD_NOT;
		}

		break;
	case DEBUG_CMD_READ :
		if((UNS8)DEBUG_CMD_DATA[DEBUG_CMD_PARS_MASK] < DEBUG_EXEC_READ_MAX){
			if(DebugReadExecRtnTable[(UNS8)DEBUG_CMD_DATA[DEBUG_CMD_PARS_MASK]]((UNS8*)DEBUG_CMD_DATA + DEBUG_DATA_PARS_MASK)){
				DEBUG_PORT_CMD = DEBUG_CMD_NOT;
			}
			DEBUG_DelayShutdown();
		}else{
			DEBUG_PORT_CMD = DEBUG_CMD_NOT;
		}
	default:
		break;
	}
}


/*****************************************************************************
 *	DEBUG_EDRGetAccDataFlag:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get caw flag
 *
 *	RETURNS: bool
 *
 *****************************************************************************
 */

BOOL DEBUG_EDRGetAccDataFlag(void)
{

	return EDR_CawDataFlag;
}


/*****************************************************************************
 *	DEBUG_UartWakeup:
 *
 *	PARAMETERS: UNS8
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Uart Wakeup call
 *
 *	RETURNS: bool
 *
 *****************************************************************************
 */
void DEBUG_UartWakeup(UNS8 dat)
{
	if(dat!= 0){
		EDR_BleActiveFlg = TRUE;
		EDR_BleActiveCnt = 0;
		Disp_UpdateData(TRIGGE_TYPE_KEY);
	}
}




/*****************************************************************************
 *	DEBUG_EDRAccDataSave:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Acc data save call
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */

void DEBUG_EDRAccDataSave(void)
{
//	if(ACC_IsStatusGood()){
//		if(FRONT_ALGO_ON == TRUE){
//			if(EDR_AccDataFlag  == TRUE){
//				if(EDR_AccDataInx < 500){
//					EDR_AccDataRamTest[EDR_AccDataInx].X = CAW.AccData.X;
//					EDR_AccDataRamTest[EDR_AccDataInx].Y = CAW.AccData.Y;
//					EDR_AccDataRamTest[EDR_AccDataInx].Z = AccData_Z;
//						EDR_AccDataInx++;
//
//				}else{
//					EDR_AccDataFlag = FALSE;
//					EDR_CawDataFlag = TRUE;
//				}
//
//			}
//		}else{
//
//		}
//	}

}



/*****************************************************************************
 *	DEBUG_ReadRecRam:
 *
 *	PARAMETERS: UNS8 *
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Read edr ram data
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 DEBUG_ReadRecRam(UNS8 *buff)
{
	BOOL ret = TRUE;
	if(0 == memcmp(buff,&debug_passkey[PASS_EDR_RD],DEBUG_PASS_LEN)){
		ret = REC_ReadRecRam();
	}else{
		debug("Password error!\r\n");
	}
	return ret;
}

/*****************************************************************************
 *	DEBUG_ReadRecFlash:
 *
 *	PARAMETERS: UNS8 *
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Read edr flash data
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 DEBUG_ReadRecFlash(UNS8 *buff)
{
	BOOL ret = TRUE;
	if(0 == memcmp(buff,&debug_passkey[PASS_EDR_RD],DEBUG_PASS_LEN)){
		ret = REC_ReadRecFlash();
	}else{
		debug("Password error!\r\n");
	}
	return ret;
}


/*****************************************************************************
 *	DEBUG_WriteEraseEEM:
 *
 *	PARAMETERS: UNS8 *
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Write erase Flash data
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 DEBUG_WriteEraseEEM(UNS8 *buff)
{
//	UNS8 key[] = {0x36,0x35,0x34,0x33,0x34};
//	if(0 == memcmp(buff,key,5)){
//		MEM_EraseAllPartData();
//	}

	return 1;
}

/*****************************************************************************
 *	DEBUG_WriteEraseEDR:
 *
 *	PARAMETERS: UNS8 *
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Write erase edr data
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 DEBUG_WriteEraseEDR(UNS8 *buff)
{
	BOOL ret = TRUE;
	if(0 == memcmp(buff,&debug_passkey[PASS_EDR_ER],DEBUG_PASS_LEN)){
		ret = REC_ClrFlashRam();
	}else{
		debug("Password error!\r\n");
	}
	return ret;
}


/*****************************************************************************
 *	DEBUG_WriteEraseID:
 *
 *	PARAMETERS: UNS8 *
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Write erase id data
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 DEBUG_WriteEraseID(UNS8 *buff)
{
//	UNS8 key[] = {0x34,0x35,0x36,0x33,0x32};
//	if(0 == memcmp(buff,key,5)){

//	}
	return 1;
}


/*****************************************************************************
 *	DEBUG_WriteRecNot:
 *
 *	PARAMETERS: UNS8 *
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: NULL
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 DEBUG_WriteRecNot(UNS8 *buff)
{

	return 1;
}

/*****************************************************************************
 *	DEBUG_ReadRecUnlock:
 *
 *	PARAMETERS: UNS8 *
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Read edr unlock data
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 DEBUG_ReadRecUnlock(UNS8 *buff)
{
	BOOL ret = TRUE;
	if(0 == memcmp(buff+1,&debug_passkey[PASS_EDR_RD],DEBUG_PASS_LEN)){
		ret = REC_ReadRecUnLockRam(buff[0]);
	}else{
		debug("Password error!\r\n");
	}
	return ret;
}





