/*
 * disp.c
 *
 *  Created on: Dec 14, 2023
 *      Author: liuzhimin
 */

#include "main.h"



#define KEY_MATURITY_CNT_MAX  				(1)

#define DISP_WAKE_INIT_TIME_MAX  			(1000)
#define DISP_WAKE_ACTIVE_TIME_MAX  			(1)

#ifdef SLEEP_TEST_MODE
#define DISP_POLLING_ACTIVE_TIME_MAX  		(1000)
#define DISP_KEY_ACTIVE_TIME_MAX  			(1000)
#else
#define DISP_POLLING_ACTIVE_TIME_MAX  		(5000)
#define DISP_KEY_ACTIVE_TIME_MAX  			(5000)
#endif


#define DISP_FLASH_TIMER_MAX  				(250)

#define DISP_LED_OUT_ALL_OFF  			    (0XFFFFFFFF)
#define DISP_LED_OUT_ALL_ON  			    (0X00000000)


#define DISP_FAN_MODE_MASK		(0x03)
#define DISP_FAN_MODE			DispCtrlPkg.ledMode.LedData.Dword
#define DISP_FAN_LED1			DispCtrlPkg.ledMode.LedData.Bit.led1
#define DISP_FAN_LED2			DispCtrlPkg.ledMode.LedData.Bit.led2
#define DISP_FAN_LED3			DispCtrlPkg.ledMode.LedData.Bit.led3
#define DISP_FAN_STATUS			DispCtrlPkg.ledMode.DispStatus



typedef enum {
	LED_LEVEL_SOS_ON = 0x0,
	LED_LEVEL_SOS_OFF =!LED_LEVEL_SOS_ON,
} LED_LEVEL_SOS_E;


typedef enum {
	LED_FLASH_OFF = 0x0,
	LED_FLASH_ON =!LED_FLASH_OFF,
} LED_FLASH_E;


typedef enum {
	LED_FLASH_STATE_NOT = 0x0,
	LED_FLASH_STATE_INIT = !LED_FLASH_STATE_NOT,
} LED_FLASH_STATE_E;


typedef enum {
	LED_LEVEL_VOT5V_ON = 0x0,
	LED_LEVEL_VOT5V_OFF =!LED_LEVEL_VOT5V_ON,
} LED_LEVEL_VOT5V_E;


typedef enum {
	LED_LEVEL_ISOFIX_R_OFF = 0x3,
	LED_LEVEL_ISOFIX_R_GREEN = 0x2,
	LED_LEVEL_ISOFIX_R_READ = 0x1,
	LED_LEVEL_ISOFIX_R_ALL = 0x0,
} LED_LEVEL_ISOFIX_R_E;

typedef enum {
	LED_LEVEL_ISOFIX_L_OFF = 0x3,
	LED_LEVEL_ISOFIX_L_GREEN = 0x1,
	LED_LEVEL_ISOFIX_L_READ = 0x2,
	LED_LEVEL_ISOFIX_L_ALL = 0x0,
} LED_LEVEL_ISOFIX_L_E;

typedef enum {
	LED_LEVEL_DIR_F_OFF = 0x3,
	LED_LEVEL_DIR_F_GREEN = 0x1,
	LED_LEVEL_DIR_F_READ = 0x2,
	LED_LEVEL_DIR_F_ALL = 0x0,
} LED_LEVEL_DIR_F_E;

typedef enum {
	LED_LEVEL_DIR_B_OFF = 0x3,
	LED_LEVEL_DIR_B_GREEN = 0x1,
	LED_LEVEL_DIR_B_READ = 0x2,
	LED_LEVEL_DIR_B_ALL = 0x0,
} LED_LEVEL_DIR_B_E;

typedef enum {
	LED_LEVEL_LEG_OFF = 0x3,
	LED_LEVEL_LEG_GREEN = 0x2,
	LED_LEVEL_LEG_READ = 0x1,
	LED_LEVEL_LEG_ALL = 0x0,
} LED_LEVEL_LEG_E;


typedef enum {
	LED_LEVEL_BUCKLE_OFF = 0x3,
	LED_LEVEL_BUCKLE_GREEN = 0x1,
	LED_LEVEL_BUCKLE_READ = 0x2,
	LED_LEVEL_BUCKLE_ALL= 0x0,
} LED_LEVEL_BUCKLE_E;

typedef enum {
	LED_LEVEL_HEAD_OFF 	= 0x3,
	LED_LEVEL_HEAD_1 	= 0x1,
	LED_LEVEL_HEAD_2  	= 0x2,
	LED_LEVEL_HEAD_ALL	= 0x0,
} LED_LEVEL_HEAD_E;

typedef enum {
	LED_LEVEL_BATVOLT_OFF 	= 0x3,
	LED_LEVEL_BATVOLT_GREEN = 0x1,
	LED_LEVEL_BATVOLT_READ 	= 0x2,
	LED_LEVEL_BATVOLT_ALL= 0x0,
} LED_LEVEL_BATVOLT_E;

typedef enum {
	LED_LEVEL_STATE_OFF = 0,
	LED_LEVEL_STATE_GREEN,
	LED_LEVEL_STATE_READ,
	LED_LEVEL_STATE_MAX,
} LED_LEVEL3_STATE_E;


typedef enum {
	LED_LEVEL_OFF = 0x3,
} LED_LEVEL_ALL_E;

typedef enum {
	LED_LEVEL_FAN_OFF = 0x7,
	LED_LEVEL_FAN_1 = 0x6,
	LED_LEVEL_FAN_2 = 0x4,
	LED_LEVEL_FAN_3 = 0x0,
}LED_LEVEL_STATE_E;

typedef enum {
	LED_LEVEL_AIR_OFF = 0x1,
	LED_LEVEL_AIR_ON = 0x0,
}LED_AIR_STATE_E;

typedef enum {
	LED_LEVEL_CH_ISOFIX_R = 0,
	LED_LEVEL_CH_ISOFIX_L,
	LED_LEVEL_CH_DIRF,
	LED_LEVEL_CH_DIRB,
	LED_LEVEL_CH_LEG,
	LED_LEVEL_CH_BUCKLE,
	LED_LEVEL_CH_lOW,
	LED_LEVEL_CH_SOS,
	LED_LEVEL_CH_5V,
	LED_LEVEL_CH_FAN,
	LED_LEVEL_CH_AIR,
	LED_LEVEL_CH_HEAD,
	LED_LEVEL_CH_MAX,
} LED_LEVEL_CH_E;

const uint8_t Led_Level_tab[][4] = {
{
	LED_LEVEL_ISOFIX_R_OFF,

	LED_LEVEL_ISOFIX_R_GREEN,
	LED_LEVEL_ISOFIX_R_READ,
	LED_LEVEL_ISOFIX_R_ALL,
},
{
	LED_LEVEL_ISOFIX_L_OFF,
	LED_LEVEL_ISOFIX_L_GREEN,
	LED_LEVEL_ISOFIX_L_READ,
	LED_LEVEL_ISOFIX_L_ALL,
},
{
	LED_LEVEL_DIR_F_OFF,
	LED_LEVEL_DIR_F_GREEN,
	LED_LEVEL_DIR_F_READ,
	LED_LEVEL_DIR_F_ALL,
},
{
	LED_LEVEL_DIR_B_OFF,
	LED_LEVEL_DIR_B_GREEN,
	LED_LEVEL_DIR_B_READ,
	LED_LEVEL_DIR_B_ALL,
},
{
	LED_LEVEL_LEG_OFF,
	LED_LEVEL_LEG_GREEN,
	LED_LEVEL_LEG_READ,
	LED_LEVEL_LEG_ALL,
},
{
	LED_LEVEL_BUCKLE_OFF,
	LED_LEVEL_BUCKLE_GREEN,
	LED_LEVEL_BUCKLE_READ,
	LED_LEVEL_BUCKLE_ALL,
},
{
	LED_LEVEL_BATVOLT_OFF,
	LED_LEVEL_BATVOLT_GREEN,
	LED_LEVEL_BATVOLT_READ,
	LED_LEVEL_BATVOLT_ALL,
},
{
	LED_LEVEL_SOS_OFF,
	LED_LEVEL_SOS_ON,
	LED_LEVEL_SOS_OFF,
	LED_LEVEL_SOS_OFF,
},
{
	LED_LEVEL_VOT5V_OFF,
	LED_LEVEL_VOT5V_ON,
	LED_LEVEL_VOT5V_OFF,
	LED_LEVEL_VOT5V_OFF,
},
{
	LED_LEVEL_FAN_OFF,
	LED_LEVEL_FAN_1,
	LED_LEVEL_FAN_2,
	LED_LEVEL_FAN_3,
},
{
	LED_LEVEL_AIR_OFF,
	LED_LEVEL_AIR_ON,
	LED_LEVEL_AIR_OFF,
	LED_LEVEL_AIR_OFF,
},
{
	LED_LEVEL_HEAD_ALL,
	LED_LEVEL_HEAD_OFF,
	LED_LEVEL_HEAD_ALL,
	LED_LEVEL_HEAD_ALL,
}

};



typedef	union
{
	struct
	{
		UNS32 SW_ISOFIXR:2;
		UNS32 SW_ISOFIXL:2;
		UNS32 SW_DIRF:2;
		UNS32 SW_DIRB:2;
		UNS32 SW_LEG:2;
		UNS32 SW_PILLOW:2;
		UNS32 SW_BUCKLE:2;
		UNS32 SW_SOS:2;
		UNS32 SW_5V:2;
		UNS32 FAN:2;
		UNS32 AIR:1;
		UNS32 Reserved:11;
	} Bit;
	UNS32 Dword;

} SwDispData_u;



typedef	union
{
	struct
	{
		UNS32 Sos:1;
		UNS32 LowPow:2;
		UNS32 Dir_B:2;
		UNS32 Isofix_L:2;
		UNS32 Buckle:2;
		UNS32 Leg:2;
		UNS32 Dir_F:2;
		UNS32 Isofix_R:2;
		UNS32 Vot5:1;
		UNS32 Reserved1:1;
		UNS32 Fan:3;
		UNS32 Air:1;
		UNS32 Head:2;
		UNS32 Reserved:9;
	} Bit;
	UNS32 Dword;

} LedDispData_u;



typedef enum
{
	POLLING_NO_ACTIVE = 0,
	POLLING_ACTIVE,
} KEY_POLLING_ACTIVE_E;



typedef enum
{
	DISP_DIR_ERROR_NO = 0,/*Both the front and rear switches are not missing*/
	DISP_DIR_ERROR_ALL,/*Simultaneously opening the front and rear switches*/
	DISP_DIR_FRONT,/*Front switch on*/
	DISP_DIR_BACK,/*Rear switch on*/
} DISP_DIR_E;



typedef	union
{
	struct
	{
		UNS8 	led1:1;
		UNS8 	led2:1;
		UNS8 	led3:1;
		UNS8	Reserved:5;
	} Bit;
	UNS8 Dword;
} DispFanLed_u;

typedef struct
{
	UNS8 DispStatus;
	DispFanLed_u LedData;

} DispFanLedMode_u;



/* Test_DISP_CTRL_PACKAGE_T */
typedef struct
{
	/* Disp deploy number */
	UNS32 SwichStatus;
	LedDispData_u LedDispData;
	DispFanLedMode_u ledMode;
	UNS16 DeployTimes;
	DISP_MODULE_STATUS_E ModuleStatus;
	/* For next LED module state  */
//	LED_MODULE_STATUS_E LEDNextModuleStatus;
	/* Trigger type  */
	DISP_TRIGGE_TYPE_E  TriggeFlag;
	/* Active  */
	KEY_POLLING_ACTIVE_E  KEYPollingActiveStatus;
	/* Data err */
	UNS8 LowDp;
	UNS8 ErrorCnt;
	LED_FLASH_E FlashFlag;
	UNS16 FlashTimes;
	UNS8 MaturityCnt;
	//UNS8 PauseTimes;
	UNS8 Reserved[1];

} DISP_CTRL_PACKAGE_T;

static DISP_CTRL_PACKAGE_T DispCtrlPkg;

/*****************************************************************************
 *	Disp_RamInit:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Module initialization
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void Disp_RamInit(void)
{
	(void)memset(&DispCtrlPkg, 0, sizeof(DispCtrlPkg));
	DispCtrlPkg.SwichStatus = 0XFFFF;
}

/*****************************************************************************
 *	Disp_UpdateData:
 *
 *	PARAMETERS: UNS8
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Update data
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void Disp_UpdateData(DISP_TRIGGE_TYPE_E  trtyp)
{
	if(FALSE == PWR_getBatEmptyStats()){
		DispCtrlPkg.TriggeFlag = trtyp;
	}
}

/*****************************************************************************
 *	Led_RefreshData:
 *
 *	PARAMETERS: UNS16
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Refresh data
 *
 *	RETURNS: ERR
 *
 *****************************************************************************
 */

SPI_ERROR_E Led_RefreshData(UNS16 DispData)
{
	return SPI1_TransmitBuffer(DispData);
}

/*****************************************************************************
 *	Disp_RestFlashData:
 *
 *	PARAMETERS: UNS16
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: reset flash data
 *
 *	RETURNS: ERR
 *
 *****************************************************************************
 */
void Disp_RestFlashData(void)
{
	DispCtrlPkg.FlashTimes = 0;
	DispCtrlPkg.FlashFlag = LED_FLASH_OFF;
}


/*****************************************************************************
 *	Disp_FlashData:
 *
 *	PARAMETERS: UNS16
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get flash data
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
LED_FLASH_E Disp_FlashData(LED_FLASH_STATE_E stat)
{
	LED_FLASH_E  flas = LED_FLASH_OFF;
	if(stat == LED_FLASH_STATE_NOT){
		if(DispCtrlPkg.FlashFlag == LED_FLASH_ON){
			DispCtrlPkg.FlashTimes++;
			if(DispCtrlPkg.FlashTimes > DISP_FLASH_TIMER_MAX){
				DispCtrlPkg.FlashTimes = 0;
				DispCtrlPkg.FlashFlag = LED_FLASH_OFF;
				flas = LED_FLASH_ON;
			}
		}else{
			DispCtrlPkg.FlashTimes++;
			if(DispCtrlPkg.FlashTimes > DISP_FLASH_TIMER_MAX){
				DispCtrlPkg.FlashTimes = 0;
				DispCtrlPkg.FlashFlag = LED_FLASH_ON;
				flas = LED_FLASH_ON;}
		}
	}else{
		DispCtrlPkg.FlashTimes = 0;
		DispCtrlPkg.FlashFlag = LED_FLASH_OFF;
	}

	return flas;
}




/*****************************************************************************
 *	Disp_ThreeSwitchLogicConver:
 *
 *	PARAMETERS: UNS16
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Three switch logic
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 Disp_ThreeSwitchLogicConver(LED_LEVEL_CH_E ch,SWITCH_STATUS_E sw)
{
	UNS8 BakDat;
	switch(sw)
	{
		case SWITCH_STATUS_NO_INSEST:
			if(DispCtrlPkg.FlashFlag == LED_FLASH_ON){
				BakDat = Led_Level_tab[ch][LED_LEVEL_STATE_READ];
			}else{
				BakDat = Led_Level_tab[ch][LED_LEVEL_STATE_OFF];}
			break;
		case SWITCH_STATUS_ON:
			BakDat = Led_Level_tab[ch][LED_LEVEL_STATE_GREEN];
			break;
		case SWITCH_STATUS_OFF:
			BakDat = Led_Level_tab[ch][LED_LEVEL_STATE_READ];
			break;
	}
	return BakDat;
}

/*****************************************************************************
 *	Disp_LockSwitchLogicConver:
 *
 *	PARAMETERS: UNS16
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: lock switch logic
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */

UNS8 Disp_LockSwitchLogicConver(LED_LEVEL_CH_E ch,SWITCH_STATUS_E sw)
{
	UNS8 BakDat;
	switch(sw)
	{
		case ADK2_VALUE_MODE_1:
			if(DispCtrlPkg.FlashFlag == LED_FLASH_ON){
				BakDat = Led_Level_tab[ch][LED_LEVEL_STATE_READ];
			}else{
				BakDat = Led_Level_tab[ch][LED_LEVEL_STATE_OFF];}
			break;
		case ADK2_VALUE_MODE_ON:
			BakDat = Led_Level_tab[ch][LED_LEVEL_STATE_GREEN];
			break;
		case ADK2_VALUE_MODE_OFF:
			BakDat = Led_Level_tab[ch][LED_LEVEL_STATE_READ];
			break;
	}
	return BakDat;
}
/*****************************************************************************
 *	Disp_LowSwitchLogicConver:
 *
 *	PARAMETERS: UNS16
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Low switch logic
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 Disp_LowSwitchLogicConver(LED_LEVEL_CH_E ch,BOOL sw)
{
	UNS8 BakDat;
	if(sw == TRUE){
		if(DispCtrlPkg.FlashFlag == LED_FLASH_ON){
			BakDat = Led_Level_tab[ch][LED_LEVEL_STATE_GREEN];
		}else{
			BakDat = Led_Level_tab[ch][LED_LEVEL_STATE_OFF];
		}
	}else{
		BakDat = Led_Level_tab[ch][LED_LEVEL_STATE_OFF];
	}
	return BakDat;
}

/*****************************************************************************
 *	Disp_HeadSwitchLogicConver:
 *
 *	PARAMETERS: UNS16
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: dir switch logic
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */

UNS8 Disp_HeadSwitchLogicConver(DISP_DIR_E dir,SWITCH_STATUS_E sw)
{
	UNS8 BakDat = 0;
	if(sw == SWITCH_STATUS_NO_INSEST){
		if(DispCtrlPkg.FlashFlag == LED_FLASH_ON){
			BakDat = LED_LEVEL_HEAD_ALL;
		}else{
			BakDat = LED_LEVEL_HEAD_OFF;
		}
	}else{
		if((dir == DISP_DIR_FRONT) && (sw != SWITCH_STATUS_OFF)){
			BakDat = LED_LEVEL_HEAD_ALL;
		}else{
			BakDat = LED_LEVEL_HEAD_OFF;
		}
	}

	return BakDat;
}
/*****************************************************************************
 *	Disp_CodeProcess:
 *
 *	PARAMETERS: UNS16
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Display data processing
 *
 *	RETURNS: UNS16
 *
 *****************************************************************************
 */
LedDispData_u Disp_CodeProcess(SwDispData_u SwVal)
{

	SWITCH_STATUS_E SwRL;
	DISP_DIR_E dir;
	LedDispData_u ledData;
	ledData.Dword = DISP_LED_OUT_ALL_OFF;

	/*Judging seat direction*/
	dir = AdKey_IsFrontORBackGate(SwVal.Bit.SW_DIRF,SwVal.Bit.SW_DIRB);
	/*Determine two ISOFIX*/
	SwRL = AdKey_IsLeftAndRightGate(SwVal.Bit.SW_ISOFIXR,SwVal.Bit.SW_ISOFIXL);
	/*Display separately in different directions*/
	switch(dir)
	{
		case DISP_DIR_FRONT:
			/*Front and back display*/
			ledData.Bit.Dir_F = Disp_ThreeSwitchLogicConver(LED_LEVEL_CH_DIRF,SwVal.Bit.SW_DIRF);
			/*ISOFIX display*/
			ledData.Bit.Isofix_R = Disp_ThreeSwitchLogicConver(LED_LEVEL_CH_ISOFIX_R,SwRL);
			break;
		case DISP_DIR_BACK:
			/*Front and back display*/
			ledData.Bit.Dir_B = Disp_ThreeSwitchLogicConver(LED_LEVEL_CH_DIRB,SwVal.Bit.SW_DIRB);
			/*ISOFIX display*/
			ledData.Bit.Isofix_L = Disp_ThreeSwitchLogicConver(LED_LEVEL_CH_ISOFIX_L,SwRL);
			break;
		case DISP_DIR_ERROR_ALL:
			/*ISOFIX display*/
		case DISP_DIR_ERROR_NO:
			/*ISOFIX display*/
			ledData.Bit.Dir_F = Led_Level_tab[LED_LEVEL_CH_DIRF][LED_LEVEL_STATE_READ];
			ledData.Bit.Dir_B = Led_Level_tab[LED_LEVEL_CH_DIRB][LED_LEVEL_STATE_READ];

			ledData.Bit.Isofix_R = Disp_ThreeSwitchLogicConver(LED_LEVEL_CH_ISOFIX_R,SwRL);
			ledData.Bit.Isofix_L = Disp_ThreeSwitchLogicConver(LED_LEVEL_CH_ISOFIX_L,SwRL);

			break;
		default:
			break;
	}
	if(SwVal.Bit.SW_DIRF == SWITCH_STATUS_NO_INSEST){
		ledData.Bit.Dir_F = Disp_ThreeSwitchLogicConver(LED_LEVEL_CH_DIRF,SwVal.Bit.SW_DIRF);
	}
	if(SwVal.Bit.SW_DIRB == SWITCH_STATUS_NO_INSEST){
		ledData.Bit.Dir_B = Disp_ThreeSwitchLogicConver(LED_LEVEL_CH_DIRB,SwVal.Bit.SW_DIRB);
	}

	/*Leg display*/
	ledData.Bit.Leg = Disp_ThreeSwitchLogicConver(LED_LEVEL_CH_LEG,SwVal.Bit.SW_LEG);
	/*Buckle display*/
//	ledData.Bit.Buckle = Led_Level_tab[LED_LEVEL_CH_BUCKLE][(SwVal.Bit.SW_BUCKLE== ADK2_VALUE_MODE_ON)?1:2];
	ledData.Bit.Buckle = Disp_LockSwitchLogicConver(LED_LEVEL_CH_BUCKLE,SwVal.Bit.SW_BUCKLE);
	/*5V display*/
	ledData.Bit.Vot5 = Led_Level_tab[LED_LEVEL_CH_5V][SwVal.Bit.SW_5V];

	ledData.Bit.Fan = Led_Level_tab[LED_LEVEL_CH_FAN][SwVal.Bit.FAN];
	ledData.Bit.Air = Led_Level_tab[LED_LEVEL_CH_AIR][SwVal.Bit.AIR];
	ledData.Bit.Head = Disp_HeadSwitchLogicConver(dir,SwVal.Bit.SW_PILLOW);

	if(DispCtrlPkg.LowDp ==TRUE){
		ledData.Bit.LowPow = Disp_LowSwitchLogicConver(LED_LEVEL_CH_lOW,DispCtrlPkg.LowDp);
		ledData.Bit.Sos = LED_LEVEL_SOS_OFF;
	}else{
		ledData.Bit.LowPow = Led_Level_tab[LED_LEVEL_CH_lOW][(BAT_STATUS_LOW == PWR_GetLowBat()?1:2)];
		ledData.Bit.Sos = (BleConnSts == TRUE)? LED_LEVEL_SOS_ON:LED_LEVEL_SOS_OFF;
	}


	return ledData;
}
/*****************************************************************************
 *	Disp_GetModuleStatus:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get display status
 *
 *	RETURNS: UNS16
 *
 *****************************************************************************
 */
DISP_MODULE_STATUS_E Disp_GetModuleStatus(void)
{
	return DispCtrlPkg.ModuleStatus;
}

/*****************************************************************************
 *	Disp_DelayOutput:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Delay Output
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void Disp_DelayOutput(void)
{
	if(DispCtrlPkg.ModuleStatus == DISP_STATUS_ACTIVE)
	DispCtrlPkg.DeployTimes = DISP_KEY_ACTIVE_TIME_MAX;
}
/*****************************************************************************
 *	Disp_GetModeAirBagLed:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get mode airbag show
 *
 *	RETURNS: bool
 *
 *****************************************************************************
 */
BOOL Disp_GetModeAirBagLed()
{
	BOOL airled = 0;
	if(REC_GetFireOnFlg())
		airled = DispCtrlPkg.FlashFlag;
	else
		airled = AdKey_GetDeviceStatus();
	return airled;
}


/*****************************************************************************
 *	Disp_AllRedFlash:
 *
 *	PARAMETERS: UNS8
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Set disp flash
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */
UNS32 Disp_AllRedFlash(void)
{
	ADK_SWITCH_U sw = {0};
	sw.Bit.SW_BUCKLE = 2;
	return sw.Dword;
}

/*****************************************************************************
 *	Disp_Routine:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Main module execution call
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */
void Disp_Routine(void)
{
	SwDispData_u SwVal;
	ADK_SWITCH_U Swtmp;
	LedDispData_u ledData = {0};
	uint8_t upflg = LED_FLASH_OFF;

	switch(DispCtrlPkg.ModuleStatus)
	{
		case DISP_STATUS_INIT :
			(void)LV595_Enable();

			DispCtrlPkg.ModuleStatus = DISP_STATUS_TEST;
			DispCtrlPkg.DeployTimes = DISP_WAKE_INIT_TIME_MAX;
			DispCtrlPkg.TriggeFlag = TRIGGE_TYPE_POLLING;

			break;
		case DISP_STATUS_TEST:
			if(DispCtrlPkg.DeployTimes > 0){
				DispCtrlPkg.DeployTimes--;
				ledData.Dword = DISP_LED_OUT_ALL_ON;
			/*Timing back to inactive state*/
			}else{
				DispCtrlPkg.ModuleStatus = DISP_STATUS_NRST;
				ledData.Dword = DISP_LED_OUT_ALL_OFF;
			}

			(void)LV595_UpdateData(ledData.Dword);
			break;
		case DISP_STATUS_NRST :
			DispCtrlPkg.ModuleStatus = DISP_STATUS_ACTIVE;
			DispCtrlPkg.DeployTimes = DISP_WAKE_ACTIVE_TIME_MAX;
			DispCtrlPkg.TriggeFlag = TRIGGE_TYPE_POLLING;
			(void)LV595_Enable();
			break;
		case DISP_STATUS_ACTIVE:
			if(DispCtrlPkg.DeployTimes > 0){
				if(REC_GetFireOnFlg() == FALSE)
				DispCtrlPkg.DeployTimes--;
			/*Timing back to inactive state*/
			}else{
				DispCtrlPkg.FlashFlag = LED_FLASH_ON;
					DispCtrlPkg.FlashTimes = 0;
				DispCtrlPkg.ModuleStatus = DISP_STATUS_INACTIVE;
				(void)LV595_UpdateData(DISP_LED_OUT_ALL_OFF);

				break;
			}

			/*No Wait*/
		/*Active and inactive paths*/
		case DISP_STATUS_INACTIVE:
			/*Obtain switch data*/
			Swtmp = AdKey_GetSwitchAll();
			SwVal.Dword = Swtmp.Dword;
			SwVal.Bit.FAN = Fan_GetMode();
			SwVal.Bit.AIR = Disp_GetModeAirBagLed();

			DispCtrlPkg.LowDp = FALSE;

			if(DispCtrlPkg.TriggeFlag == TRIGGE_TYPE_POLLING)
			{
				/*Prevent jumping*/
				if(DispCtrlPkg.SwichStatus != SwVal.Dword){
					if(DispCtrlPkg.MaturityCnt < KEY_MATURITY_CNT_MAX){
						DispCtrlPkg.MaturityCnt++;
					}else{
						/*to active state*/
						DispCtrlPkg.ModuleStatus = DISP_STATUS_ACTIVE;
						/*Processing data display*/
						DispCtrlPkg.SwichStatus = SwVal.Dword;
						if(LV595_STATUS_OFF ==LV595_GetStatus() ){
							LV595_Enable();
						}else{upflg = LED_FLASH_ON;}

						/*wait key data*/
						DispCtrlPkg.MaturityCnt = 0;
						/*switch time*/
						DispCtrlPkg.DeployTimes = DISP_POLLING_ACTIVE_TIME_MAX;

					}
				}
				else{DispCtrlPkg.MaturityCnt = 0;}
			}else if(DispCtrlPkg.TriggeFlag ==TRIGGE_TYPE_KEY){
				DispCtrlPkg.MaturityCnt = 0;

				if(DispCtrlPkg.ModuleStatus == DISP_STATUS_ACTIVE)
				{
					/*Trigger type return polling*/
					DispCtrlPkg.TriggeFlag = TRIGGE_TYPE_POLLING;
					/*Processing data display*/
					DispCtrlPkg.SwichStatus = SwVal.Dword;

					upflg = LED_FLASH_ON;
				}else{

					if(LV595_STATUS_OFF ==LV595_GetStatus()){
						LV595_Enable();
					}
				}
				/*to active state*/
				DispCtrlPkg.ModuleStatus = DISP_STATUS_ACTIVE;
				/*switch time*/
				DispCtrlPkg.DeployTimes = DISP_KEY_ACTIVE_TIME_MAX;
			}

#ifndef		NO_RES_CHECK_TEST_ENABLE
			if((FALSE == DIAG_GetAirBagFualt()) &&(REC_GetFireOnFlg() == FALSE)
					&&((PWR_GetLowBat() != BAT_STATUS_LOW ||(ADK2_VALUE_MODE_ON == AdKey_GetSwitchVal(ADKEY_SW2_5V))))){
				SwVal.Dword = Disp_AllRedFlash();
				SwVal.Bit.FAN = Fan_GetMode();
				DispCtrlPkg.LowDp = TRUE;
			}
#endif

			if(TRUE == PWR_getBatEmptyStats()){
				DispCtrlPkg.DeployTimes = 0;
				DispCtrlPkg.FlashTimes = 0;
				DispCtrlPkg.ModuleStatus = DISP_STATUS_INACTIVE;
				(void)LV595_UpdateData(DISP_LED_OUT_ALL_OFF);
			}else{
				if((upflg == LED_FLASH_ON )|| (LED_FLASH_ON == Disp_FlashData(LED_FLASH_STATE_NOT))){
					ledData = Disp_CodeProcess(SwVal);
					/*Brushing data*/
					if(DispCtrlPkg.ModuleStatus == DISP_STATUS_INACTIVE){
						ledData.Dword = DISP_LED_OUT_ALL_OFF;
					}
					(void)LV595_UpdateData(ledData.Dword);
					upflg = LED_FLASH_OFF;
				}
			}
			if(DispCtrlPkg.ModuleStatus == DISP_STATUS_INACTIVE){
				if(LV595_STATUS_OFF !=LV595_GetStatus()){
					LV595_Disable();
				}
			}


			break;
		default:
			break;
	}

}



