/*
 * fan.c
 *
 *  Created on: Dec 14, 2023
 *      Author: liuzhimin
 */

#include "main.h"



#define FAN_INIT_TIME    		(1000)


#define FAN_PWM_MODE_OFF      	(0)
#define FAN_PWM_MODE_1      	(40)
#define FAN_PWM_MODE_2      	(70)
#define FAN_PWM_MODE_3      	(100)
#define FAN_PWM_MAX      		(100)

#define FAN_ON_SEAT_TIM_MAX     (9000)



typedef	union
{
	struct
	{
		UNS8 	led1:1;
		UNS8 	FanHeat:1;
		UNS8 	TempThres:2;
		UNS8	Reserved:4;
	} Bit;
	UNS8 Dword;
} FANTempData_u;


typedef struct
{
	UNS32 					TempDelayCnt;
	FANTempData_u			FanTemp;
	FAN_MODE_E  			FanMode;
	UNS8 					FanOut;
	UNS8 					FanFlag;
	UNS8 					FanLevel;
	UNS8 					Reserved[3];
} FAN_CTRL_PACKAGE_T;

FAN_CTRL_PACKAGE_T FanCtrlPkg;
/*****************************************************************************
 *	Fan_RamInit:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: module initialization
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */
void Fan_RamInit(void)
{
    /* Initialize */
    (void) memset(&FanCtrlPkg, 0, sizeof(FanCtrlPkg));
    FanCtrlPkg.FanFlag = TRUE;
	BSP_SetPC14ToOutput();
}

/*****************************************************************************
 *	Fan_SetFanLed:
 *
 *	PARAMETERS: UNS8
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Fan LED lights displayed separately
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */
void Fan_SetFanLed(UNS8 mode)
{
	Disp_SetFanLed(mode);
}
/*****************************************************************************
 *	Fan_SetPwmOutput:
 *
 *	PARAMETERS: UNS8
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Set pwm output
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */
void Fan_SetPwmOutput(UNS8 Duty)
{
	UNS8 MaxDut;
	MaxDut = Duty;
	/*Limit size*/
	if(Duty > FAN_PWM_MAX){
		MaxDut = FAN_PWM_MAX;}

	if(FanCtrlPkg.FanOut != Duty){
		FanCtrlPkg.FanOut = Duty;
		/*Bottom level output*/
		BSP_SetPwm3Compare(MaxDut);
	}
}
/*****************************************************************************
 *	Fan_GetMode:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get fan mode
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
FAN_MODE_E Fan_GetMode(void)
{
	return FanCtrlPkg.FanMode;
}
/*****************************************************************************
 *	Fan_Mode:
 *
 *	PARAMETERS: UNS8
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Set fan mode
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */
void Fan_Mode(UNS8 mode)
{
	if(ADK2_VALUE_MODE_ON == AdKey_GetSwitchVal(ADKEY_SW2_5V))
	{
		Disp_UpdateData(TRIGGE_TYPE_KEY);
		switch(mode)
		{
			case FAN_MODE_1 :
			case FAN_MODE_2 :
			case FAN_MODE_3 :
				FanCtrlPkg.FanMode = mode;
				AL_SetAlarmOut(FanCtrlPkg.FanMode);
				break;
			case FAN_MODE_OFF :
				FanCtrlPkg.FanMode = mode;
				AL_SetAlarmOut(AL_MODE_1S);
				break;
			default:
				break;

		}
		BSP_SetPC14ToOutput();

	}
}
/*****************************************************************************
 *	Fan_SwitchNextMode:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Button response mode switching
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */

void Fan_SwitchNextMode(void)
{

	if(ADK2_VALUE_MODE_ON == AdKey_GetSwitchVal(ADKEY_SW2_5V) && (FALSE == PWR_getBatEmptyStats()))
	{
		Disp_UpdateData(TRIGGE_TYPE_KEY);
#if 0/*20240813章总变更风扇档位顺序*/
		if(FanCtrlPkg.FanMode < FAN_MODE_3){
			FanCtrlPkg.FanMode++;
			AL_SetAlarmOut(FanCtrlPkg.FanMode);
		}else{
			FanCtrlPkg.FanMode = FAN_MODE_OFF;
			AL_SetAlarmOut(AL_MODE_1S);
		}
#else
		if(FanCtrlPkg.FanMode == FAN_MODE_OFF){
			FanCtrlPkg.FanMode = FAN_MODE_3;
			AL_SetAlarmOut(FAN_MODE_3);
		}else if((FanCtrlPkg.FanMode > FAN_MODE_OFF) &&(FanCtrlPkg.FanMode < FAN_MODE_MAX)){
			FanCtrlPkg.FanMode--;
			if(FanCtrlPkg.FanMode == FAN_MODE_OFF){
				AL_SetAlarmOut(AL_MODE_1S);
			}else{
				AL_SetAlarmOut(FanCtrlPkg.FanMode);
			}
		}else{
			FanCtrlPkg.FanMode = FAN_MODE_OFF;
			AL_SetAlarmOut(AL_MODE_1S);
		}
#endif
		BSP_SetPC14ToOutput();
	}
}


/*****************************************************************************
 *	Fan_OffMode:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Off fan
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */
void Fan_OffMode(void)
{
	FanCtrlPkg.FanMode = FAN_MODE_OFF;
	Fan_SetPwmOutput(FAN_PWM_MODE_OFF);
}
/*****************************************************************************
 *	Fan_GetFanOutPwm:
 *
 *	PARAMETERS: UNS8
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get fan pwm
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 Fan_GetFanOutPwm(UNS8 mode)
{
	UNS8 OutPwm = FAN_PWM_MODE_OFF;
	switch(FanCtrlPkg.FanMode)
	{
		case FAN_MODE_OFF:
			/*Mode Off PWM value*/
			OutPwm = FAN_PWM_MODE_OFF;
			break;
		case FAN_MODE_1:
			/*Mode 1 PWM value*/
			OutPwm = FAN_PWM_MODE_1;
			break;
		case FAN_MODE_2:
			/*Mode 2 PWM value*/
			OutPwm = FAN_PWM_MODE_2;
			break;
		case FAN_MODE_3:
			/*Mode 3 PWM value*/
			OutPwm = FAN_PWM_MODE_3;
			break;
		default:
			break;
	}
	return OutPwm;
}

/*****************************************************************************
 *	Fan_Routine:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Module main loop
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void Fan_Routine(void)
{
	if(ADK2_VALUE_MODE_ON == AdKey_GetSwitchVal(ADKEY_SW2_5V) && (FALSE == PWR_getBatEmptyStats())){
		UNS8 FanOutDuty = Fan_GetFanOutPwm(FanCtrlPkg.FanMode);
		Fan_SetPwmOutput(FanOutDuty);
		if((BleConnSts == 0 ) &&(ADK2_VALUE_MODE_OFF == AdKey_GetSwitchVal(ADKEY_SW2_SOS)) && (FanCtrlPkg.FanMode != FAN_MODE_OFF)){
			FanCtrlPkg.TempDelayCnt++;
			if(FanCtrlPkg.TempDelayCnt < FAN_ON_SEAT_TIM_MAX){

			}else{
				FanCtrlPkg.TempDelayCnt = 0;
				FanCtrlPkg.FanMode = FAN_MODE_OFF;
				Fan_SetPwmOutput(FAN_PWM_MODE_OFF);
			}

		}else{
			FanCtrlPkg.TempDelayCnt = 0;
		}

	}else{
		/*off*/
		FanCtrlPkg.FanMode = FAN_MODE_OFF;
		Fan_SetPwmOutput(FAN_PWM_MODE_OFF);
		FanCtrlPkg.TempDelayCnt = 0;
	}
	/*TEMP_Routine();*/
}

































