/*
 * keymodule.c
 *
 *  Created on: Dec 1, 2023
 *      Author: liuzhimin
 */

#include "main.h"

/*********** IO KEY ***********/

#define KEY_SW1_MSK             	(0x01)  /*key1*/
#define KEY_SW2_MSK             	(0x02)  /*key2*/

/*Key thresh*/
#define KEY_BASE_CNT  				(50)
#define KEY_MAX_CNT  				(0xff)
#define KEY_PUSH_BUTTON1()       	BSP_GPIO_Read(GPIO_IN_SW)
#define KEY_PUSH_BUTTON2()       	BSP_GPIO_Read(GPIO_IN_TOUCH)

#define KEY_ACTIVE_CNT  			(200)

typedef struct
{
	UNS8 UsedKey1;
	UNS8 UsedKey2;
	UNS8 OldKey1;
	UNS8 OldKey2;
	UNS8 KeyCount1;
	UNS8 KeyCount2;

	IOKEY_ACTIVE_E ActiveFlag;
	UNS16 ActiveCnt;
	//UNS8 Reserved[1];

} IOKEY_CTRL_PACKAGE_T;

IOKEY_CTRL_PACKAGE_T  IOKeyCtrlPkg;


/*****************************************************************************
 *	IOKey_GetModuleStatus:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get module status
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
IOKEY_ACTIVE_E IOKey_GetModuleStatus(void)
{
	return IOKeyCtrlPkg.ActiveFlag;
}


/*****************************************************************************
 *	IOKey_filter:
 *
 *	PARAMETERS: UNS8
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Key filter
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 IOKey_filter(UNS8 key)
{
	uint8_t newkey = 0;
	if((key & KEY_SW1_MSK) == KEY_SW1_MSK){
        IOKeyCtrlPkg.ActiveFlag = IOKEY_STATUS_ACTIVE;
        IOKeyCtrlPkg.ActiveCnt = 0;
    	if(IOKeyCtrlPkg.KeyCount1 < KEY_MAX_CNT){
    		IOKeyCtrlPkg.KeyCount1 ++;}
        if (IOKeyCtrlPkg.KeyCount1 == KEY_BASE_CNT) {
        	newkey |= KEY_SW1_MSK;
        }
	}else{
		IOKeyCtrlPkg.KeyCount1 = 0;
	}


	if((key & KEY_SW2_MSK) == KEY_SW2_MSK){
        IOKeyCtrlPkg.ActiveFlag = IOKEY_STATUS_ACTIVE;
        IOKeyCtrlPkg.ActiveCnt = 0;
    	if(IOKeyCtrlPkg.KeyCount2 < KEY_MAX_CNT){
    		IOKeyCtrlPkg.KeyCount2 ++;
    	}
        if (IOKeyCtrlPkg.KeyCount2 == KEY_BASE_CNT) {
        	newkey |= KEY_SW2_MSK;
        }
	}else{
		IOKeyCtrlPkg.KeyCount2 = 0;
	}

    if(IOKeyCtrlPkg.ActiveFlag == IOKEY_STATUS_ACTIVE){
    	IOKeyCtrlPkg.ActiveCnt++;
    	if(IOKeyCtrlPkg.ActiveCnt >  KEY_ACTIVE_CNT){
    		IOKeyCtrlPkg.ActiveFlag = IOKEY_STATUS_INACTIVE;
    	}
    }

	return newkey;
}
/*****************************************************************************
 *	IOKey_ReadValue:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Read Key
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 IOKey_ReadValue(void)
{
	uint8_t keys = 0;
	if(KEY_PUSH_BUTTON2()){
		keys |= KEY_SW2_MSK;
	}
	if(Bit_RESET == KEY_PUSH_BUTTON1()){
		keys |= KEY_SW1_MSK;
	}

	return IOKey_filter(keys);
 }


/*****************************************************************************
 *	IOKey_RamInit:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Module initialization
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */
void IOKey_RamInit(void)
{

	(void)memset(&IOKeyCtrlPkg, 0, sizeof(IOKeyCtrlPkg));

}


/*****************************************************************************
 *	IOKey_Routine:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Module main loop
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */
void IOKey_Routine(void)
{
	uint8_t key_sw = 0;
	key_sw = IOKey_ReadValue();
	if((key_sw & KEY_SW1_MSK) == KEY_SW1_MSK)
	{
		/*Single trigger button 1 event*/
		Disp_UpdateData(TRIGGE_TYPE_KEY);
	}
	if((key_sw & KEY_SW2_MSK) == KEY_SW2_MSK)
	{
		/*Single trigger button 1 event*/
		Fan_SwitchNextMode();
	}
}


