/*
 * power.c
 *
 *  Created on: Dec 1, 2023
 *      Author: liuzhimin
 */

#include "main.h"




#define BAT_FULL_VOLE  			(2922)
#define BAT_LOW_VOLE  			((BAT_FULL_VOLE-BAT_LOW_VOLE)/10)
#define BAT_EMPTY_VOLE  		(1715)


#define BAT_LOW_BASE_CNT 		(30)
#define BAT_NOT_BASE_CNT 		(2500)
#define BAT_INIT_BASE_CNT 		(100)
#define BAT_LIST_BASE_CNT 		(250)//(500)
#define BAT_DOWN_BASE_CNT 		(500)
#define BAT_LIST_DELAY_CNT 		(10)


#define BAT_SUM_MASK			(100)

#define ELEC_PTC_MAX 		(100)
#define ELEC_PTC_UNDEF		(80)
#define ELEC_PTC_MIN 		(10)
#define ELEC_PTC_LOW 		(0)


#define SLEEP_COUNTDOWD_TIME  	(1)
#define HIGH_LOOP_IDLE_TIME  	(1500)

#define SYSTEM_CLOCK_TIMEROUT 1000


typedef enum
{
	BAT_CFG_INIT = 0,
	BAT_CFG_SAMP,
	BAT_CFG_LISTEN,
	BAT_CFG_DELAY,
	BAT_CFG_LOCK,
} BAT_CFG_E;



typedef	union
{
	struct
	{
		UNS16 ElecState:3;
		UNS16 ElecPerc:7;
		UNS16 CfgStep:5;
		UNS16 Reserved:1;
	} Bit;
	UNS16 Dword;
} BAT_CFG_PAK_U;

typedef	union
{
	struct
	{
		UNS8 OnFlag:1;
		UNS8 OffFlag:1;
		UNS8 Reserved:6;
	} Bit;
	UNS8 Dword;
} HIGH_PWR_PAK_U;


typedef struct
{
	UNS32               		VoltSum;
	BAT_CFG_PAK_U               Electric;
	UNS16                		ElecFullVlot;
	UNS16                		LowBatCnt;
	UNS16                		ElecCnt;
	UNS16                		DelayCnt;
	UNS16                		IdleCnt;				/*for delay */
	HIGH_PWR_PAK_U              HighPwr;
	UNS8						ElecUpdateFlg;
	UNS8              			WakeEvent;
	UNS8  						PWRMode;
	UNS8  						InitDelay;
	UNS8  						UartDelay;
	UNS8  						AccPwr;
	UNS8  						BleWake;
	UNS8  						FirstPowerOn;
	UNS8  						TestAccDelayFlg;
	UNS16  						InitDelayCnt;
	UNS16  						TestAccDelayCnt;
	UNS16  						accWakeDelayCnt;
	UNS32						PowerSupplyOnfreq;
	UNS8						PowerSupplyOnErr;
	UNS16						PowerSupplyOnCnt;

	UNS8 						Reserved[2];
} POWER_CTRL_PACKAGE_T;

/** PRIVATE DATA **/
POWER_CTRL_PACKAGE_T PowerCtrlPkg;

#define BAT_CFG_STEP   PowerCtrlPkg.Electric.Bit.CfgStep
#define ELEC_STATE     PowerCtrlPkg.Electric.Bit.ElecState
#define PWR_WORK_MODE  PowerCtrlPkg.PWRMode
#define BAT_SUM		   PowerCtrlPkg.VoltSum
#define ELEC_CNT	   PowerCtrlPkg.ElecCnt
#define ELEC_DELAY	   PowerCtrlPkg.DelayCnt
#define ELEC_FLG	   PowerCtrlPkg.ElecUpdateFlg
#define ELEC_FULL	   PowerCtrlPkg.ElecFullVlot
#define ELEC_PCT	   PowerCtrlPkg.Electric.Bit.ElecPerc
#define PWR_ACC_MODE   PowerCtrlPkg.AccPwr
#define PWR_BLE_WAKE   PowerCtrlPkg.BleWake
#define PWR_ON_FREQ    PowerCtrlPkg.PowerSupplyOnfreq
#define PWR_ON_ERR     PowerCtrlPkg.PowerSupplyOnErr
#define PWR_ON_CNT     PowerCtrlPkg.PowerSupplyOnCnt


#define PWR_TEST_FLG   PowerCtrlPkg.TestAccDelayFlg
#define PWR_TEST_CNT   PowerCtrlPkg.TestAccDelayCnt
POWER_MODE_E PWR_GetLoopMode(void);
MODULE_STATUS_E PWR_GetModuleStatus(void);
void SystemClock_4MConfig(void);
void SystemClock_80MConfig(void);
void PWR_DelayInit(void);
void PWR_PowerSupplyOnFreq(void);
void SystemClock_Config(void);
void MX_GPIO_Init(void );
void MX_GPDMA1_Init(void );

/*****************************************************************************
 *	PWR_RamInit:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Module initialization
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */
void PWR_RamInit(void)
{
    /* Initialize */
    (void) memset(&PowerCtrlPkg, 0, sizeof(PowerCtrlPkg));
    ELEC_FULL = BAT_FULL_VOLE;
    ELEC_PCT = ELEC_PTC_UNDEF;
    PWR_ACC_MODE = TRUE;

//    PWR_PowerSupplyOnFreq();

}

/*****************************************************************************
 *	PWR_ResetSleepTime:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Reset Sleep Time
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */
void PWR_ResetSleepTime(void)
{
	PowerCtrlPkg.IdleCnt = SLEEP_COUNTDOWD_TIME;
}

/*****************************************************************************
 *	PWR_SetKeyWakeEvent:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Set WakeEvent Key
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */

void PWR_SetKeyWakeEvent(void)
{
	PowerCtrlPkg.WakeEvent = WAKE_KEY;
}

/*****************************************************************************
 *	PWR_SetSensorWakeEvent:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Set WakeEvent Sensor
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */
void PWR_SetSensorWakeEvent(void)
{
	PowerCtrlPkg.WakeEvent = WAKE_SENSOR;
}


/*****************************************************************************
 *	PWR_GetLowBat:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get low power status
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
BAT_STATUS_E PWR_GetLowBat(void)
{
	return ELEC_STATE;
}


/*****************************************************************************
 *	PWR_GetElecPtc:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get power status
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
BAT_STATUS_E PWR_GetElecPtc(UNS8* pct)
{
	if(ELEC_STATE == BAT_STATUS_UNREAD)	{
		*pct = ELEC_PTC_UNDEF;
	}else{*pct = ELEC_PCT;}
	return ELEC_STATE;
}



/*****************************************************************************
 *	PWR_GetElecLevel:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get power status
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 PWR_GetElecLevel(void)
{
	UNS8 PCT;
	if(ELEC_STATE == BAT_STATUS_UNREAD)	{
		PCT = ELEC_PTC_UNDEF;
	}else{PCT = ELEC_PCT;}
	return PCT;
}

/*****************************************************************************
 *	PWR_GetElectricity:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get Electricity
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 PWR_GetElectricity(UNS32 batVolt)
{
	UNS16 pct;

	if(PWR_WORK_MODE == POWER_LOOP_MODE_NORMAL){
		batVolt = (batVolt > 20)?batVolt - 20:batVolt ;
	}

	if(batVolt >= ELEC_FULL){
		pct = ELEC_PTC_MAX;
	}else if(batVolt < BAT_EMPTY_VOLE){
		pct = ELEC_PTC_LOW;
	}else{
		pct = ((batVolt - BAT_EMPTY_VOLE)*100)/(ELEC_FULL - BAT_EMPTY_VOLE);
	}
	return (UNS8)pct;
}
/*****************************************************************************
 *	PWR_EleSampfilte:
 *
 *	PARAMETERS: UNS16
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Sampling filtering
 *
 *	RETURNS: UNS16
 *
 *****************************************************************************
 */
UNS16 PWR_EleSampfilte(UNS16 AdcVal)
{
	AdcVal = AdcVal*2;
	/*Take the average value to calculate the voltage*/
	BAT_SUM = BAT_SUM + AdcVal - (BAT_SUM / BAT_SUM_MASK);
	return (UNS16)(BAT_SUM / BAT_SUM_MASK);
}
/*****************************************************************************
 *	PWR_ElectricityMonitor:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Electricity Monitor
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */
void PWR_ElectricityMonitor(void)
{
	UNS16 BatVlot, BatVal,AdConver;
	UNS8 Ptc;

	BatVal = ADC_GetChannelValue(ADC_BUF_BAT_E);
 	AdConver = ADC_GetChannelValue(ADC_BUF_BAT);
	GET_CH_VOLT(BatVal,AdConver,BatVlot);

	/*Sampling filtering*/
	BatVlot = PWR_EleSampfilte(BatVlot);
	switch(BAT_CFG_STEP)
	{
		/*Power on and air run 8 times*/
		case BAT_CFG_INIT:
			ELEC_STATE = BAT_STATUS_UNREAD;
			if(ELEC_CNT < BAT_NOT_BASE_CNT){
				ELEC_CNT++;
			}else{
				BAT_CFG_STEP = 	BAT_CFG_SAMP;
				ELEC_CNT = 0;
			}
			break;
			/*Power on and air run 8 times*/
		case BAT_CFG_SAMP:
			/*Initial sampling of open circuit voltage as the initial battery charge*/
			ELEC_STATE = BAT_STATUS_UNREAD;
			/*The higher the collected voltage shall prevail*/
				/*Count 10 times*/
			if(ELEC_CNT < BAT_INIT_BASE_CNT){
				ELEC_CNT++;
			}else{
				ELEC_CNT = 0;

				/*Save power after voltage stabilization*/
				ELEC_PCT = PWR_GetElectricity(BatVlot);
				if(ELEC_PCT > ELEC_PTC_MIN ){
					ELEC_STATE = BAT_STATUS_FULL;
				}else if(ELEC_PCT == 0){
					ELEC_STATE = BAT_STATUS_EMPTY;
				}else{
					ELEC_STATE = BAT_STATUS_LOW;
				}
				BAT_CFG_STEP = 	BAT_CFG_LISTEN;
			}
			break;

		case BAT_CFG_DELAY:
			if(ELEC_CNT < BAT_LIST_DELAY_CNT){
				ELEC_CNT++;
			}else{
				ELEC_CNT = 0;
				BAT_CFG_STEP = 	BAT_CFG_LISTEN;
			}
			break;
		case BAT_CFG_LISTEN:
			/**Electricity meter mode*/
			Ptc = PWR_GetElectricity(BatVlot);

			if(Ptc < ELEC_PCT){
				ELEC_DELAY++;
				if(ELEC_DELAY > BAT_LIST_BASE_CNT){
					ELEC_DELAY = 0;
					ELEC_PCT = Ptc;
				}
			}else{
				ELEC_DELAY = 0;
			}

			/*Low voltage monitoring*/
			if(ELEC_PCT <= ELEC_PTC_MIN){
				/*Enter low battery sign*/
				ELEC_STATE = BAT_STATUS_LOW;
				if(ELEC_PCT == 0){
					BAT_CFG_STEP = BAT_CFG_LOCK;
					  LL_USART_DisableIT_RXNE(UART4);
					  NVIC_DisableIRQ(EXTI7_IRQn);
					  LL_EXTI_DeInit();
				}
			}
			break;
		case BAT_CFG_LOCK:
			/*Once entered, it will be locked*/
			ELEC_STATE = BAT_STATUS_EMPTY;
			ELEC_CNT = ELEC_PTC_LOW;
			BAT_CFG_STEP = BAT_CFG_LOCK;
			ELEC_PCT = 0;
			LL_GPIO_ResetOutputPin(GPIOC,BT_Reset_PC12_Pin);
			break;
		default:
			break;
	}

}
/*****************************************************************************
 *	PWR_Uart4EnterConfig:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Serial Port Configuration
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void PWR_Uart4EnterConfig(void)
{
	LL_EXTI_InitTypeDef EXTI_InitStruct = {0};
	LL_GPIO_InitTypeDef GPIO_InitStruct = {0};

	GPIO_InitStruct.Pin =  LL_GPIO_PIN_11;
	GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = LL_GPIO_PULL_UP;
	LL_GPIO_Init(GPIOC, &GPIO_InitStruct);


	LL_EXTI_SetEXTISource(LL_EXTI_EXTI_PORTC, LL_EXTI_EXTI_LINE11);

	EXTI_InitStruct.Line_0_31 = LL_EXTI_LINE_11;
	EXTI_InitStruct.LineCommand = ENABLE;
	EXTI_InitStruct.Mode = LL_EXTI_MODE_IT;
	EXTI_InitStruct.Trigger = LL_EXTI_TRIGGER_FALLING;
	LL_EXTI_Init(&EXTI_InitStruct);


//	LL_GPIO_SetPinPull(GPIOC, LL_GPIO_PIN_11, LL_GPIO_PULL_UP);
//	LL_GPIO_SetPinMode(GPIOC, LL_GPIO_PIN_11, LL_GPIO_MODE_INPUT);

	NVIC_SetPriority(EXTI11_IRQn, NVIC_EncodePriority(NVIC_GetPriorityGrouping(),0, 0));
	NVIC_EnableIRQ(EXTI11_IRQn);

}
/*****************************************************************************
 *	PWR_getBatEmptyStats:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Obtain empty battery power
 *
 *	RETURNS: BOOT   true:empty
 *
 *****************************************************************************
 */
BOOL PWR_getBatEmptyStats(void)
{
	BOOL ret = FALSE;
	if(ELEC_STATE == BAT_STATUS_EMPTY){
		ret = TRUE;
	}
	return ret;
}

/*****************************************************************************
 *	PWR_WakeUpEnterConfig:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Serial Port Configuration
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void PWR_WakeUpEnterConfig(void)
{

	LL_EXTI_InitTypeDef EXTI_InitStruct = {0};
	  LL_GPIO_InitTypeDef GPIO_InitStruct = {0};

	  /* GPIO Ports Clock Enable */
	  LL_AHB2_GRP1_EnableClock(LL_AHB2_GRP1_PERIPH_GPIOA);
	  LL_AHB2_GRP1_EnableClock(LL_AHB2_GRP1_PERIPH_GPIOB);
	  LL_AHB2_GRP1_EnableClock(LL_AHB2_GRP1_PERIPH_GPIOC);
	  LL_AHB2_GRP1_EnableClock(LL_AHB2_GRP1_PERIPH_GPIOD);
	  LL_AHB2_GRP1_EnableClock(LL_AHB2_GRP1_PERIPH_GPIOH);



	  /*PC13 PC14 PC15 OUT*//*LOW_Power 2.93    Change_OFF 0   BANK_EN 29.42    */
	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_14;//|LL_GPIO_PIN_15;
	  GPIO_InitStruct.Mode = LL_GPIO_MODE_ALTERNATE;
//	  GPIO_InitStruct.Pull = LL_GPIO_PULL_UP;
	  LL_GPIO_Init(GPIOC, &GPIO_InitStruct);


//	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_13;
//	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
//	  GPIO_InitStruct.Pull = LL_GPIO_PULL_UP;
//	  LL_GPIO_Init(GPIOC, &GPIO_InitStruct);
	  BSP_GPIO_Write(GPIO_OUT_ADC_POWER,1);//OFF
	  BSP_GPIO_Write(GPIO_OUT_LED_POWER,1);//OFF
//	  BSP_GPIO_Write(GPIO_OUT_CHANGE_OFF,0);//OFF null





	  /*OSC PH0 PH1*//*1.2   1.2    */
	  GPIO_InitStruct.Pin = LL_GPIO_PIN_0|LL_GPIO_PIN_1;
	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
	  GPIO_InitStruct.Pull = LL_GPIO_PULL_DOWN;
	  LL_GPIO_Init(GPIOH, &GPIO_InitStruct);

	  /*adc 0 1 2 3 PC0 PC1 PC2 PC3*//* VBAT ,ISOFIX  ISOFIX 反向  */
	  GPIO_InitStruct.Pin = LL_GPIO_PIN_0|LL_GPIO_PIN_1|LL_GPIO_PIN_2|LL_GPIO_PIN_3;
	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
	  GPIO_InitStruct.Pull = LL_GPIO_PULL_DOWN;
	  LL_GPIO_Init(GPIOC, &GPIO_InitStruct);

	  /*adc 4 5 6 7 8 9 PA0 PA1 PA2 PA3 PA4 PA6*//* 反向 支腿  头托  sos  ntc ver */
	  GPIO_InitStruct.Pin = LL_GPIO_PIN_0|LL_GPIO_PIN_1|LL_GPIO_PIN_2|LL_GPIO_PIN_3|LL_GPIO_PIN_4|LL_GPIO_PIN_6;
	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
	  GPIO_InitStruct.Pull = LL_GPIO_PULL_DOWN;
	  LL_GPIO_Init(GPIOA, &GPIO_InitStruct);

	  /*adc PB2*//*  5v    */
	  GPIO_InitStruct.Pin = LL_GPIO_PIN_1|LL_GPIO_PIN_2;
	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
	  GPIO_InitStruct.Pull = LL_GPIO_PULL_DOWN;
	  LL_GPIO_Init(GPIOB, &GPIO_InitStruct);

//	  /*SPI1  SCK_PA5 PA7_MOSI  *//*    */
	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_5|LL_GPIO_PIN_7;
//	  GPIO_InitStruct.Mode = LL_GPIO_MODE_OUTPUT;
//	  GPIO_InitStruct.Speed = LL_GPIO_SPEED_FREQ_LOW;
//	  GPIO_InitStruct.OutputType = LL_GPIO_OUTPUT_PUSHPULL;
	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
	  GPIO_InitStruct.Pull = LL_GPIO_PULL_DOWN;
	  LL_GPIO_Init(GPIOA, &GPIO_InitStruct);

	  /*SPI1 CS_PB0 *//*    */
	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_0;
//	  GPIO_InitStruct.Mode = LL_GPIO_MODE_OUTPUT;
//	  GPIO_InitStruct.Speed = LL_GPIO_SPEED_FREQ_LOW;
//	  GPIO_InitStruct.OutputType = LL_GPIO_OUTPUT_PUSHPULL;
	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
	  GPIO_InitStruct.Pull = LL_GPIO_PULL_DOWN;
	  LL_GPIO_Init(GPIOB, &GPIO_InitStruct);

	  LL_GPIO_ResetOutputPin(GPIOA, LL_GPIO_PIN_5|LL_GPIO_PIN_7);
	  LL_GPIO_ResetOutputPin(GPIOB,LL_GPIO_PIN_0);

	  /*UART DEBUG  TX_PC4   RX_PC5*//* 20240220 Debug: input error!   */
	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_4|LL_GPIO_PIN_5;
	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
	  GPIO_InitStruct.Pull = LL_GPIO_PULL_UP;
	  LL_GPIO_Init(GPIOC, &GPIO_InitStruct);


	  /*FAN_PWM PB1    BUZ_PWM PB10 *//*    */
	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_10;
	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
	  GPIO_InitStruct.Pull = LL_GPIO_PULL_DOWN;
//	  GPIO_InitStruct.Mode = LL_GPIO_MODE_OUTPUT;
//	  GPIO_InitStruct.Speed = LL_GPIO_SPEED_FREQ_VERY_HIGH;
//	  GPIO_InitStruct.OutputType = LL_GPIO_OUTPUT_PUSHPULL;
	  LL_GPIO_Init(GPIOB, &GPIO_InitStruct);
	  LL_GPIO_ResetOutputPin(GPIOB, LL_GPIO_PIN_10);

	  /*FAN*/
//	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_7;
//	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
//	  GPIO_InitStruct.Pull = LL_GPIO_PULL_DOWN;
//	  LL_GPIO_Init(GPIOC, &GPIO_InitStruct);
//	  LL_GPIO_ResetOutputPin(GPIOC, LL_GPIO_PIN_7);

	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_7;
	  GPIO_InitStruct.Mode = LL_GPIO_MODE_OUTPUT;
	  GPIO_InitStruct.Pull = LL_GPIO_PULL_NO;
	  LL_GPIO_Init(GPIOC, &GPIO_InitStruct);
	  LL_GPIO_ResetOutputPin(GPIOC, LL_GPIO_PIN_7);


	  /*SPI2 CS1_PA12   SCK_PA13   MISO_PA14   MOSI_PA15*//*    */
//	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_12|LL_GPIO_PIN_13|LL_GPIO_PIN_14|LL_GPIO_PIN_15;
//	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
//	  GPIO_InitStruct.Pull = LL_GPIO_PULL_NO;
//	  LL_GPIO_Init(GPIOB, &GPIO_InitStruct);
//
//	  /*SPI2 CS2_PC6  *//*    */
//	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_6;
//	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
//	  GPIO_InitStruct.Pull = LL_GPIO_PULL_NO;
//	  LL_GPIO_Init(GPIOC, &GPIO_InitStruct);
	  /*MCU_IN2 PC8  */
	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_8;
	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
	  GPIO_InitStruct.Pull = LL_GPIO_PULL_NO;
	  LL_GPIO_Init(GPIOC, &GPIO_InitStruct);


	  /*锁扣_PC9  */
	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_9;
	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
	  GPIO_InitStruct.Pull = LL_GPIO_PULL_DOWN;
	  LL_GPIO_Init(GPIOC, &GPIO_InitStruct);



#ifndef IO_LOW_POWER_ENABLE
	  /*MCU_CT1  */


	  LL_GPIO_SetOutputPin(GPIOC, LL_GPIO_PIN_15);
	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_15;
	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
	  GPIO_InitStruct.Pull = LL_GPIO_PULL_DOWN;
	  LL_GPIO_Init(GPIOC, &GPIO_InitStruct);

	  /*BST_CTL2  PA10  !*/

//	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_10;
//	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
//	  GPIO_InitStruct.Pull = LL_GPIO_PULL_UP;
//	  LL_GPIO_Init(GPIOA, &GPIO_InitStruct);


	  /*BST_CTL4  PA8  max !*/
//	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_8;
//	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
//	  GPIO_InitStruct.Pull = LL_GPIO_PULL_NO;
//	  LL_GPIO_Init(GPIOA, &GPIO_InitStruct);



#endif



	  /*BST_CTL3  PA9  NC!*/
//	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_9;
//	  GPIO_InitStruct.Mode = LL_GPIO_MODE_ALTERNATE;
//	  GPIO_InitStruct.Pull = LL_GPIO_PULL_NO;
//	  LL_GPIO_Init(GPIOA, &GPIO_InitStruct);
//	  LL_GPIO_ResetOutputPin(GPIOA, LL_GPIO_PIN_9);
	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_9;
	  GPIO_InitStruct.Mode = LL_GPIO_MODE_OUTPUT;
	  GPIO_InitStruct.Pull = LL_GPIO_PULL_UP;
	  LL_GPIO_Init(GPIOA, &GPIO_InitStruct);
	  LL_GPIO_SetOutputPin(GPIOA, LL_GPIO_PIN_9);
//	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_9;
//	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
//	  GPIO_InitStruct.Pull = LL_GPIO_PULL_NO;
//	  LL_GPIO_Init(GPIOA, &GPIO_InitStruct);
//	  LL_GPIO_ResetOutputPin(GPIOA, LL_GPIO_PIN_9);




	  /*JT PB4 PB3    */
//	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_3|LL_GPIO_PIN_4;
//	  GPIO_InitStruct.Mode = LL_GPIO_MODE_ALTERNATE;
//	  GPIO_InitStruct.Pull = LL_GPIO_PULL_NO;
//	  LL_GPIO_Init(GPIOB, &GPIO_InitStruct);

#ifndef	  DEBUG_PA13_14_AALTER_TEST_MODE
	  /*JT PA13 PA14 PA15  *//*////////////////////////////////////////////////////////   */
	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_15|LL_GPIO_PIN_14|LL_GPIO_PIN_13;
	  GPIO_InitStruct.Mode = LL_GPIO_MODE_ALTERNATE;
	  GPIO_InitStruct.Pull = LL_GPIO_PULL_NO;
#else
	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_15;
	  GPIO_InitStruct.Mode = LL_GPIO_MODE_ALTERNATE;
	  GPIO_InitStruct.Pull = LL_GPIO_PULL_NO;
#endif
//	  GPIO_InitStruct.Mode = LL_GPIO_MODE_OUTPUT;
//	  GPIO_InitStruct.Speed = LL_GPIO_SPEED_FREQ_LOW;
//	  GPIO_InitStruct.OutputType = LL_GPIO_OUTPUT_PUSHPULL;
	  LL_GPIO_Init(GPIOA, &GPIO_InitStruct);


	  /*UART BLE EXT_UP *//*    */
//	  GPIO_InitStruct.Pin =  LL_GPIO_PIN_10|LL_GPIO_PIN_11|LL_GPIO_PIN_12;
//	  GPIO_InitStruct.Mode = LL_GPIO_MODE_INPUT;
//	  GPIO_InitStruct.Pull = LL_GPIO_PULL_NO;
//	  LL_GPIO_Init(GPIOC, &GPIO_InitStruct);




	  /**/
	  LL_EXTI_SetEXTISource(LL_EXTI_EXTI_PORTA, LL_EXTI_EXTI_LINE12);

	  /**/
	  LL_EXTI_SetEXTISource(LL_EXTI_EXTI_PORTB, LL_EXTI_EXTI_LINE5);

	  /**/
//	  LL_EXTI_SetEXTISource(LL_EXTI_EXTI_PORTB, LL_EXTI_EXTI_LINE9);

	  /**/
	  EXTI_InitStruct.Line_0_31 = LL_EXTI_LINE_12;
	  EXTI_InitStruct.LineCommand = ENABLE;
	  EXTI_InitStruct.Mode = LL_EXTI_MODE_IT;
	  EXTI_InitStruct.Trigger = LL_EXTI_TRIGGER_RISING;
	  LL_EXTI_Init(&EXTI_InitStruct);

	  /**/
	  EXTI_InitStruct.Line_0_31 = LL_EXTI_LINE_5;
	  EXTI_InitStruct.LineCommand = ENABLE;
	  EXTI_InitStruct.Mode = LL_EXTI_MODE_IT;
	  EXTI_InitStruct.Trigger = LL_EXTI_TRIGGER_FALLING;
	  LL_EXTI_Init(&EXTI_InitStruct);

	  /**/
//	  EXTI_InitStruct.Line_0_31 = LL_EXTI_LINE_9;
//	  EXTI_InitStruct.LineCommand = ENABLE;
//	  EXTI_InitStruct.Mode = LL_EXTI_MODE_IT;
//	  EXTI_InitStruct.Trigger = LL_EXTI_TRIGGER_RISING;
//	  LL_EXTI_Init(&EXTI_InitStruct);




	  /**/
//	  LL_GPIO_SetPinPull(GPIOB, LL_GPIO_PIN_9, LL_GPIO_PULL_DOWN);

	  /**/
//	  LL_GPIO_SetPinMode(GPIOB, LL_GPIO_PIN_9, LL_GPIO_MODE_INPUT);



	  /**/
	  LL_GPIO_SetPinPull(GPIOA, LL_GPIO_PIN_12, LL_GPIO_PULL_DOWN);

	  /**/
	  LL_GPIO_SetPinPull(GPIOB, LL_GPIO_PIN_5, LL_GPIO_PULL_UP);

	  /**/
	  LL_GPIO_SetPinMode(GPIOA, LL_GPIO_PIN_12, LL_GPIO_MODE_INPUT);

	  /**/
	  LL_GPIO_SetPinMode(GPIOB, LL_GPIO_PIN_5, LL_GPIO_MODE_INPUT);



	  LL_EXTI_ClearRisingFlag_0_31(LL_EXTI_LINE_9);
	  /* EXTI interrupt init*/
	  NVIC_SetPriority(EXTI5_IRQn, NVIC_EncodePriority(NVIC_GetPriorityGrouping(),0, 0));
	  NVIC_EnableIRQ(EXTI5_IRQn);
	  NVIC_SetPriority(EXTI9_IRQn, NVIC_EncodePriority(NVIC_GetPriorityGrouping(),0, 0));
	  NVIC_EnableIRQ(EXTI9_IRQn);
	  NVIC_SetPriority(EXTI12_IRQn, NVIC_EncodePriority(NVIC_GetPriorityGrouping(),0, 0));
	  NVIC_EnableIRQ(EXTI12_IRQn);





	  /**/
	  LL_EXTI_SetEXTISource(LL_EXTI_EXTI_PORTB, LL_EXTI_EXTI_LINE7);

	  /**/
	  EXTI_InitStruct.Line_0_31 = LL_EXTI_LINE_7;
	  EXTI_InitStruct.LineCommand = ENABLE;
	  EXTI_InitStruct.Mode = LL_EXTI_MODE_IT;
	  EXTI_InitStruct.Trigger = LL_EXTI_TRIGGER_FALLING;
	  LL_EXTI_Init(&EXTI_InitStruct);

	  /**/
	  LL_GPIO_SetPinPull(GPIOB, LL_GPIO_PIN_7, LL_GPIO_PULL_NO);

	  /**/
	  LL_GPIO_SetPinMode(GPIOB, LL_GPIO_PIN_7, LL_GPIO_MODE_INPUT);

	  NVIC_SetPriority(EXTI7_IRQn, NVIC_EncodePriority(NVIC_GetPriorityGrouping(),0, 0));
	  NVIC_EnableIRQ(EXTI7_IRQn);


}




/*****************************************************************************
 *	PWR_WakeUpExitInit:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Wake up and exit initialization
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void PWR_WakeUpExitInit(void)
{
	LL_EXTI_InitTypeDef EXTI_InitStruct = {0};
	LL_GPIO_InitTypeDef GPIO_InitStruct = {0};


	LL_EXTI_DeInit();

	/* EXTI interrupt init*/
	  /**/
	  EXTI_InitStruct.Line_0_31 = LL_EXTI_LINE_5;
	  EXTI_InitStruct.LineCommand = DISABLE;
	  EXTI_InitStruct.Mode = LL_EXTI_MODE_IT;
	  EXTI_InitStruct.Trigger = LL_EXTI_TRIGGER_RISING;
	  LL_EXTI_Init(&EXTI_InitStruct);
	  NVIC_DisableIRQ(EXTI5_IRQn);

	/**/
	LL_GPIO_SetPinPull(GPIOA, LL_GPIO_PIN_12, LL_GPIO_PULL_DOWN);
	/**/
	LL_GPIO_SetPinPull(GPIOB, LL_GPIO_PIN_5, LL_GPIO_PULL_DOWN);
	/**/
	LL_GPIO_SetPinMode(GPIOA, LL_GPIO_PIN_12, LL_GPIO_MODE_INPUT);
	/**/
	LL_GPIO_SetPinMode(GPIOB, LL_GPIO_PIN_5, LL_GPIO_MODE_INPUT);





	GPIO_InitStruct.Pin =  LL_GPIO_PIN_7;
	GPIO_InitStruct.Pull = LL_GPIO_PULL_NO;
	GPIO_InitStruct.Mode = LL_GPIO_MODE_OUTPUT;
	GPIO_InitStruct.Speed = LL_GPIO_SPEED_FREQ_LOW;
	GPIO_InitStruct.OutputType = LL_GPIO_OUTPUT_PUSHPULL;
	LL_GPIO_Init(GPIOC, &GPIO_InitStruct);


	LL_GPIO_ResetOutputPin(GPIOC, LL_GPIO_PIN_7);


	MX_GPIO_Init();

	MX_SPI1_Init();
	MX_SPI2_Init();
	MX_TIM2_InitPeriod(SYS_RATE_4M);
	MX_TIM3_Init();
	MX_RTC_Init();

	MX_UART4_Init();

	MX_GPDMA1_Init();
	MX_ADC1_Init();
	BSP_StartPwm();
}


/*****************************************************************************
 *	PWR_80MWakeUpExitInit:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: High frequency wake-up exit initialization
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void PWR_80MWakeUpExitInit(void)
{
	LL_EXTI_InitTypeDef EXTI_InitStruct = {0};
	LL_GPIO_InitTypeDef GPIO_InitStruct = {0};

	/* EXTI interrupt init*/
	/**/
	EXTI_InitStruct.Line_0_31 = LL_EXTI_LINE_5;
	EXTI_InitStruct.LineCommand = DISABLE;
	EXTI_InitStruct.Mode = LL_EXTI_MODE_IT;
	EXTI_InitStruct.Trigger = LL_EXTI_TRIGGER_RISING;
	LL_EXTI_Init(&EXTI_InitStruct);
	NVIC_DisableIRQ(EXTI5_IRQn);

//	/**/
//	LL_EXTI_SetEXTISource(LL_EXTI_EXTI_PORTA, LL_EXTI_EXTI_LINE12);
//	/**/
//	LL_EXTI_SetEXTISource(LL_EXTI_EXTI_PORTB, LL_EXTI_EXTI_LINE5);
	/**/
	LL_GPIO_SetPinPull(GPIOA, LL_GPIO_PIN_12, LL_GPIO_PULL_DOWN);
	/**/
	LL_GPIO_SetPinPull(GPIOB, LL_GPIO_PIN_5, LL_GPIO_PULL_DOWN);
	/**/
	LL_GPIO_SetPinMode(GPIOA, LL_GPIO_PIN_12, LL_GPIO_MODE_INPUT);
	/**/
	LL_GPIO_SetPinMode(GPIOB, LL_GPIO_PIN_5, LL_GPIO_MODE_INPUT);
	//	  /**/
	//	  LPGPIO_InitStruct.Pin = LL_LPGPIO_PIN_10;
	//	  LPGPIO_InitStruct.Mode = LL_LPGPIO_MODE_OUTPUT;
	//	  LL_LPGPIO_Init(LPGPIO1, &LPGPIO_InitStruct);




	GPIO_InitStruct.Pin =  LL_GPIO_PIN_7;
	GPIO_InitStruct.Pull = LL_GPIO_PULL_NO;
	GPIO_InitStruct.Mode = LL_GPIO_MODE_OUTPUT;
	GPIO_InitStruct.Speed = LL_GPIO_SPEED_FREQ_LOW;
	GPIO_InitStruct.OutputType = LL_GPIO_OUTPUT_PUSHPULL;
	LL_GPIO_Init(GPIOC, &GPIO_InitStruct);
	LL_GPIO_ResetOutputPin(GPIOC, LL_GPIO_PIN_1);




	MX_GPIO_Init();

	MX_SPI1_Init();
	MX_SPI2_Init();
	MX_TIM2_InitPeriod(SYS_RATE_80M);
	MX_TIM3_Init();
	MX_RTC_Init();

	MX_UART4_Init();

	MX_GPDMA1_Init();
	MX_ADC1_Init();
	BSP_StartPwm();
}

/*****************************************************************************
 *	PWR_GetModuleStatus:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Status judgment of each module
 *
 *	RETURNS:Inactive or active
 *
 *****************************************************************************
 */
MODULE_STATUS_E PWR_GetModuleStatus(void)
{
	MODULE_STATUS_E status = MODULE_STATUS_ACTIVE;
	/*Disp delay activity detection*/
	if((DISP_STATUS_INACTIVE == Disp_GetModuleStatus())
	/*ADK activity detection*/
	&&(ADK_STATUS_INACTIVE == ADKey_GetModuleStatus())
	/*IOKEY activity detection*/
	&&(IOKEY_STATUS_INACTIVE == IOKey_GetModuleStatus())
	/*Bottom level parallel port chip shutdown*/
	&&(LV595_STATUS_OFF ==LV595_GetStatus())
//	&&(ADK2_VALUE_MODE_OFF == AdKey_GetSwitchVal(ADKEY_SW2_5V))
	&&(FAN_MODE_OFF == Fan_GetMode())
	&&(FRONT_ALGO_ON != TRUE)
#ifndef SLEEP_TEST_MODE
	&&(AL_GetFlag() != AL_STATUS_ON)
#endif
//	&&(DEBUG_ReadEDRfire() == FALSE)//20240227 update
	&&(TRUE != PWR_getBatEmptyStats())
	&&(EDR_BleActiveFlg == FALSE)
	&&(WAKEUP_TEST == 1)
	/*&& Sensor activity*/
	){
		status = MODULE_STATUS_INACTIVE;
	}

	return status;
}

/*****************************************************************************
 *	PWR_EnterSleepCountdown:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Sleep core function
 *
 *	RETURNS: NORMAL OR  HIGH
 *
 *****************************************************************************
 */

POWER_MODE_E PWR_EnterSleepCountdown(void)
{
	/*After entering sleep, the flag directly returns to normal*/
	POWER_MODE_E Mode = POWER_LOOP_MODE_NORMAL;
#ifdef	SLEEP_JMP_HIGH_TEST_MODE
	PowerCtrlPkg.WakeEvent = WAKE_SENSOR;
#else
	PowerCtrlPkg.WakeEvent = WAKE_SWITCH;
#endif


	/*Once again, make sure there are no incidents!!!*/
	if(MODULE_STATUS_INACTIVE == PWR_GetModuleStatus()){

		PowerCtrlPkg.accWakeDelayCnt = 0;
		/*gpio*/
		PWR_WakeUpEnterConfig();

		NVIC_DisableIRQ(GPDMA1_Channel0_IRQn);
		NVIC_DisableIRQ(ADC1_IRQn);
		NVIC_DisableIRQ(USART3_IRQn);
		NVIC_DisableIRQ(UART4_IRQn);
		BAT_CFG_STEP = 	BAT_CFG_DELAY;
		ELEC_CNT = 0;

		PowerCtrlPkg.InitDelayCnt = 0;

//		LL_DMA_DisableChannel(GPDMA1, LL_DMA_CHANNEL_0);



		LL_USART_DisableIT_RXNE_RXFNE(USART3);
		LL_USART_Disable(USART3);
		LL_USART_DeInit(USART3);

		LL_USART_DisableIT_RXNE_RXFNE(UART4);
		LL_USART_Disable(UART4);
		LL_USART_DeInit(UART4);

		/*PWR_Uart4EnterConfig();*/

		LL_SPI_DeInit(SPI1);
		LL_SPI_DeInit(SPI2);


		LL_ADC_Disable(ADC1);
		LL_ADC_DeInit(ADC1);


		/**/
#ifdef AUTO_WAKEUP_ENABLE
		  /* Disable RTC registers write protection */
		  LL_RTC_DisableWriteProtection(RTC);

		  /* Enable wake up counter and wake up interrupt */
		  /* Note: Periodic wakeup interrupt should be enabled to exit the device
		     from low-power modes.*/
		  LL_RTC_EnableIT_WUT(RTC);
		  LL_RTC_WAKEUP_Enable(RTC);

		  /* Enable RTC registers write protection */
		  LL_RTC_EnableWriteProtection(RTC);

#endif /* AUTO_WAKEUP_ENABLE */

//		if(PWR_WORK_MODE != POWER_LOOP_MODE_HIGH)
//		{
		  /** Request to enter STOP 0 mode
			* Following procedure describe in STM32U5xx Reference Manual
			* See PWR part, section Low-power modes, STOP 0 mode
			*/
		  /* Set STOP 0 mode when CPU enters deepsleep */
		  LL_PWR_SetPowerMode(LL_PWR_STOP2_MODE);

		  /* Set SLEEPDEEP bit of Cortex System Control Register */
		  LL_LPM_EnableDeepSleep();

		  /* Request Wait For Interrupt */
		  __WFI();

//		}

#ifdef AUTO_WAKEUP_ENABLE
		/* Disable RTC registers write protection */
		LL_RTC_DisableWriteProtection(RTC);

		LL_RTC_DisableIT_WUT(RTC);
		LL_RTC_WAKEUP_Disable(RTC);

		/* Enable RTC registers write protection */
		LL_RTC_EnableWriteProtection(RTC);
#endif /* AUTO_WAKEUP_ENABLE */

		  NVIC_SetPriorityGrouping(NVIC_PRIORITYGROUP_4);
		  NVIC_SetPriority(SysTick_IRQn, NVIC_EncodePriority(NVIC_GetPriorityGrouping(),15, 0));
		  LL_AHB3_GRP1_EnableClock(LL_AHB3_GRP1_PERIPH_PWR);

		if(PowerCtrlPkg.HighPwr.Bit.OnFlag == TRUE){
			PowerCtrlPkg.WakeEvent = WAKE_SENSOR;
			PowerCtrlPkg.HighPwr.Bit.OnFlag = FALSE;
		}
		if(PowerCtrlPkg.WakeEvent ==  WAKE_SENSOR)
		{
			SystemClock_80MConfig();
			PWR_80MWakeUpExitInit();
			BSP_FeedWWDG();
			MX_TIM1_InitPeriod(SYS_RATE_80M);
			MX_WWDG_Init();
			MX_USART3_UART_Init();
			PowerCtrlPkg.InitDelay = TRUE;
			PowerCtrlPkg.IdleCnt = 0;
			Mode = POWER_LOOP_MODE_HIGH;

		}else if((PowerCtrlPkg.WakeEvent ==  WAKE_SWITCH)||(PowerCtrlPkg.WakeEvent ==  WAKE_KEY)){

			SystemClock_4MConfig();
			PWR_WakeUpExitInit();
			BSP_FeedWWDG();
			MX_TIM1_InitPeriod(SYS_RATE_4M);
			MX_WWDG_SET_PRE(SYS_RATE_4M);

			MX_USART3_UART_Init();
			Mode = POWER_LOOP_MODE_NORMAL;
			if(PowerCtrlPkg.WakeEvent ==  WAKE_KEY){
			DIAG_AirBagUpdate();}
		}
		PowerCtrlPkg.WakeEvent = WAKE_SWITCH;
		EDR_AccDataInx = 0;
		EDR_AccDataFlag  = TRUE;
		BSP_FeedWWDG();


	}
	Disp_RestFlashData();
	return Mode;
}



/*****************************************************************************
 *	PWR_GetUartInitDelayState:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get uart init state
 *
 *	RETURNS: TRUE OR  FALSE
 *
 *****************************************************************************
 */
BOOL PWR_GetUartInitDelayState(void)
{
	BOOL ret = 	PowerCtrlPkg.UartDelay;
	PowerCtrlPkg.UartDelay = TRUE;
	return ret;
}



/*****************************************************************************
 *	PWR_UpdateHighState:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Update high, INT call
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void PWR_UpdateHighState(void)
{
	if(PWR_WORK_MODE != POWER_LOOP_MODE_HIGH)
	{PowerCtrlPkg.HighPwr.Bit.OnFlag = TRUE;}
	PowerCtrlPkg.IdleCnt = 0;
}

/*****************************************************************************
 *	PWR_UpdateBleState:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Update ble data, INT call
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void PWR_UpdateBleState(void)
{
	PWR_BLE_WAKE = TRUE;
}

/*****************************************************************************
 *	PWR_UpdateHighPower:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Update power
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void PWR_UpdateHighPower(void)
{
	if(PowerCtrlPkg.HighPwr.Bit.OnFlag == TRUE){
		PWR_WORK_MODE = POWER_LOOP_MODE_HIGH;
		PowerCtrlPkg.HighPwr.Bit.OnFlag = FALSE;
		SystemClock_80MConfig();
		MX_TIM1_InitPeriod(SYS_RATE_80M);
		MX_TIM2_InitPeriod(SYS_RATE_80M);
		BSP_StartPwm();
		MX_WWDG_Init();
	}
}


/*****************************************************************************
 *	PWR_HighPowerDelay:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Power delay
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void PWR_HighPowerDelay(void)
{
	PowerCtrlPkg.IdleCnt = 0;

}


/*****************************************************************************
 *	PWR_OffHighPower:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Power off call
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void PWR_OffHighPower(void)
{
	if((PowerCtrlPkg.HighPwr.Bit.OffFlag == TRUE) && (FRONT_ALGO_ON == FALSE)){
		if((PWR_GetModuleStatus())){
			PowerCtrlPkg.IdleCnt = 0;
			PowerCtrlPkg.HighPwr.Bit.OffFlag =  FALSE;
		}else{
			if(TRUE == ACC_ClearSRC()){
				PWR_WORK_MODE = POWER_LOOP_MODE_SLEEP;
				PowerCtrlPkg.IdleCnt = SLEEP_COUNTDOWD_TIME;
				PowerCtrlPkg.HighPwr.Bit.OffFlag = FALSE;
			}
		}
	}else{
		PowerCtrlPkg.HighPwr.Bit.OffFlag =  FALSE;
	}
}


/*****************************************************************************
 *	PWR_ExitHighPower:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Power exit call
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void PWR_ExitHighPower(void)
{

	PowerCtrlPkg.HighPwr.Bit.OffFlag = TRUE;
}

/*****************************************************************************
 *	PWR_AccTestDelayCall:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: ACC test call
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void PWR_AccTestDelayCall(void)
{
	PWR_TEST_CNT = 0;
}

/*****************************************************************************
 *	PWR_AccCurrentManagement:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: ACC power Management
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void PWR_AccCurrentManagement(void)
{

}
/*****************************************************************************
 *	PWR_Manage:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Module main loop
 *
 *	RETURNS: VOID
 *
 *****************************************************************************
 */
void PWR_Manage(void)
{
	UNS8 AccPowerStatus = FALSE;
	PWR_ElectricityMonitor();
	switch(PWR_WORK_MODE)
	{
	case POWER_LOOP_MODE_HIGH:

		if((MODULE_STATUS_INACTIVE == PWR_GetModuleStatus())
				&& (FRONT_ALGO_ON == FALSE)){
		   if(TRUE == ACC_IsStatusGoodSleep()){
				if(PowerCtrlPkg.FirstPowerOn == FALSE){
					PowerCtrlPkg.IdleCnt = HIGH_LOOP_IDLE_TIME;
					PowerCtrlPkg.FirstPowerOn = TRUE;
				}
				if(PowerCtrlPkg.IdleCnt < HIGH_LOOP_IDLE_TIME ){
#ifndef LOOP_HIGH_MODE_TEST_ENABLE
					PowerCtrlPkg.IdleCnt++;
#endif
				}else{
						/*Exit high-speed mode*/
						PWR_ExitHighPower();}
			}else{

			}
		}else{
			PowerCtrlPkg.IdleCnt = 0;
		}
		break;
	case POWER_LOOP_MODE_NORMAL:
		if(MODULE_STATUS_INACTIVE == PWR_GetModuleStatus()){
			/*Countdown enters sleep mode*/
			if(PowerCtrlPkg.IdleCnt > 0){
				PowerCtrlPkg.IdleCnt--;
				PWR_WORK_MODE = POWER_LOOP_MODE_NORMAL;
				break;
			}
		}else{PowerCtrlPkg.IdleCnt = SLEEP_COUNTDOWD_TIME;
		if(PWR_WORK_MODE != POWER_LOOP_MODE_SLEEP){
			/*Non sleep return*/
			break;
		}
		}
	case POWER_LOOP_MODE_SLEEP:
		POWER_MODE_E Mode = POWER_LOOP_MODE_NORMAL;
		LL_TIM_ClearFlag_UPDATE(TIM1);
		LL_TIM_DisableIT_UPDATE(TIM1);
		/*acc power switch*/
		if(TRUE ==  AdKey_GetAccIntStatus()){
			/*On ACC power supply*/
			LL_GPIO_ResetOutputPin(GPIOB, LL_GPIO_PIN_4 );
			/*Determine whether the sensor is in salary status*/
			if((POWER_LOOP_MODE_SLEEP == PWR_GetLoopMode()
				&& ACCEL_IsStatusGood() == TRUE)){
				/*Module completed and entered sleep mode*/
				AccPowerStatus = TRUE;
			}
		}else{
			/*Off ACC power supply*/
			if(!LL_GPIO_IsInputPinSet(GPIOB,LL_GPIO_PIN_4)){
				LL_GPIO_SetOutputPin(GPIOB, LL_GPIO_PIN_4 );
				/*Reset ram*/
				 ACCEL_RamInit();
			}
			/*enter sleep mode*/
			AccPowerStatus = TRUE;
		}
		/*Entering Sleep*/
		if((TRUE == ACC_INT_IsStatusGood())
				&& (TRUE == ACC_ClearSRC()
				&& (AccPowerStatus == TRUE))){
			Mode = PWR_EnterSleepCountdown();
		}
		LL_TIM_EnableIT_UPDATE(TIM1);//ok
		/*modified mode*/
		PWR_WORK_MODE = Mode;
		PowerCtrlPkg.IdleCnt = SLEEP_COUNTDOWD_TIME;
		break;

	default:
		break;
	}
}


/*****************************************************************************
 *	PWR_SetLoopMode:
 *
 *	PARAMETERS: VOID
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Power management interface
 *
 *	RETURNS: power mode
 *
 *****************************************************************************
 */
void PWR_SetLoopMode(POWER_MODE_E mode)
{
	PWR_WORK_MODE = mode;
}

/*****************************************************************************
 *	PWR_GetLoopMode:
 *
 *	PARAMETERS: power mode
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Obtain power management interface
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
POWER_MODE_E PWR_GetLoopMode(void)
{
	return PWR_WORK_MODE;
}

/*****************************************************************************
 *	SystemClock_4MConfig:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: System initialization 4m
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void SystemClock_4MConfig(void)
{
	  UNS32 TimerOut = SYSTEM_CLOCK_TIMEROUT;
	  LL_FLASH_SetLatency(LL_FLASH_LATENCY_0);
	  while((LL_FLASH_GetLatency()!= LL_FLASH_LATENCY_0)&&(--TimerOut)){}

	  LL_PWR_SetRegulVoltageScaling(LL_PWR_REGU_VOLTAGE_SCALE4);
	  TimerOut = SYSTEM_CLOCK_TIMEROUT;
	  while ((LL_PWR_IsActiveFlag_VOS() == 0)&&(--TimerOut)){}

	  LL_RCC_HSI_Enable();
	  TimerOut = SYSTEM_CLOCK_TIMEROUT;
	   /* Wait till HSI is ready */
	  while((LL_RCC_HSI_IsReady() != 1)&&(--TimerOut)){}


	  LL_RCC_HSI_SetCalibTrimming(16);
	  LL_RCC_LSI_Enable();

	   /* Wait till LSI is ready */

	  LL_RCC_MSIS_Enable();
	  TimerOut = SYSTEM_CLOCK_TIMEROUT;
	   /* Wait till MSIS is ready */
	  while((LL_RCC_MSIS_IsReady() != 1)&&(--TimerOut)){}


	  LL_RCC_MSI_EnableRangeSelection();
	  LL_RCC_MSIS_SetRange(LL_RCC_MSISRANGE_4);
	  LL_RCC_MSI_SetCalibTrimming(16, LL_RCC_MSI_OSCILLATOR_1);
	  LL_PWR_EnableBkUpAccess();
	  LL_RCC_SetSysClkSource(LL_RCC_SYS_CLKSOURCE_HSI);
	  TimerOut = SYSTEM_CLOCK_TIMEROUT;
	   /* Wait till System clock is ready */
	  while((LL_RCC_GetSysClkSource() != LL_RCC_SYS_CLKSOURCE_STATUS_MSIS)&&(--TimerOut)){}


	  LL_RCC_SetAHBPrescaler(LL_RCC_SYSCLK_DIV_1);
	  LL_RCC_SetAPB1Prescaler(LL_RCC_APB1_DIV_1);
	  LL_RCC_SetAPB2Prescaler(LL_RCC_APB2_DIV_1);
	  LL_RCC_SetAPB3Prescaler(LL_RCC_APB3_DIV_1);

//	  LL_Init1msTick(4000000);

//	  LL_SetSystemCoreClock(4000000);
	  LL_SetSystemCoreClock(16000000);
//	  LL_RCC_HSI_EnableInStopMode();
}

/*****************************************************************************
 *	SystemClock_80MConfig:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: System initialization 80m
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void SystemClock_80MConfig(void)
{
	  UNS32 TimerOut = SYSTEM_CLOCK_TIMEROUT;
	 LL_FLASH_SetLatency(LL_FLASH_LATENCY_2);
	  while((LL_FLASH_GetLatency()!= LL_FLASH_LATENCY_2)&&(--TimerOut)){}

	  LL_PWR_SetRegulVoltageScaling(LL_PWR_REGU_VOLTAGE_SCALE2);
	  TimerOut = SYSTEM_CLOCK_TIMEROUT;
	  while ((LL_PWR_IsActiveFlag_VOS() == 0)&&(--TimerOut)){}

	  LL_RCC_HSI_Enable();
	  TimerOut = SYSTEM_CLOCK_TIMEROUT;
	   /* Wait till HSI is ready */
	  while((LL_RCC_HSI_IsReady() != 1)&&(--TimerOut)){}


	  LL_RCC_HSI_SetCalibTrimming(16);
	  LL_RCC_LSI_Enable();

	  LL_RCC_MSIK_Enable();
	  TimerOut = SYSTEM_CLOCK_TIMEROUT;
	   /* Wait till MSIK is ready */
	  while((LL_RCC_MSIK_IsReady() != 1)&&(--TimerOut)){}

	  LL_RCC_MSI_EnableRangeSelection();
	  LL_RCC_MSIK_SetRange(LL_RCC_MSIKRANGE_4);
	  LL_RCC_MSI_SetCalibTrimming(16, LL_RCC_MSI_OSCILLATOR_1);
	  LL_PWR_EnableBkUpAccess();
	  LL_RCC_PLL1_ConfigDomain_SYS(LL_RCC_PLL1SOURCE_HSI, 4, 80, 4);
	  LL_RCC_PLL1_EnableDomain_SYS();
	  LL_RCC_SetPll1EPodPrescaler(LL_RCC_PLL1MBOOST_DIV_1);
	  LL_RCC_PLL1_SetVCOInputRange(LL_RCC_PLLINPUTRANGE_4_8);
	  LL_RCC_PLL1_Enable();
	  TimerOut = SYSTEM_CLOCK_TIMEROUT;
	   /* Wait till PLL is ready */
	  while((LL_RCC_PLL1_IsReady() != 1)&&(--TimerOut)){}

	  LL_RCC_SetSysClkSource(LL_RCC_SYS_CLKSOURCE_PLL1);
	  TimerOut = SYSTEM_CLOCK_TIMEROUT;
	   /* Wait till System clock is ready */
	  while((LL_RCC_GetSysClkSource() != LL_RCC_SYS_CLKSOURCE_STATUS_PLL1)&&(--TimerOut)){}

	  LL_RCC_SetAHBPrescaler(LL_RCC_SYSCLK_DIV_1);
	  LL_RCC_SetAPB1Prescaler(LL_RCC_APB1_DIV_1);
	  LL_RCC_SetAPB2Prescaler(LL_RCC_APB2_DIV_1);
	  LL_RCC_SetAPB3Prescaler(LL_RCC_APB3_DIV_1);

//	  LL_Init1msTick(80000000);

	  LL_SetSystemCoreClock(80000000);
//	  LL_RCC_MSIK_EnableInStopMode();
}



/*****************************************************************************
 *	PWR_IntAllEnable:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Enable interruption
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */

void PWR_IntAllEnable(void)
{
	LL_USART_EnableIT_RXNE(UART4);
	LL_USART_EnableIT_RXNE(USART3);
}


/*****************************************************************************
 *	PWR_IntAllDisable:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Disable interrupts
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void PWR_IntAllDisable(void)
{
	LL_USART_DisableIT_RXNE_RXFNE(USART3);
	LL_USART_DisableIT_RXNE_RXFNE(UART4);
}

/*****************************************************************************
 *	PWR_GetPowerSupplyOnfreq:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Obtain the number of times the power supply has been powered on
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
UNS32 PWR_GetPowerSupplyOnfreq(void)
{
	return PWR_ON_FREQ;
}

/*****************************************************************************
 *	PWR_GetPowerSupplyOnfreq:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Record the number of times the power is turned on, main call
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void PWR_PowerSupplyOnFreq(void)
{
#define POWER_RECORD_MAX   	0xFFFFFFFE
#define WR_REPEAT_MAX   	3
#define PWR_ON_CNT_MAX   	1000
	UNS8 timeout =0;
//	LL_GPIO_TogglePin(GPIOA, LL_GPIO_PIN_15);
	if(PWR_ON_CNT < PWR_ON_CNT_MAX){
		++PWR_ON_CNT;
		if(PWR_ON_CNT != PWR_ON_CNT_MAX){
			return;
		}
		UNS32 pwron = MX_Flash_RD_PowerOnRecordFlag();
		if(POWER_RECORD_MAX != pwron){
			pwron++;
			PWR_ON_FREQ = pwron;
			while((0 != MX_Flash_WR_PowerOnRecordFlag(pwron))&&(timeout < WR_REPEAT_MAX)){
				timeout++;
				if(pwron == MX_Flash_RD_PowerOnRecordFlag()){
					break;
				}
			}
			if(timeout == WR_REPEAT_MAX){
				PWR_ON_ERR = 1;
			}
		}else{
			PWR_ON_FREQ = POWER_RECORD_MAX;
		}
	}
}




