
/*
 * rec.c
 *
 *  Created on: Jul 8, 2024
 *      Author: liuzhimin
 */




#include "main.h"

#define REC_FIRE_FLASH_ADDR   				((uint32_t)0x0802F000)
#define REC_LOCK_FLASH_ADDR   				((uint32_t)0x0802F080)
#define REC_FLASH_NULL_ECC   				((uint32_t)0xFFFFFFFF)
#define REC_FLASH_MAX   					((uint32_t)0x00002000)


#define REC_ACCDATA_UNLOCK_LEN_MAX  		600
#define REC_ACCDATA_ROUND_LEN_MAX  			1400
#define REC_ACCDATA_LOCK_LEN_MAX   			650
#define REC_ALGORITHM_START_TIME_MAX   		10

#define REC_STATE_TIME_MASK   				(0x7FFFFF)
#define REC_STATE_BIT1_MASK   				(0x1)
#define REC_STATE_BIT3_MASK   				(0x7)

#define REC_ACCDATA_UNLOCK_REM_LEN_MAX   	100
#define REC_ACCDATA_UNLOCK_FILTE_MASK   	1
#define REC_ACCDATA_UNLOCK_START_FILTE   	4
#define REC_ACCDATA_UNLOCK_START_THRESHOLD  45

#define REC_ACCDATA_UNLOCK_EXIT_MAX		  	100

typedef enum
{
	REC_FIRE_TYPE_OFF = 0XFFFFFFFF,
	REC_FIRE_TYPE_ON  = 0X542A542A,

} REC_FIRE_KEY_TYPE_E;

typedef enum
{
	REC_STEP_PRINTF_0 = 0,
	REC_STEP_PRINTF_1,
	REC_STEP_PRINTF_2,
	REC_STEP_PRINTF_3,
	REC_STEP_PRINTF_4,
	REC_STEP_PRINTF_5,
	REC_STEP_PRINTF_6,
} REC_STEP_PRINTF_E;

typedef enum
{
	REC_CMD_NULL = 0,
	REC_CMD_ON,
	REC_CMD_REFRESH,
	REC_CMD_OFF,

} REC_CMD_E;


typedef	union
{
	struct
	{
		UNS32 Int:1;
		UNS32 Ready:1;
		UNS32 Algo:1;
		UNS32 AirCheck:3;
		UNS32 UI:3;
		UNS32 Timer:23;
	} Bit;
	UNS32 Dword;
} REC_STATE_U;



typedef struct
{
	AXIS_DATA_T Dat;
	REC_STATE_U State;
} REC_ARRAY_T;

 typedef struct
 {
	UNS32 FireCheckSum;
	UNS32 DateCheckSum;
 	UNS32 AlogTime[REC_ALGORITHM_START_TIME_MAX];
 	UNS32 FireTime;
 	UNS32 OffMosTime;
 	REC_FIRE_KEY_TYPE_E FireSignalFlag;
 	UNS8 TimeInx;
 	UNS8 TrigFlag;
 	UNS8 MosFlag;
 	UNS8 SaveFlag;

	UNS8 Reserved[1];
 } REC_DAT_T;


/* Test_REC_CTRL_PACKAGE_T */
typedef struct
{
	REC_DAT_T Dat;
	/* Data err */
	UNS32 FireCrc;
	UNS32 DataCrc;
	UNS16 RecRoundInx;
	UNS16 RecRoundDelay;
	UNS16 RecRoundTime;
	UNS16 RecLockDelay;
	UNS8  RecFlg;
	UNS8  RecAlgoOnTime;
	UNS8  RecAlgoOnDat;
	UNS8  PauseTimes;
 	UNS8  RecStep;
	UNS16 RecPrintStep;
	UNS16 RecPrintSubStep;
	UNS16 RecPrintLen;
	UNS16 RecPrintMax;
	UNS16 RecClrStep;
	UNS16 RecUnLockArrayInx;
	UNS16 RecUnLockBlockInx;
	UNS8  RecUnLockBlockFlg;
	UNS8  RecUnLockRemFlg;
	S16   RecUnLockFilte[REC_ACCDATA_UNLOCK_START_FILTE];
	UNS8  RecUnLockOn;
	UNS8  RecUnLockFilteInx;
	UNS16 RecUnLockExitCnt;
	UNS8 Reserved[4];

} REC_CTRL_PACKAGE_T;



volatile REC_ARRAY_T *FLASH_READ_LOCK_DATA_ADDR = (REC_ARRAY_T *)REC_LOCK_FLASH_ADDR;
volatile REC_DAT_T *FLASH_READ_FIRE_ADDR 		= (REC_DAT_T *)REC_FIRE_FLASH_ADDR;


static REC_CTRL_PACKAGE_T RecCtrlPkg;
static REC_ARRAY_T REC_AccDataLockRamBuff[REC_ACCDATA_LOCK_LEN_MAX] = {0};
static REC_ARRAY_T REC_AccDataRoundRamBuff[REC_ACCDATA_ROUND_LEN_MAX] = {0};
static REC_ARRAY_T REC_AccDataUnlockRamBuff[REC_ALGORITHM_START_TIME_MAX][REC_ACCDATA_UNLOCK_LEN_MAX] = {0};

#define REC_ROUND_INX     			RecCtrlPkg.RecRoundInx
#define REC_LOCK_DELAY 				RecCtrlPkg.RecLockDelay
#define REC_ALGO_ON_DAT 			RecCtrlPkg.RecAlgoOnDat
#define REC_ALGO_ON_TIME 			RecCtrlPkg.RecAlgoOnTime
#define REC_ROUND_TIME				RecCtrlPkg.RecRoundTime
#define REC_FIRE_SIGNAL_FLG			RecCtrlPkg.Dat.FireSignalFlag
#define REC_FIRE_TRIG_FLG			RecCtrlPkg.Dat.TrigFlag
#define REC_FIRE_MOS_FLG			RecCtrlPkg.Dat.MosFlag
#define REC_FIRE_SAVE_FLG			RecCtrlPkg.Dat.SaveFlag
#define REC_FIRE_TIME_INX			RecCtrlPkg.Dat.TimeInx
#define REC_FIRE_TIME_GROUP			RecCtrlPkg.Dat.AlogTime
#define REC_FIRE_TIME_FLG			RecCtrlPkg.Dat.FireTime
#define REC_FIRE_OFFMOS_TIME		RecCtrlPkg.Dat.OffMosTime
#define REC_CHECKSUM_FIRE_FLG		RecCtrlPkg.Dat.FireCheckSum
#define REC_CHECKSUM_DATA_FLG		RecCtrlPkg.Dat.DateCheckSum
#define REC_ROUND_STEP				RecCtrlPkg.RecStep
#define REC_FIRE_DATA				RecCtrlPkg.Dat
#define REC_FIRE_CRC				RecCtrlPkg.FireCrc;
#define REC_DATE_CRC				RecCtrlPkg.DataCrc;
#define REC_ACC_RAM					REC_AccDataRoundRamBuff
#define REC_ACC_LOCK				REC_AccDataLockRamBuff
#define REC_ACC_UNLOCK				REC_AccDataUnlockRamBuff
#define REC_CLR_STEP				RecCtrlPkg.RecClrStep
#define REC_UNLOCK_ARRAY_INX		RecCtrlPkg.RecUnLockArrayInx
#define REC_UNLOCK_BLOCK_INX		RecCtrlPkg.RecUnLockBlockInx
#define REC_UNLOCK_BLOCK_FLG		RecCtrlPkg.RecUnLockBlockFlg
#define REC_UNLOCK_REM_FLG			RecCtrlPkg.RecUnLockRemFlg


#define REC_FLS_ALOG_TIME			FLASH_READ_FIRE_ADDR->AlogTime
#define REC_FLS_DATA_CHECKSUM		FLASH_READ_FIRE_ADDR->DateCheckSum
#define REC_FLS_FIRE_CHECKSUM		FLASH_READ_FIRE_ADDR->FireCheckSum
#define REC_FLS_FIRE_SIGNAL_FLG		FLASH_READ_FIRE_ADDR->FireSignalFlag
#define REC_FLS_FIRE_TIME			FLASH_READ_FIRE_ADDR->FireTime
#define REC_FLS_OFF_MOS_FLG			FLASH_READ_FIRE_ADDR->MosFlag
#define REC_FLS_OFF_MOS_TIME		FLASH_READ_FIRE_ADDR->OffMosTime
#define REC_FLS_SAVE_FLG			FLASH_READ_FIRE_ADDR->SaveFlag
#define REC_FLS_TIME_INX			FLASH_READ_FIRE_ADDR->TimeInx
#define REC_FLS_TRIGFLG				FLASH_READ_FIRE_ADDR->TrigFlag
#define REC_PRINT_STEP  			RecCtrlPkg.RecPrintStep
#define REC_PRINT_SUB  				RecCtrlPkg.RecPrintSubStep
#define REC_PRINT_LEN  				RecCtrlPkg.RecPrintLen
#define REC_PRINT_MAX  				RecCtrlPkg.RecPrintMax

#define REC_UNLOCK_START_FILTE		RecCtrlPkg.RecUnLockFilte
#define REC_UNLOCK_ON  				RecCtrlPkg.RecUnLockOn
#define REC_UNLOCK_FILTE_INX  		RecCtrlPkg.RecUnLockFilteInx
#define REC_UNLOCK_EXIT_CNT  		RecCtrlPkg.RecUnLockExitCnt



#define REC_FOUND_LOCK_TIME_MAX    		(450)/*20241018update*/

#define REC_FLASH_16BYTE_OFFSET_MASK   	(2)
#define REC_FLASH_LENBYTE_OFFSET_MASK   (8)






typedef UNS8 (*FUNCTION_PTR)(void);


typedef enum
{
	REC_SAVE_STEP_INIT = 0,
	REC_SAVE_STEP_WAIT,
	REC_SAVE_STEP_ECC_CHECK,
	REC_SAVE_STEP_DATA_CRC,
	REC_SAVE_STEP_DATA_WRITE,
	REC_SAVE_STEP_DATA_CHECK,
	REC_SAVE_STEP_FIRE_CRC,
	REC_SAVE_STEP_FIRE_WRITE,
	REC_SAVE_STEP_FIRE_CHECK,
	REC_SAVE_STEP_LOCK,
	REC_SAVE_STEP_MAX,
} REC_SAVE_STEP_E;


UNS8 REC_SaveStepInit(void);
UNS8 REC_SaveStepFireCrc(void);
UNS8 REC_SaveStepFireWrite(void);
UNS8 REC_SaveStepFireCheck(void);
UNS8 REC_SaveStepDataCrc(void);
UNS8 REC_SaveStepDataWrite(void);
UNS8 REC_SaveStepDataCheck(void);
UNS8 REC_SaveStepLock(void);
UNS8 REC_SaveStepWait(void);
UNS8 REC_SaveStepEccCheck(void);


static const FUNCTION_PTR RecExecRtnTable[REC_SAVE_STEP_MAX] ={
		REC_SaveStepInit,
		REC_SaveStepWait,
		REC_SaveStepEccCheck,
		REC_SaveStepDataCrc,
		REC_SaveStepDataWrite,
		REC_SaveStepDataCheck,
		REC_SaveStepFireCrc,
		REC_SaveStepFireWrite,
		REC_SaveStepFireCheck,
		REC_SaveStepLock,
};

/*****************************************************************************
 *	REC_UserCheckSum:
 *
 *	PARAMETERS: Address and length of data to be verified
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Obtain validation value.
 *
 *	RETURNS: 32bit Verification code
 *
 *****************************************************************************
 */

UNS32 REC_UserCheckSum(UNS32 *Data,UNS16 len)
{
	UNS32 checkSum = 0;
	UNS16 Inx = 0;
	UNS16 Lenmax = len/4;
	while(Inx < Lenmax){
		checkSum += Data[Inx++];
	}
	return checkSum;
}


/*****************************************************************************
 *	REC_UserCheckEcc:
 *
 *	PARAMETERS: Address and length of data to be verified
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Flash data ecc detection.
 *
 *	RETURNS: UNS8  TRUE  or  FALSE
 *
 *****************************************************************************
 */
UNS8 REC_UserCheckEcc(UNS32* addr,UNS32 len)
{
	UNS8 err = FALSE;
	UNS32 lenmax = len/4;
	for(UNS32 *ad = addr;ad < (addr + lenmax);ad++){
		if(*ad != REC_FLASH_NULL_ECC){
			err = TRUE;
			break;
		}
	}

	return err;
}



/*****************************************************************************
 *	REC_SaveStepEccCheck:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Import flash data to RAM.
 *
 *	RETURNS: UNS8  TRUE  or  FALSE
 *
 *****************************************************************************
 */

UNS8 REC_SaveStepInit(void)
{
#ifndef	REC_ON_FLASH_SAVE_TEST_ENABLE
	if((REC_FLS_FIRE_SIGNAL_FLG == REC_FIRE_TYPE_ON)
			|| ((REC_FLS_FIRE_CHECKSUM != REC_FLASH_NULL_ECC)
			&& (REC_FLS_DATA_CHECKSUM != REC_FLASH_NULL_ECC)
			)){
		UNS32 FireCRC = REC_UserCheckSum((UNS32 *)FLASH_READ_FIRE_ADDR + REC_FLASH_16BYTE_OFFSET_MASK,
				sizeof(REC_DAT_T) - REC_FLASH_LENBYTE_OFFSET_MASK);
		UNS32 DataCRC = REC_UserCheckSum((UNS32 *)FLASH_READ_LOCK_DATA_ADDR,sizeof(REC_ACC_LOCK));

		if((FireCRC == REC_FLS_FIRE_CHECKSUM) && (DataCRC == REC_FLS_DATA_CHECKSUM )){
			for(int i = 0 ; i< REC_ACCDATA_LOCK_LEN_MAX ;i++){
				REC_FIRE_DATA = *FLASH_READ_FIRE_ADDR;
				REC_ACC_LOCK[i].Dat = FLASH_READ_LOCK_DATA_ADDR[i].Dat;
				REC_ACC_LOCK[i].State.Dword = FLASH_READ_LOCK_DATA_ADDR[i].State.Dword;
			}

		}
		REC_FIRE_DATA.FireSignalFlag = REC_FIRE_TYPE_ON;
		REC_ROUND_STEP = REC_SAVE_STEP_LOCK;
	}else{
		if(REC_UserCheckEcc((UNS32*)REC_FIRE_FLASH_ADDR,REC_FLASH_MAX)){
			MX_FlashErasePage(REC_FIRE_FLASH_ADDR);
		}
		REC_ROUND_STEP = REC_SAVE_STEP_WAIT;
	}

#else
	REC_ROUND_STEP = REC_SAVE_STEP_WAIT;
#endif
	return TRUE;
}


/*****************************************************************************
 *	REC_SaveStepEccCheck:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Detecting ignition data.
 *
 *	RETURNS: UNS8  TRUE  or  FALSE
 *
 *****************************************************************************
 */
UNS8 REC_SaveStepEccCheck(void)
{
	if((REC_FIRE_DATA.FireSignalFlag == REC_FIRE_TYPE_ON)&&(REC_FIRE_SAVE_FLG == TRUE)){
		if(REC_UserCheckEcc((UNS32*)REC_FIRE_FLASH_ADDR,REC_FLASH_MAX)){
			MX_FlashErasePage(REC_FIRE_FLASH_ADDR);
		}
		REC_ROUND_STEP = REC_SAVE_STEP_DATA_CRC;
	}

	return TRUE;
}

/*****************************************************************************
 *	REC_SaveStepDataCrc:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Detecting ignition data.
 *
 *	RETURNS: UNS8  TRUE  or  FALSE
 *
 *****************************************************************************
 */
UNS8 REC_SaveStepDataCrc(void)
{
	if((REC_FIRE_DATA.FireSignalFlag == REC_FIRE_TYPE_ON)&&(REC_FIRE_SAVE_FLG == TRUE)){
		REC_CHECKSUM_DATA_FLG = REC_UserCheckSum((UNS32 *)REC_ACC_LOCK,sizeof(REC_ACC_LOCK));
		REC_ROUND_STEP = REC_SAVE_STEP_DATA_WRITE;
	}

	return TRUE;
}


/*****************************************************************************
 *	REC_SaveStepDataWrite:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Write ACC data to Flash.
 *
 *	RETURNS: UNS8  TRUE  or  FALSE
 *
 *****************************************************************************
 */
UNS8 REC_SaveStepDataWrite(void)
{
	if((REC_FIRE_DATA.FireSignalFlag == REC_FIRE_TYPE_ON)&&(REC_FIRE_SAVE_FLG == TRUE)){
		MX_FlashProgram(REC_LOCK_FLASH_ADDR,(uint32_t *)REC_ACC_LOCK,sizeof(REC_ACC_LOCK));
		REC_ROUND_STEP = REC_SAVE_STEP_DATA_CHECK;
	}
	return TRUE;
}

/*****************************************************************************
 *	REC_SaveStepDataCheck:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Obtain ignition data and perform testing.
 *
 *	RETURNS: UNS8  TRUE  or  FALSE
 *
 *****************************************************************************
 */
UNS8 REC_SaveStepDataCheck(void)
{
	UNS32 crc;
	if((REC_FIRE_DATA.FireSignalFlag == REC_FIRE_TYPE_ON)&&(REC_FIRE_SAVE_FLG == TRUE)){
		crc = REC_UserCheckSum((UNS32 *)FLASH_READ_LOCK_DATA_ADDR,sizeof(REC_ACC_LOCK));
		if(crc == REC_CHECKSUM_DATA_FLG){
			REC_ROUND_STEP = REC_SAVE_STEP_FIRE_CRC;
		}else{
			REC_ROUND_STEP = REC_SAVE_STEP_ECC_CHECK;
		}
	}
	return TRUE;
}


/*****************************************************************************
 *	REC_SaveStepFireCrc:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Test ignition data validation.
 *
 *	RETURNS: UNS8  TRUE  or  FALSE
 *
 *****************************************************************************
 */
UNS8 REC_SaveStepFireCrc(void)
{
	if((REC_FIRE_DATA.FireSignalFlag == REC_FIRE_TYPE_ON)&&(REC_FIRE_SAVE_FLG == TRUE)){
		REC_CHECKSUM_FIRE_FLG = REC_UserCheckSum((UNS32 *)&REC_FIRE_DATA+REC_FLASH_16BYTE_OFFSET_MASK,
				sizeof(REC_DAT_T)-REC_FLASH_LENBYTE_OFFSET_MASK);
		REC_ROUND_STEP = REC_SAVE_STEP_FIRE_WRITE;
	}
	return TRUE;
}

/*****************************************************************************
 *	REC_SaveStepFireWrite:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Write ignition information to flash.
 *
 *	RETURNS: UNS8  TRUE  or  FALSE
 *
 *****************************************************************************
 */

UNS8 REC_SaveStepFireWrite(void)
{
	if((REC_FIRE_DATA.FireSignalFlag == REC_FIRE_TYPE_ON)&&(REC_FIRE_SAVE_FLG == TRUE)){
		MX_FlashProgram(REC_FIRE_FLASH_ADDR,(uint32_t *)&REC_FIRE_DATA,sizeof(REC_FIRE_DATA));/*0.125*/
		REC_ROUND_STEP = REC_SAVE_STEP_FIRE_CHECK;
	}
	return TRUE;
}


/*****************************************************************************
 *	REC_SaveStepLock:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Locked state after ignition, no action.
 *
 *	RETURNS: UNS8  TRUE  or  FALSE
 *
 *****************************************************************************
 */
UNS8 REC_SaveStepFireCheck(void)
{
	UNS32 crc;
	if((REC_FIRE_DATA.FireSignalFlag == REC_FIRE_TYPE_ON)&&(REC_FIRE_SAVE_FLG == TRUE)){
		crc = REC_UserCheckSum((UNS32 *)REC_FIRE_FLASH_ADDR+REC_FLASH_16BYTE_OFFSET_MASK,
				sizeof(REC_FIRE_DATA)-REC_FLASH_LENBYTE_OFFSET_MASK);
		if(crc == REC_CHECKSUM_FIRE_FLG){
			REC_ROUND_STEP = REC_SAVE_STEP_LOCK;
		}else{
			REC_ROUND_STEP = REC_SAVE_STEP_ECC_CHECK;
			MX_FlashErasePage(REC_FIRE_FLASH_ADDR);
		}
	}
	return TRUE;
}



/*****************************************************************************
 *	REC_SaveStepLock:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Locked state after ignition, no action.
 *
 *	RETURNS: UNS8  TRUE  or  FALSE
 *
 *****************************************************************************
 */
UNS8 REC_SaveStepLock(void)
{
	return TRUE;
}


/*****************************************************************************
 *	REC_AlgoOnTimeRec:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Record the time of each algorithm startup.
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */

extern S16 AccData_Z;
void REC_AlgoOnTimeRec(void)
{
	if(FRONT_ALGO_ON != REC_ALGO_ON_TIME){
		REC_ALGO_ON_TIME = FRONT_ALGO_ON;
		if((REC_FIRE_SIGNAL_FLG != REC_FIRE_TYPE_ON)&&(REC_ALGO_ON_TIME == TRUE)){
			REC_FIRE_TIME_INX++;
			if(REC_FIRE_TIME_INX >= REC_ALGORITHM_START_TIME_MAX){
				REC_FIRE_TIME_INX = 0;
			}
			REC_FIRE_TIME_GROUP[REC_FIRE_TIME_INX] = cTASK_GetSysTimer();
		}
	}
}

/*****************************************************************************
 *	REC_GetTimeDifference:
 *
 *	PARAMETERS: 32bit Time
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Calculate the difference.
 *
 *	RETURNS: 32bit Time
 *
 *****************************************************************************
 */
UNS32 REC_GetTimeDifference(UNS32 time)
{
	UNS32 rettime = 0,currenttime = cTASK_GetSysTimer();
	if(time < currenttime){
		rettime = currenttime - time;
	}else if(time > currenttime){
		rettime = currenttime + (0XFFFFFFFF - time);
	}
	return rettime;
}


/*****************************************************************************
 *	REC_SaveStepWait:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Polling data and moving data.
 *
 *	RETURNS: UNS8  TRUE  or  FALSE
 *
 *****************************************************************************
 */
UNS8 REC_SaveStepWait(void)
{
	REC_AlgoOnTimeRec();

#if 1

	if(FRONT_ALGO_ON == TRUE){
		REC_ALGO_ON_DAT = TRUE;

	}
	if(REC_UNLOCK_ON == FALSE){
		REC_UNLOCK_START_FILTE[REC_UNLOCK_FILTE_INX++] = CAW.AccData.X;
		REC_UNLOCK_FILTE_INX &= REC_ACCDATA_UNLOCK_FILTE_MASK;
		S16 AccFilte = (REC_UNLOCK_START_FILTE[0]+REC_UNLOCK_START_FILTE[1]) >> 1;
		if(AccFilte >= REC_ACCDATA_UNLOCK_START_THRESHOLD){
			REC_UNLOCK_ON = TRUE;
			REC_UNLOCK_EXIT_CNT = 0;
			if(REC_UNLOCK_BLOCK_FLG == FALSE){
				REC_UNLOCK_BLOCK_FLG = TRUE;
				REC_UNLOCK_REM_FLG = TRUE;
			}
		}
	}else{
		REC_UNLOCK_EXIT_CNT++;
		if(REC_UNLOCK_EXIT_CNT >= REC_ACCDATA_UNLOCK_EXIT_MAX){
			REC_UNLOCK_EXIT_CNT = REC_ACCDATA_UNLOCK_EXIT_MAX;
			REC_UNLOCK_ON = FALSE;
		}

	}
#else
	if(FRONT_ALGO_ON == TRUE){
		REC_ALGO_ON_DAT = TRUE;
		if(REC_UNLOCK_BLOCK_FLG == FALSE){
			REC_UNLOCK_BLOCK_FLG = TRUE;
			REC_UNLOCK_REM_FLG = TRUE;
		}
	}
#endif



//#ifdef		REC_UNLOCK_RAM_TEST_ENABLE
#if		1
	if(REC_FIRE_SIGNAL_FLG == REC_FIRE_TYPE_ON){
#else
	if((REC_ALGO_ON_DAT == TRUE) || (REC_FIRE_SIGNAL_FLG == REC_FIRE_TYPE_ON)){
#endif
		if((REC_FIRE_TRIG_FLG == FALSE) && (REC_FIRE_SIGNAL_FLG == REC_FIRE_TYPE_ON)){
			REC_FIRE_TRIG_FLG = TRUE;
			REC_FIRE_TIME_FLG = cTASK_GetSysTimer();
			REC_ROUND_TIME = 0;//REC_GetTimeDifference(REC_FIRE_TIME_GROUP[REC_FIRE_TIME_INX]);
		}
		REC_ROUND_TIME++;
		UNS16 rouinx = REC_ROUND_INX;
		if(REC_ROUND_TIME >= REC_FOUND_LOCK_TIME_MAX){
			UNS16 Inx = (REC_ACCDATA_LOCK_LEN_MAX - 1);
			while(Inx > 0){
				REC_ACC_LOCK[Inx].Dat  = REC_ACC_RAM[rouinx].Dat;
				REC_ACC_LOCK[Inx].State.Dword   = REC_ACC_RAM[rouinx].State.Dword;
				if(rouinx == 0 ){rouinx  = REC_ACCDATA_ROUND_LEN_MAX - 1;
				}else{rouinx--;}
				Inx--;
			}

			if(REC_FIRE_SIGNAL_FLG == REC_FIRE_TYPE_ON){
#ifndef	EDR_ON_FLASH_SAVE_TEST_ENABLE
				if(REC_FLS_FIRE_SIGNAL_FLG != REC_FIRE_TYPE_ON){
				REC_ROUND_STEP = REC_SAVE_STEP_ECC_CHECK;
				REC_FIRE_SAVE_FLG = TRUE;
				}
#else
				REC_ROUND_STEP = REC_SAVE_STEP_LOCK;
#endif
			}else{
				if(FRONT_ALGO_ON == TRUE){
//					REC_ROUND_TIME = REC_GetTimeDifference(REC_FIRE_TIME_GROUP[REC_FIRE_TIME_INX]);
					REC_ROUND_TIME = 0;
				}else{
					REC_ALGO_ON_DAT = FALSE;
					REC_ROUND_TIME = 0;
				}
			}
		}
	}
#ifndef		REC_UNLOCK_RAM_TEST_ENABLE
		//if((REC_UNLOCK_BLOCK_FLG == TRUE)&&(REC_FIRE_SIGNAL_FLG != REC_FIRE_TYPE_ON)){
		if(REC_UNLOCK_BLOCK_FLG == TRUE){/*20240913更新*/
			if(REC_UNLOCK_REM_FLG == TRUE){
				UNS16 Inx = REC_ACCDATA_UNLOCK_REM_LEN_MAX;
				UNS16 rouinx = REC_ROUND_INX;
				do{
					Inx--;
					REC_AccDataUnlockRamBuff[REC_UNLOCK_BLOCK_INX][Inx].Dat  		= REC_ACC_RAM[rouinx].Dat;
					REC_AccDataUnlockRamBuff[REC_UNLOCK_BLOCK_INX][Inx].State.Dword = REC_ACC_RAM[rouinx].State.Dword;
					if(rouinx == 0 ){rouinx  = REC_ACCDATA_ROUND_LEN_MAX - 1;
					}else{rouinx--;}

				}while(Inx > 0);
				REC_UNLOCK_REM_FLG = FALSE;
				REC_UNLOCK_ARRAY_INX = REC_ACCDATA_UNLOCK_REM_LEN_MAX - 1;
			}else{
				REC_AccDataUnlockRamBuff[REC_UNLOCK_BLOCK_INX][REC_UNLOCK_ARRAY_INX].Dat = REC_ACC_RAM[REC_ROUND_INX].Dat;
				REC_AccDataUnlockRamBuff[REC_UNLOCK_BLOCK_INX][REC_UNLOCK_ARRAY_INX].State = REC_ACC_RAM[REC_ROUND_INX].State;
			}
			REC_UNLOCK_ARRAY_INX++;
			if(REC_UNLOCK_ARRAY_INX >= REC_ACCDATA_UNLOCK_LEN_MAX){
				REC_UNLOCK_ARRAY_INX = 0;

#ifndef     	REC_UNLOCK_CONTINUOUS_SAVE_TEST_ENABLE
				REC_UNLOCK_BLOCK_FLG = FALSE;
#else
				//if(FRONT_ALGO_ON != TRUE){
				if(REC_UNLOCK_ON != TRUE){
				REC_UNLOCK_BLOCK_FLG = FALSE;
				}
#endif
				REC_UNLOCK_BLOCK_INX++;
				if(REC_UNLOCK_BLOCK_INX >= REC_ALGORITHM_START_TIME_MAX){
					REC_UNLOCK_BLOCK_INX = 0;
				}
			}
		}
#endif
	return TRUE;
}


/*****************************************************************************
 *	REC_GetFireOnFlg:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Obtain the ignition flag.
 *
 *	RETURNS:  UNS8  TRUE  or  FALSE
 *
 *****************************************************************************
 */

UNS8 REC_GetFireOnFlg(void)
{
	UNS8 ret = FALSE;
	if(REC_FIRE_DATA.FireSignalFlag == REC_FIRE_TYPE_ON){
		ret = TRUE;
	}
	return ret;
}


/*****************************************************************************
 *	REC_ClrFlashRam:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Clear flash RAM and various tags.
 *
 *	RETURNS: UNS8  TRUE  or  FALSE
 *
 *****************************************************************************
 */
UNS8 REC_ClrFlashRam(void)
{
	UNS8 ret = FALSE;
	if(REC_CLR_STEP == 0){
		(void)memset(&REC_AccDataLockRamBuff, 0, sizeof(REC_AccDataLockRamBuff));
		REC_CLR_STEP++;
	}else if(REC_CLR_STEP == 1){
		MX_FlashErasePage(REC_FIRE_FLASH_ADDR);
		DEBUG_EDRfireFlagClr();
		DPL_AlgorithmStartup();
		REC_CLR_STEP++;
	}else if(REC_CLR_STEP == 2){
		(void)memset(&REC_AccDataUnlockRamBuff[REC_PRINT_MAX], 0, sizeof(REC_AccDataUnlockRamBuff[0]));
		REC_PRINT_MAX++;
		if(REC_PRINT_MAX >= REC_ALGORITHM_START_TIME_MAX){
			REC_PRINT_MAX = 0;
			REC_CLR_STEP++;
		}
	}else if(REC_CLR_STEP == 3){
		(void)memset(&RecCtrlPkg, 0, sizeof(REC_CTRL_PACKAGE_T));
		ret = TRUE;
		BSP_ProgramEnableCall();
		debug("Erased successfully!  \r\n");
	}else{
		ret = TRUE;
	}
	return ret;
}


/*****************************************************************************
 *	REC_RamInit:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Record process initialization program.
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */

void REC_RamInit(void)
{
	(void)memset(&RecCtrlPkg, 0, sizeof(RecCtrlPkg));
	(void)memset(&REC_ACC_LOCK, 0, sizeof(REC_ACC_LOCK));
	(void)memset(&REC_ACC_RAM, 0, sizeof(REC_ACC_RAM));

}


/*****************************************************************************
 *	REC_Routine:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Record the main running process.
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void REC_Routine(void)
{
	REC_ACC_RAM[REC_ROUND_INX].Dat.X = CAW.AccData.X;
	REC_ACC_RAM[REC_ROUND_INX].Dat.Y = CAW.AccData.Y;
	REC_ACC_RAM[REC_ROUND_INX].Dat.Z = CAW.AccData.Z;
	REC_ACC_RAM[REC_ROUND_INX].State.Bit.Int =  (REC_STATE_BIT1_MASK & LL_GPIO_IsInputPinSet(GPIOB,LL_GPIO_PIN_9));
	REC_ACC_RAM[REC_ROUND_INX].State.Bit.Ready = (REC_STATE_BIT1_MASK & AdKey_GetDeviceStatus());
	REC_ACC_RAM[REC_ROUND_INX].State.Bit.Algo =  (REC_STATE_BIT1_MASK & FRONT_ALGO_ON);
	REC_ACC_RAM[REC_ROUND_INX].State.Bit.AirCheck = (REC_STATE_BIT3_MASK &DIAG_GetAirBagStep());
	REC_ACC_RAM[REC_ROUND_INX].State.Bit.UI = (REC_STATE_BIT3_MASK &Disp_GetModuleStatus());
	REC_ACC_RAM[REC_ROUND_INX].State.Bit.Timer = (REC_STATE_TIME_MASK & cTASK_GetSysTimer());

	if(REC_ROUND_STEP <REC_SAVE_STEP_MAX){
	RecExecRtnTable[REC_ROUND_STEP]();
	}
	REC_ROUND_INX++;
	if(REC_ROUND_INX >= REC_ACCDATA_ROUND_LEN_MAX){
		REC_ROUND_INX = 0;
	}
}

/*****************************************************************************
 *	REC_SignalFireRecord:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Ignition mark.
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void REC_SignalFireRecord(void)
{
	if(REC_FIRE_SIGNAL_FLG != REC_FIRE_TYPE_ON){
		REC_FIRE_SIGNAL_FLG = REC_FIRE_TYPE_ON;
	}

}

/*****************************************************************************
 *	REC_SignalOffMosRecord:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Record the time of closing the record.
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void REC_SignalOffMosRecord(void)
{
	if((REC_FIRE_MOS_FLG == FALSE)&&(REC_FIRE_SIGNAL_FLG == REC_FIRE_TYPE_ON)){
		REC_FIRE_MOS_FLG = TRUE;
		REC_FIRE_OFFMOS_TIME = cTASK_GetSysTimer();
	}
}

/*****************************************************************************
 *	REC_ReadRecRamClr:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: erase status.
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void REC_ReadRecRamClr(void)
{
	REC_PRINT_STEP = 0;
	REC_PRINT_SUB  = 0;
	REC_PRINT_LEN  = 0;
	REC_PRINT_MAX  = 0;

}


/*****************************************************************************
 *	REC_ReadRecRam:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Polling to print RAM records.
 *
 *	RETURNS: UNS8  TRUE  or  FALSE
 *
 *****************************************************************************
 */
UNS8 REC_ReadRecRam(void)
{

	if(REC_PRINT_STEP == REC_STEP_PRINTF_0){
		DEBUG_PrintfID();
		REC_PRINT_STEP++;
		REC_PRINT_LEN = 0;
	}else if(REC_PRINT_STEP == REC_STEP_PRINTF_1){

		debug("Rec ram \r\n");
		debug("Fire state: %d \r\n",(REC_FIRE_SIGNAL_FLG == REC_FIRE_TYPE_ON)?1:0);
		debug("Fire time: %8d%8d%8d \r\n",(int)(REC_FIRE_TIME_FLG - REC_FIRE_TIME_GROUP[REC_FIRE_TIME_INX]),
				(int)REC_FIRE_TIME_FLG,
				(int)REC_FIRE_TIME_FLG& REC_STATE_TIME_MASK);
		debug("Off mos time: %8d%8d \r\n",(int)REC_FIRE_OFFMOS_TIME,(int)REC_FIRE_OFFMOS_TIME& REC_STATE_TIME_MASK);
		debug("Alog on end: %d \r\n",REC_FIRE_TIME_INX );

		REC_PRINT_STEP++;
		REC_PRINT_LEN = 0;
	}else if(REC_PRINT_STEP == REC_STEP_PRINTF_2){
		debug("Alog on %d: %8d%8d\r\n",(int)REC_PRINT_LEN,
				(int)REC_FIRE_TIME_GROUP[REC_PRINT_LEN],
				(int)REC_FIRE_TIME_GROUP[REC_PRINT_LEN]& REC_STATE_TIME_MASK);
		REC_PRINT_LEN++;
		if(REC_PRINT_LEN >=REC_ALGORITHM_START_TIME_MAX){
			REC_PRINT_LEN = 1;
			REC_PRINT_STEP++;
			debug("index    x      y      z      time int ready algo air ui\r\n");
		}
	}else if(REC_PRINT_STEP == REC_STEP_PRINTF_3){
		if(REC_PRINT_LEN < REC_ACCDATA_LOCK_LEN_MAX){
			debug("%4d%8d%8d%8d%8d%2d%2d%2d%4d%4d \r\n",
			REC_PRINT_LEN,
			(int)REC_ACC_LOCK[REC_PRINT_LEN].Dat.X,
			(int)REC_ACC_LOCK[REC_PRINT_LEN].Dat.Y,
			(int)REC_ACC_LOCK[REC_PRINT_LEN].Dat.Z,
			(int)REC_ACC_LOCK[REC_PRINT_LEN].State.Bit.Timer,
			(int)REC_ACC_LOCK[REC_PRINT_LEN].State.Bit.Int,
			(int)REC_ACC_LOCK[REC_PRINT_LEN].State.Bit.Ready,
			(int)REC_ACC_LOCK[REC_PRINT_LEN].State.Bit.Algo,
			(int)REC_ACC_LOCK[REC_PRINT_LEN].State.Bit.AirCheck,
			(int)REC_ACC_LOCK[REC_PRINT_LEN].State.Bit.UI
		);


			REC_PRINT_LEN++;
		}else{
			REC_PRINT_STEP++;
		}
	}else if(REC_PRINT_STEP == REC_STEP_PRINTF_4){
		debug("-------------------\r\n");
		return TRUE;
	}
	return FALSE;
}



/*****************************************************************************
 *	REC_ReadRecFlash:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Polling to print FLASH records.
 *
 *	RETURNS: UNS8  TRUE  or  FALSE
 *
 *****************************************************************************
 */
UNS8 REC_ReadRecFlash(void)
{
	if(REC_PRINT_STEP == REC_STEP_PRINTF_0){
		DEBUG_PrintfID();
		REC_PRINT_STEP++;
		REC_PRINT_LEN = 0;
	}else if(REC_PRINT_STEP == REC_STEP_PRINTF_1){
		debug("Rec flash \r\n");
		debug("Fire state: %d \r\n",(REC_FLS_FIRE_SIGNAL_FLG == REC_FIRE_TYPE_ON)?1:0);
		debug("Fire time: %8d%8d%8d \r\n",(int)(REC_FLS_FIRE_TIME - REC_FLS_ALOG_TIME[REC_FLS_TIME_INX]),
				(int)REC_FLS_FIRE_TIME,
				(int)REC_FLS_FIRE_TIME& REC_STATE_TIME_MASK);
		debug("Off mos time: %8d%8d \r\n",(int)REC_FLS_OFF_MOS_TIME,(int)REC_FLS_OFF_MOS_TIME& REC_STATE_TIME_MASK);
		debug("Alog on end: %d \r\n",(REC_FLS_TIME_INX)>10?-1:REC_FLS_TIME_INX);
		REC_PRINT_STEP++;
		REC_PRINT_LEN = 0;
	}else if(REC_PRINT_STEP == REC_STEP_PRINTF_2){
		debug("Alog on %d: %8d%8d\r\n",(int)REC_PRINT_LEN,
				(int)REC_FLS_ALOG_TIME[REC_PRINT_LEN],
				(int)REC_FLS_ALOG_TIME[REC_PRINT_LEN]& REC_STATE_TIME_MASK );
		REC_PRINT_LEN++;
		if(REC_PRINT_LEN >=REC_ALGORITHM_START_TIME_MAX){
			REC_PRINT_LEN = 1;
			REC_PRINT_STEP++;
			debug("index  x  y  z  time  int  ready  algo  air  ui  \r\n");
		}
	}else if(REC_PRINT_STEP == REC_STEP_PRINTF_3){
		if(REC_PRINT_LEN < REC_ACCDATA_LOCK_LEN_MAX){
			debug("%4d%8d%8d%8d%8d%2d%2d%2d%4d%4d \r\n",
			REC_PRINT_LEN,
			(int)FLASH_READ_LOCK_DATA_ADDR[REC_PRINT_LEN].Dat.X,
			(int)FLASH_READ_LOCK_DATA_ADDR[REC_PRINT_LEN].Dat.Y,
			(int)FLASH_READ_LOCK_DATA_ADDR[REC_PRINT_LEN].Dat.Z,
			(int)FLASH_READ_LOCK_DATA_ADDR[REC_PRINT_LEN].State.Bit.Timer,
			(int)FLASH_READ_LOCK_DATA_ADDR[REC_PRINT_LEN].State.Bit.Int,
			(int)FLASH_READ_LOCK_DATA_ADDR[REC_PRINT_LEN].State.Bit.Ready,
			(int)FLASH_READ_LOCK_DATA_ADDR[REC_PRINT_LEN].State.Bit.Algo,
			(int)FLASH_READ_LOCK_DATA_ADDR[REC_PRINT_LEN].State.Bit.AirCheck,
			(int)FLASH_READ_LOCK_DATA_ADDR[REC_PRINT_LEN].State.Bit.UI
			);
			REC_PRINT_LEN++;
		}else{
			REC_PRINT_STEP++;
		}
	}else if(REC_PRINT_STEP == REC_STEP_PRINTF_4){
		debug("-------------------\r\n");
		return TRUE;
	}
	return FALSE;
}


/*****************************************************************************
 *	REC_ReadRecUnLockRam:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Non locking event printing.
 *
 *	RETURNS: UNS8  TRUE  or  FALSE
 *
 *****************************************************************************
 */

UNS8 REC_ReadRecUnLockRam(UNS8 block)
{
	if(REC_PRINT_STEP == REC_STEP_PRINTF_0){
		DEBUG_PrintfID();
		REC_PRINT_STEP++;
		REC_PRINT_LEN = 0;
	}else if(REC_PRINT_STEP == REC_STEP_PRINTF_1){
		if(block == REC_ALGORITHM_START_TIME_MAX){
			REC_PRINT_STEP  = REC_STEP_PRINTF_4;
			REC_PRINT_LEN = 0;
			debug("block  x  y  z  time  int  ready  algo  air  ui  \r\n");
		}else if(block == (REC_ALGORITHM_START_TIME_MAX + 1)){
			debug("-------------------\r\n");
			debug("Rec unlock block: %d \r\n",REC_PRINT_MAX);
			debug("index  x  y  z  time  int  ready  algo  air  ui  \r\n");
			REC_PRINT_MAX  = 0;
			REC_PRINT_LEN = 0;
			REC_PRINT_STEP  = REC_STEP_PRINTF_5;
		}else if(block > (REC_ALGORITHM_START_TIME_MAX+1)){
			debug("Rec unlock block: err!\r\n");
			return TRUE;
		}else{
			REC_PRINT_STEP++;
			debug("-------------------\r\n");
			debug("Rec unlock block: %d \r\n",block);
			debug("index  x  y  z  time  int  ready  algo  air  ui  \r\n");
			REC_PRINT_LEN = 0;
		}
	}else if(REC_PRINT_STEP == REC_STEP_PRINTF_2){
		if(REC_PRINT_LEN < REC_ACCDATA_UNLOCK_LEN_MAX){
			debug("%4d%8d%8d%8d%8d%2d%2d%2d%4d%4d \r\n",
			REC_PRINT_LEN,
			(int)REC_ACC_UNLOCK[block][REC_PRINT_LEN].Dat.X,
			(int)REC_ACC_UNLOCK[block][REC_PRINT_LEN].Dat.Y,
			(int)REC_ACC_UNLOCK[block][REC_PRINT_LEN].Dat.Z,
			(int)REC_ACC_UNLOCK[block][REC_PRINT_LEN].State.Bit.Timer,
			(int)REC_ACC_UNLOCK[block][REC_PRINT_LEN].State.Bit.Int,
			(int)REC_ACC_UNLOCK[block][REC_PRINT_LEN].State.Bit.Ready,
			(int)REC_ACC_UNLOCK[block][REC_PRINT_LEN].State.Bit.Algo,
			(int)REC_ACC_UNLOCK[block][REC_PRINT_LEN].State.Bit.AirCheck,
			(int)REC_ACC_UNLOCK[block][REC_PRINT_LEN].State.Bit.UI
			);
			REC_PRINT_LEN++;
		}else{
			REC_PRINT_STEP++;
		}
	}else if(REC_PRINT_STEP == REC_STEP_PRINTF_3){
		debug("-------------------\r\n");
		return TRUE;
	}else if(REC_PRINT_STEP == REC_STEP_PRINTF_4){
		if(REC_PRINT_LEN < REC_ALGORITHM_START_TIME_MAX){
			debug("%4d%8d%8d%8d%8d%2d%2d%2d%4d%4d \r\n",
			REC_PRINT_LEN,
			(int)REC_ACC_UNLOCK[REC_PRINT_LEN][0].Dat.X,
			(int)REC_ACC_UNLOCK[REC_PRINT_LEN][0].Dat.Y,
			(int)REC_ACC_UNLOCK[REC_PRINT_LEN][0].Dat.Z,
			(int)REC_ACC_UNLOCK[REC_PRINT_LEN][0].State.Bit.Timer,
			(int)REC_ACC_UNLOCK[REC_PRINT_LEN][0].State.Bit.Int,
			(int)REC_ACC_UNLOCK[REC_PRINT_LEN][0].State.Bit.Ready,
			(int)REC_ACC_UNLOCK[REC_PRINT_LEN][0].State.Bit.Algo,
			(int)REC_ACC_UNLOCK[REC_PRINT_LEN][0].State.Bit.AirCheck,
			(int)REC_ACC_UNLOCK[REC_PRINT_LEN][0].State.Bit.UI
			);
			REC_PRINT_LEN++;
		}else{
			REC_PRINT_STEP = REC_STEP_PRINTF_3;
		}
	}else if(REC_PRINT_STEP == REC_STEP_PRINTF_5){
		if(REC_PRINT_LEN < REC_ACCDATA_UNLOCK_LEN_MAX){
			debug("%4d%8d%8d%8d%8d%2d%2d%2d%4d%4d \r\n",
			REC_PRINT_LEN,
			(int)REC_ACC_UNLOCK[REC_PRINT_MAX][REC_PRINT_LEN].Dat.X,
			(int)REC_ACC_UNLOCK[REC_PRINT_MAX][REC_PRINT_LEN].Dat.Y,
			(int)REC_ACC_UNLOCK[REC_PRINT_MAX][REC_PRINT_LEN].Dat.Z,
			(int)REC_ACC_UNLOCK[REC_PRINT_MAX][REC_PRINT_LEN].State.Bit.Timer,
			(int)REC_ACC_UNLOCK[REC_PRINT_MAX][REC_PRINT_LEN].State.Bit.Int,
			(int)REC_ACC_UNLOCK[REC_PRINT_MAX][REC_PRINT_LEN].State.Bit.Ready,
			(int)REC_ACC_UNLOCK[REC_PRINT_MAX][REC_PRINT_LEN].State.Bit.Algo,
			(int)REC_ACC_UNLOCK[REC_PRINT_MAX][REC_PRINT_LEN].State.Bit.AirCheck,
			(int)REC_ACC_UNLOCK[REC_PRINT_MAX][REC_PRINT_LEN].State.Bit.UI
			);
			REC_PRINT_LEN++;
		}else{

			REC_PRINT_MAX++;
			if(REC_PRINT_MAX >= REC_ALGORITHM_START_TIME_MAX){
				REC_PRINT_MAX = 0;
				REC_PRINT_STEP = REC_STEP_PRINTF_3;
				return TRUE;
			}
			debug("-------------------\r\n");
			debug("Rec unlock block: %d \r\n",REC_PRINT_MAX);
			debug("index  x  y  z  time  int  ready  algo  air  ui  \r\n");
			REC_PRINT_LEN = 0;
		}

	}
	return FALSE;
}










