/*
 * temp.c
 *
 *  Created on: Mar 19, 2024
 *      Author: liuzhimin
 */

#include "main.h"


#define TEMP_VAL_MIN  		(-40)
#define TEMP_VAL_MAX  		(127)
#define TEMP_INX_MIN  		(0)
#define TEMP_INX_MAX 		(230)
#define TEMP_OFFSET  		(40)
#define TEMP_QUERIES_CNT  	(5)




typedef enum
{
	TEMP_MODE_INIT = 0,
	TEMP_MODE_NORMAL,
}TEMP_MODE_E;



/* Test_TEMP_CTRL_PACKAGE_T */
typedef struct
{
	TEMP_MODE_E  tempMode;
	S16 		tempData;
	S16 		tempOld;
	UNS16 		tempSampCnt;
	UNS16 		PauseTimes;
	UNS32 		tempSum;
	UNS16 		MaturityCnt;
	UNS8 		ErrorCnt;
	UNS8 		Reserved[1];

} TEMP_CTRL_PACKAGE_T;

static TEMP_CTRL_PACKAGE_T TempCtrlPkg;


#define TEMP_MASK_MAX   	16
#define TEMP_INIT_MAX   	20
#define TEMP_UPDATE_MAX   	10
#define TEMP_MODE	   		TempCtrlPkg.tempMode
#define TEMP_DATA	   		TempCtrlPkg.tempData
#define TEMP_UPDATE	   		TempCtrlPkg.tempOld
#define TEMP_SAMP_CNT  		TempCtrlPkg.PauseTimes
#define TEMP_SAMP_SUM  		TempCtrlPkg.tempSum
#define TEMP_UPDATE_CNT	    TempCtrlPkg.MaturityCnt




const UNS16 temp_table[]={
3954 	,
3951 	,
3946 	,
3939 	,
3930 	,
3920 	,
3909 	,
3896 	,
3883 	,
3869 	,
3854 	,
3838 	,
3822 	,
3805 	,
3788 	,
3770 	,
3752 	,
3733 	,
3714 	,
3694 	,
3674 	,
3653 	,
3632 	,
3610 	,
3588 	,
3565 	,
3541 	,
3516 	,
3491 	,
3465 	,
3438 	,
3411 	,
3382 	,
3353 	,
3322 	,
3291 	,
3259 	,
3226 	,
3192 	,
3157 	,
3121 	,
3085 	,
3047 	,
3009 	,
2970 	,
2930 	,
2889 	,
2848 	,
2806 	,
2764 	,
2723 	,
2678 	,
2634 	,
2590 	,
2545 	,
2500 	,
2455 	,
2410 	,
4092 	,
2319 	,
2274 	,
2228 	,
2183 	,
2137 	,
2092 	,
2048 	,
2003 	,
1959 	,
1914 	,
1871 	,
1828 	,
1785 	,
1742 	,
1700 	,
1659 	,
1618 	,
1578 	,
1538 	,
1499 	,
1461 	,
1423 	,
1386 	,
1349 	,
1313 	,
1278 	,
1244 	,
1210 	,
1177 	,
1144 	,
1113 	,
1082 	,
1051 	,
1022 	,
993 	,
965 	,
937 	,
910 	,
884 	,
859 	,
834 	,
810 	,
787 	,
764 	,
742 	,
720 	,
699 	,
679 	,
659 	,
640 	,
621 	,
603 	,
586 	,
569 	,
552 	,
536 	,
521 	,
506 	,
491 	,
477 	,
463 	,
450 	,
437 	,
425 	,
413 	,
401 	,
390 	,
379 	,
368 	,
358 	,
347 	,
338 	,
328 	,
319 	,
310 	,
302 	,
293 	,
285 	,
278 	,
270 	,
263 	,
256 	,
248 	,
242 	,
235 	,
229 	,
223 	,
217 	,
211 	,
205 	,
200 	,
195 	,
189 	,
184 	,
179 	,
175 	,
170 	,
165 	,
161 	,
157 	,
153 	,
149 	,
145 	,
141 	,
137 	,
134 	,
130 	,
127 	,
124 	,
120 	,
117 	,
114 	,
111 	,
108 	,
105 	,
103 	,
100 	,
97 	,
94 	,
92 	,
90 	,
87 	,
85 	,
83 	,
81 	,
78 	,
76 	,
74 	,
72 	,
70 	,
68 	,
67 	,
65 	,
63 	,
61 	,
60 	,
58 	,
57 	,
55 	,
52 	,
52 	,
51 	,
49 	,
48 	,
47 	,
45 	,
44 	,
43 	,
42 	,
41 	,
39 	,
38 	,
37 	,
36 	,
35 	,
34 	,
33 	,
32 	,
31 	,
30 	,
30 	,
29 	,
28 	,
27 	,
26 	,
26 	,
25 	,
24 	,
23 	,
22 	,
22 	,
21 	,
20 	,
20 	,
19 	,
19 	,
18 	,
18 	,
17 	,
16 	,
16 	,
16 	,
};

/*****************************************************************************
 *	TEMP_NtcFaultDetect:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: NTC Fault Detect
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 TEMP_NtcFaultDetect(void)
{
	uint16_t adc;
	/*Obtain ADC temperature value*/
	adc = ADC_GetChannelValue(ADC_BUF_NTC);
	TEMP_NTC_FAILURE_DIAG_T err = TEMP_NTC_NO_ERROR;
    if(adc >= TEMP_NTC_OPEN_VLALUE_MAX){
    	err = TEMP_NTC_NO_OPEN_ERROR;
	}else if(adc <  TEMP_NTC_OPEN_VLALUE_MIN){
    	err = TEMP_NTC_NO_SHORT_GND_ERROR;
	}
    return err;
}


/*****************************************************************************
 *	TEMP_ConvertAdc:
 *
 *	PARAMETERS: UNS16
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: ADC to temp
 *
 *	RETURNS: S16
 *
 *****************************************************************************
 */
S16 TEMP_ConvertAdc(UNS16 adc){
    UNS16 inx;
    UNS16  mid,min,max;
    /*Filter out anomalies*/
    if(adc > temp_table[TEMP_INX_MIN])      return (TEMP_VAL_MIN);
    if(adc < temp_table[TEMP_INX_MAX])      return (TEMP_VAL_MAX);

    min = TEMP_INX_MIN;
    max = TEMP_INX_MAX;
   /* Check the temperature gauge*/
    for(int qes = 0; qes < TEMP_QUERIES_CNT; qes++ ){
    	mid = min / 2 + max / 2;
        if(adc <= temp_table[mid]) min = mid;
        else  max = mid;
    }
    /*Secondary table lookup*/
    inx = min;
    if(adc == temp_table[min])  inx = min;
    else if(adc == temp_table[max])  inx = max;
    else{
    	/*Check the table again*/
        while(min <= max){
            min++;
            if(adc == temp_table[min])   {inx = min; break;}
            else if(adc > temp_table[min]){
                min--;
                inx = min;
                break;}
        }
    }
    return (inx - TEMP_OFFSET);
}

/*****************************************************************************
 *	TEMP_Sampfilte:
 *
 *	PARAMETERS: UNS16
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Temp data filte
 *
 *	RETURNS: UNS16
 *
 *****************************************************************************
 */
UNS16 TEMP_Sampfilte(UNS16 AdcVal)
{
	UNS16 temp = 0;
	TEMP_SAMP_SUM = TEMP_SAMP_SUM + AdcVal - (TEMP_SAMP_SUM / TEMP_MASK_MAX);
	if(TEMP_MODE == TEMP_MODE_INIT){
		temp = 2319;
		TEMP_SAMP_CNT++;
		if(TEMP_SAMP_CNT > TEMP_INIT_MAX){
			TEMP_SAMP_CNT = 0;
			TEMP_MODE = TEMP_MODE_NORMAL;
		}
	}else{
		TEMP_SAMP_CNT = 0;
		temp = (UNS16)(TEMP_SAMP_SUM / TEMP_MASK_MAX);
	}
	return temp;
}


/*****************************************************************************
 *	TEMP_RamInit:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Module initialization
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void TEMP_RamInit(void)
{
	(void)memset(&TempCtrlPkg, 0, sizeof(TempCtrlPkg));
}

/*****************************************************************************
 *	TEMP_Routine:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Module main loop
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void TEMP_Routine(void)
{
	uint16_t adc;

	/*Obtain ADC temperature value*/
	adc = ADC_GetChannelValue(ADC_BUF_NTC);
	adc = TEMP_Sampfilte(adc);
	/*Conversion temperature value*/
	TEMP_UPDATE= TEMP_ConvertAdc(adc);


	if(TEMP_MODE == TEMP_MODE_INIT){
		TEMP_UPDATE_CNT = 0;
		TEMP_DATA = TEMP_UPDATE;
	}else{
		if(TEMP_DATA != TEMP_UPDATE){
			TEMP_UPDATE_CNT++;
			if(TEMP_UPDATE_CNT > TEMP_UPDATE_MAX){
				TEMP_DATA = TEMP_UPDATE;
				TEMP_UPDATE_CNT = 0;
			}
		}else{
			TEMP_UPDATE_CNT = 0;
		}
	}
}

/*****************************************************************************
 *	TEMP_GetTempValue:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get temperature interface
 *
 *	RETURNS: S16
 *
 *****************************************************************************
 */
S16 TEMP_GetTempValue(void)
{

	return TEMP_DATA;
}













