/*
 * test.c
 *
 *  Created on: Mar 19, 2024
 *      Author: liuzhimin
 */


#include "main.h"
#include "../BLE/BLE.h"


UNS8 test_step = 0;
UNS8 diag_step = 0;

AXIS_DATA32BIT_T testacc[ACC_MAX];


/*****************************************************************************
 *	TSET_Read:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: test interface
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */

UNS8 TSET_Read(void)
{
	UNS16 AdConver ;
	switch(test_step){
	case 0:
		uint32_t uid[4];
		VER_GetChipId(uid);
		debug("id/%08X%08X%08X/\r\n",(int)uid[0],(int)uid[1],(int)uid[2]);
	break;
	case 1:
		printf("ves/%s\r\nv2es/%d%01d%01d-%s/\r\n",VERSION,YEAR,MONTH,DAY,BUILD__);
		break;
	case 2:debug("isofix_l/%d/%d/\r\n",AdKey_GetSwitchVal(ADKEY_SW3_ISOFIX0),ADC_GetChannelValue(ADC_BUF_ISOFIXR));break;
	case 3:debug("isofix_r/%d/%d/\r\n",AdKey_GetSwitchVal(ADKEY_SW3_ISOFIX1),ADC_GetChannelValue(ADC_BUF_ISOFIXL));break;
	case 4:debug("leg/%d/%d/\r\n",AdKey_GetSwitchVal(ADKEY_SW3_LEG),ADC_GetChannelValue(ADC_BUF_LEG));break;
	case 5:debug("front/%d/%d/\r\n",AdKey_GetSwitchVal(ADKEY_SW3_DIR0),ADC_GetChannelValue(ADC_BUF_DIRF));break;
	case 6:debug("back/%d/%d/\r\n",AdKey_GetSwitchVal(ADKEY_SW3_DIR1),ADC_GetChannelValue(ADC_BUF_DIRB));break;
	case 7:debug("lock/%d/\r\n",AdKey_GetSwitchVal(ADKEY_SW2_BUCKLE));break;
	case 8:debug("seat/%d/%d/\r\n",AdKey_GetSwitchVal(ADKEY_SW2_SOS),ADC_GetChannelValue(ADC_BUF_SOS));break;
	case 9:debug("head/%d/%d/\r\n",AdKey_GetSwitchVal(ADKEY_SW3_PILLOW),ADC_GetChannelValue(ADC_BUF_PILLOW));break;
	case 10:debug("5v/%d/%d/\r\n",AdKey_GetSwitchVal(ADKEY_SW2_5V),ADC_GetChannelValue(ADC_BUF_5V));break;
	case 11:
		UNS16 AdVal = ADC_GetChannelValue(ADC_BUF_VER);
		AdConver = ADC_GetChannelValue(ADC_BUF_BAT);
		UNS16 vBattMonData;
		GET_CH_VOLT(AdVal,AdConver,vBattMonData);
		vBattMonData = vBattMonData/3.127;
		debug("ver/%d/%d/\r\n",vBattMonData,ADC_GetChannelValue(ADC_BUF_VER));
	break;
	case 12:
		UNS16 BatVlot, BatVal;
		BatVal = ADC_GetChannelValue(ADC_BUF_BAT_E);
		AdConver = ADC_GetChannelValue(ADC_BUF_BAT);
		GET_CH_VOLT(BatVal,AdConver,BatVlot);
		debug("bat/%d/%d/\r\n",PWR_GetElecLevel(),BatVlot*2);//,ADC_GetChannelValue(ADC_BUF_BAT_E));

		break;
	case 13:debug("ntc/%d/%d/\r\n",TEMP_GetTempValue(),ADC_GetChannelValue(ADC_BUF_NTC));break;
	case 14:
		UNS16 res = DIAG_DiagAirBagRes();

		UNS16 ad =  DIAG_DiagAirBagAdc();
		debug("air/%d/%d/%d/\r\n",DIAG_GetAirBagFualt(),res,ad);
		break;
	case 15:

		AXIS_DATA_T axis[2];
		BOOL Asat= ACC_INT_IsStatusGood();
		ACCEL_GetDualSensorData(axis);
		printf("acc/%d/%d/%d/%d/%d/%d/%d/\r\n",Asat,
				axis[ACC_MASTER].X,axis[ACC_MASTER].Y,axis[ACC_MASTER].Z,
				axis[ACC_SLAVE].X,axis[ACC_SLAVE].Y,axis[ACC_SLAVE].Z);
		break;
	case 16:
		printf("key/%d/%d/\r\n",BSP_GPIO_Read(GPIO_IN_SW),BSP_GPIO_Read(GPIO_IN_TOUCH));
		break;

	default:test_step = 0;return TRUE;break;
	}
	test_step++;
 return FALSE;
}





typedef UNS8 (*FUNCTION_PTR)(UNS8 *);

UNS8 TEST_Not(UNS8 *buff);
UNS8 TEST_ReadID(UNS8 *buff);
UNS8 TEST_WriteID(UNS8 *buff);
UNS8 TEST_ReadSN(UNS8 *buff);
UNS8 TEST_WriteSN(UNS8 *buff);
UNS8 TEST_ReadSN2(UNS8 *buff);
UNS8 TEST_WriteSN2(UNS8 *buff);
UNS8 TEST_ReadAccData(UNS8 *buff);
UNS8 TEST_ReadBuildPrint(UNS8 *buff);
UNS8 TEST_ReadHardwareVer(UNS8 *buff);

UNS8 TEST_ReadAccOffsetFlash(UNS8 *buff);
UNS8 TEST_WriteAccOffsetFlash(UNS8 *buff);
UNS8 TEST_ReadAccOffsetReg(UNS8 *buff);
UNS8 TEST_WriteAccOffsetReg(UNS8 *buff);
UNS8 TEST_ReadPowerFlg(UNS8 *buff);
UNS8 TEST_ReadVer(UNS8 *buff);

UNS8 TEST_ReadSNChar(UNS8 *buff);
UNS8 TEST_WriteSNChar(UNS8 *buff);

typedef enum
{
	TEST_CMD_NOT = 0,
	TEST_CMD_READ_ID,
	TEST_CMD_WRITE_ID,
	TEST_CMD_READ_SN,
	TEST_CMD_WRITE_SN,
	TEST_CMD_READ_SN2,
	TEST_CMD_WRITE_SN2,
	TEST_CMD_READ_ACC_DATA,
	TEST_CMD_READ_POWERFLG,
	TEST_CMD_READ_VER,
	TEST_CMD_READ_ACC_OFFSET_FLASH,
	TEST_CMD_WRITE_ACC_OFFSET_FLASH,
	TEST_CMD_READ_ACC_OFFSET_REG,
	TEST_CMD_WRITE_ACC_OFFSET_REG,
	TEST_CMD_READ_SN_CHAR,
	TEST_CMD_WRITE_SN_CHAR,
	TEST_CMD_READ_BUILD_PRINT,
	TEST_CMD_READ_HARDWARE_VER,
	TEST_CMD_MAX,
} DEBUG_CMD_E;

static const FUNCTION_PTR TEST_ExecRtnTable[TEST_CMD_MAX] ={
	TEST_Not,
	TEST_ReadID,
	TEST_WriteID,
	TEST_ReadSN,
	TEST_WriteSN,
	TEST_ReadSN2,
	TEST_WriteSN2,
	TEST_ReadAccData,
	TEST_ReadPowerFlg,
	TEST_ReadVer,
	TEST_ReadAccOffsetFlash,
	TEST_WriteAccOffsetFlash,
	TEST_ReadAccOffsetReg,
	TEST_WriteAccOffsetReg,
	TEST_ReadSNChar,
	TEST_WriteSNChar,
	TEST_ReadBuildPrint,
	TEST_ReadHardwareVer,
};



/*****************************************************************************
 *	TSET_HandleEx:
 *
 *	PARAMETERS: UNS8*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Test the operation of the main module
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 TSET_HandleEx(UNS8* packet)
{
	UNS8 ret = TRUE;
	UNS8 cmdid = packet[TEST_CMD_NOT];
	if(cmdid < TEST_CMD_MAX){
		ret = (TEST_ExecRtnTable[cmdid](packet + TEST_CMD_READ_ID));
	}
	return ret;
}

/*****************************************************************************
 *	TSET_HandleEx:
 *
 *	PARAMETERS: UNS8*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: NOT
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 TEST_Not(UNS8 *buff)
{
	 return TRUE;
}

/*****************************************************************************
 *	TEST_ReadID:
 *
 *	PARAMETERS: UNS8*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Test the read id
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 TEST_ReadID(UNS8 *buff)
{
	UNS32 devid = MX_Flash_RD_DevID();
	debug("\r\nid$%010d$\r\n",(int)devid);
	return TRUE;
}

/*****************************************************************************
 *	TEST_WriteID:
 *
 *	PARAMETERS: UNS8*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Test the write id
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 TEST_WriteID(UNS8 *buff)
{
	UNS32 id = 0;
	id = buff[0];
	id |= (buff[1] << 8);
	id |= (buff[2] << 16);
	id |= (buff[3] << 24);
	/* Write SN to BLE module. */
	memcpy(deviceID, buff, 4);
	BtWakeTimeCnt = 1;
	LL_GPIO_ResetOutputPin(BT_WAKE_PB6_GPIO_Port, BT_WAKE_PB6_Pin); //Set Wake-Up pin to low
	txAtAdvdat.enable = 1;
	MX_Flash_WR_DevID(id);
	//UNS32 devid = MX_Flash_RD_DevID();
	//debug("\r\nid$%010d$\r\n",(int)devid);
	return TRUE;
}

/*****************************************************************************
 *	TEST_ReadSN:
 *
 *	PARAMETERS: UNS8*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Test the read sn
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 TEST_ReadSN(UNS8 *buff)
{
	UNS32 devsn = MX_Flash_RD_DevSN();
	debug("\r\nsn$%010d$\r\n",(int)devsn);
	return TRUE;
}

/*****************************************************************************
 *	TEST_WriteSN:
 *
 *	PARAMETERS: UNS8*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Test the write sn
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 TEST_WriteSN(UNS8 *buff)
{

	UNS32 sn = 0;
	sn = buff[0];
	sn |= (buff[1] << 8);
	sn |= (buff[2] << 16);
	sn |= (buff[3] << 24);
	MX_Flash_WR_DevSN(sn);
	debug("\r\nOK$\r\n");
	return TRUE;
}

/*****************************************************************************
 *	TEST_ReadSN2:
 *
 *	PARAMETERS: UNS8*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Test the read sn
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 TEST_ReadSN2(UNS8 *buff)
{
	UNS32 devsn = MX_Flash_RD_DevSN2();
	debug("\r\nsn2$%d$\r\n",(int)devsn);
	return TRUE;
}

/*****************************************************************************
 *	TEST_ReadSN2:
 *
 *	PARAMETERS: UNS8*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Test the write sn2
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 TEST_WriteSN2(UNS8 *buff)
{

	UNS32 sn = 0;
	sn = buff[0];
	sn |= (buff[0] << 8);
	sn |= (buff[0] << 16);
	sn |= (buff[0] << 24);
	MX_Flash_WR_DevSN2(sn);
	debug("\r\nOK$\r\n");
	return TRUE;
}

UNS8 TEST_ReadSNChar(UNS8 *buff)
{
	uint8_t buffer[20];
	MX_Flash_RD_SN_CharBuffer(buffer);
	debug("\r\nSN$%s$\r\n",buffer);
	return TRUE;
}
UNS8 TEST_WriteSNChar(UNS8 *buff)
{
	UNS8 SN_buffer[20];
	memcpy(SN_buffer,buff,20);
	if(0 == MX_Flash_WR_SN_CharBuffer(SN_buffer)){
		debug("OK$\r\n");
	}else{
		debug("ERR$\r\n");
	}
	return TRUE;
}

/*****************************************************************************
 *	TEST_ReadACC:
 *
 *	PARAMETERS: UNS8*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Test the acc data
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 TEST_ReadAccData(UNS8 *buff)
{


	AXIS_DATA_T axis[ACC_MAX];
	ACCEL_GetDualSensorData(axis);
#if 1
#define READ_ACC_OFFSET_MAX   32
	if(test_step < READ_ACC_OFFSET_MAX){
		testacc[ACC_MASTER].X += axis[ACC_MASTER].X;
		testacc[ACC_MASTER].Y += axis[ACC_MASTER].Y;
		testacc[ACC_MASTER].Z += axis[ACC_MASTER].Z;
		testacc[ACC_SLAVE].X += axis[ACC_SLAVE].X;
		testacc[ACC_SLAVE].Y += axis[ACC_SLAVE].Y;
		testacc[ACC_SLAVE].Z += axis[ACC_SLAVE].Z;
		test_step++;
	}else if(test_step >= READ_ACC_OFFSET_MAX){
		axis[ACC_MASTER].X = testacc[ACC_MASTER].X /READ_ACC_OFFSET_MAX;
		axis[ACC_MASTER].Y = testacc[ACC_MASTER].Y /READ_ACC_OFFSET_MAX;
		axis[ACC_MASTER].Z = testacc[ACC_MASTER].Z /READ_ACC_OFFSET_MAX;
		axis[ACC_SLAVE].X = testacc[ACC_SLAVE].X /READ_ACC_OFFSET_MAX;
		axis[ACC_SLAVE].Y = testacc[ACC_SLAVE].Y /READ_ACC_OFFSET_MAX;
		axis[ACC_SLAVE].Z = testacc[ACC_SLAVE].Z /READ_ACC_OFFSET_MAX;
		debug("acc$%d$%d$%d$%d$%d$%d$\r\n",
				(int)axis[ACC_MASTER].X,
				(int)axis[ACC_MASTER].Y,
				(int)axis[ACC_MASTER].Z,
				(int)axis[ACC_SLAVE].X,
				(int)axis[ACC_SLAVE].Y,
				(int)axis[ACC_SLAVE].Z
				);
		testacc[ACC_MASTER].X = 0;
		testacc[ACC_MASTER].Y = 0;
		testacc[ACC_MASTER].Z = 0;
		testacc[ACC_SLAVE].X = 0;
		testacc[ACC_SLAVE].Y = 0;
		testacc[ACC_SLAVE].Z = 0;

		test_step = 0;
		 return TRUE;
	}
	PWR_AccTestDelayCall();
	 return FALSE;
#else
#define READ_ACC_OFFSET_MAX   100
		debug("acc$%d$%d$%d$%d$%d$%d$\r\n",
				(int)axis[ACC_MASTER].X,
				(int)axis[ACC_MASTER].Y,
				(int)axis[ACC_MASTER].Z,
				(int)axis[ACC_SLAVE].X,
				(int)axis[ACC_SLAVE].Y,
				(int)axis[ACC_SLAVE].Z
				);
	 test_step++;
		if(test_step > READ_ACC_OFFSET_MAX){
			test_step = 0;
			 return TRUE;
		}
		PWR_AccTestDelayCall();
		 return FALSE;
#endif
}

/*****************************************************************************
 *	TEST_ReadAccOffsetFlash:
 *
 *	PARAMETERS: UNS8*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Test reading sensor offset flash
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */

UNS8 TEST_ReadAccOffsetFlash(UNS8 *buff)
{
	debug("Offset$%d$%d$%d$%d$%d$%d$\r\n",0,0,0,0,0,0);
	 return TRUE;
}

/*****************************************************************************
 *	TEST_WriteAccOffsetFlash:
 *
 *	PARAMETERS: UNS8*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Test writing sensor offset flash
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 TEST_WriteAccOffsetFlash(UNS8 *buff)
{

	debug("OK$\r\n");
	return TRUE;
}

/*****************************************************************************
 *	TEST_ReadAccOffsetReg:
 *
 *	PARAMETERS: UNS8*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Test reading sensor offset register
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 TEST_ReadAccOffsetReg(UNS8 *buff)
{
	debug("Offset$%d$%d$%d$%d$%d$%d$\r\n",0,0,0,0,0,0);
	 return TRUE;
}





/*****************************************************************************
 *	TEST_WriteAccOffsetReg:
 *
 *	PARAMETERS: UNS8*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Test writing sensor offset register
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 TEST_WriteAccOffsetReg(UNS8 *buff)
{
	if(buff[0] == 0x7f){
		buff[1] &= 0x0f;
		buff[2] &= 0x0f;
		buff[3] &= 0x0f;
		buff[4] &= 0x0f;
		buff[5] &= 0x0f;
		buff[6] &= 0x0f;
		debug("OK$\r\n");
	}
	 return TRUE;
}


/*****************************************************************************
 *	TEST_ReadPowerFlg:
 *
 *	PARAMETERS: UNS8*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Test the power record
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 TEST_ReadPowerFlg(UNS8 *buff)
{
	UNS32 pwr_on =	PWR_GetPowerSupplyOnfreq();
	debug("\r\npwr$%d$\r\n",(int)pwr_on);
	 return TRUE;
}


/*****************************************************************************
 *	TEST_ReadPowerFlg:
 *
 *	PARAMETERS: UNS8*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Test the version
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 TEST_ReadVer(UNS8 *buff)
{
	uint32_t uid[4];
	VER_GetChipId(uid);
	debug("ver$%08X%08X%08X%08X$%s%d%01d%01d-%s$\r\n",
			(int)uid[2],(int)uid[1],(int)uid[0],(int)uid[3],
			VERSION,YEAR,MONTH,DAY,BUILD__);
	 return TRUE;
}

/*****************************************************************************
 *	TEST_ReadBuildPrint:
 *
 *	PARAMETERS: UNS8*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Test the version
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 TEST_ReadBuildPrint(UNS8 *buff)
{
	UNS8 pass[13] = {0x59,0x58,0x32,0x30,0x32,0x34,0x52,0x44,0x56,0x45,0x52,0x00,0x00};//YX2024RDVER
	if(0 == memcmp(buff,pass,13)){
		uint32_t uid[4];
		VER_GetChipId(uid);
		debug("%08X%08X%08X%08X/%s%d%01d%01d-%s /%s\r\n",
				(int)uid[2],(int)uid[1],(int)uid[0],(int)uid[3],
				VERSION,YEAR,MONTH,DAY,BUILD__,
				HARDVERSION);
	}else{
		debug("Password error!\r\n");
	}

	return TRUE;
}
/*****************************************************************************
 *	TEST_ReadHardwareVer:
 *
 *	PARAMETERS: UNS8*
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Test the Hardware version
 *
 *	RETURNS: UNS8
 *
 *****************************************************************************
 */
UNS8 TEST_ReadHardwareVer(UNS8 *buff)
{
	UNS8 pass[13] = {0x32,0x30,0x32,0x34,0x52,0x44,0x56,0x45,0x52,0x00,0x00};//2024RDVER
	if(0 == memcmp(buff,pass,10)){
		debug("%s\r\n",HARDVERSION);
	}else{
		debug("Password error!\r\n");
	}

	return TRUE;
}
