/*
 * ver.c
 *
 *  Created on: Mar 7, 2024
 *      Author: liuzhimin
 */

#include "main.h"

#define VER_UID_ADDRESS 		0x08004000
#define VER_DATA_ADDRESS 		0x08003F90

#define VER_DATA_0_CFG  		0x73726576
#define VER_DATA_1_CFG  		0x3a6e6f69




volatile const char __RO_BL_INFO_DATE__[]    __attribute__((section(".bl_info.date"))) = { "version:"VERSION};
volatile const char __RO_BL_INFO_VERSION__[] __attribute__((section(".bl_info.ver"))) = {""__DATE__"/"__TIME__"-"BUILD__"//$"HARDVERSION};
volatile const char __RO_BL_INFO_DETAIL__[]  __attribute__((section(".bl_info.detailed"))) = {"end"};


#define VER_INIT(__NAME) \
	if (__RO_BL_INFO_VERSION__[0] == 'A' && __RO_BL_INFO_VERSION__[2] == 'A'\
			&& __RO_BL_INFO_DATE__[1] == 'A') {\
		__NAME = 0;\
	}\


typedef struct __attribute__((__packed__))
{
    uint32_t uid[4];
}ver_uid_t;

typedef struct __attribute__((__packed__))
{
    uint32_t data[2];
}ver_data_t;

volatile ver_uid_t *vercfg = (ver_uid_t *)VER_UID_ADDRESS;
volatile ver_data_t *verdatacfg = (ver_data_t *)VER_DATA_ADDRESS;


/*****************************************************************************
 *	VER_CRC32:
 *
 *	PARAMETERS: uint32_t crc_in, uint8_t byte
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Convert 32bitCRC
 *
 *	RETURNS: uint32_t
 *
 *****************************************************************************
 */
uint32_t VER_CRC32(uint32_t crc_in, uint8_t byte)
{
  uint32_t crc = crc_in;
  uint32_t in = byte | 0x100;

  do
  {
    crc <<= 1;
    in <<= 1;
    if(in & 0x100)
      ++crc;
    if(crc & 0x10000)
      crc ^= 0x1021;
  }

  while(!(in & 0x10000));

  return crc;
}
/*****************************************************************************
 *	VER_CRC:
 *
 *	PARAMETERS: uint32_t crc_in, uint32_t byte
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Convert CRC
 *
 *	RETURNS: uint32_t
 *
 *****************************************************************************
 */
uint32_t VER_CRC(const uint32_t* p_data, uint32_t size)
{
  uint32_t crc = 0;
  const uint32_t* dataEnd = p_data+size;

  while(p_data < dataEnd)
    crc = VER_CRC32(crc, *p_data++);

  crc = VER_CRC32(crc, 0);
  crc = VER_CRC32(crc, 0);

  return crc;
}


/*****************************************************************************
 *	VER_CONFG:
 *
 *	PARAMETERS: void
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Version and ID detection
 *
 *	RETURNS: TRUE OR  FALSE
 *
 *****************************************************************************
 */
UNS8 VER_CONFG(void)
{
	uint32_t uid[4] = {0};
	VER_INIT(uid[3]);
	uid[0] = (uint32_t)(READ_REG(*((uint32_t *)UID_BASE)));
	uid[1] = (uint32_t)(READ_REG(*((uint32_t *)(UID_BASE + 4U))));
	uid[2] = (uint32_t)(READ_REG(*((uint32_t *)(UID_BASE + 8U))));
	uid[3] = VER_CRC(uid,3);
	if((vercfg->uid[0] == uid[0])
		&&(vercfg->uid[1] == uid[1])
		&&(vercfg->uid[2] == uid[2])
		&&(vercfg->uid[3] == uid[3])){
		if((verdatacfg->data[0] == VER_DATA_0_CFG)
			&&(verdatacfg->data[1] == VER_DATA_1_CFG)){

			return TRUE;
		}
	}
	return FALSE;
}
/*****************************************************************************
 *	VER_GetChipId:
 *
 *	PARAMETERS: uint32_t *uid
 *
 *	CONSTRAINTS: None
 *
 *	DESCRIPTION: Get ID chip
 *
 *	RETURNS: void
 *
 *****************************************************************************
 */
void VER_GetChipId(uint32_t *uid)
{
	uid[0] = vercfg->uid[0];
	uid[1] = vercfg->uid[1];
	uid[2] = vercfg->uid[2];
	uid[3] = vercfg->uid[3];
}


